/*
 * $Id: rvananal.vs,v 1.9 93/11/02 16:13:52 josh Stab $
 *
 * This is a Viewscript encapsulation of the Vantage analzyer as applied
 * to cosimulation.  This requires that VANTAGE_VSS and PATH be set up
 * to look at the IR10/van_vss subdirectory off of $WDIR.  It also means
 * the library list is found in vsslib.ini.
 */

/*
 * For cosimulation, VANTAGE_VSS is set to $WDIR/van_vss.  However,
 * if there is no van_vss underneath one of the directories listed in
 * $WDIR, we use the environment variable VANTAGE_VSS, which the user
 * must set to run VSS 4.2 from Vantage proper (no cosim).  Note that
 * even if the user sets this, if there is a $WDIR/van_vss, we'll use
 * that to do cosimulation.
 */
if (!isbound (`VANTAGE_VSS, current_package ()) || (VANTAGE_VSS == nil)) {
    VANTAGE_VSS := vscript.vsSrchdirs ("van_vss", vscript.vsAttr.Dir);
    if (VANTAGE_VSS == nil)
        VANTAGE_VSS = vscript.vsSrchdirs ("IR10/van_vss", vscript.vsAttr.Dir);
    if (VANTAGE_VSS == nil)
        VANTAGE_VSS = vscript.getenv ("VANTAGE_VSS");

    if (VANTAGE_VSS == nil)
        error ("Could not find $WDIR/van_vss, or $VANTAGE_VSS", nil);

    /*
     * Likewise, we look for system libraries, in $WDIR/vhdllibs, provided
     * that directory exists.  Otherwise, we look in VANTAGE_VSS/pgm/libs,
     * however it was we got VANTAGE_VSS set up (see above).
     */
    SYS_PATH := vscript.vsSrchdirs ("vhdllibs", vscript.vsAttr.Dir);
    if ((SYS_PATH == nil) && (VANTAGE_VSS != nil))
        SYS_PATH = vscript.vsDnpath (vscript.vsDnpath (VANTAGE_VSS, "pgm"),
                                     "libs");

    /*
     * Finally, set up a new PATH environment variable, and in fact a
     * complete environment variable list for spawning vantage programs.
     */
    PATH := vscript.vsDnpath (vscript.vsDnpath (VANTAGE_VSS, "pgm"), "dir")
        + ":" + vscript.getenv ("PATH");
    SYS_ENV := list ("VANTAGE_VSS=" + VANTAGE_VSS,
                     "PATH=" + PATH,
                     "WDIR=" + vscript.getenv ("WDIR"));

    ld_path := vscript.getenv ("LD_LIBRARY_PATH");
    if (ld_path != nil)
        SYS_ENV = cons ("LD_LIBRARY_PATH=" + ld_path, SYS_ENV);

    define_enum := vcWv.MakeEnum;
    load ("vananalz.vs");

    function vssAnalyze() {
        /* Find a window, preferably the viewframe window */
        window := nil;
        region := nil;
        foreach (w; vcWin.Loop ()) {
            window = w;
            region = vcWin.OwnerRegion (w);
            if (vcRegion.Fac (region) == vframe.fac)
                break;
        } /* foreach */

        if (window != nil) {
            event := vcEvent.Create (0, 0, '0', "", window, region, nil);
            vcCmd.Run (analyze_without_setup_cmd, window, event, nil,
                       TRUE);
        } /* if */
    } /* function vssAnalyze */
} /* if */

vssAnalyze ();
