SOURCE_TIME             := 0;

LBR_FLAG_SOURCE_FILE    := 1;
LBR_FLAG_RCHK           := 2;
LBR_FLAG_DBGLVL         := 3;
LBR_FLAG_C_OPTLVL       := 4;
LBR_FLAG_VSS_VER_MAJOR  := 5;
LBR_FLAG_VSS_VER_MINOR  := 6;
LBR_FLAG_VHDL_OPTLVL    := 7;
LBR_FLAG_LANG           := 8;

MAX_LBR_FLAG            := 8;

VIF_TIME                := 9;
BIN_TIME                := 10;

NUM_TMS_FIELDS          := 11;

function read_lib_tms(fn) {
    tms := nil;
    if (fexists (fn)) {
        f := fopen (fn, "rb");

        /* The file starts with 4 bytes of magic */
        if ((fgetchar (f) != '\252') ||
            (fgetchar (f) != '\252') ||
            (fgetchar (f) != '\252') ||
            (fgetchar (f) != '\252'))
            error ("Bad magic number in library timestamp file", fn);

        version := fget32 (f, vlIntVT);
        if (version == 1) {
            tms = make_vector (NUM_TMS_FIELDS, nil);
            tms[SOURCE_TIME] = fget32 (f, vlIntVT);
            tms[BIN_TIME]    = fget32 (f, vlIntVT);
            tms[VIF_TIME]    = fget32 (f, vlIntVT);

            num_flags := fget32 (f, vlIntVT);
            data_length := fget32 (f, vlIntVT);
            declare flag_type;

            if (data_length == 0) {  /* older versions used this field */
                while ((flag_type = fget32 (f, vlIntVT)) > 0) {
                    if (flag_type == LBR_FLAG_SOURCE_FILE)
                        tms[flag_type] = fgetline (f);
                    else if ((flag_type > 0) && (flag_type <= MAX_LBR_FLAG))
                        tms[flag_type] = fget32 (f, vlIntVT);
                    else
                        /*
                         * Just silently fail.  The user *really* doesn't care!
                         * error ("invalid flag in tms file " + fn, flag_type)
                         */
                        ;
                } /* while */
            } /* if */
        } /* if */

        fclose (f);
    } /* if */

    tms;
} /* function read_lib_tms */

if (FALSE) {
    function tst(fn) {
        tms := read_lib_tms (fn);
        if (tms != nil) {
            function s(x)
                convert (vlStringVT, x);

            (" src=" + int_to_date (tms[SOURCE_TIME]) +
             " bin=" + int_to_date (tms[BIN_TIME]) +
             " vif=" + int_to_date (tms[VIF_TIME]) +
             " file=" + s(tms[LBR_FLAG_SOURCE_FILE]) +
             " range_check=" + s(tms[LBR_FLAG_RCHK]) +
             " c_opt_level=" + s(tms[LBR_FLAG_C_OPTLVL]) +
             " version=" + sprintf ("%d.%d",
                                    tms[LBR_FLAG_VSS_VER_MAJOR],
                                    tms[LBR_FLAG_VSS_VER_MINOR]) +
             " vhdl_opt_level=" + s(tms[LBR_FLAG_VHDL_OPTLVL]));
        } /* if */
        else
            nil;
    } /* function tst */
} /* if */
