/*
 *  $Revision: 1.4 $
 *
 *  The Core vcListEdit facility.
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

function dummy(args...);

/*\vscmuserDocpage vcListEdit.Cmd -- Create a command for use with vcListEdit

DESCRIPTION
     Like vcWv.Cmd, but you need only specify the parameters, using
     vcWv.Param.  The facility, function, name, and title are filled
     in for you.

SYNOPSIS
     cmd := vcListEdit.Cmd (
         vcWv.Param (...),
         vcWv.Param (...)...);
*/
function Cmd(args...)
    vcCmd.Create (fac, dummy, nil, nil, args);

wired_add := Add;

/*\vscmuserDocpage vcListEdit.Add -- Add a field value to the list editor

DESCRIPTION
     Used to programmatically preload values into the list editor.
     This routine is called once for each field in each row to receive
     initial values when a list editor is instantiated.  These values
     should be preloaded before the list editor is displayed in a window.

     ts and data specify the type and value of the field.  The remaining
     arguments can be omitted.  They are used to specify
     different facilities and/or preferences for specific fields of
     specific rows.

NOTE
     If any of the other vcListEdit routines are called while a
     row is incomplete, the row will be completed with default values.

     All facilities used should have a _Get method so that the values
     can be retrieved from the editor regions.

SYNOPSIS
     vcListEdit.Add (vcRegionVT list_editor, vscmVT value
                     [, vcFacilityVT fac] [, preferences...]);
*/
function Add(args...) {
    l :=        car (args);     args = cdr (args);
    data :=     car (args);     args = cdr (args);
    fac := nil;
    if ((args != nil) && istype (car (args), vcFacilityVT)) {
        fac = car (args);
        args = cdr (args);
    } /* if */
    wired_add (l, data, fac, args);
} /* function Add */
        
function rfunc(routine) func (l, e) routine (vcEvent.Region (e));;

vcBind.Set (bindings, '\f16', vcWv.Cmd (fac, rfunc (Copy), "copy"));
vcBind.Set (bindings, '\f18', vcWv.Cmd (fac, rfunc (Paste), "paste"));
vcBind.Set (bindings, '\f20', vcWv.Cmd (fac, rfunc (Cut), "cut"));
vcBind.Set (bindings, '\delete', vcWv.Cmd (fac, rfunc (Delete), "delete"));
