/*
 *  $Revision: 1.19 $
 *
 *  Viewscript UI definitions for the Core vcPicklist facility.
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

armed_bindings := vcBind.Create ();

declare save_bindings;

activate_cmd := vcWv.Cmd (fac,
                          func(picklist, ev)
                          {
                              highlight (picklist, ev);
                              r := vcEvent.Region (ev);
                              vcRegion.SetGrab (r, r);
                              save_bindings = vcRegion.Bindings (r);
                              vcRegion.SetBindings (r, armed_bindings);
                          }, "Highlight");

fire_cmd := vcWv.Cmd (fac,
                      func (picklist, ev)
                      {
                          r := vcEvent.Region (ev);
                          vcRegion.SetGrab (r, nil);
                          vcRegion.SetBindings (r, save_bindings);
                          save_bindings = nil;
                          set (picklist, ev);
                      }, "Select");

function bind(key, action) {
    vcBind.Set (bindings, key, action);
    vcBind.Set (ReturnBindings, key, action);
} /* function bind */

bind ('\leftdown', activate_cmd);
bind ('\rightdown', activate_cmd);
vcBind.Set (armed_bindings, '\leftup', fire_cmd);
vcBind.Set (armed_bindings, '\rightup', fire_cmd);
vcBind.Set (armed_bindings, '\move', vcWv.Cmd (fac, highlight));

right_cmd := vcWv.Cmd (fac, right, "right");
up_cmd := vcWv.Cmd (fac, up, "up");
left_cmd := vcWv.Cmd (fac, left, "left");
down_cmd := vcWv.Cmd (fac, down, "down");
dblclick_cmd := vcWv.Cmd (fac, dblclick, "dblclick");
home_cmd := vcWv.Cmd (fac,
                      func(p, e) {
                          declare nrows, ncols;
                          Dimensions (vcEvent.Region (e), &nrows, &ncols);
                          if (ncols == 1)
                              SetPick (vcEvent.Region (e), 0);
                      },
                      "home");
end_cmd := vcWv.Cmd (fac,
                     func(p, e) {
                         declare nrows, ncols;
                         Dimensions (vcEvent.Region (e), &nrows, &ncols);
                         if (ncols == 1)
                             SetPick (vcEvent.Region (e), nrows - 1);
                     },
                     "end");
space_cmd := vcWv.Cmd (fac, space, "space");

bind ('\right', right_cmd);
bind ('\^F', right_cmd);
bind ('\left', left_cmd);
bind ('\^B', left_cmd);
bind ('\up', up_cmd);
bind ('\^P', up_cmd);
bind ('\down', down_cmd);
bind ('\^N', down_cmd);
bind ('\home', home_cmd);
bind ('\end', end_cmd);
bind (' ', space_cmd);

vcBind.Set (ReturnBindings, '\enter', dblclick_cmd);
vcBind.Set (ReturnBindings, '\leftdbl', dblclick_cmd);
vcBind.Set (ReturnBindings, '\rightdbl', dblclick_cmd);

function Scroll3(args...) Scroll3VA (args);

/*\vscmuserDocpage vcPicklist.Items -- iterator to walk through picklist items

DESCRIPTION
     Used to execute the body of a foreach on each item in a picklist.

SYNOPSIS
     foreach (row, col, index, is_set, visual, data; vcPicklist.Items (pl))
         ...

     The visual is either a region or a string, depending on the
     picklist.  Blank entries of a 2-D picklist will not be iterated.
*/
iterator Items(picklist) {
    declare n_rows, n_cols, r, c, i, item;
    Dimensions (picklist, &n_rows, &n_cols);
    i = 0;
    for (r = 0; r < n_rows; ++r)
        for (c = 0; c < n_cols; ++c) {
            item = vcPicklist.Get (picklist, r, c);
            if (item != nil)
                iterate (r, c, i, vcPicklist.IsSet (picklist, i),
                         car (item), cdr (item));
            ++i;
        } /* for */
} /* iterator Items */
