/*
 * Name:     VDRAWBT.VS
 * Facility: VDRAW
 * 
 * Copied initially from vplace.vs
 *
 * Functional Description: 
 *    Viewscript file for Viewdraw
 *
 * Environment:
 *     SunOS 4.1 & CORE 5.0
 *     
 * Author:  
 *   Ron Maxwell, February 12, 1991  (vplace.vs)
 *
 * Modified by: 
 * Who            Version   When      What
 *
 * PJS
 * 
 * (c) Copyright 1991, 1992 by Viewlogic Systems, Inc.
 */

/* 
 * Viewdraw 5.0 Globals.  All are declared in vdraw.vs but are assigned
 * values here for easier user customization.
 *
 * Fast draw mode, center zoom area on Zoom In (but not Zoom Out).
 */

if (read_only() != vlTrueFalse.True)
    enable_fast_mode    = 1;
else
    enable_fast_mode    = 0;

zoom_in_and_center  = YES;
auto_scroll_enabled = YES;

move_warp_cursor    = 5;        /* 5 - Center cursor in bounding box. */
copy_warp_cursor    = move_warp_cursor;
add_warp_cursor     = 0;
bpaste_warp_cursor  = 0;
rotate_warp_cursor  = -1;
scale_warp_cursor   = 5;
pan_warp_cursor     = NO;   /* Set to YES for warping of cursor to middle */

enable_gestures     = 1;  /* -1 for middle, 1 for shift middle.  0 for off. */

/*
 * GESTURE BINDINGS
 */

/* Refer to vcGesture.vs for values of specific gestures. */

vcGesture.Set (gestures, vcGesture.Z,    zoom_to_area_cmd);
vcGesture.Set (gestures, vcGesture.Line, zoom_to_area_cmd);
vcGesture.Set (gestures, vcGesture.O,    out_menu_cmd);
vcGesture.Set (gestures, vcGesture.M,    move_cmd);
vcGesture.Set (gestures, vcGesture.C,    copy_cmd);
vcGesture.Set (gestures, vcGesture.P,    full_cmd);

/* vcGesture.Set (gestures, 0x2233, vcWin.wclose); */
/* vcGesture.Set (gestures, 0x2211, in_cmd); */
/* vcGesture.Set (gestures, 0x3311, scroll_cmd); */
/* vcGesture.Set (gestures, 0x1211, delete_cmd);  */ /* Check mark */
/* vcGesture.Set (gestures, 0x2122, pan_cmd); */     /* P (more or less) */
/* vcGesture.Set (gestures, 0x2211, undo_cmd); */    /* U (more or less) */
/* vcGesture.Set (gestures, 0x2133, redo_cmd); */

/*
 * Keyboard accelerators (key bindings)
 */

vcBind.Set (bindings, 'a',   attr_kbd_cmd);
vcBind.Set (bindings, 'A',   arc_cmd);
vcBind.Set (bindings, '\^A', cattr_all_cmd);
vcBind.Set (bindings, '\_a', add_popup_cmd);
vcBind.Set (bindings, 'b',   bus_cmd);
vcBind.Set (bindings, 'B',   box_cmd);
vcBind.Set (bindings, 'c',   copy_cmd);
vcBind.Set (bindings, 'C',   circle_cmd);
/* Don't assign anything to Control-C ('\^C') */
vcBind.Set (bindings, '\_c', change_popup_cmd);
vcBind.Set (bindings, 'd',   delete_cmd);
vcBind.Set (bindings, '\delete',   delete_cmd);
/* vcBind.Set (bindings, 'D',   delete_cmd); */
vcBind.Set (bindings, '\_e', edit_popup_cmd);
/* vcBind.Set (bindings, 'f',   noop_cmd); */
vcBind.Set (bindings, '\^F', fast_draw_cmd);
vcBind.Set (bindings, '\_f', file_popup_cmd);
vcBind.Set (bindings, '\_a', add_popup_cmd);
vcBind.Set (bindings, 'i',   do_comp_cmd);
vcBind.Set (bindings, '\_i', info_popup_cmd); 
/* vcBind.Set (bindings, 'k',   ktext_cmd); */
vcBind.Set (bindings, 'l',   label_kbd_cmd);
vcBind.Set (bindings, 'L',   line_cmd);
vcBind.Set (bindings, '\^L', levelpush_cmd);
vcBind.Set (bindings, '\_l', level_popup_cmd);
vcBind.Set (bindings, 'm',   move_cmd);
vcBind.Set (bindings, 'n',   net_cmd);
vcBind.Set (bindings, 'o',   oattr_kbd_cmd);
vcBind.Set (bindings, 'p',   pin_cmd);
vcBind.Set (bindings, '\_p', props_popup_cmd);
vcBind.Set (bindings, '\_s', select_popup_cmd);
vcBind.Set (bindings, 'r',   redo_cmd);
vcBind.Set (bindings, 's',   save_cmd);
vcBind.Set (bindings, 'S',   string_cmd);
vcBind.Set (bindings, 't',   text_cmd);
vcBind.Set (bindings, '\_t', text_popup_cmd);
vcBind.Set (bindings, 'u',   undo_cmd);
vcBind.Set (bindings, '\_u', utils_popup_cmd);
vcBind.Set (bindings, '\_v', view_popup_cmd);
vcBind.Set (bindings, 'w',   write_cmd);
vcBind.Set (bindings, 'W',   writeto_cmd);
vcBind.Set (bindings, 'z',   zoom_cmd);
vcBind.Set (bindings, 'Z',   zselect_menu_cmd);

/*
 * FUNCTION KEY BINDINGS
 */

/* vcBind.Set (bindings, '\F4',  scroll_cmd); */ /* CORE uses F4 */
vcBind.Set (bindings, '\F5',  refresh_cmd);
vcBind.Set (bindings, '\F6',  pan_cmd);
/* vcBind.Set (bindings, '\F7',  in_cmd); */
vcBind.Set (bindings, '\F7',  in_menu_cmd);
/* vcBind.Set (bindings, '\F8',  out_cmd); */
vcBind.Set (bindings, '\F8',  out_menu_cmd);
vcBind.Set (bindings, '\F9',  zoom_cmd);
vcBind.Set (bindings, '\F0',  full_cmd);

vcBind.Set (bindings, '\#\F9', store_region_cmd);
vcBind.Set (bindings, '\#\F0', recall_region_cmd);

vcBind.Set (bindings, '\#\F1',  in_and_center_cmd);

/* Setup keys for display refresh interruption
 *
 * F4 == scroll
 * F5 == refresh
 * F6 == pan
 * F7 == in
 * F8 == out
 * F9 == zoom
 * F0 (F 10) == full
 */
vcWv.Interrupts (fac, '\e', '\F4', '\F5', '\F6', '\F7', '\F8', '\F9', '\F0');

/*
 * MOUSE BUTTON BINDINGS 
 */

/* CFI sez:  Control == Add and Shift == Un */

vcBind.Set (bindings, '\leftdown',   select_cmd);     /* leftdown */
vcBind.Set (bindings, '\^\leftdown', addselect_cmd);  /* Control leftdown */
vcBind.Set (bindings, '\#\leftdown', unselect_cmd);   /* Shift leftdown */
vcBind.Set (bindings, '\rightdown',  (  read_only() == vlTrueFalse.True
                                      ? VO_popup_cmd
                                      : sch_popup_cmd));

function init_middle_button()
{
    if (enable_gestures == 1) {

        vcBind.Set (bindings, '\#\middledown', vcGesture.StartCmd);

        if (enable_fast_mode == 1) {

            vcBind.Set (bindings, '\middledown',     do_fast_draw_cmd);
            /* vcBind.Set (bindings, vcGesture.Default, do_fast_draw_cmd); */
            vcGesture.Set (gestures, vcGesture.Default, do_fast_draw_cmd);
        }
        else {

            vcBind.Set (bindings, '\middledown',     noopmode_cmd);
            /* vcBind.Set (bindings, vcGesture.Default, noopmode_cmd); */
            vcGesture.Set (gestures, vcGesture.Default, noopmode_cmd);
        }
    }
    else if (enable_gestures == -1) {

        if (enable_fast_mode == 1)
            /* vcBind.Set (bindings, vcGesture.Default, do_fast_draw_cmd); */
            vcGesture.Set (gestures, vcGesture.Default, do_fast_draw_cmd);
        else
            /* vcBind.Set (bindings, vcGesture.Default, noopmode_cmd); */
            vcGesture.Set (gestures, vcGesture.Default, noopmode_cmd);

        vcBind.Set (bindings, '\middledown',   vcGesture.StartCmd);
        vcBind.Set (bindings, '\#\middledown', vcGesture.StartCmd);
    }
    else {

        if (enable_fast_mode == 1)
            vcBind.Set (bindings, '\middledown', do_fast_draw_cmd);
        else
            vcBind.Set (bindings, '\middledown', noopmode_cmd);

        vcBind.Set (bindings, '\#\middledown', noop_cmd);
    }

    cmd_sch = vcBind.Get (bindings, '\middledown');

} /* function init_middle_button */

init_middle_button ();

vcBind.Set (bindings, '\^\middledown', net_cmd);
