/* ------------------------------------------------------------------------ *\
 * $Id: vnavtop.vs,v 1.9 92/11/14 15:30:03 bobdavis Stab Locker: dave $
 * The definition of the pick facility for the top level window --
 * this facility appears twice there, once for the schematic, once for
 * "remote tools".
 *
 * Author: Bob Davis for George Deak
 * Just ask George.
 *
 * Copyright (c) 1992 by Viewlogic Systems, Inc.
 * change log omitted to avoid huge comments
\* ------------------------------------------------------------------------ */
function
_Create(args, region)
{
    Context := struct { Title; Nav; Picklist;
                        TheList; Index; SelItem; HasBeenFound };

/*set_stdout(TRUE);*/
/*printf("args:%s\n", args);*/
/*set_stdout(FALSE);*/

    Context.Title = car(args);
    Context.Nav = cadr(args);
    Context.TheList = caddr(args);
    Context.Picklist = nil;

/*printf ("Context.TheList: %s\n", Context.TheList);*/

    /* these will be changed by vcPicklistNotify & SetSelectedString */
    Context.Index = 0;
    Context.SelItem = car(Context.TheList);
    Context.HasBeenFound = FALSE;

    /* set up the title */
    Title := vcRegion.Create("", vcLabel.fac, Context.Title);
    SubTitle := vcRegion.Create("", vcLabel.fac, "Type: Item");
    vcRegion.Font(SubTitle, vgFont.mediumb);

    /* set up the buttons */
    AttachedB := vcWv.LabelButtonCreate("Attached", "Attached",
                                        nil, AttachedCmd, nil);

    EquivalentB := vcWv.LabelButtonCreate("Equivalent", "Equivalent",
                                          nil, EquivalentCmd, nil);

    SheetsOrContentsB := vcWv.LabelButtonCreate("Sheets", "Sheets",
                                                nil, SheetsCmd, nil);

    VisitB := vcWv.LabelButtonCreate("Visit", "Visit(next)",
                                     nil, VisitCmd, nil);

    /* Cancel never does anything -- they're here to be greyed out
     * so the ui of the top picklist looks similar to that for normal
     * picklists.
     */
    CancelB := vcWv.LabelButtonCreate("Cancel", "Cancel",
                                      nil, CancelCmd, nil);
    vcRegion.Enable(CancelB, FALSE, FALSE);

    HelpB := vcWv.LabelButtonCreate("Help", "Help",
                                    nil, HelpCmd, nil);
    /*vcRegion.Enable(HelpB, FALSE, FALSE);*/

    Buttons := vcTable.Make("buttons", 2, 3,
                            AttachedB, EquivalentB, SheetsOrContentsB,
                            VisitB, CancelB, HelpB);

    /* create picklist region */
    PickSelector := List2Picklist(Context.TheList, Context,
                                  Buttons);

    /* the table
     * find-cancel-help on the last line similar to the arrangements
     * of other dialogs.
     */
    ListSelector := vcTable.Make("listselector", 4, 1,
                                 Title,
                                 SubTitle,
                                 PickSelector,
                                 Buttons);

    vcRegion.AddChild(region, ListSelector);

    return(Context);
} /* _Create */

function
Update(Context, region, TheList)
{
/*set_stdout(TRUE);*/
/*printf("Update\n");*/
/*printf("Context:%s\n", Context);*/
/*printf("region:%s\n", region);*/
/*printf("TheList:%s\n", TheList);*/
/*set_stdout(FALSE);*/

    Context.TheList = TheList;

    /* may need to recreate the pick selector */
    PickSelector := vcTable.Get(region, 2, 0);

    Buttons := vcTable.Get(region, 3, 0);

    NewPicklist := List2Picklist(TheList, Context, Buttons);

    vcTable.Set(region, 2, 0, NewPicklist);
} /* Update */

function
List2Picklist(TheList, Context, Buttons)
{
    AttachedB := vcTable.Get(Buttons, 0, 0);
    EquivalentB := vcTable.Get(Buttons, 0, 1);
    SheetsOrContentsB := vcTable.Get(Buttons, 0, 2);
    VisitB := vcTable.Get(Buttons, 1, 0);

    declare NArray;
    Picklist := vcRegion.Create("picklist", vcPicklist.fac, nil,
                                rows <- 10,
                                columns <- 1,
                                exclusive <- TRUE,
                                enhanced_notify <- TRUE,
                                will_scroll <- TRUE);
    {
        item := TheList;
        declare stuff;
        NArray = 0;
        while (item != nil) {
            stuff = car(item);
/*set_stdout(TRUE);*/
/*printf("item:%s\n", item);*/
/*printf("stuff:%s\n", stuff);*/
/*printf("car(stuff):%s\n", car(stuff));*/
/*printf("cadr(stuff):%s\n", cadr(stuff));*/
/*set_stdout(FALSE);*/
            vcPicklist.Add(Picklist,
                           sprintf("%s: %s",
                                   vnav.FindType2String(cadr(stuff)),
                                   car(stuff)),
                           stuff);
            item = cdr(item);
            ++NArray;
        } /* while */
    }
    /* upon calls via vcPicklistNotify, this updates the PicklistContext stuff
     * above so the correct item is indicated.
     */
    vcPicklist.Notify(Picklist, func(Stuff, on_or_off, index) {
        if (on_or_off) {
            Context.Index = index;
            Context.SelItem = Stuff;
            Context.HasBeenFound = FALSE;
        } /* if */
        GreyButtons(Stuff, Buttons,
                    AttachedB, EquivalentB, SheetsOrContentsB, VisitB);
    });
    GreyButtons(car(Context.TheList), Buttons,
                AttachedB, EquivalentB, SheetsOrContentsB, VisitB);
    Context.Picklist = Picklist;

    ideal := NArray;
    min := 4;                   /* min could be a pref I suppose */
    if (ideal < min)
        ideal = min;
    PickSelector := vcPicklist.Scroll("pickselector", Picklist,
                                      min, ideal, 0);

    /* not a basketball technique */
    vcPicklist.SetPick(Picklist, 0);

    PickSelector;
} /* List2Picklist */

function
GreyButtons(TheItem, Buttons,
            AttachedB, EquivalentB, SheetsOrContentsB, VisitB)
{
    Attached := FALSE;
    Equivalent := FALSE;
    SheetsOrContents := FALSE;
    Visit := FALSE;

    if (TheItem != nil) {
        Visit = TRUE;

        FindType := cadr(TheItem);

        if (FindType == vnav.vlnFindType.Net) {
            Attached = TRUE;
            Equivalent = TRUE;
            SheetsOrContents = TRUE; /* sheets */

            SheetsOrContentsB = vcWv.LabelButtonCreate("Sheets", "Sheets",
                                                       nil, SheetsCmd, nil);
        } /* if */
        else if (   FindType == vnav.vlnFindType.Component
                 || FindType == vnav.vlnFindType.Component_PinName
                 || FindType == vnav.vlnFindType.RefDes_PinNumber) {
            Attached = TRUE;

            SheetsOrContentsB = vcWv.LabelButtonCreate("Sheets", "Sheets",
                                                       nil, SheetsCmd, nil);
        } /* else if */
        else if (FindType == vnav.vlnFindType.RefDes) {
            Attached = TRUE;
            SheetsOrContents = TRUE; /* contents */

            SheetsOrContentsB = vcWv.LabelButtonCreate("Contents", "Contents",
                                                       nil, ContentsCmd, nil);
        } /* else if */
    } /* if */

    /* last arg of these indicates whether or not to refresh */
    Refresh := TRUE;
    vcRegion.Enable(AttachedB, Attached, Refresh);
    vcRegion.Enable(EquivalentB, Equivalent, Refresh);
    vcRegion.Enable(SheetsOrContentsB, SheetsOrContents, Refresh);
    vcRegion.Enable(VisitB, Visit, Refresh);

    vcTable.Set(Buttons, 0, 2, SheetsOrContentsB);
} /* GreyButtons */

/* ---------------- */

function
Visit(Context, Event)
{
    /*DumpIt("Visit", Context);*/

    if (Context.HasBeenFound)
        vcPicklist.SetPick(Context.Picklist, Context.Index + 1);
    Context.HasBeenFound = TRUE;

    vnav.DoVisit(Context.Nav, cadr(Context.SelItem), car(Context.SelItem));
} /* Visit */

VisitCmd := vcWv.Cmd(fac, Visit, "Visit");

function
Attached(Context, Event)
{
    /*DumpIt("Attached", Context);*/

    vnav.AttachedFcn(Context.Nav, vcEvent.Window(Event),
                     cadr(Context.SelItem), car(Context.SelItem));
} /* Attached */

AttachedCmd := vcWv.Cmd(fac, Attached, "Attached");

function
Equivalent(Context, Event)
{
    /*DumpIt("Equivalent", Context);*/

    if (cadr(Context.SelItem) == vnav.vlnFindType.Net)
        vnav.EquivalentFcn(Context.Nav, vcEvent.Window(Event),
                           car(Context.SelItem));
} /* Equivalent */

EquivalentCmd := vcWv.Cmd(fac, Equivalent, "Equivalent");

function
Sheets(Context, Event)
{
    /*DumpIt("Sheets", Context);*/

    if (cadr(Context.SelItem) == vnav.vlnFindType.Net)
        vnav.SheetsFcn(Context.Nav, vcEvent.Window(Event),
                       car(Context.SelItem));
} /* Sheets */

SheetsCmd := vcWv.Cmd(fac, Sheets, "Sheets");

function
Contents(Context, Event)
{
    /*DumpIt("Contents", Context);*/

    if (cadr(Context.SelItem) == vnav.vlnFindType.RefDes)
        vnav.ContentsFcn(Context.Nav, vcEvent.Window(Event),
                         car(Context.SelItem));
} /* Contents */

ContentsCmd := vcWv.Cmd(fac, Contents, "Contents");

/* ---------------- */

CancelCmd := vcWv.Cmd(fac, func(d,e);, "Cancel");

HelpCmd := vcWv.Cmd(fac, func(d,e)
                    vcAlert.SpawnHelp(vcEvent.Window(e),
                                      "vnav",
                                      "The Rest of the Top Window");,
                    "Help");

/* ---------------- */

function
EnableCommands(Loaded,          /* loaded a design or not */
               Pcb)             /* pcb mode */
{
    vcCmd.Enable(VisitCmd, Loaded);
    vcCmd.Enable(AttachedCmd, Loaded);
    vcCmd.Enable(EquivalentCmd, Loaded);
    vcCmd.Enable(SheetsCmd, Loaded);
    vcCmd.Enable(ContentsCmd, Loaded && Pcb);
} /* EnableCommands */

/* ---------------- */

MAXCOL := 78;

function
DumpIt(str, Context)
{
    declare col, length;
    declare This, ThisStr;
    Item := Context.TheList;

/*printf("Context %s\n", Context);*/
/*printf("Context.Title %s\n", Context.Title);*/
/*printf("Context.TheList %s\n", Context.TheList);*/
/*printf("Context.SelItem %s\n", Context.SelItem);*/
/*printf("Item %s\n", Item);*/

    printf("%s\n", str);
    printf("{\n   ");

    col = 4;                    /* indent level */
    while(1) {
        This = car(Item);
        ThisStr = sprintf("(%s:%s)",
                          vnav.FindType2String(cadr(This)), car(This));
        length = string_length(ThisStr);
        col += length + 1;
        if (col > MAXCOL) {
            printf("\n    ");
            col = 4 + length;
        } /* if */
        else
            printf(" ");
        printf(ThisStr);

        Item = cdr(Item);
        if (!Item)
            break;
    } /* while(1) */
    printf("\n}\n");

    printf("%d %s\n", Context.Index, Context.SelItem);
} /* DumpIt */

load("pickht.vs");              /* help text for buttons */
