/*
 *  $Revision: 1.67.2.2 $
 *
 *  Alert Box facility.
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 */

/* try to prevent stray double-clicks from activating the OK button */
vcBind.Set (bindings, '\leftdbl', vcWv.Cmd (fac, func(d,e);));

trackitdown_cmd := vcWv.Cmd (fac, func (alert, event) _trackitdown (alert););
vcBind.Set (bindings, 't',   trackitdown_cmd);
vcBind.Set (bindings, '\^T', trackitdown_cmd);
vcBind.Set (bindings, 'v',   trackitdown_cmd);
vcBind.Set (bindings, '\^V', trackitdown_cmd);

/*
 * We need to have the vcAlert.AlertCancel command defined for
 * upward compatibility of macros recorded before this button ran
 * the vcWin.close command.
 */
vcWv.Cmd (fac, func (alert, event)
                   vcWin.Close (vcEvent.Window (event));,
          "AlertCancel");
vcBind.Set (bindings, 'c', vcWin.wclose);
vcBind.Set (bindings, '\^C', vcWin.wclose);

/*
 * We need to register this command with the alert box C code for
 * macro recordability based on commands.
 */
SetCloseCommand (vcWin.wclose);

helpitout_cmd := vcWv.Cmd (fac, func (alert, event) _helpitout (alert););
vcBind.Set (bindings, 'h',   helpitout_cmd);
vcBind.Set (bindings, '\^H', helpitout_cmd);
vcBind.Set (bindings, '?',   helpitout_cmd);

/*
 * This variable wants to be global to track, track_edit, etc.
 *
 * It is also advertised, through the man page for AddVisit.
 * DO NOT RENAME IT!!!
 */
declare last_regexp;


/*
 * Default function which gets called when a user hits the Track button.
 * Note the parameter type declarations, which will force ViewScript to
 * perform type checking.  The `data' parameter is not typed, and is
 * reserved for future extension of this protocol.
 *
 * RETURN VALUE should be nil on success, otherwise the number of the
 * error which occurred..
 */
function track(vlStringVT app, integer sev, integer num, vlStringVT loc, data)
{
    declare l, return_value;

    /*printf ("app=%s, sev=%s, num=%s, loc=\"%s\"\n", app, sev, num, loc);*/

    /*
     * Make sure we have a trackable location here, before anything else.
     */
    if (string_eq (loc, ""))
        return_value = 1111; /* Not a trackable location */

    /*
     * If ViewScript caused this error, call Viewbugger!
     */
    else if (string_eq (app, "vscript")) {
        reg := vcRegion.Create ("ViewScript Debugger", vcVbg.fac, nil);
        vcWin.Create (reg, vcWin.NoMacros);
        return_value = nil; /* success */
    }

    /*
     * Otherwise look up in the assoc list `track_alist' for a
     * regexp matching `loc', and call the corresponding track
     * function.
     */
    else {
        for (l = track_alist;
             (l != nil) &&
             (vscript.re_exec(last_regexp = vscript.re_compile(caar(l)),loc)
                  == FALSE);
             l = cdr (l))
            ;

        if (l == nil)
            return_value = 1111; /* Not a trackable location */
        else {
            /*printf ("We got a match!\n");*/
            /*printf ("l=%s\n", l);*/
            /* Note that we have to use this intermediate var `funk' */
            funk := cadar (l);
            return_value = funk (app, sev, num, loc, data);
        }
    }

    return_value;
    /* DON'T PUT ANY CODE HERE -- the return value indicates success */

} /* track */


/*
 * track_vtext, vt_window_death -- support for tracking errors using
 *     a vtext window.
 */
declare vt, vt_win;

function vt_window_death(win) {
    vt_win = nil;
    vt = nil;
}

function track_vtext(app, sev, num, loc, data) {
    /*
     * Get the strings matching the first and second subexpressions,
     * which are the filename and linenumber respectively.
     * Note: `last_regexp' was set inside function track.
     */
    filename := vscript.re_getmatch (last_regexp, 1);
    linenum := vscript.re_getmatch (last_regexp, 2);

    if (vt == nil) {
        vt = vcRegion.Create ("error tracker", vtx.fac, filename,
                              load_file <- TRUE,
                              height <- 20);
        vt_win = vcWin.Create (vt);
        vcWin.DestroyNotify (vt_win, vt_window_death);
    }
    else if (vtx.GetBufferName (vt) != filename)
        vtx.Load (vt, filename);
    if (linenum != nil) {
        vtx.SelectLine (vt, linenum);
        vtx.Update (vt);
    }

    /*
     * Return value is important!
     */
    nil;

} /* track_vtext */


/*
 * track_vnav -- support for tracking errors that are best handled
 *     by the Navigator.
 *
 * HACK_ALERT: The error handling within this and other track_*
 * functions is horrendous due to the fact that vscript support for
 * error reporting is so lame.
 *
 * HACK_ALERT: The navigator *may* be sold separately.  In that case
 * we'd need to grey out the "Visit(Next)" button for those messages
 * which can't be tracked without the navigator.  To do this we
 * would need to:
 *     1.  Implement a vcAlert.IsTrackable() function here, and call
 *         it from alert.c:is_trackable().
 *     2.  Attempt once to contact the navigator through IPC, then
 *         try to spawn it.  If we don't make contact, then all
 *         future calls to IsTrackable() should return FALSE for
 *         navigator-type messages.  The code to spawn the navigator
 *         should be here and NOT inside IsTrackable() because this
 *         function needs to return in a reasonable time, and after
 *         we spawned the navigator, we would have to wait for it to
 *         come up, then send it this error location.  It would be
 *         OK for IsTrackable() to return TRUE the first time even
 *         though there may not be a navigator.
 */
vnav_cntx := nil;
tried_to_start_vnav := FALSE;
function track_vnav(app, sev, num, loc, data) {
    return_value := nil;

    /*
     * Get the strings matching the second and third subexpressions,
     * which are the design name and location info respectively.
     * Note: `last_regexp' was set inside function track.
     */
    design_name := vscript.re_getmatch (last_regexp, 2);
    location_info := vscript.re_getmatch (last_regexp, 3);

    /*
     * Time to get more picky about the format of the location info.
     * Until now, we've just verified that it looks kind of like a
     * ViewDraw object.  We do this checking here so that we can
     * report a better error message.
     */
    VerifyVdrawLocation (location_info, loc);
    /* NO RETURN if not valid location */

    /***if (vipc.CntxGetTid (vipc.GetCntxDflt ()) == vipc.TidNAVIGATOR) {***/
    /*
     * If the `vnav symbol is bound, then this is the navigator
     * process.  Find a window with vnav in it and call
     * vnav.VisitFcn() directly.
     */
    if (isbound (`vnav, nil)) {
        declare vnav_region;

        catch (`window_found)
            foreach (win; vcWin.Loop ())
                if ((vnav_region = vcRegion.FindFac (vcWin.Region (win),
                                                     vnav.fac)) != nil)
                    throw (`window_found);
        exception
            /* do nothing */;

        if (vnav_region == nil)
            return_value = 1113; /* can't find a Navagator window */
        else
            /* make sure navigator is returning the right stuff! */
            return_value =
                vnav.VisitFcn (vnav_region,
                               vcEvent.Create (0, 0, '0', "",
                                               vcRegion.Window (vnav_region),
                                               vnav_region,
                                               vcRegion.Area (vnav_region)),
                               design_name, location_info);
    }
    /*
     * Otherwise we need to try to IPC the navigator.
     */
    else {
        if (vnav_cntx == nil)
            vnav_cntx = vipc.CntxCreate (vipc.GidDFLT, vipc.UidDFLT,
                                         vipc.FidDFLT, vipc.TidNAVIGATOR,
                                         vipc.HnmANY, vipc.PidANY,
                                         vipc.CidANY);
        if (vipc.Send (vipc.MsgCMD, vipc.GetCntxDflt (), vnav_cntx,
                       sprintf ("visit %s %s", design_name, location_info),
                       vipc.FlgDIRECT)
            == vipc.FAILURE)
        {
            return_value = 1112; /* can't contact the navigator */
/*
            function do_exit(mode, code)
                if (mode == vcSysMode.Exited)
                    printf ("vnav exited with status %d\n", code);
                else
                    printf ("vnav received signal %d; terminated\n", code);

            printf ("Spawning \"vnav %s\"...\n", design_name);
            sys_obj := vcSys.Command (sprintf ("vnav %s", design_name),
                                      nil, nil, nil, do_exit, nil);
*/
        }
    }

    /*
     * Return value is important!
     */
    return_value;

} /* track_vnav */


/*
 * VerifyVdrawLocation -- called by track_vnav, this fn makes sure
 *     that `loc' is a valid location or it reports an error
 *
 *   `loc' - location info without design name
 *   `full_loc' - full location info including design name
 *
 * Note that the regexp checking performed herein is more stringent
 * than that done by the regexp in `track_alist'.  This is done so
 * that we can report a better error if we can't parse the location
 * string.  If the location doesn't match the first regexp, we can
 * only give the message "unrecognized location"; but here, we know
 * that the location looks like a Vdraw location, so we can give a
 * better message, something  like "Bad error location for net --
 * "blah".  A net location must match the regular expression -- "...".
 */
vdraw_loc_alist := nil;
function VerifyVdrawLocation(loc, full_loc) {
    declare token;
    err := nil;

    /*
     * Init all the crazy regexps that we'll be using.
     *
     * BEWARE.  This is pretty hairy regexp stuff, especially things
     * like the fact that you must quote backslashes from both
     * ViewScript and the regexp functions.
     */
    if (vdraw_loc_alist == nil) {
        str_posint    := "[0-9]+";
        str_item      := "[^\\\\]+";
        str_item_path := sprintf ("(%s\\\\)*%s", str_item, str_item);
        str_refdes    := "[A-Za-z][^ \t]*[0-9]+";

        str_obj := sprintf ("object %s", str_item_path);
        str_net := sprintf ("net %s", str_item_path);
        str_com := sprintf ("com %s", str_item_path);
        str_pin := sprintf ("pin %s.%s", str_item_path, str_item);
        str_pkg := sprintf ("pkg %s", str_refdes);
        str_ppn := sprintf ("ppn %s\\[%s\\]", str_refdes, str_posint);
        str_sheet := sprintf ("sheet %s +%s\\.%s", str_item_path,
                              str_item, str_posint);
        str_sym := sprintf ("sym %s", str_item_path);

        re_obj := vscript.re_compile (str_obj);
        re_net := vscript.re_compile (str_net);
        re_com := vscript.re_compile (str_com);
        re_pin := vscript.re_compile (str_pin);
        re_pkg := vscript.re_compile (str_pkg);
        re_ppn := vscript.re_compile (str_ppn);
        re_sheet := vscript.re_compile (str_sheet);
        re_sym := vscript.re_compile (str_sym);

        vdraw_loc_alist = list (list ("object", re_obj),
                                list ("net", re_net),
                                list ("com", re_com),
                                list ("pin", re_pin),
                                list ("pkg", re_pkg),
                                list ("ppn", re_ppn),
                                list ("sheet", re_sheet),
                                list ("sym", re_sym));
    }

    string_token (loc, &token, " ");
    match := assoc (token, vdraw_loc_alist);
    if (match == FALSE)
        vlEx.Throw2 (vlEx.FAILURE, "vcAlert", vlErrSev.Error,
                     sprintf ("vcAlert.VerifyVdrawLocation does not support location \"%s\".  How did we get here?", full_loc));
    if (!vscript.re_exec (cadr (match), loc))
        vlEx.Throw2 (vlEx.FAILURE, "vcAlert", vlErrSev.Error,
                     sprintf ("Bad error location for %s -- \"%s\".\nA %s location must match this regular expression", token, full_loc, token));

    /*
     * OK, everything's verified.  We can go on with life now.
     */
    TRUE;

} /* VerifyVdrawLocation */


/*\vscmuserDocpage vcAlert.AddVisit -- add visit function to Error Navigation system
 *
 * SYNOPSIS
 *      voidVT vcAlert.AddVisit(regexp, visit_function)
 *          vlStringVT regexp;        // when a location matches this...
 *          closureVT visit_function; // ...this func will be called
 *
 * DESCRIPTION
 *      Add a visit function to the Error Navigation system, to be
 *      used for all errors with location fields that match the
 *      specified regular expression.
 *
 *      The Error Navigation system maintains an association list of
 *      (RegularExpression VisitFunction) pairs.  When the user hits
 *      the "Visit(Next)" button in an Alert Box, this list is
 *      searched for the first regular expression matching the
 *      location field of the current error report. If any RE
 *      matches, the associated visit function is called.
 *
 *      This function, then, constructs a list from parameters
 *      `regexp' and `visit_function', and prepends that list to the
 *      system's association list.
 *
 * THE VISIT FUNCTION
 *      `visit_function' will be called with 5 arguments, and should
 *      be declared as follows:
 *
 *          function visit_function(source, severity, number,
 *                                  location, data)
 *
 *      `source' is the first field in an error report; where this
 *      error message came from.  `severity' is the intensity of the
 *      error, and may be given to you as either an integer or an
 *      enumerated type.  `number' is the error message number; zero
 *      indicates that there is no message number.  `location' is
 *      the location field, without the leading <colon><space> and
 *      trailing <colon> that are field delimiters in all error
 *      reports.  `data' is currently unused, and reserved for
 *      future extension of the protocol.
 *
 *      If you detect an error from within the function, do not
 *      return, but instead signal an error using the function
 *      `error2' or `error'.
 *
 *      `visit_function' MUST return nil upon successful
 *      completion.  Non-nil return values are reserved for future
 *      use, and may cause an error to be reported.  Note that it is
 *      inefficient to use the ViewScript `return' statement; it is
 *      sufficient to end your function with the statment:
 *
 *          nil;
 *
 * WRITING A GOOD REGEXP
 *      The regexp should be anchored against the beginning and end
 *      of string using the '^' and '$' characters, respectively.
 *      The regexp "file (.*)" would match successfully against the
 *      string "there is some file somewhere", matching the
 *      substring "file somewhere".  This is because the regexp was
 *      not expressed to match only at the beginning of the string.
 *      The regexp "^file (.*)$" would NOT match the above example.
 *
 *      Note also that the regexp "^file (.*)$" contains a
 *      subexpression, which could be used inside your visit
 *      function to get the filename.  The Error Navigation software
 *      compiles a regexp object, `vcAlert.last_regexp', out of your
 *      regexp string.  You can take advantage of this knowledge by
 *      calling `vscript.re_getmatch' as soon as your visit function
 *      gets control.  If your regexp was specified as "^file  (.*)$",
 *      you could get the string that matched the first
 *      subexpression, "(.*)", with the following code fragment:
 *
 *          filename := vscript.re_getmatch (vcAlert.last_regexp, 1);
 *
 *      See the man page for vscript.re_getmatch for a fuller
 *      explanation of this stuff.
 *
 * EXAMPLE
 *      // This code would likely exist in your facility's VS file...
 *
 *      function myfunc(source, severity, number, location, data)
 *      {
 *          printf ("myfunc(%s, %s, %s, %s, %s)\n",
 *                  source, severity, number, location, data);
 *          filename := vscript.re_getmatch (vcAlert.last_regexp, 1);
 *          line_number := vscript.re_getmatch (vcAlert.last_regexp, 2);
 *          printf ("filename=%s, line_number=%s\n", filename, line_number);
 *      }
 *
 *      vcAlert.AddVisit ("^file (.*), line (.*)$", myfunc);
 *
 *      error2 ("testing", vlErrSev.Error, 1234, "file /etc/rc, line 3",
 *              "somebody screwed up");
 *
 * CAUTIONS
 *      The regular expressions used by this function are full
 *      (egrep-style) regular expressions, which is COMPLETELY
 *      DIFFERENT from shell meta-syntax; i.e. pretty hairy stuff
 *      for the uninitiated.
 *
 *      Passing a regular expression which is overly generic (i.e.
 *      matches more than the intended set of location fields) will
 *      of course cause `visit_function' to be called, overriding
 *      the default behavior.
 */
function AddVisit(vlStringVT regexp, closureVT visit_function)
    track_alist = cons (list (regexp, visit_function), track_alist);


/*
 * `track_alist' is an association list, that is, a list of 2
 * element lists.  The car of each cons cell is a regular expression
 * describing a location string.  The cdr is the function which gets
 * called when a track is done on a message with a location string
 * matching the regular expression.
 */
track_alist := list (list ("^file (.+), line ([0-9]+)$", track_vtext),
                     list ("^file (.+), ([0-9]+)$",      track_vtext),
                     list ("^(.+), line ([0-9]+)$",      track_vtext),
                     list ("^file (.+)$",                track_vtext),

                     /*
                      * We're not so picky yet with the location
                      * info.  If it looks like a ViewDraw object,
                      * then we call `track_vnav', and it gets more
                      * picky there with the location info.  This
                      * way, we're able to report a much more
                      * reasonable error message.
                      *
                      * IF A REGEXP IS ADDED HERE that maps to a
                      * call to `track_vnav', then the more explicit
                      * regexp and error message should also be
                      * added to `VerifyVdrawLocation'.
                      */
                     list ("^((.+) )?(object .+)$", track_vnav),
                     list ("^((.+) )?(net .+)$", track_vnav),
                     list ("^((.+) )?(com .+)$", track_vnav),
                     list ("^((.+) )?(pin .+)$", track_vnav),
                     list ("^((.+) )?(pkg .+)$", track_vnav),
                     list ("^((.+) )?(ppn .+)$", track_vnav),
                     list ("^((.+) )?(sheet .+)$", track_vnav),
                     list ("^((.+) )?(sym .+)$", track_vnav));
