/*
 *  $Revision: 1.40.2.1 $
 *
 *  Viewscript UI for the Core vcCmdline facility.
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

function bind(keys, fun, func_name) {
    function do_key(key) {
        vcBind.Set (bindings, key, cmd);
        vcBind.Set (ReturnBindings, key, cmd);
    } /* function do_key */

    cmd := vcWv.Cmd (fac, fun, func_name);
    if (iscons (keys))
        for (; keys != nil; keys = cdr (keys))
            do_key (car (keys));
    else
        do_key (keys);
    cmd;
} /* function bind */

bind ('\^A',    beginning_of_line,  "beginning_of_line");
bind ('\^D',    del_char,           "del_char");
bind ('\^E',    end_of_line,        "end_of_line");
bind ('\^U',    del_to_beginning,   "del_to_beginning");
bind ('\^K',    del_to_end,         "del_to_end");
DelAllCmd := bind ('\^W',    del_all,            "del_all");

DelPrevCmd :=
    bind (list ('\177', '\^H'),   del_previous,       "del_previous");

/* Beta upward compatibility synonyms. */
vcCmdtable.Add (commands, "del_previousH", DelPrevCmd);
vcCmdtable.Add (commands, "del_previous177", DelPrevCmd);

bind (list ('\^B', '\left'),    back_char,      "back_char");
bind (list ('\^F', '\right'),   forward_char,   "forward_char");
bind (list ('\^N', '\down'),    next_line,      "next_line");
bind (list ('\^P', '\up'),      prev_line,      "prev_line");

process_cmd := vcWv.Cmd (fac, process, "process");
vcBind.Set (ReturnBindings, '\enter', process_cmd);
vcBind.Set (ReturnBindings, '\^M', process_cmd);
vcBind.Set (ReturnBindings, '\^J', process_cmd);
            

ins_char_cmd := vcWv.Cmd (fac, ins_char, "ins_char");
foreach (i; vcWv.Printable ())
{
    vcBind.Set (bindings, i, ins_char_cmd);
    vcBind.Set (ReturnBindings, i, ins_char_cmd);
}

/*
 * Open Look Drag&Drop support.
 */
function _Drop(cmdline, event)
    set (cmdline, vcEvent.String (event));

/*
 * Cut and paste from the X cut-buffer.
 */
cut_bindings := vcBind.Create ();

function paste(cmdline, event)
    _Insert (cmdline, vcWv.FetchCutBuffer (wv));

bind (list ('\middledown', '\f18', '\insert'), paste, "paste");
bind (list ('\delete', '\f20'), cut, "cut");

declare start_event, start_bindings;

bind ('\leftdown', func(cmd, ev)
                   {
                       start_event = ev;
                       r := vcEvent.Region (ev);
                       vcRegion.SetGrab (r, r);
                       start_bindings = vcRegion.Bindings (r);
                       vcRegion.SetBindings (r, cut_bindings);
                       hstart (cmd, ev);
                   }, "hstart");

vcBind.Set (cut_bindings, '\move', vcWv.Cmd (fac, hcont));
vcBind.Set (cut_bindings, '\leftup',
            vcWv.Cmd (fac,
                      func(cmd, ev)
                      {
                          r := vcEvent.Region (ev);
                          vcRegion.SetGrab (r, nil);
                          vcRegion.SetBindings (r, start_bindings);
                          hcont (cmd, ev);
                          if (!vcWv.ClickTogether (wv, start_event, ev))
                              vcRegion.StoreCutBuffer (Region (cmd),
                                                       hbuf (cmd));
                          hend (cmd, ev);
                      }, "hend"));

_ClearSelection := hclr;

set := Set;                     /* Upward compatibility */

vcWv.MakeEnum ("vcFormatVT", list ("Octal", "Decimal", "Hex"));

function CvtFormat(window, code, format) {
    if (format == vcFormat.Octal)
        code = vcWin.Eval (window, "0" + code + ";");
    else if (format == vcFormat.Hex)
        code = vcWin.Eval (window, "0x" + code + ";");
    else
        code = vcWin.Eval (window, code + ";");
    code;
} /* function CvtFormat */

function quoted_insert(cmdline, event, code, format) {
    window := vcEvent.Window (event);
    code = CvtFormat (window, code, format);
    ins_char (cmdline, vcEvent.Create (vcEvent.X (event),
                                       vcEvent.Y (event),
                                       int2char (code + 0),
                                       vcEvent.String (event),
                                       window,
                                       vcEvent.Region (event),
                                       nil));
} /* function quoted_insert */

quote_cmd := vcWv.Cmd (fac, quoted_insert, "quote", "Insert Character Code",
                       vcWv.Param ("Character Code", vlStringVT, vcWv.Sticky,
                                   ""),
                       vcWv.Param ("Numerical Format", vcFormatVT, vcWv.Sticky,
                                   vcFormat.Decimal, vcWv.Secondary));

vcBind.Set (bindings, '\^Q', quote_cmd);
vcBind.Set (ReturnBindings, '\^Q', quote_cmd);
