/*
 *  $Revision: 1.10.2.2 $
 *
 *  Viewlogic Proprietary Information
 *
 *  Copyright (c) 1993 by Viewlogic Systems, Inc.
 *
 */

/*
 * Apply a ViewDraw command to a specific error.
 */
function Apply(el_region, f) {
    e := vcRegion.Get (picklist (el_region), vscmVT);
    if (e == nil)
        vlEx.Throw2 (vlEx.FAILURE, "vcErrList", vlErrSev.Error,
                     "There is no current error");
    visit (el_region, e, f);
} /* function Apply */

set_visit_function (func (el, e) visit (el, e, nil););

highlight_interrupt := nil;

function highlight_obj(c, w, pop_viewdraw) {
    function start_flashing() {
        if (pop_viewdraw)
            vcWin.Show (w);
        vdraw_region := vcWin.OwnerRegion (w);
        mbb := vdraw.mbb_selected (vdraw_region);
        vp := vcRegion.Viewport (vdraw_region, nil, 0);
        bounds := vgU.User (vp);
        count := 2;

        next_timer := nil;

        highlight_interrupt = func(nt) next_timer = nt;;

        timer := vcWv.Timer (wv, 250, func() {
            function lines() {
                vgU.Box (vp, mbb);
                vgU.Line (vp,
                          vgRect.Xmin (mbb), vgRect.Ymin (mbb),
                          vgRect.Xmin (bounds), vgRect.Ymin (bounds));
                vgU.Line (vp,
                          vgRect.Xmax (mbb), vgRect.Ymin (mbb),
                          vgRect.Xmax (bounds), vgRect.Ymin (bounds));
                vgU.Line (vp,
                          vgRect.Xmin (mbb), vgRect.Ymax (mbb),
                          vgRect.Xmin (bounds), vgRect.Ymax (bounds));
                vgU.Line (vp,
                          vgRect.Xmax (mbb), vgRect.Ymax (mbb),
                          vgRect.Xmax (bounds), vgRect.Ymax (bounds));
                vcWv.FlushDisplay (wv);
            } /* function lines */

            rop := vgU.Rasterop (vp, vgRasterop.Xor);
            color := vgU.Color (vp, vgColor.Yellow);
            fstyle := vgU.Fillstyle (vp, vgFill.Solid);
            lines ();
            sleep (1);
            lines ();
            vgU.Rasterop (vp, rop);
            vgU.Color (vp, color);
            vgU.Fillstyle (vp, fstyle);

            --count;
            if ((count == 0) ||
                ((next_timer != nil) && ((count % 2) == 0)))
            {
                vcWv.CancelTimer (timer);
                highlight_interrupt = nil;
                if (next_timer != nil)
                    next_timer ();
            } /* if */
        },
                             TRUE);
    } /* function start_flashing */

    if (highlight_interrupt != nil)
        highlight_interrupt (start_flashing);
    else
        start_flashing ();
} /* function highlight_obj */

vdraw_ctx := nil;
my_ctx := nil;

function visit(el, e, do_more_stuff) {
    location := vlErr.Location (e);
    if (string_eq (location, ""))
        vlEx.Throw2 (vlEx.FAILURE, "vcErrList", vlErrSev.Error,
                     "Error has no location field");
    
    if (isbound (`vdraw, nil)) {
        w := vcRegion.Window (picklist (el));
        vcWin.WithBusy (w, func() {
            vdraw.SelectLocation (location, func(c, w) {
                if (do_more_stuff != nil)
                    do_more_stuff (c, w, vlErr.App (e), location);
                else
                    highlight_obj (c, w,
                                   vcRegion.PrefGet (region (el),
                                                     "pop_viewdraw"));
            });
        });
    } /* if */
/*      (vdraw.gen_vdraw_cmd (design)) ("SelectPath " + design + " " + path);*/
    else {
        if (vdraw_ctx == nil) {
            vdraw_ctx = vipc.CntxCreate (vipc.GidANY, vipc.UidANY,
                                         vipc.FidDFLT, vipc.TidVDRAW,
                                         vipc.HnmANY, vipc.PidANY,
                                         vipc.CidANY);
            my_ctx = vitc.SrcCntxCreate ("vcErrList", "");
        } /* if */

        function send_cmd_to_viewdraw(str)
            if (vitc.Send ("Command Line", my_ctx, vdraw_ctx, str,
                           vipc.FlgDIRECT) != vitc.Success)
                vlEx.Throw2 (vlEx.FAILURE, "vcErrList", vlErrSev.Error,
                             "Error selecting Viewdraw path: " + location +
                             "\n+" + vipc.ErrGetStr ());
            
        send_cmd_to_viewdraw ("locate " + location);
        if (do_more_stuff != nil)
            do_more_stuff (send_cmd_to_viewdraw, nil, vlErr.App (e), location);
    } /* else */
    set_last_visited (el, e);
} /* function visit */

function visit_next(el, event) {
    declare e, n_rows, n_cols;
    pl := picklist (el);

    vcPicklist.Dimensions (pl, &n_rows, &n_cols);

    if (n_rows > 0) {
        e = vcRegion.Get (pl, vscmVT);
        if (e == nil) {
            vcPicklist.SetPick (pl, 0);
            e = vcRegion.Get (pl, vscmVT);
        } /* if */
        else if (e == last_visited (el)) {
            vcPicklist.down (pl, event);
            e = vcRegion.Get (pl, vscmVT);
        } /* if */
        visit (el, e, nil);
    } /* else */
} /* function visit_selected */
    
visit_next_cmd := vcWv.Cmd (fac, visit_next, "visit_next");
vcCmd.SetHelp (visit_next_cmd,
         "Visit the next error in the list, or the first if none is selected");


/*
 * Return current error test.
 */
function CurrentTest(el) {
    e := vcRegion.Get (picklist (el), vlErrVT);
    if (e == nil)
        vlEx.Throw2 (vlEx.FAILURE, "vcErrList", vlErrSev.Error,
                     "There is no current error");
    vlErr.App (e);
} /* function CurrentTest */
