/*
 *  $Revision: 1.4.1.2 $  $Date: 1994/11/28 19:55:29 $
 *
 *  Useful editor functions for the VDMF class definitions.
 *
 *  Copyright (c) 1994 by Viewlogic Systems, Inc.
 */



/*
 * NOTE: This file is loaded into the top level package.  It's loaded from the
 * vdmMaster.scm file.
 */



/*
 * Declare ITC source contexts used by the EditViewDraw function.
 */
vdraw_context := vitc.CntxCreate (vipc.GidDFLT, vipc.UidDFLT,
                                  vipc.FidDFLT, vitc.TidVDRAW,
                                  vitc.NameDFLT, vipc.HnmANY,
                                  vipc.PidANY, vipc.CidANY);

my_context    := vitc.SrcCntxCreate ("vdmEdit", "VDMF Editors");



/*\Docpage vdmEditViewDraw -- ViewDraw editor.
 *
 * DESCRIPTION
 *      It handles the following situations:
 *
 *      1. ViewData is loaded into the vdraw package.
 *
 *      2. We are not in ViewDraw, but ViewDraw is already up an running and
 *         just needs to open a new window, instead of invoking yet another
 *         process.
 *
 *      3. ViewDraw is not running, and we need another process.
 */
function vdmEditViewDraw(obj, outProc, errProc, exitProc)
{
    declare cmd;

    location     := vdm.ObjGetLocation (obj);
    name         := vdm.ObjGetName (obj);
    class_name   := vdm.ClassGetName (vdm.ObjGetClass (obj));
    is_schematic := TRUE;

    if (string_eq (class_name, "Schematic"))
        cmd = "locate - sch " + name;
    else
    {
        cmd = "locate - sym " + name;
        is_schematic = FALSE;
    }


    /*
     * If this call is being made from ViewDraw, just call it.
     */
    if (isbound (`vdraw, current_package ()))
    {
        if (is_schematic)
            vdraw.schematic (name);
        else
            vdraw.symbol (name);
    }

    /*
     * Else assume that ViewDraw is up and running and send the command to it.
     * If that fails, spawn a new ViewDraw, register to receive a tool start
     * message from it and then send the command to it.
     */
    else if (vitc.Send (vitc.MsgCMD, my_context, vdraw_context, cmd,
                        vipc.FlgDIRECT) != vitc.Success)
    {
        if (is_schematic)
            vcSys.Command ("viewdraw -sch " + name, nil, outProc, errProc,
                           exitProc, nil);
        else
            vcSys.Command ("viewdraw -sym " + name, nil, outProc, errProc,
                           exitProc, nil);
    }
    else if (! null (exitProc))
        exitProc (vcSysMode.Exited, 0);
}



/*\Docpage vdmEditDefault -- Default VDM editor.
 * 
 * DESCRIPTION
 *      The vcWin.TextEdit function does most of the work in figuring out
 *      which editor to use.  If it's unavailable, we try to pick off the
 *      value of the EDITOR environment variable and spawn a subprocess.  If
 *      that fails, an error is signaled to the user and the function exits.
 */
function vdmEditDefault(dataObj, outProc, errProc, exitProc)
{
    path := vdm.ObjGetPhysicalDescriptor (dataObj);

    if (isbound (`vcWin, current_package ()))
        vcWin.TextEdit (nil, path, nil, exitProc, FALSE, FALSE);
    else if ((editor := vscript.getenv ("EDITOR")) != nil)
        vcSys.Command (editor + " " + path, nil, outProc, errProc, exitProc,
                       nil);
    else
    {
        vlErr.Out2 ("vdmEdit", vlErrSev.Error,
                    "No default editor available to edit " + path);
        if (! null (exitProc))
            exitProc (vcSysMode.Exited, 1);
    }
}
