;;;; $Revision: 1.3.1.1 $   $Date: 1994/11/07 21:11:53 $
;;;;
;;;; DESCRIPTION
;;;;      Viewlogic Project Manager initialization file.  Use this
;;;;      file to define all your project-related issues: locations,
;;;;      storage managers, work areas, groups, etc.
;;;;
;;;;      There can be two VPM initialization files, the master and
;;;;      the user customizations.  This concept is borrowed from the
;;;;      Powerview facilities, where there's a master GUI facility
;;;;      file, vdraw.vs for example, and the user customization file,
;;;;      vdrawus.vs, containing the user's own modifications to the
;;;;      GUI.
;;;;
;;;;      The vpm.scm file is the master initialization file.  The
;;;;      central CAD group should set it up with enough information
;;;;      for every user to do their work.  Each user can then create
;;;;      their own vpmus.scm file to include things like additional
;;;;      location definitions for the user's own data.
;;;;
;;;; NOTE
;;;;      At the first release of the VDMF libraries, only the
;;;;      ViewScript/SCM syntax is supported for the VPM
;;;;      initialization files.  It is recommended that you use the
;;;;      Location menu commands to edit the project and location
;;;;      information and let the ViewData facility write the changes
;;;;      for you.
;;;;
;;;; CAUTION
;;;;      To maintain both upward and backward compatibility, all the
;;;;      Project Management specific functions will be called via the
;;;;      vpm|Process function.  The first argument will serve as a
;;;;      dispatcher to the appropriate callback.  The rest of the
;;;;      arguments are passed to the callback as a variable number
;;;;      argument list.
;;;;
;;;;      You can add any additional ViewScript code that you want.
;;;;      Just remember that if you want to call any of the VPM
;;;;      functions, you must do it through the vpm|Process function.
;;;;
;;;;      For optimization purposes, the initialization files are
;;;;      loaded directly into the `vpm' package.  Therefore, you
;;;;      don't need to include the vpm prefix when calling the
;;;;      Process function. 
;;;;
;;;; SYNOPSIS
;;;;      (vpm|Process "DispatchKey" arg1 arg2 ... argn)
;;;;
;;;; Copyright (c) 1994 by Viewlogic Systems, Inc.
;;;;



;;;
;;; Set the internal version number.
;;;
(Process 'SetVersion 5.3)


;;;
;;; Add class files.  A class file is a file containing the VDMF class
;;; definitions for the design data.  There is no limit on the number
;;; of class files that you might want to include.  A class file is a
;;; natural way to define all the class definitions used by the whole
;;; group or even the entire company.  There's no limit on the size of
;;; the file either.
;;;
;;; Note that the class files must be loaded before the class drawers
;;; are defined: otherwise the drawer definitions would refer to
;;; classes which haven't been defined yet.  Also note that the master
;;; class bunch, vdmMaster, containing definitions for the Viewlogic
;;; data types is loaded by default through the vdm.master_file
;;; preference.
;;;
;(Process 'AddClassFile "Your-Own-Class-File-Name")



;;;
;;; Declare which storage managers are available.  There are three
;;; default storage manager encapsulations available with the VDMF
;;; library: RCS, CVS and the file system.
;;;
;;; You need to specify two pieces of information for each storage
;;; manager: the encapsulation and the display name of the storage
;;; manager.  A storage manager can be encapsulated either as a
;;; ViewScript file containing the definitions of the encapsulation
;;; methods or a ViewScript package containing the methods.  If the
;;; storage manager was encapsulated as file, the first argument
;;; should be a string representing the encapsulation filename (not the
;;; pathname, just the filename).  If the storage manager was
;;; encapsulated using the package method, the first argument should
;;; be the symbol whose value is the ViewScript package.
;;;
;;; The second argument is the "pretty" name, as it will be displayed
;;; by the ViewData facility when you use it to edit a location
;;; definition.  It just happens that most of the encapsulation
;;; filenames are not easily understood as to what exactly they are
;;; encapsulating, so the "pretty" string offers you a chance to
;;; disambiguate the user.
;;;
(Process 'AddStorageMgr "vdmRCS" "RCS")
(Process 'AddStorageMgr "vdmCVS" "CVS")
(Process 'AddStorageMgr  "vdmFile" "File system")



;;;
;;; Define class drawers.  A class drawer is a collection of class
;;; definitions and/or other class drawers.  It's used to collect
;;; class definitions, possibly dispersed throughout many different
;;; class files, into a single set of related classes.  For example,
;;; you can setup a class drawer for all the simulation models, or all
;;; the schematic files.  The drawer can then be used as the class
;;; filter during the location definition.
;;;
;;; A class drawer definition consists of the drawer's name and a
;;; ViewScript list containing the names of classes or other drawers.
;;; The names must be represented as strings and the list must be
;;; flat.  Drawer definitions can contain references to other drawers
;;; and there's no limit on the length of such definition chains.
;;;
;(Process 'AddDrawer "ViewDraw Data" `("Schematic" "Symbol"))



;;;
;;; Set the search space.  A search space is defined as a set of
;;; location definitions.  A location is defined via the 'AddLocation
;;; keyword.  It takes four pieces of information to define a
;;; location:
;;;
;;; 1. Location alias
;;;
;;; Shorthand name for the location.  The alias is used by the ViewData
;;; facility when displaying the location search space.  Use the
;;; vpm.MapFindLocation(man) function to locate an existing location
;;; Id, i.e. to convert from the location's alias to a location object.
;;; 
;;; 2. Storage manager specification (filename or package)
;;; 
;;; Each physical location is assumed to be under the control of a
;;; storage manager, even if that storage manager is a file system.
;;; The storage manager information can either be encapsulated as a
;;; ViewScript file or as a set of methods defined inside a ViewScript
;;; package.  See the VDMF's Integrator's Manual for more details on
;;; storage manager integration.  The two encapsulation methods are
;;; mutually exclusive and this function uses the following rule to
;;; distinguish between them: if the package is not nil, it means
;;; that the storage manager encapsulation was done via the package.
;;; If the package is equal to nil, then it means that the
;;; encapsulation was done via the file.  The reason for this
;;; heuristic is that no sane integrator would create a storage
;;; manager encapsulation in the ViewScript's top level package, nil.
;;; 
;;; 3. Location string
;;; 
;;; The location string is the actual description of the physical
;;; location of the data.  The format of the string is dictated by the
;;; specific storage manager integration and is defined in the
;;; _vdmInit method for that storage manager.  Again, see the VDMF's
;;; Integrator's Manual for more details on how to encapsulate a
;;; storage manager.
;;; 
;;; Warning: the location string is NOT synonymous with the pathname.
;;; The semantics of the string are dictated by the storage manager.
;;; For example, for RCS, the location string is composed of two
;;; tokens: the first token represents the pathname of the storage
;;; area.  The second token represents the pathname of the work area.
;;; For Atria's ClearCase, the string is again composed of two tokens,
;;; but the first token represents the name of the "view" and the
;;; second token represents the directory pathname.  The encapsulation
;;; of the default file system interprets the location string as a
;;; single token: the directory pathname.  And so on.
;;; 
;;; 4. Location class filter
;;; 
;;; To speed up the display of the data information in the ViewData
;;; facility, you can notify the system about a particular set of data
;;; types that are to be found in the given location.  The VDMF
;;; library will then not look for any other types of data.  Class
;;; filters are especially useful when you have many different class
;;; types and you are not interested in the ViewData facility wasting
;;; time determining if there are any instances of those class types
;;; in the location when you know that there aren't any to begin with.
;;; You may specify nil to signify that there's no class filter
;;; for this location.
;;;
;(Process 'AddLocation "builtin" "vdmFile" "/import/wvlib/builtin"
;         `("Schematic" "Symbol"))
