/*
 *  $Revision: 1.48.1.10 $    $Date: 1995/04/17 23:13:14 $
 *
 *  This is the ViewScript file for the C version of vcBrowse.  It
 *  supersedes in RCS the implemenetation that was completely in
 *  ViewScript.
 *
 *  Copyright (c) 1995 by Viewlogic Systems, Inc.
 */

function busy(f)
    func(args...)
        vcWin.WithBusy (vcEvent.Window (cadr (args)), func() apply (f, args););
    ;

/*\intDocpage obj_cmd_immediate -- Make a "fast" command.
 *
 * DESCRIPTION
 *      Make a command that runs another command on the currently selected
 *      object, without doing any prompting.  It's intended to be used from
 *      the pop-up menu for edit/expand/collapse.
 */
function obj_cmd_immediate(cmd)
    vcWv.Cmd (fac, func(view, event)
              vcCmd.Run (cmd, view, event, list (GetLastSelectedPath (view)),
                         TRUE););


expand_cmd := vcWv.Cmd (fac, busy (expand), "expand", "Expand",
                        vcWv.Param ("Path", vlStringVT, vcWv.Dynamic,
                                    GetLastSelectedPath));

collapse_cmd := vcWv.Cmd (fac, busy (collapse), "collapse", "Collapse",
                          vcWv.Param ("Path", vlStringVT, vcWv.Dynamic,
                                      GetLastSelectedPath));

edit_cmd := vcWv.Cmd (fac, busy (edit), "edit", "Edit",
                      vcWv.Param ("Path", vlStringVT, vcWv.Dynamic,
                                  GetLastSelectedPath));
edit_popup_cmd := obj_cmd_immediate (edit_cmd);

select_cmd := vcWv.Cmd (fac, select, "brw_select", "Select",
                        vcWv.Param ("Path", vlStringVT, vcWv.Dynamic,
                                    GetLastSelectedPath),
                        vcWv.Param ("Flag", vlOnOffVT, vcWv.Sticky, TRUE));


function menu(view, event, path) {
    menu := PathGetMenu (view, path);
    if (menu == nil)
        menu = obj_menu;
    vcMenuPop.Create (menu, event);
} /* function menu_cmd */
menu_cmd := vcWv.Cmd (fac, menu, "menu", "Pop up Menu",
                      vcWv.Param ("Path", vlStringVT, vcWv.Dynamic,
                                  GetLastSelectedPath));
vcCmd.EnableMacro (menu_cmd, FALSE);
    
/*
 * In order to implement right-mouse-button pop-up menus over picklists,
 * we have to copy the picklist bindings and modify them.  We also need
 * to fake out the picklist into thinking the left mouse button was pressed
 * and released over the picklist in the appropriate location so that the
 * last_info_selected field gets set by our Notify callback, so we know
 * which menu to pop up.
 */
browse_picklist_bindings := nil;

function picklist_select_xy(pl, event) {
    vcPicklist.SetXY (vcRegion.FindFac (vcEvent.Region (event),
                                        vcPicklist.fac),
                      vcEvent.X (event),
                      vcEvent.Y (event),
                      vcPicklist.Select.Replace);
/*
    vcPicklist.DeselectAll (vcEvent.Region (event));
    vcCmd.Run (vcBind.Get (vcPicklist.bindings, '\leftdown'), pl, event, nil,
               FALSE);
    vcCmd.Run (vcBind.Get (vcPicklist.armed_bindings, '\leftup'),
               pl, event, nil, FALSE);
*/
} /* function picklist_select_xy */
    

function popup_menu_over_picklist(pl, event) {
    picklist_select_xy (pl, event);
    window := vcEvent.Window (event);
    region := vcWin.OwnerRegion (window);
    browse_region := vcRegion.FindFac (region, fac);
    view := vcRegion.GetData (browse_region);

    menu (view, event, GetLastSelectedPath (view));
} /* function popup_menu_over_picklist */
popup_menu_over_picklist_cmd := vcWv.Cmd (vcPicklist.fac,
                                          popup_menu_over_picklist,
                                          "PopupMenu");
vcCmd.EnableMacro (popup_menu_over_picklist_cmd, FALSE);

function setup_browse_picklist_bindings() {
    browse_picklist_bindings = vcBind.Duplicate (vcPicklist.bindings);
    vcBind.Set (browse_picklist_bindings, '\rightdown',
                popup_menu_over_picklist_cmd);
    setup_browse_picklist_bindings = nil;
} /* function setup_browse_picklist_bindings */

function set_picklist_bindings(picklist_region) {
    if (setup_browse_picklist_bindings != nil)
        setup_browse_picklist_bindings ();
    vcRegion.SetBindings (picklist_region, browse_picklist_bindings);
} /* function set_picklist_bindings */

function drag_current(view, event) {
    picklist_region := vcRegion.FindFac (vcEvent.Region (event),
                                         vcPicklist.fac);
    picklist_select_xy (vcRegion.GetData (picklist_region), event);
    vcCmd.Run (drag_cmd, view, event, list (GetLastSelectedPath (view)), TRUE);
} /* function start_drag */
drag_current_cmd := vcWv.Cmd (fac, drag_current);
vcBind.Set (bindings, '\middledown', drag_current_cmd);

function do_drag(view, event, path) {
    drag_data := PathGetDragInfo (view, path);
    if (drag_data != nil) {
        <data, icon> := drag_data;
        declare first_icon;
        string_token (icon, &first_icon, " ");
        vcWv.StartDrag (wv, data, first_icon);
    } /* if */
} /* function do_drag */
drag_cmd := vcWv.Cmd (fac, do_drag, "drag", "Start Dragging Object",
                      vcWv.Param ("Path", vlStringVT,
                                  vcWv.Dynamic, GetLastSelectedPath));

function enable_cmds() {
    /*
     * Determine whether the Edit command should be enabled for this node.
     * This happens independently of the fact whether the node is being
     * selected or deselected.
     */
    vcCmd.Enable (edit_popup_cmd, TRUE);/*info.class.edit != NULL*/

} /* enable_cmds */

function ClassSet(class, args...) {
    children := nil;
    get_info := nil;
    edit := nil;

    function vscm_children(obj, add_child, add_child_data) {
        foreach (child, class; children (obj, nil))
            add_child (add_child_data, class, child);
    } /* function vscm_children */

    function vscm_edit(obj, editing_done, editing_done_data) {
        edit (obj,
              func(args...) editing_done (editing_done_data);,
              class);
    } /* function vscm_edit */

    function vscm_get_info(obj, report_info, report_info_data) {
        foreach (args...; get_info (obj)) {
            declare name, val;
            if (cdr (args) == nil) {
                val = string_token (car (args), &name, ":=");
                if ((string_length (val) > 0) &&
                    ((val[0] == ':') || (val[0] == '=')))
                    val = string_trim (substring (val, 1, 0));
            } /* if */
            else
                <name, val> = args;

            report_info (report_info_data, name, val);
        } /* foreach */
    } /* function vscm_get_info */

    foreach (name_val; vscript.list_iter (args)) {
        name := car (name_val);
        if (name == `children) {
            children = cdr (name_val);
            set_cdr (name_val, vscm_children);
        } /* if */
        else if (name == `get_info) {
            get_info = cdr (name_val);
            set_cdr (name_val, vscm_get_info);
        } /* else if */
        else if (name == `edit) {
            edit = cdr (name_val);
            set_cdr (name_val, vscm_edit);
        } /* else if */
    } /* foreach */

    ClassSet1 (class, args);
} /* function ClassSet */

/*
 * The old viewscript-based vcBrowse expected the object "bitmap"
 * function to return a single string bitmap, or a list of bitmaps.
 * The C-based one deals easier with just a string of space-separated
 * bitmaps.
 */
function bitmap_translate(old_bitmap_function) {
    func(obj) {
        declare bitmaps;
        if (isstring (old_bitmap_function))
            bitmaps = old_bitmap_function;
        else
            bitmaps = old_bitmap_function (obj);
        if (ispair (bitmaps)) {
            s := nil;
            foreach (bitmap; vscript.list_iter (bitmaps)) {
                if (s == nil)
                    s = bitmap;
                else
                    s += " " + bitmap;
            } /* foreach */
            s;
        } /* if */
        else
            bitmaps;
    }; /* func */
} /* function bitmap_translate */

/*
 * Functions for defining new classes in Viewscript
 */
function ClassCreate(name, args...) {
    type := assq (`type, args);
    if (type) {
        args = vscript.delete_list_element (args, type);
        type = /**** HACK ALERT vscmVT ******/ cdr (type);
    } /* if */
    else
        type = vscmVT;

    bitmap := assq (`bitmap, args);
    if (bitmap) {
        set_car (bitmap, `get_icon);
        set_cdr (bitmap, bitmap_translate (cdr (bitmap)));
    } /* if */

    drag_data := assq (`drag_data, args);
    if (drag_data)
        set_car (drag_data, `get_drag_data);
    
    drag_icon := assq (`drag_icon, args);
    if (drag_icon) {
        set_car (drag_icon, `get_drag_icon);
        set_cdr (drag_icon, func(obj) cdr (drag_icon););
    } /* if */

    class := ClassCreate1 (name, type, nil);
    apply (ClassSet, cons (class, args));
    class;
} /* function ClassCreate */


/*
 * The following code "replaces" the vcBrowse.Obj* functions with versions
 * that will allow the ommission of the "class" arguments, for upward
 * compatibility.  This kludge is dependent on a deliberate blurring in
 * vcBrowse's object hashing & comparison functions, which look only
 * at a classes type, not the class ID itself, which certainly won't
 * match when we default it to what we have below.
 */

null_class := ClassCreate ("vcBrowseDummyClass");

/*
 * Many of the functions take (view,obj,class).  These all can be
 * handled the same way:
 */
function obj_proc(f)
    func(view, obj, args...) {
        f (view, obj, null (args) ? null_class : car (args));
    }; /* func */

ObjIsPresent    = obj_proc (ObjIsPresent);
ObjSelect       = obj_proc (ObjSelect);
ObjUnSelect     = obj_proc (ObjUnSelect);
ObjExpand       = obj_proc (ObjExpand);
ObjCollapse     = obj_proc (ObjCollapse);
ObjEdit = obj_proc (ObjEdit);
ObjMoveUp       = obj_proc (ObjMoveUp);
ObjMoveDown     = obj_proc (ObjMoveDown);
ObjRemove       = obj_proc (ObjRemove);
ObjGetColor     = obj_proc (ObjGetColor);
ObjGetName      = obj_proc (ObjGetName);
ObjNewWindow    = obj_proc (ObjNewWindow);
ObjViewCreate   = obj_proc (ObjViewCreate);
ObjMoveFirst    = obj_proc (ObjMoveFirst);

NewWindow       := ObjNewWindow;

/*
 * Other functions have to be specially treated.
 */
wired_ObjShow := ObjShow;
ObjShow = func(view, obj, class...) {
    if (class == nil) {
        obj = car (obj);
        class = null_class;
    } /* if */
    else
        class = car (class);
    wired_ObjShow (view, obj, class);
};

wired_ObjRecalcChildren := ObjRecalcChildren;
ObjRecalcChildren = func(view, obj, args...) {
    declare self, class;
    if (length (args) == 2)
        <class, self> = args;
    else {
        self = car (args);
        class = null_class;
    } /* else */
    wired_ObjRecalcChildren (view, obj, class, self);
};

wired_ObjInsert := ObjInsert;
ObjInsert = func(view, parent, args...) {
    declare parent_class, obj, class;
    if (length (args) == 3)
        <parent_class, obj, class> = args;
    else {
        parent_class = null_class;
        <obj, class> = args;
    } /* else */
    wired_ObjInsert (view, parent, parent_class, obj, class);
};

wired_ObjGetParent := ObjGetParent;
ObjGetParent = func(view, obj, args...) {
    if (args == nil)
        car (wired_ObjGetParent (view, obj, null_class));
    else
        wired_ObjGetParent (view, obj, car (args));
};

/*
 * This ones an iterator, but it's easier to fix it if we treat it
 * as a function that takes a proc as the first arg.
 */
wired_ObjLoop := ObjLoop;
ObjLoop = func(proc, view, parent, args...) {
    class := (args == nil) ? null_class : car (args);
    wired_ObjLoop (proc, view, parent, class);
};


function DeselectAll(view, event)
    vcPicklist.DeselectAll (vcEvent.Region (event));

DeselectAllCmd := vcWv.Cmd (fac, DeselectAll);

obj_menu    := vcWv.MakeMenu ("obj_menu",
                              vcWv.Pin (),
                              list ("Edit", edit_popup_cmd),
                              list ("Expand", obj_cmd_immediate (expand_cmd)),
                              list ("Collapse",
                                    obj_cmd_immediate (collapse_cmd)));

vcBind.Set (bindings, '\leftdown', DeselectAllCmd);
