/*
 *  $Revision: 1.21.1.2 $   $Date: 1995/03/15 04:07:31 $
 *
 *  This file implements a facility for editing a filename with a popup file
 *  selection, dialog hooked to a button.
 *
 *  Copyright (c) 1992,1995 by Viewlogic Systems, Inc.
 */


/* register a function + arg that will be called whenever a change occurs
 * in the FilePop editor */
function NotifyOnChange(t, f, a)
{
    this := vcRegion.GetData (t);

    this.thefunc = f;
    this.thearg  = a;
}



/* run the notification function associated with this instance */
function run_notify_function(this)
    if (this.thefunc != nil)
        (this.thefunc) (this.thearg);



function make_file_select(this, event)
{
    table   := this.thetable;
    cmdline := vcTable.Get (table, 0, 0);
    button  := vcTable.Get (table, 0, 1);
    region  := vcRegion.Parent (table);

    filter  := vcRegion.PrefGet (region, "filter");
    deflt   := vcRegion.Get (cmdline, vlStringVT);

    /*
     * If the user invoked this with the '?' key,
     * intelligently merge the default and the filter.
     * If the filter is not specified, or begins with '*', and
     * if the default file has a partial name, then merge...
     */
    if ((vcEvent.Ch (event) == '?') &&
        ((string_length (filter) == 0) || (filter[0] == '*')) &&
        (string_length (deflt) > 0) &&
        ((parsed := vscript.vsParse (deflt)) != nil) &&
        ((name := caddr (parsed)) != nil) &&
        (string_length (name) > 0))
        filter = name + filter;

    file_select_args := vcRegion.PrefCopyChanged (region, vcFileSelect.fac);

    if ((file_select_args == nil) || !assq (`read_now, file_select_args))
        file_select_args = cons (read_now <- TRUE, file_select_args);

    fs := vcRegion.Create1 ("", vcFileSelect.fac, deflt, file_select_args);
    vcRegion.SetFocus (fs, fs);


    function call_on_change(str)
    {
        /*
         * Use vcRegion.Get instead of taking the string from the
         * command line so the pathname gets prepended -- the
         * command line may only have the filename, without the path.
         */

        /* the vcFilePop region changed: call any associated function */
        run_notify_function (this);

        if (!string_eq (str, ""))
            vcCmd.Run (reset_value_cmd, table, event,
                       list (vcRegion.Get (fs, vlStringVT)), TRUE);
    } /* function call_on_change */

    vcCmdline.CallOnChange (fs, call_on_change);


    dismiss_button := vcRegion.Create ("buttons", vcButnRow.fac,
       list (list ("Accept", "Accept", func();, list (close_window <- TRUE))));

    /* choose an appropriate name for the dialog box */
    sel_style := vcRegion.PrefGet (region, "select_type");
    if (sel_style == vcFileSelect.Style.Files)
        name := "Select File";
    else if (sel_style == vcFileSelect.Style.Directories)
        name := "Select Directory";
    else
        name := "Select File Or Directory";

    t := vcTable.Make (name, 2, 1,
                       fs,
                       dismiss_button);

    this.popup = vcWin.Popup (vcEvent.Window (event), t, fs, vcWin.NoMacros);
    vcRegion.SetFocus (t, fs);
    vcRegion.Enable (button, FALSE, TRUE);


    function destroyfunc(r)
       vcRegion.Enable (button, TRUE, TRUE);


    vcRegion.DestroyNotify (fs, func(r)
                                if (this.popup != nil)
                                {
                                    vcRegion.Enable (button, TRUE, TRUE);
                                    this.popup = nil;
                                });
} /* function make_file_select */


browse_cmd := vcWv.Cmd (fac, make_file_select, "Browse");
vcCmd.EnableMacro (browse_cmd, FALSE);



function _Create(filename, region)
{
    cmdline := vcRegion.Create (vcRegion.Name (region) + "_cmdline",
                                vcCmdline.fac, filename,
                                macro_on <- vcRegion.MacroOn (region));
    button  := vcWv.LabelButtonCreate ("", "Browse...", nil,
                                       browse_cmd, nil);

    vcRegion.FocusInhibit (cmdline);
    vcRegion.FocusInhibit (button);

    table   := vcTable.Create ("", 1, 2);
    vcTable.Add (table, cmdline);
    vcTable.Add (table, button);
    vcRegion.AddChild (region, table);

    this := struct {
        thetable = table;
        thefunc  = nil;   /* this instance's notification function */
        popup    = nil;
        thearg   = nil    /* arg to pass to that function */
           
    };


    /* the vcFilePop region changed: call any associated function */
    function call_on_change2(str)
        run_notify_function (this);

    vcCmdline.CallOnChange (cmdline, call_on_change2);


    function tab_completion(str)
        if ((complete := vcFileSelect.Complete (str)) == nil)
            vcWin.VisualBell (vcRegion.Window (region));
        else
            vcCmdline.Set (cmdline, complete);

    vcCmdline.CallOnComplete (cmdline, tab_completion);

    this;
} /* function _Create */

   

function _Set(this, filename)
{
    table   := this.thetable;
    cmdline := vcTable.Get (table, 0, 0);
    vcRegion.Set (cmdline, filename);
}



function _Get(this, region, ts)
{
    table   := this.thetable;
    cmdline := vcTable.Get (table, 0, 0);
    vcRegion.Get (cmdline, ts);
}



function _Destroy(this)
    if (this.popup != nil)
    {
        w := this.popup;
        this.popup = nil;
        vcWin.Destroy (w);
    }



/*
 * This command is run whenever file selection box changes values,
 * so that the new value is recorded in the macro file in a coherent
 * manner.  We do this in lieu of making the file selection popup
 * macro recordable -- those don't record so well.
 */
function reset_value(table, event, value)
{
    cmdline := vcTable.Get (table, 0, 0);
    vcCmdline.Set (cmdline, value);
}


reset_value_cmd := vcWv.Cmd (fac, reset_value, "reset", "ResetValue",
                             vcWv.Param ("", vlStringVT, vcWv.Constant, ""));


vcBind.Set (bindings, '?', browse_cmd);
vcBind.Set (bindings, '\rightdown', browse_cmd);

    
vcFac.CopyPrefs (vcFileSelect.fac, fac);

/* register this editor so a default value may be omitted */
vcDialog.RegisterNoDefault (vcFilePop.fac, "", NotifyOnChange);

