/*
 * $Revision: 1.1.1.1 $
 *
 * vcNotebook facility
 */
/*\vscmuserSection vcNotebook. -- notebook tab facility
 *
 * DESCRIPTION
 *     This facility provides a set up labelled notebook tabs,
 *     which allow the user to flip among panels of controls.
 *     This uses the vcNoteTab.(man) facility, and that facility's
 *     API may be used to control these regions. (A vcNotebook 
 *     region may be passed right into a vcNoteTab function.)
 *
 *     The user creates a vcNotebook region, and then adds regions
 *     to it, each associated with a string, which appears on the
 *     tab. The user may then flip through the regions.
 *
 *     Tab labels may be 'formatted' as described in the vcStaticText.
 *     man page.
 *
 *     Tabs may be enabled and disabled, via the vcNoteTab API.
 *
 * INITIALIZATION DATA
 *     vgAlignVT - the alignment of the tabbed regions
 *
 * SEE ALSO
 *     vcNoteTab.(man)
 */

function _Create (align, rgn) {

    o := vcRegion.Create ("", vcOverlay.fac, align);
    nt := vcRegion.Create (vcRegion.Name (rgn), vcNoteTab.fac, o,
                  macro_on <- vcRegion.MacroOn (rgn));
    t := vcTable.Make ("", 2, 1, nt, o);

    vcRegion.AddChild (rgn, t);

    vcNoteTab.CallOnSelect (nt, func (r) {
                                   vcOverlay.SetCurrent (o, r);
                                   if (r == nil)
                                       vcRegion.QueueForRefresh (rgn);});

    struct {
        overlay = o;
        notetab = nt
    };
}

/*\vscmuserDocpage vcNotebook.AddEntry -- add region to notebook
 *
 * DESCRIPTION
 *     This function adds entries to the notebook. The child region
 *     is placed among the flippable regions, and is associated with
 *     the tab with the given label. 
 *
 * SYNOPSIS
 *     vlVoidVT vcNotebook.AddEntry (vcRegionVT notebook, 
 *                                   vlStringVT str,
 *                                   vcRegionVT child)
 */
function AddEntry (rgn, str, child) {
    data := vcRegion.GetData (rgn);
    vcRegion.AddChild (data.overlay, child);
    vcNoteTab.AddEntry (data.notetab, str, child);
}


