/* ---------------------------------------------------------------------- *\
 * Module -- mpfload.vs
 *
 *      Facility:      Vtime
 *
 *      Environment:   SUN4, Powerview 5.2
 *
 *      Authors:       Mani Alagappan,  Dec 1, 1993
 *                     Jeff Hare
 *
 *      Description:   vtime facility vs file
 *       
 *
 *      Copyright (c) 1993 by Viewlogic Systems, Inc.
 *
\* ---------------------------------------------------------------------- */
/*\Section MPF_Parser -- Motive MPF results parser
*/

msg_PathUnavailable := cons(2080, "No Path Trace display available for the last trace");
msg_McpNothingSelected := cons(2082, "Select DataPath Components to appear in the MCP first.");
msg_InfoNothingSelected := cons(2083, "Select Component(s) or a Net first.");

/*===========================================================================*/
/*==                     N e w V m p f                                     ==*/
/*===========================================================================*/
/*\Docpage NewVmpf -- Creates and initializes a Verification MPF structure.
*/
function NewVmpf() {
   struct {                  /*----------------------------------------------*/
      Key        = "VMPF";   /* Verify MPF Root Structure                    */
      RunNumber  = 0;        /* Verification run number -resets each session */
      Margins    = nil       /* List() of Margin structures                  */
   };
}


/*===========================================================================*/
/*==                     N e w T m p f                                     ==*/
/*===========================================================================*/
/*\Docpage NewTmpf -- Creates and initializes a Trace MPF structure.
*/
function NewTmpf() {
   struct {              /*--------------------------------------------------*/
      Key     = "TMPF";  /* Trace MPF Root Structure                         */
      Loc     = nil;     /* Target Part Pin of this Trace                    */
      Etype   = nil;     /* Edge of data being analyzed                      */
      Edesc   = nil;     /* Description of the constraint & data edge        */
      ModType = nil;     /* Model name of the Trace target component         */
      Constraints = nil  /* List() of Constraint structures                  */
   }; 
}


/*===========================================================================*/
/*==                     N e w C o n s t r a i n t                         ==*/
/*===========================================================================*/
/*\Docpage NewConstraint -- Creates and initializes a Constraint structure
*/
function NewConstraint(edge)
{
   struct {            /*-----------------------------------------------------*/
      Key      = "CON";/* Identify structure as a constraint structure        */
      Ctype    = nil;  /* Constraint being analyzed: S,T,P,K                  */
      ConPart  = nil;  /* Always matches the LOC Part ID                      */
      ConPin   = nil;  /* Always matches the LOC Pin  ID                      */
      ConEdge  = edge; /* Edge of Data being analyzed                         */
      ClkPart  = nil;  /* ID of the 'virtual' part containing ClkPin & RefPin */
      ClkPin   = nil;  /* Clock pin on ClkPart                                */
      ClkEdge  = nil;  /* Clock Input edge direction [R|F] for CTYPE          */
      RefPin   = nil;  /* Time reference trigger  pinname. SETHLD wrt CTYPE T */
      ClkPath  = nil;  /* List of records found in Clock Path                 */
      DataPath = nil;  /* List of records found in Data Path                  */
      Margin   = nil;  /* Margin structure  SH,PL,PH,SK                       */
      TrcMpf   = nil   /* Reference to the TrcMpf structure this belongs to   */
   };
}

/*===========================================================================*/
/*==                     N e w N e t                                       ==*/
/*===========================================================================*/
/*\Docpage NewNet  -- Creates and initializes a Net record for path display
*/
function NewNet(C) {
   struct {              /*--------------------------------------------------*/
      Key     = "NET";   /* Net structure for path display                   */
      Ptype   = nil;     /* 1=Datapath 2=Clockpath 3=Both                    */
      Net     = nil;     /* Net Name (N) or (NR) record                      */
      Idd     = nil;     /* Interconnect delays being applied (IDD*) record  */
      Constraint = C     /* Reference to the Constraint structure            */
   }; 
}

/*===========================================================================*/
/*==                     N e w I n s t                                     ==*/
/*===========================================================================*/
/*\Docpage NewInst -- Creates and initializes a Instance record for path dsp
*/
function NewInst(C) {
   struct {              /*--------------------------------------------------*/
      Key     = "INST";  /* Instance structure for path display              */
      Ptype   = nil;     /* 1=Datapath 2=Clockpath 3=Target Part             */
      Ipin    = nil;     /* Input Pin record   (I)                           */
      Opin    = nil;     /* Output Pin record  (O)                           */
      Model   = nil;     /* Model record       (M)                           */
      Dtype   = nil;     /* Delay Type record  (DT)                          */
      Constraint = C     /* Reference to the Constraint structure            */
   }; 
}


function GetConInst(lst) 
{
   n := nil;
   nd := nil;

   while (lst != nil) {
      nd = car(lst);
      /*printf("nd: [%s], Key: [%s], Ptype: [%s]\n", nd,nd.Key,nd.Ptype);*/
      if ((nd.Ptype == 3) && string_eq(nd.Key,"INST")) 
         lst = nil;
      else {
         nd = nil;
         lst = cdr(lst);
      }
   }
   /*printf("Returning %s\n",nd);*/
   nd;
}

function GetPart(i) 
{
   /*printf("GetPart: %s\n",i);*/
   if (i.Ipin != nil)
      list_ref(i.Ipin,1);
   else
      nil;
}
      
function GetIpin(i) 
{
   /*printf("GetIpin: %s\n",i);*/
   if (i.Ipin != nil)
      list_ref(i.Ipin,2);
   else
      nil;
}

function GetOpin(i) 
{
   if (i.Opin != nil)
      list_ref(i.Opin,2);
   else
      nil;
}

function GetModel(i) 
{
   if (i.Model != nil)
      list_ref(i.Model,2);
   else
      nil;
}
      
function GetDtype(i) 
{
   if (i.Dtype != nil)
      list_ref(i.Dtype,2);
   else
      nil;
}

function GetItime(i) 
{
   /*printf("GetItime: %s\n",i);*/
   if (i.Ipin != nil)
      list_ref(i.Ipin,4);
   else
      nil;
}

function GetOtime(i) 
{
   if (i.Opin != nil)
      list_ref(i.Opin,4);
   else
      nil;
}

function GetNet(i) 
{
   /*printf("GetNet: %s\n",i);*/
   if (i.Net != nil)
      list_ref(i.Net,1);
   else
      nil;
}
function GetIdd(i) 
{
   /*printf("GetIdd: %s\n",i);*/
   if (i.Idd != nil && length(i.Idd) > 3)
      list_ref(i.Idd,3);
   else
      if (i.Net != nil && length(i.Net) > 3)
         list_ref(i.Net,3);
      else
         "none";
}

function GetCMargin(c) 
{
   if (c.Margin != nil) 
      c.Margin;
   else
      nil;
}
/*===========================================================================*/
/*==                     P a r s e T r c M p f                             ==*/
/*===========================================================================*/
/*\Docpage ParseTrcMpf -- Loads a Trace MPF File into a Tmpf structure
*/
function ParseTrcMpf(d, event) 
{
   declare line;
   declare lst;
   declare more;
   declare MpfFile;
   declare Tmpf;
   declare win;

   TrcMpfs := nil;


   more    = vlTrueFalse.True;
   MpfFile = vtime.GetMtvVar(d, MtvVar.MV_file_tmp_mpf_trace, MtvValue.VALUE); 
   Tmpf    = NewTmpf();

   if (fexists(MpfFile)) {
      fp := fopen(MpfFile, "r");

      win = vcEvent.Window(event);
      vcWin.ClearStatus(win);
      vcWin.AddStatus(win,"Reading MPF path data...");

      while (more && (line = fgetline(fp)) != nil) {
         lst = nil;

         foreach(token; string_loop(line, " \t\n"))
            lst = cons(token,lst);
         lst = reverse_destructive(lst);
         
         if (string_eqi("LOC",car(lst))) 
            Tmpf.Loc = cdr(lst);
         
         else if (string_eqi("ETYPE",car(lst))) 
            Tmpf.Etype = cdr(lst);
         
         else if (string_eqi("MODTYPE",car(lst))) 
            Tmpf.ModType = cdr(lst);
         
         else if (string_eqi("CTYPE",car(lst))) {
            vcWin.ClearStatus(win);
            vcWin.AddStatus(win,"Parsing Constraints...");
            Tmpf.Constraints = ParseConstraint(fp,lst,Tmpf);
         }
         else if (string_eqi("END",car(lst))) {
            TrcMpfs = cons(Tmpf,TrcMpfs);
            /*printf("TrcMpfs: %s,  Tmpf: %s\n",TrcMpfs,Tmpf);*/
            Tmpf    = NewTmpf(); /* Make a new one in case mpf file has more than one... */
            
         }
      }
/*      PrintTrace(TrcMpfs);*/

      if (Tmpf != nil && TrcMpfs != nil) 
         GenPath(d,event,TrcMpfs);
      else {
         vcWin.ClearStatus(win);
         vcWin.AddStatus(win,"No valid Path available...");
      }


   }
   else {
      msg := sprintf(cdr(vtime.msg_PathUnavailable));
      vtime.AlertBox(d,event,car(msg_PathUnavailable),vlErrSev.Error,msg);
   }
}



/*===========================================================================*/
/*==                     P a r s e C o n s t r a i n t                   ==*/
/*===========================================================================*/
/*\Docpage ParseConstraint -- Load the next constraint block from MPF file
*/
function ParseConstraint(fp,inlst,tmpf)
{
   declare line;
   declare lst;

   more := vlTrueFalse.True;

   c := NewConstraint(tmpf.Etype);
   c.TrcMpf = tmpf;
   c.Ctype = cdr(inlst);

   while (more) {
      if ((line = fgetline(fp)) == nil) 
         more = vlTrueFalse.False;
      else {
         
         lst = nil;
         foreach(token; string_loop(line, " \t\n"))
            lst = cons(token,lst);
         
         lst = reverse_destructive(lst);
         
         if (string_eqi("CONPART",car(lst)))
            c.ConPart = cdr(lst);
         
         else if (string_eqi("CONPIN",car(lst))) 
            c.ConPin  = cdr(lst);
         
         else if (string_eqi("CLKPART",car(lst))) 
            c.ClkPart = cdr(lst);
         
         else if (string_eqi("CLKPIN",car(lst))) 
            c.ClkPin  = cdr(lst);
         
         else if (string_eqi("CLKEDGE",car(lst))) 
            c.ClkEdge = cdr(lst);
         
         else if (string_eqi("REFPIN",car(lst))) 
            c.RefPin  = cdr(lst);
         
         else if (string_eqi("CLKPATH",car(lst))) 
            c.ClkPath  = ParsePath(fp,2);
         
         else if (string_eqi("DATAPATH",car(lst))) 
            c.DataPath = ParsePath(fp,1);
         
         else if (string_eqi("SH",car(lst))) {
            more = vlTrueFalse.False;
            c.Margin = lst;
         }
         else if (string_eqi("SK",car(lst))) {
            more = vlTrueFalse.False;
            c.Margin = lst;
         }
         else if (string_eqi("PL",car(lst))) {
            more = vlTrueFalse.False;
            c.Margin = lst;
         }
         else if (string_eqi("PH",car(lst))) {
            more = vlTrueFalse.False;
            c.Margin = lst;
         }
         /*print(line);*/
      }
   }
   c;
}

/*===========================================================================*/
/*==                     P a r s e P a t h                                 ==*/
/*===========================================================================*/
/*\Docpage ParsePath -- Load an MPF Path into a list of structures.
*/
function ParsePath(fp,type)
{
   declare line;
   declare lst;
   more := vlTrueFalse.True;
   item := nil;

   while (more && (line = fgetline(fp)) != nil) {
      lst = nil;
      foreach(token; string_loop(line, " \t\n"))
         lst = cons(token,lst);

      lst = reverse_destructive(lst);
      
      if (string_eqi(")",car(lst)))
         more = vlTrueFalse.False;

      else if (string_eqi("I",car(lst))) 
         item = cons(cons(type,lst),item);

      else if (string_eqi("O",car(lst))) 
         item = cons(cons(type,lst),item);

      else if (string_eqi("M",car(lst))) 
         item = cons(cons(type,lst),item);

      else if (string_eqi("N",car(lst))) 
         item = cons(cons(type,lst),item);

      else if (string_eqi("NR",car(lst))) 
         item = cons(cons(type,lst),item);

      else if (string_eqi("PI",car(lst))) 
         item = cons(cons(type,lst),item);

      else if (string_eqi("IDD",car(lst))) 
         item = cons(cons(type,lst),item);

      else if (string_eqi("IDD_def",car(lst))) 
         item = cons(cons(type,lst),item);

      else if (string_eqi("DT",car(lst))) 
         item = cons(cons(type,lst),item);
   }
   
   reverse_destructive(item);
}



function ProcessPath(PathRecords,Constraint) 
{
   path := nil;
   Net  := nil;
   Inst := nil;
   r    := nil;
   rec  := nil;
   last := 0; /* 0=start, 1=Inst, 2=Net */

   foreach(r; vscript.list_iter(PathRecords)) {
      
      rec = cdr(r);
      
      if (string_eq("N",car(rec)) || string_eq("NR",car(rec))) {
         if (Net != nil) {
            path = cons(Net,path);
            /* printf("Storing Net %s\n",Net.Net); */
         }     
         Net = NewNet(Constraint);
         Net.Net = rec;
         Net.Ptype = car(r);
         /*print(Net.Net);*/
      }
      
      if (string_eq("IDD",car(rec)) || string_eq("IDD_def",car(rec))) {
         if (Net == nil) Net = NewNet(Constraint);
         Net.Idd = rec;
         Net.Ptype = car(r);
         /*print(Net.Idd);*/
      }
      
      if (string_eq("I",car(rec))) {
         if (Inst != nil) {
            path = cons(Inst,path);
            /*printf("Storing Inst %s\n",Inst.Ipin);*/
         }
         Inst = NewInst(Constraint);
         Inst.Ipin = rec;
         Inst.Ptype = car(r);
         /*print(Inst.Ipin);*/
      }
      
      if (string_eq("O",car(rec))) {
         if (Inst == nil) Inst = NewInst(Constraint);
         Inst.Opin = rec;
         Inst.Ptype = car(r);
         /*print(Inst.Opin);*/
      }
      
      if (string_eq("M",car(rec))) {
         if (Inst == nil) Inst = NewInst(Constraint);
         Inst.Model = rec;
         Inst.Ptype = car(r);
         /*print(Inst.Model);*/
      }
      
      if (string_eq("DT",car(rec))) {
         if (Inst == nil) Inst = NewInst(Constraint);
         Inst.Dtype = rec;
         Inst.Ptype = car(r);
         /*print(Inst.Dtype);*/
      }
      
   }
   if (Net != nil) {
      path = cons(Net,path);
      /*printf("Storing Net %s\n",Net.Net);*/
   }
   if (Inst != nil) {
      Inst.Ptype = 3;
      path = cons(Inst,path);
      /*printf("Storing Inst %s\n",Inst.Ipin);*/
   }
   path;   
}

/*===========================================================================*/
/*==                     P r i n t T m p f                                 ==*/
/*===========================================================================*/
/*\Docpage PrintTmpf  -- Prints out a Tmpf structure 
*/
function PrintTmpf(s)
{
   printf("LOC: %s\n",s.Loc);
   printf("ETY: %s\n",s.Etype);
   printf("Mod: %s\n",s.ModType);
   printf("Constraint: \n");

   c := s.Constraints;
   printf("CTYPE: %s\n",c.Ctype);
   printf("PART: %s\n",c.ConPart);
   printf("PIN: %s\n",c.ConPin);
   printf("EDGE: %s\n",c.ConEdge);
   printf("ClkPart: %s\n",c.ClkPart);
   printf("ClkPin: %s\n",c.ClkPin);
   printf("ClkEdge: %s\n",c.ClkEdge);
   printf("RefPin: %s\n",c.RefPin);
   printf("ClkPath: %s\n",c.ClkPath);
   printf("DataPath: %s\n",c.DataPath);
}

/*===========================================================================*/
/*==                     P r i n t P a t h                                 ==*/
/*===========================================================================*/
/*\Docpage PrintPath -- Prints out a path list
*/
function PrintPath(l)
{
   printf("\n\n New Path \n\n");
   foreach(s; vscript.list_iter(l)) {
      if (string_eq(s.Key,"NET")) {
         printf("N- Net: %s\n",s.Net);
         printf("N- Idd: %s\n",s.Idd);
         printf("N- Pty: %d\n",s.Ptype);
      }
      else {
         printf("I- Ipin:  %s\n",s.Ipin);
         printf("I- Opin:  %s\n",s.Opin);
         printf("I- Model: %s\n",s.Model);
         printf("I- Dtype: %s\n",s.Dtype);
         printf("I- Ptype: %d\n",s.Ptype);
      }
   }
}

/*===========================================================================*/
/*==                     P r i n t T r a c e                               ==*/
/*===========================================================================*/
/*\Docpage PrintTrace -- Prints out a path trace data structure
*/
function PrintTrace(pathlist)
{
   declare s;
   declare c;

   foreach(s; vscript.list_iter(pathlist)) {

      PrintTmpf(s);

   }
}





/*===========================================================================*/
/*==                    G e n P a t h                                      ==*/
/*===========================================================================*/
/*\Docpage GenPath  -- 
*/
function GenPath(d,e,pathlist)
{
   declare nd;

   declare s1;
   declare s2;

   declare ClockSource;
   declare DataSource;

   declare c;
   declare win;

   declare Dpath;
   declare Cpath;
   declare edges;
   declare edge_data;
   declare clkpart;
   declare datapart;

   declare s;
   declare root;
   roots := nil;


   /*--------------------------*/
   /* Create a new tree region */
   /*--------------------------*/
   pregion := vcRegion.Create("Path Trace Results",
                           path.fac,
                           cons(d,e));

   pdata := vcRegion.GetData(pregion);

   /*------------------------------------------------*/
   /* This MPF file may contain many traces appended */
   /*------------------------------------------------*/
   foreach(s; vscript.list_iter(pathlist)) {
      /*printf("Generating Path: %s\n",s);*/

      c = s.Constraints;

      if (c != nil) {
         if (c.ClkPath != nil && c.DataPath != nil) {
            
            if (c.DataPath != nil) {
               Dpath = ProcessPath(c.DataPath,c);
               /*PrintPath(Dpath);*/
            }
            
            if (c.ClkPath != nil) {
               Cpath = ProcessPath(c.ClkPath,c);
               /*PrintPath(Cpath);*/
            }
            /*printf("Edge Type: %s\n",s.Etype);*/
            if (string_eqi(car(s.Etype),"LHMAX")) {
               s.Edesc = "Setup - Rising Edge of Data";
               ndx := 3;
            } 
            else if (string_eqi(car(s.Etype),"HLMAX")) {
               s.Edesc = "Setup - Falling Edge of Data";
               ndx := 4;
            }
            else if (string_eqi(car(s.Etype),"LHMIN")) {
               s.Edesc = "Hold - Rising Edge of Data";
               ndx := 5;
            }
            else if (string_eqi(car(s.Etype),"HLMIN")) {
               s.Edesc = "Hold - Falling Edge of Data";
               ndx := 6;
            }
            
            clkpart = GetConInst(Cpath);
            datapart= GetConInst(Dpath);
            
            mgn := "";
            if ((m := GetCMargin(c)) != nil) {
               v := car(m);
               if (v[0] == 'S') {
                  if (ndx < length(m)) {
                     mval := list_ref(m,ndx);
                     if (mval[0] != 'x') 
                        if (string2number(mval) < 0)
                           mgn = sprintf("Violation by:   %s ns",mval);
                        else
                           mgn = sprintf("Margin amount:   %s ns",mval);
                     else
                        mgn = nil;
                  }
               }
            }
            if (mgn != nil) {  /* Can't draw this one without a margin. */
               ce := car(c.ClkEdge);
               if (ce != nil)
                  edgestr := ce[0] == 'R' ? "Rising Edge Clock" : "Falling Edge Clock";
               else
                  edgestr := "Unknown Clock Edge";
               root = path.RootNode(pregion,  
                                    GetPart(clkpart),
                                    GetModel(clkpart),
                                    GetIpin(clkpart),
                                    GetIpin(datapart),
                                    GetItime(clkpart),
                                    GetItime(datapart),
                                    edgestr,
                                    s.Edesc,
                                    mgn,
                                    datapart);
               
               roots = cons(root,roots);
               
               win = vcEvent.Window(e);
               vcWin.ClearStatus(win);
               vcWin.AddStatus(win,sprintf("Creating %s Path for %s",car(s.Etype),car(c.ConPart)));
               
               edges = DrawPath(Cpath,root,pregion, "Clock Path");
               
               if (edges != nil)
                  edge_data = cons(edges,edge_data);
               
               edges  = DrawPath(Dpath,root,pregion, "Data Path");
               
               if (edges != nil)
                  edge_data = cons(edges,edge_data);
            }
         }
      }
   }

   if (roots != nil) {
      vcTree.SetRoot(pdata.tree, roots);
      
      declare src;
      declare dst;
      declare data;
      declare edge;
      
      foreach (edges; vscript.list_iter(edge_data)) {
         foreach (e; vscript.list_iter(edges)) {
            src = list_ref(e,0);
            dst = list_ref(e,1);
            data= list_ref(e,2);
            edge = vcTree.GetEdge(src,dst);
            vcTree.EdgeSetItem(edge,data);
         }
      }           
      
      win = vcEvent.Window(e);
      vcWin.ClearStatus(win);
      vcWin.AddStatus(win,"Creating Path Display...");

      foreach (nd; vscript.list_iter(roots)) {
         path.DrawPath(pregion, nd);
      }
      
      vtime.DisplaySet (d, pregion, "Path");
      vcWin.ClearStatus(win);
      vcWin.AddStatus(win,"Done...");
   }
   else {
      win = vcEvent.Window(e);
      vcWin.ClearStatus(win);
      vcWin.AddStatus(win,"No Path Display Available due to unknown margin (x)");
   }
}


/*===========================================================================*/
/*==                    D r a w P a t h                                     ==*/
/*===========================================================================*/
/*\Docpage DrawPath  -- Creates and Draws a path display of a trace report
*/
function DrawPath(l,root,pregion,StubLabel)
{
   declare n;
   node := root;
   declare owner;
   declare from;
   declare to;
   declare edge;
   head := nil;
   pending := nil;
   edge_data := nil;

   foreach (n; vscript.list_iter(l)) {
      if (n.Ptype != 3) {
         if (string_eq(n.Key,"INST")) {
            owner = node;
            node = path.CompNode(pregion, 
                                 owner, 
                                 "foonet",                  /* Fix this!!!!!!!! */
                                 GetPart(n),
                                 GetModel(n),
                                 GetIpin(n),
                                 GetOpin(n),
                                 GetItime(n),
                                 GetOtime(n),
                                 GetDtype(n),                 /* Fix this!!!!!!!! */
                                 vcTree.Border.Box,n);

            if (head == nil) head = node;

            if (pending != nil) {
               edge_data = cons(list(owner,node,pending),edge_data);
               pending = nil;
            }
         }
         else {
            pending = n;
         }
      }
   }

   owner = node;
   node = path.StubNode(pregion,
                      owner,
                      StubLabel);
   
   if (pending != nil) 
      edge_data = cons(list(owner,node,pending),edge_data);
   

   edge_data;
}





   
<cmdPathDisplay,prvCmdPathDisplay> := vtime.CmdCreate(vtime.fac,
                                                      ParseTrcMpf,
                                                      "Path",
                                                      "Display a Worst Case traced path in the display region",nil);

<cmdTparse,prvcmdTparse> := vtime.CmdCreate(vtime.fac,                                   
                             ParseTrcMpf,
                             "_tparse",
                             "Parse a Trace Report and generate a critical path display.", nil);







/*===========================================================================*/
/*==                     P a r s e V r c M p f                             ==*/
/*===========================================================================*/
/*\Docpage ParseVrcMpf -- Loads a Verification MPF File into a Vmpf structure
*/
function ParseVfyMpf(d,e)
{
   declare line;
   declare lst;
   declare more;
   declare MpfFile;
   declare Vmpf;
   declare the_rest;
   declare key;

   declare m;
   declare tsr;
   declare tsf;
   declare thr;
   declare thf;
   declare l;
   declare part;
   declare pin;
   declare wrn;
   setups := nil;
   holds  := nil;
   pulses := nil;
   skews  := nil;
   line_count := 0;
   declare llen;
   declare token_count;
   more    = vlTrueFalse.True;

   return_value := nil;

   MpfFile := vtime.GetMtvVar(d, MtvVar.MV_file_tmp_mpf_report, MtvValue.VALUE);

   win := vcEvent.Window(e);
   vcWin.ClearStatus(win);
   vcWin.AddStatus(win,"Reading Verification Report data...");


   Vmpf    = NewVmpf();

   if (fexists(MpfFile)) {
      fp := fopen(MpfFile, "r");
      
      while (more && (line = fgetline(fp)) != nil) {
         lst = nil;
         line_count = line_count + 1;
         vcWin.ClearStatus(win);
         vcWin.AddStatus(win,sprintf("Processing MPF Verification Report (line %d).",line_count));
         token_count = 0;
         llen = 0;
         foreach(token; string_loop(line, " \t\n")) {
            token_count = token_count + 1;
            if (token[0] == '(') 
               llen = token_count;
            lst = cons(token,lst);
         }

         lst = reverse_destructive(lst);

         
         
         if (string_eqi("SH",car(lst))) {
            if (llen > 7) {
               l = cdr(lst);
               part = car(l); l=cdr(l);
               pin  = car(l); l=cdr(l);
               tsr  = car(l); l=cdr(l);
               tsf  = car(l); l=cdr(l);
               thr  = car(l); l=cdr(l);
               thf  = car(l);
               wrn  = l;
               
               
               /*-----------------------*/
               /* Setup Time Violations */
               /*-----------------------*/
               if (tsr[0] == 'x' || thr[0] == 'C')
                  setups = cons(sprintf("%-5s %-7s %s %-5s %s","Setup",tsr,"R",pin,part),setups);
               else
                  if (string2number(tsr) < 0)
                     setups = cons(sprintf("%-5s %-7s %s %-5s %s","Setup",tsr,"R",pin,part),setups);
               
               if (tsf[0] == 'x' || thr[0] == 'C') 
                  setups = cons(sprintf("%-5s %-7s %s %-5s %s","Setup",tsf,"F",pin,part),setups);
               else
                  if (string2number(tsf) < 0)
                     setups = cons(sprintf("%-5s %-7s %s %-5s %s","Setup",tsf,"F",pin,part),setups);
               
               
               /*----------------------*/
               /* Hold Time Violations */
               /*----------------------*/
               if (thr[0] == 'x' || thr[0] == 'C')
                  holds  = cons(sprintf("%-5s %-7s %s %-5s %s","Hold",thr,"R",pin,part),holds);
               else
                  if (string2number(thr) < 0)
                     holds  = cons(sprintf("%-5s %-7s %s %-5s %s","Hold",thr,"R",pin,part),holds);
               
               if (thf[0] == 'x' || thf[0] == 'C')
                  holds  = cons(sprintf("%-5s %-7s %s %-5s %s","Hold",thf,"F",pin,part),holds);
               else
                  if (string2number(thf) < 0)
                     holds  = cons(sprintf("%-5s %-7s %s %-5s %s","Hold",thf,"F",pin,part),holds);
               
            }
            Vmpf.Margins = cons(lst,Vmpf.Margins);  /* Just for grins. */
         }
          
         else if (string_eqi("SK",car(lst))) {
            if (llen > 5) {
               l = cdr(lst);
               part = car(l); l=cdr(l);
               pin  = car(l); l=cdr(l);
               m    = car(l);
               wrn  = l;
               
               /*-----------------*/
               /* Skew Violations */
               /*-----------------*/
               if (m[0] == 'x' || m[0] == 'C')
                  skews = cons(sprintf("%-5s %-7s %s %-5s %s","Skew",m,"-",pin,part),skews);
               else
                  if (string2number(m) < 0)
                     skews = cons(sprintf("%-5s %-7s %s %-5s %s","Skew",m,"-",pin,part),skews);
            }
            Vmpf.Margins = cons(lst,Vmpf.Margins);
         }

         else if (string_eqi("PL",car(lst))) {
            if (llen > 5) {
               l = cdr(lst);
               part = car(l); l=cdr(l);
               pin  = car(l); l=cdr(l);
               m    = car(l);
               wrn  = l;
               
               /*----------------------*/
               /* Pulse Low Violations */
               /*----------------------*/
               if (m[0] == 'x' || m[0] == 'C')
                  pulses = cons(sprintf("%-5s %-7s %s %-5s %s","Pulse",m,"L",pin,part),pulses);
               else
                  if (string2number(m) < 0)
                     pulses = cons(sprintf("%-5s %-7s %s %-5s %s","Pulse",m,"L",pin,part),pulses);
            }
            Vmpf.Margins = cons(lst,Vmpf.Margins);
         }
         else if (string_eqi("PH",car(lst))) {
            if (llen > 5) {
               l = cdr(lst);
               part = car(l); l=cdr(l);
               pin  = car(l); l=cdr(l);
               m    = car(l);
               wrn  = l;
               
               /*-----------------------*/
               /* Pulse High Violations */
               /*-----------------------*/
               if (m[0] == 'x' || m[0] == 'C')
                  pulses = cons(sprintf("%-5s %-7s %s %-5s %s","Pulse",m,"H",pin,part),pulses);
               else
                  if (string2number(m) < 0)
                     pulses = cons(sprintf("%-5s %-7s %s %-5s %s","Pulse",m,"H",pin,part),pulses);
            }
            Vmpf.Margins = cons(lst,Vmpf.Margins);
         }
         else if (string_eqi("P",car(lst))) 
            Vmpf.Margins = cons(lst,Vmpf.Margins);
         
         else if (string_eqi("RUN",car(lst))) 
            Vmpf.RunNumber = cdr(lst);
         
         else if (string_eqi("END",car(lst))) 
            more = vlTrueFalse.False;
            
      }
      
      return_value = list (Vmpf,d,e,setups, holds, pulses, skews);
      
   }   
   return_value;
}


