function init_menu() {
    vcCmd.Enable (read_wire_cmd,            vlOnOff.On);
    vcCmd.Enable (set_tech_cmd,            vlOnOff.On);
    vcCmd.Enable (set_in_tech_cmd,            vlOnOff.On);
    vcCmd.Enable (input_options_cmd,            vlOnOff.On);
    vcCmd.Enable (synth_cmd,            vlOnOff.Off);
    vcCmd.Enable (resynth_cmd,            vlOnOff.Off);
    vcCmd.Enable (wire_cmd,            vlOnOff.Off);
    vcCmd.Enable (iopad_cmd,            vlOnOff.Off);
    vcCmd.Enable (outdir_cmd,            vlOnOff.Off);
    vcCmd.Enable (report_cmd,            vlOnOff.Off);
    vcCmd.Enable (show_report_cmd,            vlOnOff.Off);
    vcCmd.Enable (map_cmd,            vlOnOff.Off);
    vcCmd.Enable (constraint_cmd,     vlOnOff.Off );
    vcCmd.Enable (set_points_cmd,        vlOnOff.Off);
} /* end of init_menu */

function close_all_commands() {

    vcCmd.Enable (read_wire_cmd,            vlOnOff.Off);
    vcCmd.Enable (set_tech_cmd,            vlOnOff.Off);
    vcCmd.Enable (set_in_tech_cmd,            vlOnOff.Off);
    vcCmd.Enable (input_options_cmd,            vlOnOff.Off);
    vcCmd.Enable (synth_cmd,            vlOnOff.Off);
    vcCmd.Enable (resynth_cmd,            vlOnOff.Off);
    vcCmd.Enable (wire_cmd,            vlOnOff.Off);
    vcCmd.Enable (iopad_cmd,            vlOnOff.Off);
    vcCmd.Enable (outdir_cmd,            vlOnOff.Off);
    vcCmd.Enable (report_cmd,            vlOnOff.Off);
    vcCmd.Enable (show_report_cmd,            vlOnOff.Off);
    vcCmd.Enable (map_cmd,            vlOnOff.Off);
    vcCmd.Enable (constraint_cmd,     vlOnOff.Off );
    vcCmd.Enable (set_points_cmd,        vlOnOff.Off);
    vcCmd.Enable (viewgen_cmd,        vlOnOff.Off);
    vcCmd.Enable (start_macro_cmd,        vlOnOff.Off);
    vcCmd.Enable (close_macro_cmd,        vlOnOff.Off);
    vcCmd.Enable (run_macro_cmd,        vlOnOff.Off);
    vcCmd.Enable (show_aliases_cmd,        vlOnOff.Off);
    vcCmd.Enable (set_alias_cmd,        vlOnOff.Off);
    vcCmd.Enable (remove_alias_cmd,        vlOnOff.Off);
    vcCmd.Enable (run_system_cmd,        vlOnOff.Off);
    vcCmd.Enable (clear_lib_cmd,        vlOnOff.Off);
    vcCmd.Enable (list_lib_cmd,        vlOnOff.Off);
    vcCmd.Enable (show_setup_cmd,        vlOnOff.Off);
    vcCmd.Enable (show_history_cmd,        vlOnOff.Off);
    vcCmd.Enable (version_cmd,        vlOnOff.Off);
    vcWin.UpdateMenus (vdes2_win);
}

vcWv.MakeEnum ("YesNoVT", list ("Yes", "No"));
inlib_list := " ";
patch_string := "V";
gndnet_list := "GND";
vddnet_list := "VDD";
glbnet_list := " ";
alias_list := " ";
cell_file_list := " ";
tri2mux_flag := YesNo.No;

function get_tri2mux_flag(data) {
   return(tri2mux_flag);
}

function set_input_options(d,e, patch, tri2, gnd, vdd, pin, aliases, cellfile) { 

   declare tri2mux_string;


   if(!string_eq( patch_string, patch )) {
       patch_string = patch;
       vcShell.TypeIn( vdes_shell, "patch_string "+patch_string, TRUE);
   }
   tri2mux_flag = tri2;
   tri2mux_string = convert(string, tri2);
   if( string_eq("Yes", tri2mux_string ))
       vcShell.TypeIn( vdes_shell, "tri2mux", TRUE);
   else
       vcShell.TypeIn( vdes_shell, "not_tri2mux", TRUE );
   if(!string_eq( gndnet_list, gnd )) {
       gndnet_list = gnd;
       vcShell.TypeIn( vdes_shell, "not_ground_net *", TRUE);
       vcShell.TypeIn( vdes_shell, "ground_net "+gndnet_list, TRUE);
   }
   if(!string_eq( vddnet_list, vdd )) {
       vddnet_list = vdd;
       vcShell.TypeIn( vdes_shell, "not_vdd *", TRUE);
       vcShell.TypeIn( vdes_shell, "vdd_net "+vddnet_list, TRUE);
   }
   if(!string_eq( glbnet_list, pin )) {
       if( !string_eq( glbnet_list, " " ))
            vcShell.TypeIn( vdes_shell, "not_pin *", TRUE );
       glbnet_list = pin;
       vcShell.TypeIn( vdes_shell, "add_pin "+glbnet_list, TRUE);
   }
   if(!string_eq( alias_list, aliases )) {
       alias_list = aliases;
       vcShell.TypeIn( vdes_shell, "hier_lib "+alias_list, TRUE);
   }
   if(!string_eq( cell_file_list, cellfile )) {
       cell_file_list = cellfile;
       vcShell.TypeIn( vdes_shell, "hier_cell "+cell_file_list, TRUE);
   }

}

input_options_cmd := vcWv.Cmd( fac, set_input_options, "input_options",
                             "input_options",
                   vcWv.Param( "Patch string", string, vcWv.Sticky, patch_string,
                               "Enter patch string"),
                   vcWv.Param( "Translate tri-states to muxes", YesNoVT, vcWv.Dynamic,
                               get_tri2mux_flag),
                   vcWv.Param( "Ground nets", string, vcWv.Sticky, gndnet_list,
                               "Enter ground net names"),
                   vcWv.Param( "Power nets", string, vcWv.Sticky, vddnet_list,
                               "Enter Power net names"),
                   vcWv.Param( "Global pins", string, vcWv.Sticky, glbnet_list,
                               "Enter global nets which should be treated as input pins"),
                   vcWv.Param( "Black box aliases", string, vcWv.Sticky, alias_list,
                               "Enter library alias names to be treated as black boxes"),
                   vcWv.Param( "Black box cell files", string, vcWv.Sticky, cell_file_list,
                               "Enter names of files that contain cells to be treated as black boxes")); 
                   
vcCmd.EnableApply (input_options_cmd, TRUE);

function call_wirm( d, e, innet ) {
   declare in, out, wire_list;

   in = convert( string, innet);
   if (!null (wire_list = vscript.vsParse (in))) {
        in = caddr (wire_list);
   }
   else
        error ("Unable to parse this file name", in);

   current_design = in;
   out = current_design + "_new";
   current_outnet = current_outdir + out;
   vscript.vsDelete( spins_file, vscript.vsFILE );
   vcShell.TypeIn (vdes_shell,"net "+ current_design+" "+out, TRUE);
   current_outnet = current_outdir + out;
   vcCmd.Enable (constraint_cmd, vlOnOff.Off);
   vcWin.UpdateMenus (vdes2_win);
   for( i := 0; i < 100; ++i );
   readpins( func(){
       enable_syn();
       vcCmd.Enable (read_wire_cmd, vlOnOff.On);
       vcCmd.Enable (constraint_cmd, vlOnOff.On );
       vcWin.UpdateMenus (vdes2_win);});

}

read_wire_cmd := vcWv.Cmd( fac, call_wirm, "read_wire_file", "read wire file",
                 vcWv.Param( "INPUT Wire file", string, vcWv.Sticky, current_design,
                             vcFileSelect.fac, dirlist <-("./wir"), filter<- "*.1", read_now <- TRUE)); 

vcCmd.EnableApply (read_wire_cmd, TRUE);
/* set input technologies */

INLIB_EXIST := FALSE;
picked_inlibs := nil;


lib_list := list( "x2000",
                  "x3000",
                  "x4000",
                  "actel",
                  "act2",
                  "lsilca",
                  "tc24sc",
                  "tc140g",
                  "cmos5",
                  "std" );

declare inlib_pick;
function call_addlib ( d, e ) {

    declare strlist;
    declare str, lib;
    declare picked_lib;
    declare new_lib;

    function cb_cancel( doenable ) {
       if (doenable == TRUE) {
          vcCmd.Enable (set_in_tech_cmd, vlOnOff.On);
       vcWin.UpdateMenus (vdes2_win); }
    }
    function cb_action() {
       declare rows, cols, r, c, border, region, child;

       picked_inlibs = nil;

       vcPicklist.Dimensions(inlib_pick, &rows, &cols);
       if( INLIB_EXIST ) vcShell.TypeIn( vdes_shell, "clear_i", TRUE );
       for( r = 0; r < rows; ++r )
            for( c = 0; c < cols; ++c ) {
                 if (vcPicklist.IsSet (inlib_pick, c + r * cols)) {
                     border = vcTable.Get (inlib_pick, r, c);
                     if (border != nil) {
                         region = nil;
                         foreach (child; vcRegion.Children (border, FALSE))
                         region = child;
                         if (region != nil) {
                             lib = vcRegion.Data (region, vlStringVT);
                             vcShell.TypeIn( vdes_shell, "add_in "
                              + lib, TRUE);
                               picked_inlibs = append1 (picked_inlibs, lib );
                         }
                     }
                 } /* if */
        } /* for */
        INLIB_EXIST = TRUE;
    }
    
    function append_to_lib_list( lib ) {
        declare cmd;

        vcPicklist.Add(p_region, lib, lib );
        test.SetByString( inlib_pick, lib);
        picked_inlibs = append1 (picked_inlibs, lib );
        lib_list = append1 ( lib_list, lib );
        cmd =  vcTable.Get( cmd_region , 0, 1);
        vcCmdline.Set( cmd, " ");

    } 

    strlist = lib_list;
    inlib_pick = p_region := vcRegion.Create("Inlib", vcPicklist.fac,
                                       nil,
                                      columns <- 2,
                                      exclusive   <- FALSE);
/*    sp_region := vcPicklist.Scroll("Inlib", p_region, 5, 2, 10); */

    for(; !null(strlist); strlist = cdr( strlist )) {
          str = car( strlist );
          vcPicklist.Add(p_region, str, str);
    }
    strlist = picked_inlibs;
    for(; !null(strlist); strlist = cdr( strlist )) {
          str = car( strlist );
          test.SetByString( inlib_pick, str);
    }

    /* create "add_lib" button */

    t_top := vcTable.Make ("", 1, 1,
                            p_region);
/*                            sp_region); */
    cmd_region := stringbutton (list (list(
                      func(s) { new_lib = s; },
                      func(){ append_to_lib_list(new_lib);},
                      "Append to input lib list:", "")));
    vcTable.Add (t_top, cmd_region ); 

    butlist := putwindow (cb_action, cb_cancel, t_top, "set_in_tech");

}

set_in_tech_cmd := vcWv.Cmd( fac, call_addlib, "set_input_tech", "set_input_tech" );
                    
output_menu := vcWv.make_menu  ("output_menu",
  list ("Set outdir", outdir_cmd),
  list ("Wire", wire_cmd),
  list ("Report", report_cmd));

constraint_menu := vcWv.make_menu  ("cons_menu",
  list ("Set I/O constraints", constraint_cmd),
  list ("Set operating points", set_points_cmd));

                    
main_menu := vcWv.make_menu  ("main_menu",
  vcWin.RedSquareEntry,
  list ("Target technology", set_tech_cmd),
  list ("Input technology", set_in_tech_cmd),
  list ("WIR options", input_options_cmd),
  list ("Input wirelist", read_wire_cmd),
  list ("Run", m_run),
  list ("Generate outputs", output_menu),
  list ("Constraints", constraint_menu),
  list ("Misc", m_misc));

  
