/*
 * $RCSfile: vdbResults.vs,v $ $Revision: 1.3 $
 *
 * The vdbResults facility -- methods + commands
 */


/*
 * Given a list of view names, return a list of view values as strings.
 * If a given view does not exist, use some default string.
 *
 * This is horendously inefficient, getting all the views this way.  I'll
 * have to fix it later.
 */
function GetViewValues(cntx, sLibCompName, lVtypes, bShowJustCompName)
{
    /*printf("GetViewValues(%s, %s, %s)\n", cntx, sLibCompName, lVtypes);*/
    declare lViewValues, sValue, sDisplay;

    /*
     * If (bShowJustCompName), first column is just: compname
     * else it's : libname ':' compname
     */
    if (bShowJustCompName
        && (i := lindex (sLibCompName, ':')))
        lViewValues = list (substring (sLibCompName, i+1, 0));
    else
        lViewValues = list (sLibCompName);

    foreach (sViewName; vscript.list_iter (lVtypes))
    {
        sValue = vdb_cb.CompGetViewValue (cntx, sLibCompName, sViewName);
        sDisplay = (null (sValue) ? "-" : sValue);
        lViewValues = cons (sDisplay, lViewValues);
    }

    /* reverse list, because we cons'd it in the wrong order */
    reverse_destructive (lViewValues);

}


/**********************************************************************/
/* _Create -- create an instance of this facility
 *
 * INITIALIZATION DATA
 *      vscmVT -- vscmVT; list o' stuff too complicated to mention; look below
 */
function _Create(lData, rgnThis)
{
    <cntx, lComps, lColumnTitles, lVtypes, bShowJustCompName> := lData;

    /* first column is: libname ':' compname */
    lColumnTitles = cons ((bShowJustCompName ? "COMP" : "LIB:COMP"),
                          lColumnTitles);
    xHeaderFont := vgFont.smallf;
    xEntryFont := vgFont.smallf;
    /*vdbForm.print_list ("lComps",        lComps);*/
    /*vdbForm.print_list ("lColumnTitles", lColumnTitles);*/
    /*vdbForm.print_list ("lVtypes",       lVtypes);*/
    /*printf ("bShowJustCompName=%s\n",bShowJustCompName);*/

    /* truncate list if it exceeds threshold preference */
    iThreshold := vcRegion.PrefGet (rgnThis, "threshold");
    sStatus := nil;
    if (length (lComps) > iThreshold)
    {
        sStatus = "List truncated to first " + number2string (iThreshold)
            + " entries";
        l := lComps;
        for (i := 1; i < iThreshold; ++i)
            l = cdr (l);;
        set_cdr (l, nil);
    }

    /* get view data */
    /* `lViewData' is cons'd backwards, but it gets righted when */
    /* we build `lPicklistData' by looping over `lViewData' */
    lViewData := nil;
    foreach (sLibCompName; vscript.list_iter (lComps))
        lViewData = cons (GetViewValues (cntx, sLibCompName, lVtypes,
                                         bShowJustCompName),
                          lViewData);
    /*printf("lViewData:%s\n",lViewData);*/

    declare i, iWidth;

    /* generate sprintf format for spreadsheet.  Each index in vector */
    /* `vMaxLength' represents the max width found for that number */
    /* column */
    vMaxLength := make_vector (length (lColumnTitles), 0);
    for (i = 0; i < length(lColumnTitles); ++i)
        vMaxLength[i] = string_length(lColumnTitles[i]);
    foreach (e; vscript.list_iter (lViewData))
        for (i = 0; i < length(lColumnTitles); ++i)
            if ((iWidth = string_length (e[i])) > vMaxLength[i])
                vMaxLength[i] = iWidth;
    sColumnFormat := "%-" + number2string(vMaxLength[0]) + "s";
    for (i = 1; i < length(lColumnTitles); ++i)
        sColumnFormat = sColumnFormat + "  %-"
            + number2string(vMaxLength[i]) + "s";
    /*printf("sColumnFormat = \"%s\"\n", sColumnFormat);*/

    lPicklistData := nil;
    lCompNames := reverse(lComps);
    foreach (e; vscript.list_iter (lViewData))
    {
        sVisual := apply (sprintf, cons (sColumnFormat, e));
        /* car(lCompNames) == "libname:compname" */
        lPicklistData = cons (cons (sVisual, car(lCompNames)),
                              lPicklistData);
        lCompNames = cdr (lCompNames);
    }
    /*print_list ("lPicklistData", lPicklistData);*/

    rgnPicklistHeader :=
        vcRegion.Create ("results list header", vcLabel.fac,
                         /* prepend " " to line things up nicely */
                         apply (sprintf, cons (" "+sColumnFormat,
                                               lColumnTitles)));
    vcRegion.Font (rgnPicklistHeader, xHeaderFont);
    vcRegion.Align (rgnPicklistHeader, vgAlign.LowerLeft);

    rgnPicklist :=
        vcRegion.Create ("results list", vcPicklist.fac, lPicklistData,
                         columns <- 1,
                         font <- xEntryFont);
    /* damn, this doesn't work like it should...
    vcRegion.Background (rgnPicklist, vgAllColor.Background, TRUE); */
    rgnResultsScr :=
        vcPicklist.Scroll3 ("results scr", rgnPicklist,
                            vcRegion.PrefGet (rgnThis, "results_min"),
                            vcRegion.PrefGet (rgnThis, "results_ideal"),
                            vcRegion.PrefGet (rgnThis, "results_max"),
                            200,
                            400,
                            0,
                            vertical <- vcScroll.Presence.Always,
                            horizontal <- vcScroll.Presence.Always);

    rgnMain :=
        vcTable.Make ("results table", 2, 1,
                      rgnPicklistHeader,
                      rgnResultsScr);
    rgnMain =
        vcRegion.Create ("margin", vcMargin.fac, rgnMain,
                         left_margin <- 10,
                         right_margin <- 10,
                         top_margin <- 10,
                         bottom_margin <- 5);

    vcRegion.AddChild (rgnThis, rgnMain);

    struct vdbResultsVT {
        vcRegionVT rgnThis;
        vcRegionVT rgnPicklist;
        vscmVT     sStatus; /* string or nil */
        vlStringVT sSelectedComp;
        vlStringVT sColumnFormat;
        vlStringVT lColumnTitles
    };
    this := struct vdbResultsVT {
        rgnThis, rgnPicklist, sStatus, "", sColumnFormat, lColumnTitles
    };

    /* must come after struct definition */
    vcPicklist.Notify (rgnPicklist,
                       func(sComp, bSet)
                       {
                           if (bSet)
                               this.sSelectedComp = sComp;
                           DisplayStatus (this);
                       });
    vcPicklist.SetPick (rgnPicklist, 0);

    /*
     * Return facility data object (struct).
     */
    this;

} /* _Create */

function _Get(this, rgn, ts)
    vcRegion.Get (this.rgnPicklist, ts);

function _Refresh(this, args...)
    DisplayStatus (this);

function DisplayStatus(this)
{
    win := vcRegion.Window (this.rgnThis);
    if (!null (win))
    {
        vcWin.ClearStatus (win);
        if (!null (this.sStatus))
        {
            vcWin.AddStatus (win, this.sStatus);
            this.sStatus = nil;
        }
    }

}


function after(this, event)
    DisplayStatus (this);


/*
 * Preferences.
 */
vlPref.Replace (prefs, "results_min", vlPositiveIntVT,
                "minimum number of entries visible in candidate list", 3);
vlPref.Replace (prefs, "results_ideal", vlPositiveIntVT,
                "default number of entries visible in candidate list", 7);
vlPref.Replace (prefs, "results_max", vlUnsignedIntVT,
                "maximum number of entries visible in candidate list", 0);
vlPref.Replace (prefs, "threshold", vlUnsignedIntVT,
                "maximum number of candidates to show in list.  0 means always show all of them.  Note that building big lists may take a long time, and this is protection against building an irrationally large list of components.", 50);
