/*
 * $RCSfile: vdbgrp.vs,v $ $Revision: 1.24 $
 *
 * ViewDatabook II -- groups
 */


/*
 * vdbAllGroupsVT -- all groups in memory
 */
struct vdbAllGroupsVT
{
    vlIntVT    iCurrGroup;      /* index of current group */
    vscmVT     vGroups          /* vector of vdbGroupVT.  CAN'T USE vectorVT
                                   though -- some ViewScript bug prevents it */
};

/*
 * vdbGroupVT -- one component group
 */
struct vdbGroupVT
{
    vscmVT     sCategory;       /* FALSE if we haven't deduced it yet,
                                   nil if we tried but couldn't */
    vscmVT     lComps;          /* list of comps in this group */
    vscmVT     lLoadedAttrs;    /* list of attrs as they were loaded */
    vscmVT     lCurrAttrs       /* list of attrs as they exist currently */
};


/*
 * RETURNS
 *      vdbAllGroupsVT (opaque type!)
 */
function GrpInitGroups(lGroups)
{
    iNumGroups := length (lGroups);
    vGroups    := make_vector (iNumGroups);

    for (i := 0; i < iNumGroups; ++i)
    {
        lThisGroup := lGroups[i];
        lComps     := car (lThisGroup);
        lAttrs     := cadr (lThisGroup);
        vGroups[i] = struct vdbGroupVT { FALSE, lComps, lAttrs, lAttrs };
    }

    /*
     * HACK_ALERT: need to implement CallOnLoad here.  This may mean
     * relocating the code that forms groups to this module (from vdraw
     * side), and calling the CallOnLoad before forming groups.
     */

    /* return value */
    struct vdbAllGroupsVT { 0, vGroups };
}


/*
function GrpNextGroup(hAllGroups, this, event)
{
    if (this.iCurrGroup + 2 > length (lGroups))
        vdbStatus (this, "No next group");
    else
        GrpSetGroup (this, this.iCurrGroup + 1);

}

function GrpPrevGroup(hAllGroups, this, event)
{
    if (this.iCurrGroup == 0)
        vdbStatus (this, "No previous group");
    else
        GrpSetGroup (this, this.iCurrGroup - 1);

}
*/


function GrpReset(hAllGroups, this)
{
    /*printf("___GrpReset________________________________________\n");*/
    hAllGroups.vGroups[hAllGroups.iCurrGroup].lCurrAttrs =
        hAllGroups.vGroups[hAllGroups.iCurrGroup].lLoadedAttrs;
    /*printf("hAllGroups.vGroups[hAllGroups.iCurrGroup] = %s\n",
           hAllGroups.vGroups[hAllGroups.iCurrGroup]);*/

    GrpDisplayAttrs (hAllGroups, this);
}


/*
 * Given the symbol name (e.g. "cap"), return the category.
 *
 * 1. Check for existence of libname attribute, e.g. PARFILE=capacitors
 * 2. Look for library with same name as the symbol, e.g. "cap"
 * 3. Check in index.par for matching line, e.g. "*cap  capacitors.par"
 * 4. If preference set, search all libraries for SYMBOL, e.g. "SYMBOL=*cap*"
 * 5. Indicate failure.
 */
function Grp_DeduceCategory(this, lAttrs)
{
    /*DEBUG("DeduceCategory: %s\n", lAttrs);*/

    /* step #1 -- see if there's a libname attribute */
    if (!null (vdbCfg.sLibNameAttr))
        if (e := assoc_string_eqi (vdbCfg.sLibNameAttr, lAttrs))
        {
            <sDev,sPath,sName,sExt> := vscript.vsParse (cadr (e));
            /*printf("*** found %s=%s\n",vdbCfg.sLibNameAttr,cadr(e));*/
            return (sName);
        }

/*
    // If we instantiated this component, there might be a special
    // attribute that we added...
    if (e := assoc_string_eqi (sVllmLibNameAttr, lAttrs))
        // Who-whee!  Adding that attr finally paid off!
        return (cadr (e));
*/

    sVtype := EncapGetVtype ();

    /* is there a SYMBOL view that we got from ViewDraw?  why not? */
    if (!(e := assoc_string_eqi (sVtype, lAttrs)))
        vdbThrow (1024, "%s not found in group of components", sVtype);

    /* step #2 */
    sName := cadr (e);
    if (!null (vllm.CntxFindLib (this.cntx, sName)))
        return (sName);

    /* step #3 */
    sLibrary := vllm.ParFindIndex (sName, TRUE);
    if (!null (sLibrary))
    {
        <sDev,sPath,sName,sExt> := vscript.vsParse (sLibrary);
        return (sName);
    }

    /* step #4 -- oh, boy, now we have to search... */
    if (!SearchAllLibrariesForSymbol())
        return (nil);

    EnsureLoaded (this, nil); /* ensure all libs are loaded */

    bHaveSearched := FALSE;
    sCategory := nil;
    ClearSearch (this);
    if ((e := assoc_string_eqi (sVtype, lAttrs))
        && !null (vllm.CntxFindVtype (this.cntx, sVtype)))
    {
        AddSearch (this, sVtype, cadr (e));
        bHaveSearched = TRUE;
    }
/*
    foreach (e; vscript.list_iter (lAttrs))
    {
        sAttr := car (e);
        sVtype := AttrToVtype (sAttr); // returns FALSE if fails
        if (sVtype
            && !null (vllm.CntxFindVtype (this.cntx, sVtype)))
        {
            AddSearch (this, sVtype, cadr (e));
            bHaveSearched = TRUE;
        }
    }
*/
    if (bHaveSearched)
    {
        <iCandidates, alCommonViews> := ExecSearch (this);
        if (e := assoc_string_eqi (OrgGetVtype (), alCommonViews))
            sCategory := cdr (e);
    }
    /* return value */
    sCategory;
}

function GrpEnableCmds(hAllGroups, this)
    ;
/* Next/prev now taken care of by tabs.
{
    declare bPrevEnabled, bNextEnabled;
    if (this.eMode != vdbMode.BackAnnotate)
    {
        bPrevEnabled = FALSE;
        bNextEnabled = FALSE;
    }
    else
    {
        bPrevEnabled = (this.iCurrGroup != 0);
        bNextEnabled = (this.iCurrGroup + 1 < length (this.lGroups));
    }
    vcCmd.Enable (cmdPrevGroup, bPrevEnabled);
    vcCmd.Enable (cmdNextGroup, bNextEnabled);

}
*/

function GrpGetAttrList(hAllGroups, i)
    hAllGroups.vGroups[i].lCurrAttrs;

function GrpDisplayAttrs(hAllGroups, this)
{
    vThisGroup := hAllGroups.vGroups[hAllGroups.iCurrGroup];

    /* This won't necessarily change the category, but it will always */
    /* reset the search */
    SearchMaybeChangeCategory (this, vThisGroup.sCategory);

    foreach (e; vscript.list_iter (vThisGroup.lCurrAttrs))
    {
        sVtype := AttrAndCategoryToVtype (vThisGroup.sCategory, car (e));
        if (! null(rgn := vlSym.Fetch (this.symVtypeToSearchField, sVtype)))
        {
            sValue := cadr (e);
            CmdlineSetValue (this, rgn, sValue);
            CmdlineSetValueSource(this, rgn, sVtype, vdbValueSource.FromVdraw);
            AddSearch (this, sVtype, sValue);
        }
    }
    MaybeAddRequiredViews (this);
    SearchExecUpdateDisplay (this);
}

/*
 * DESCRIPTION
 *      Add SYMBOL search criteria.  This happens after the user does a
 *      "Clear" -- we don't want all search criteria to be cleared.  See bug
 *      #200215.
 *
 * CAUTION
 *      Only valid in BackAnnotate mode.
 */
function GrpSearchAddEncap(hAllGroups, this)
{
    vThisGroup := hAllGroups.vGroups[hAllGroups.iCurrGroup];
    sVtype := EncapGetVtype ();

    /* is there a SYMBOL view that we got from ViewDraw?  why not? */
    if (!(e := assoc_string_eqi (sVtype, vThisGroup.lLoadedAttrs)))
        vdbThrow (1024, "%s not found in group of components", sVtype);

    sVtype := AttrAndCategoryToVtype (vThisGroup.sCategory, car (e));
    if (! null(rgn := vlSym.Fetch (this.symVtypeToSearchField, sVtype)))
    {
        sValue := cadr (e);
        CmdlineSetValue (this, rgn, sValue);
        CmdlineSetValueSource(this, rgn, sVtype, vdbValueSource.FromVdraw);
        AddSearch (this, sVtype, sValue);
    }
}

function GrpSetGroup(hAllGroups, this, iGroup)
vcWin.WithBusy (vcRegion.Window (this.rgnThis), func()
{
    vdbStatus (this, ""); /* clear status msg */

    hAllGroups.iCurrGroup = iGroup;
    lCompUids := hAllGroups.vGroups[iGroup].lComps;
    lAttrs    := hAllGroups.vGroups[iGroup].lCurrAttrs;
/*\BeginDebugVS
    printf("___Begin: GrpSetGroup %d________________________________________\n", iGroup);
    printf("hAllGroups.iCurrGroup = %s\n",hAllGroups.iCurrGroup);
    printf("lCompUids             = %s\n",lCompUids);
    printf("lAttrs                = %s\n",lAttrs);
\EndDebugVS*/

    /*
     * Enable/disable next/prev buttons as appropriate.
     */
    EnableCmds (this);

    /*
     * Select components in tool.
     */
    _Track (lCompUids);

    /*
     * Get category of group.  FALSE means "hasn't been initialized".
     */
    sCategory := hAllGroups.vGroups[iGroup].sCategory;
    if (sCategory == FALSE)
    {
        sCategory = Grp_DeduceCategory (this, lAttrs);
        /*
         * Handle case where library name is bogus.
         */
        if (null (sCategory))
        {
            vdbAlert (nil, nil, sprintf ("Cannot deduce library name from component information"),
                      vlErrSev.Error, nil);
            sCategory = FALSE;
        }
        else if (vdbCfg.eOrg == vdbOrganized.ByLibrary
                 && null (vllm.CntxFindLib (this.cntx, sCategory)))
        {
            vdbAlert (nil, nil, sprintf ("No such library: %s", sCategory),
                      vlErrSev.Error, nil);
            sCategory = FALSE;
        }
        /*
         * Remember the category so we don't have to go through that again.
         */
        hAllGroups.vGroups[iGroup].sCategory = sCategory;
    }

    /* FALSE category means "couldn't find it" -- error */
    if (sCategory == FALSE)
        vcRegion.Enable (this.rgnSearchArea, FALSE, TRUE);
    else
    {
        /*
         * Now that we've got the search fields set up per the selected
         * category and cleared, we're ready to start filling them in with
         * the attrs from the current group (after mapping the attrs to vtypes
         * of course).
         */
        GrpDisplayAttrs (hAllGroups, this);

        /* needed? */
        vcRegion.SetFocus (this.rgnThis, this.rgnSearchArea);
    }

});

function GrpGetCurrGroup(hAllGroups)
    hAllGroups.iCurrGroup;

function GrpGetNumGroups(hAllGroups)
    length (hAllGroups.vGroups);

function GrpGetComps(hAllGroups)
    hAllGroups.vGroups[hAllGroups.iCurrGroup].lComps;
