/*
 *  $Id: vdfopen.vs,v 1.7 92/04/08 19:29:19 surma Stab $
 * 
 *  (c) Copyright 1992 by Viewlogic Systems, Inc.
 */

/* global */
fopen_num_cols := 4;   /* can be overriden in vdrawus.vs */

ddes_table    := nil;

function remove_duplicates(list)
{
    declare cur_str, next_str;
    cur_ptr := nil;
    next_ptr := nil;

    cur_ptr = list;
    next_ptr = cdr (cur_ptr); 
    while (!null (cur_ptr) && !null (next_ptr)) {
        cur_str = car (cur_ptr);
        next_str = car (next_ptr);

        /* remove the next string from the list if it matches the
         * current string, otherwise move the current list ptr
         */
        if (string_eq (cur_str, next_str))
            set_cdr (cur_ptr, cdr (next_ptr));
        else
            cur_ptr = cdr (cur_ptr);

        next_ptr = cdr (cur_ptr);
    }

    list;
}

/*\Function build_filepicklist -- Build a picklist of symbol or schematic names
 */
function build_filepicklist(data_type, library, filter, name)
{
    declare str;

    /* This number represents the number of columns in the file open
     * dialog box.  You may change it and do a "viewdraw -writestate"
     * to update the state file (viewdraw.stb).
     */
    count :=  0;

    templist := nil;
    sorted_list := nil;
    masterlist := nil;

    /* hash table of aliases from data.des */
    ddes_table = open_data_des ();

    /* build a list of all files, substituting aliases where appropriate */
    foreach (str; files (data_type, library, "*", 0)) {

        if (ddes_table != nil)
            str = convertalias (str);

        /* apply the filter to the list */
        if (vdraw.vlStrMat (filter, str))
            templist = cons (str, templist);
    }

    if (templist != nil)
        sorted_list = vscript.Sort (templist, string_le);
        
    sorted_list = remove_duplicates (sorted_list);

    templist = nil;
    
    while (!null (sorted_list)) {
        str = car (sorted_list);
        
        /* This fancy stuff is done to make n columns.  Before, when 
           vcPicklist.Add was being used, the number of columns was 
           hard-coded in the vcRegion.Create statement.  Now, when 
           building the list prior to the create, we must make the
           list itself reflect the # of columns.  Just change the 
           values of the mod statements to change the number of cols.
           */

        templist = cons (cons (str, str), templist);

        if (++count % fopen_num_cols == 0) {
            templist = reverse_destructive (templist);
            masterlist = cons (templist, masterlist);
            templist = nil;
        }

        sorted_list = cdr (sorted_list);
    }

    if (count % fopen_num_cols != 0) {
        templist = reverse_destructive (templist);
        masterlist = cons (templist, masterlist);
    }

    masterlist = reverse_destructive (masterlist);
    picklist := vcRegion.Create ("", vcPicklist.fac, masterlist);

    picklist;

} /* function build_filepicklist */


function fopen_create_cmdline(default, onreturn)
{
    cmd_line := vcRegion.Create1 ("", vcCmdline.fac, default, 
                                  list (cons (`width, 30)));

    vcCmdline.CallOnReturn (cmd_line, onreturn);

    cmd_line;

} /* function fopen_create_cmdline */


function create_entrybox(prompt, cmdline)
{
    /* create table for symbol name command line  */
    text_entry := vcTable.Create ("", 1, 2);

    vcTable.Add (text_entry, label (prompt));
    vcTable.Add (text_entry, cmdline);

    text_entry;

} /* function create_entrybox */


function build_and_set_files(win, table, row,
                             datatype, library, filter, notify)
{
    if (win != nil)
        vcWin.Busy (win, TRUE);

    /* Set the last library for open_data_des */
    set_lastlib(library);

    picklist := build_filepicklist (datatype, library, filter, "");
    vcPicklist.Notify (picklist, notify);

    vcTable.Set (table, row, 0, vcPicklist.Scroll ("", picklist, 4, 6, 0));

    if (win != nil)
        vcWin.Busy (win, FALSE);

} /* function build_and_set_files */


function build_datatype_pick(data_type, notify)
{
    dtr := vcTable.Create ("Datatype", 1, 1);

    picklist := vcRegion.Create1 ("DT", vcPicklist.fac, nil,
                                  list (cons (`exclusive, TRUE),
                                        cons (`rows, 1),
                                        cons (`columns, 2)));

    vcPicklist.Add (picklist, "Schematics", vqDataType.Schematic);
    vcPicklist.Add (picklist, "Symbols",    vqDataType.Symbol);

    vcPicklist.Notify (picklist, notify);

    vcTable.Add (dtr, picklist);

    /* Center the 2 columns within the 1 row of Table - dtr. */
    vcTable.Col (dtr, -1, "Invisible", 2, 2, 4, "MiddleCenter");

    vcPicklist.SetPick (picklist, data_type);

    dtr;

} /* static vcRegionID build_datatype_pick */


function build_pathpicklist(directory)
{
    count := 0;
    pick  := 0;

    picklist := vcRegion.Create1 ("", vcPicklist.fac, nil,
                                  list (cons (`exclusive, TRUE),
                                        cons (`rows, 5),
                                        cons (`columns, 1)));

    foreach (str; libraries ()) {

        vcPicklist.Add (picklist, str, str);
        if (   string_eq (str, directory)
            || string_eq (str, "(" + directory + ")"))

            pick = count;

        ++count;
    }

    vcPicklist.SetPick (picklist, pick);

    fopen_libpicklist = picklist;

} /* function build_pathpicklist */


/* ---------------------------------------------------------------------- */
/* Globals (for history) */

fopen_window   := nil;
fopen_name     := "";
fopen_datatype := vqDataType.Schematic;
fopen_lastlib  := nil;
fopen_is_apply := FALSE;
fopen_libpicklist := nil;
fopen_scrollp  := nil;
fopen_table    := nil;
fopen_cmdline  := nil;

load("vdaddini.vs");

function set_lastlib(lib)
{
    fopen_lastlib = lib;
}

/*\Function build_fopen_dialog -- Create File Open dialog box.
 */
function build_fopen_dialog(parent, data_type)
{
    declare picklist;

    fopen_table = nil;
    fopen_cmdline = nil;

    if (fopen_window != nil)
        vcWin.Close (fopen_window);

    if (fopen_lastlib == nil)
        fopen_lastlib = getprimarylib ();

    function fopen_onreturn(e, str)
    {
        declare name;

        if (   (lindex (str, '*') == FALSE)
            && (lindex (str, '?') == FALSE)) {

            apply_already_set := FALSE;

            if (fopen_is_apply == TRUE)
                apply_already_set = TRUE;
            else
                fopen_is_apply = TRUE;

            unwind_protect {
                
                vcWin.ClearStatus (vcEvent.Window (e));
                
                fopen_name = str;
                
                /* If fopen_name contains a colon, we assume that there is
                 * a library name preceding it.  Do not prepend fopen_lastlib
                 * in this case.
                 */
                
                if (   (lindex (fopen_name, ':') == FALSE)
                    && (lindex (fopen_lastlib, '(') != FALSE))
                    name = fopen_lastlib + ":" + fopen_name;
                else
                    name = fopen_name;
                
                if (fopen_datatype == vqDataType.Schematic)
                    vcCmd.Run (sch_cmd, nil, e, list (name), TRUE);
                else
                    vcCmd.Run (sym_cmd, nil, e, list (name), TRUE);
            }
            cleanup {
                if (!apply_already_set)
                    fopen_is_apply = FALSE;
            }
        }
        else {

            vcWin.ClearStatus (vcEvent.Window (e));
            build_and_set_files (fopen_window, fopen_table, 0, fopen_datatype,
                                 fopen_lastlib, str, fopen_set_line);
        }

    } /* function fopen_onreturn */

    function fopen_set_line(str, on_off)
    {
        if (on_off == vlOnOff.On) {

            vcCmdline.Set (fopen_cmdline, str);
            fopen_name = str;
        }

    } /* function fopen_set_line */

    function fopen_set_library(library, on_off)
    {
        if (on_off == vlOnOff.On) {

            fopen_lastlib = library;
            build_and_set_files (fopen_window, fopen_table, 0, fopen_datatype,
                                 fopen_lastlib, "*", fopen_set_line);
        }
    } /* function fopen_set_library */

    function datatype_change(which, on_off)
    {
        if ((fopen_window != nil) && (on_off == vlOnOff.On)) {

            vcWin.ClearStatus (fopen_window);
            vcWin.AddStatus (fopen_window, "Building list...");

            fopen_datatype = which;
            build_and_set_files (fopen_window, fopen_table, 0, fopen_datatype,
                                 fopen_lastlib, "*", fopen_set_line);

            vcWin.ClearStatus (fopen_window);
        }

    } /* function datatype_change */

    function fopen_buttons(accept_f, apply_f, cancel_f)
    {
        function edit_order_f(e) {
            if (getwindowcount () > 0)
                w := getcurrentwindow ();
            else
                w := fopen_window;
            
            e := vcEvent.Create (0, 0, ' ', "", w, vcWin.Region (w), nil);

            vcCmd.Run (lib_order_cmd, nil, e, nil, TRUE);
        }

        buttons := vcRegion.Create ("Buttons", vcButnRow.fac,
            list (list ("Accept", "Accept", accept_f, 
                        list (close_window <- TRUE)),
                  list ("Apply", "Apply", apply_f, nil),
                  list ("SearchOrder", "SearchOrder", edit_order_f, nil),
                  list ("Cancel", "Cancel", cancel_f,
                        list (close_window <- TRUE))));

        /* RETURN VALUE IS LAST EXPRESSION EVALUATED -- DON'T ADD CODE HERE */

    } /* function fopen_buttons */

    function fopen_cancel()
    {
        if (getwindowcount () < 1) {

            e := vcEvent.Create (0, 0, ' ', "", fopen_window,
                                 vcWin.Region (fopen_window), nil);

            vcCmd.Run (vcWin.QuitCmd, nil, e, nil, TRUE);
        }

    } /* function fopen_cancel */

    function fopen_apply()
    {
        declare w;

        fopen_is_apply = TRUE;

        unwind_protect {
            if (getwindowcount () > 0)
                w = getcurrentwindow ();
            else
                w = fopen_window;
            
            e := vcEvent.Create (0, 0, ' ', "", w, vcWin.Region (w), nil);
            
            fopen_name = vcRegion.Get (fopen_cmdline, vlStringVT);
            
            fopen_name = convertalias (fopen_name);
            
            fopen_onreturn (e, fopen_name);
        }
        cleanup 
            fopen_is_apply = FALSE;

    } /* function fopen_apply */

    function fopen_accept()
    {
        fopen_apply ();
        fopen_cancel ();

    } /* function fopen_apply */

    fopen_datatype = data_type;

    if (parent != nil)
        vcWin.Busy (parent, TRUE);

    dbox := vcTable.Create ("ViewDraw 5.3.2 File Open", 5, 1);

    picklist = build_filepicklist (fopen_datatype,
                                   fopen_lastlib, "*", fopen_name);
    vcPicklist.Notify (picklist, fopen_set_line);

    vcTable.Add (dbox, vcPicklist.Scroll ("", picklist, 4, 6, 0));

    /* vcTable.Add (dbox, vcRubber.Create ("", 0, 0)); */

    fopen_cmdline = fopen_create_cmdline ("", fopen_onreturn);
    vcTable.Add (dbox, create_entrybox ("Enter name: ", fopen_cmdline));

    vcTable.Add (dbox, build_datatype_pick (data_type, datatype_change));

    picklist = build_pathpicklist (fopen_lastlib);
    vcPicklist.Notify (picklist, fopen_set_library);

    /* don't change max to zero for libraries */
    fopen_scrollp = vcPicklist.Scroll ("", picklist, 4, 6, 8);
    vcTable.Add (dbox, fopen_scrollp);

    vcTable.Add (dbox,
                 fopen_buttons (fopen_accept, fopen_apply, fopen_cancel));

    fopen_table  = dbox;

    /* make sure kbd focus is in the cmdline region */
    vcRegion.SetFocus (dbox, fopen_cmdline);

    /* ensure a slightly prettier layout */
    dbox = vcRegion.Create ("ViewDraw 5.3.2 File Open", vcMargin.fac, dbox);

    fopen_window = vcWin.Create (dbox,
                                 (vcWin.NoCmdline
                                  + vcWin.NoMenus
                                  + vcWin.NoBindings));

    /* set the fopen_window to nil when window closed */
    vcWin.DestroyNotify (fopen_window,
                       func(a) {fopen_window = nil; /* fopen_table = nil; */});

    if (parent != nil)
        vcWin.Busy (parent, FALSE);

} /* function build_fopen_dialog */


/* ---------------------------------------------------------------------- */

function splitlibcomp(libcompptr)
{
    declare lib, colon;

    colon = lindex (*libcompptr, ':');
    
    if (colon == FALSE)
        return ("");

    else {

        lib = substring (*libcompptr, 0, colon);
        *libcompptr = substring (*libcompptr, (colon + 1), 0);

        return (lib);
    }

} /* function splitlibcomp */

ccomp_window := nil;

function build_ccomp_dialog(display, parent, default)
{
    declare picklist, dbox, ccomp_table, ccomp_cmdline, ccomp_name, ccomp_lib;

    if (ccomp_window != nil)
        vcWin.Close (ccomp_window);

    /* ccomp_table   = nil; */
    /* ccomp_cmdline = nil; */

    ccomp_lib  = splitlibcomp (&default);
    ccomp_name = default;

    set_lastlib(ccomp_lib);

    function ccomp_onreturn(event, str)
    {
        declare name;

        if (   (lindex (str, '*') == FALSE)
            && (lindex (str, '?') == FALSE)) {
            
            ccomp_name = str;

            if (lindex (ccomp_name, ':') == FALSE)
                name = ccomp_lib + ":" + ccomp_name;
            else
                name = ccomp_name;

            vcCmd.Run (chgcomp_cmd, display, event, list (name), TRUE);
        }
        vcWin.ClearStatus (vcEvent.Window (event));

    } /* function ccomp_onreturn */

    function ccomp_apply()
    {
        if (getwindowcount () > 0) {

            w := getcurrentwindow ();
            r := vcWin.Region (w);

            if (w != nil) {

                event := vcEvent.Create (0, 0, ' ', "", w, r, nil);
                ccomp_name = vcRegion.Get (ccomp_cmdline, vlStringVT);

                /* Convert the alias first */
                ccomp_name = convertalias (ccomp_name);

                ccomp_onreturn (event, ccomp_name);

            }
        }

    } /* function ccomp_apply */

    function ccomp_accept()
    {
        ccomp_name = vcRegion.Get (ccomp_cmdline, vlStringVT);

        if (string_eq (ccomp_name, "") != TRUE) {

            ccomp_apply ();
            vcWin.Close (ccomp_window);
        }
    } /* function ccomp_apply */

    function ccomp_set_line(str, on_off)
    {
        if (on_off == vlOnOff.On) {

            vcCmdline.Set (ccomp_cmdline, str);
            ccomp_name = str;
        }

    } /* function ccomp_set_line */

    function ccomp_set_library(library, on_off)
    {
        if (on_off == vlOnOff.On) {

            ccomp_lib = library;
            build_and_set_files (ccomp_window, ccomp_table, 0,
                                 vqDataType.Symbol, ccomp_lib,
                                 "*", ccomp_set_line);
        }

    } /* function ccomp_set_library */

    function ccomp_buttons(accept_f, apply_f)
    {
        buttons := vcRegion.Create ("Buttons", vcButnRow.fac,
            list (list ("Accept", "Accept", accept_f, nil),
                  list ("Apply", "Apply", apply_f, nil),
                  list ("Cancel", "Cancel",
                        func () vcWin.Close (ccomp_window);, nil)));

        /* RETURN VALUE IS LAST EXPRESSION EVALUATED -- DON'T ADD CODE HERE */

    } /* function ccomp_buttons */

    if (parent != nil)
        vcWin.Busy (parent, TRUE);

    dbox = vcTable.Create ("ViewDraw 5.3.2 Change Component", 4, 1);

    picklist = build_filepicklist (vqDataType.Symbol,
                                   ccomp_lib, "*", ccomp_name);
    vcPicklist.Notify (picklist, ccomp_set_line);

    vcTable.Add (dbox, vcPicklist.Scroll ("", picklist, 4, 6, 0));

    ccomp_cmdline = fopen_create_cmdline ("", ccomp_onreturn);
    vcTable.Add (dbox, create_entrybox ("Enter name: ", ccomp_cmdline));

    vcCmdline.Set (ccomp_cmdline, ccomp_name);

    picklist = build_pathpicklist (ccomp_lib);
    vcPicklist.Notify (picklist, ccomp_set_library);

    fopen_scrollp = vcPicklist.Scroll ("", picklist, 4, 6, 8);

    vcTable.Add (dbox, fopen_scrollp);

    vcTable.Add (dbox, ccomp_buttons (ccomp_accept, ccomp_apply));

    ccomp_table  = dbox;

    ccomp_window = vcWin.Create (dbox,
                                 (vcWin.NoCmdline
                                  + vcWin.NoMenus
                                  + vcWin.NoBindings));

    /* set the ccomp_window to nil when window closed */
    vcWin.DestroyNotify (ccomp_window,
                       func(a) {ccomp_window = nil; /* ccomp_table = nil; */});

    if (parent != nil)
        vcWin.Busy (parent, FALSE);

} /* function build_ccomp_dialog */


/* ---------------------------------------------------------------------- */

function convertalias (str)
{
    declare name, lib, alias, length;

    if (ddes_table == nil)
        return (str);

    colon := lindex (str, ':');
    
    if (colon != FALSE) {
        length = string_length (str);
        name = substring (str, (colon + 1), length);
        lib = substring (str, 0, colon);
    }
    else
        name = str;

    alias = vlSym.Fetch (ddes_table, name);

    if (alias == nil)
        name = str;        
    else {
        if (lib != nil)
            name = sprintf ("%s:%s", lib, alias);
        else
            name = alias;
    }

    name;
}

function build_scomppicklist(filter)
{
    templist := nil;

    /* hash table of aliases from data.des */
    ddes_table = open_data_des ();

    foreach (str; shsyms (filter, 0)) {
        str = convertalias (str);
        templist = cons (cons (str, str), templist);
    }

    templist = reverse_destructive (templist);
    picklist := vcRegion.Create ("", vcPicklist.fac, templist);

    picklist;

} /* function build_scomppicklist */


function build_seloption_pick(pick, notify)
{
    table := vcTable.Create ("SelectOption", 1, 1);

    picklist := vcRegion.Create1 ("SO", vcPicklist.fac, nil,
                                  list (cons (`exclusive, TRUE),
                                        cons (`rows, 1),
                                        cons (`columns, 3)));

    vcPicklist.Add (picklist, "Select",    vqSelectOption.Select);
    vcPicklist.Add (picklist, "AddSelect", vqSelectOption.AddSelect);
    vcPicklist.Add (picklist, "Unselect",  vqSelectOption.Unselect);

    vcPicklist.Notify (picklist, notify);

    vcTable.Add (table, picklist);

    /* Center the columns within the 1 row of table. */
    vcTable.Col (table, -1, "Invisible", 2, 2, 4, "MiddleCenter");

    vcPicklist.SetPick (picklist, pick);

    table;

} /* static vcRegionID build_seloption_pick */


scomp_window := nil;

function build_scomp_dialog(parent)
{
    declare picklist, dbox, scomp_table, scomp_option;

    if (scomp_window != nil)
        vcWin.Close (scomp_window);

    /* scomp_table  = nil; */

    scomp_option = vqSelectOption.Select;

    function scomp_doit(str, on_off)
    {
        if ((on_off == vlOnOff.On) && (getwindowcount () > 0)) {

            w := getcurrentwindow ();
            r := vcWin.Region (w);

            /* Need to get the data (a.k.a. display) from window. */

            display := wingetdisplay (w);

            event := vcEvent.Create (0, 0, ' ', "", w, r, nil);

            str = convertalias (str);

            /* Do the selection here. */
            vcCmd.Run (selcomp_cmd, display, event,
                       list (str, scomp_option), TRUE);
        }

    } /* function scomp_doit */

    function scomp_chg_option(which, on_off)
    {
        if (on_off == vlOnOff.On)
            scomp_option = which;
    }

    function scomp_buttons()
    {
        buttons := vcRegion.Create ("Buttons", vcButnRow.fac,
            list (list ("Dismiss", "Dismiss",
                        func () vcWin.Close (scomp_window);, nil)));

        /* RETURN VALUE IS LAST EXPRESSION EVALUATED -- DON'T ADD CODE HERE */

    } /* function scomp_buttons */

    if (parent != nil)
        vcWin.Busy (parent, TRUE);

    dbox = vcTable.Create ("ViewDraw 5.3.2 Select Component", 3, 1);
    picklist = build_scomppicklist ("*");
    vcPicklist.Notify (picklist, scomp_doit);

    vcTable.Add (dbox, vcPicklist.Scroll ("", picklist, 4, 6, 8));

    vcTable.Add (dbox, build_seloption_pick (scomp_option, scomp_chg_option));

    vcTable.Add (dbox, scomp_buttons ());

    scomp_table = dbox;

    scomp_window = vcWin.Create (dbox,
                                 (vcWin.NoCmdline
                                  + vcWin.NoMenus
                                  + vcWin.NoBindings));

    /* set the scomp_window to nil when window closed */
    vcWin.DestroyNotify (scomp_window,
                         func(a) {
                             scomp_window = nil;
                             /* scomp_table = nil; */
                         });

    if (parent != nil)
        vcWin.Busy (parent, FALSE);

} /* function build_scomp_dialog */

/*\Function open_data_des -- Build a symbol table of filename aliases from data.des file
 */
function open_data_des()
{
    declare name, alias, slash;

    slash = vscript.vsConvPath ("/");
    t := nil;
    
    fullpath := nil;
    libpath := nil;
    libname := nil;

    if (string_eq (fopen_lastlib, "*memory*"))
        fopen_lastlib = getprimarylib ();

    if (fopen_lastlib != nil) {
        if (lindex (fopen_lastlib, '(') != FALSE) {
            libname = substring (fopen_lastlib, 1, 
                                 (string_length (fopen_lastlib) -1));
            libpath = getfirstlibpath (libname);
        }
        else {
            libname = fopen_lastlib;
            libpath = getfirstlibpath (libname);
        }

        if ((libpath == nil) && !string_eq (fopen_lastlib, "*memory*"))
            libpath = fopen_lastlib;
        
        if (libpath != nil) {
            
            fullpath = libpath + slash + "data.des";
        }
    }

    while ((fullpath != nil) && (fexists (fullpath))) {

        fp := fopen (fullpath, "r");
        
        if (fp != nil) {
            
            if (t == nil)
                t = vlSym.StrCreate (29);
            
            line := fgetline (fp);
            while (!null (line)) {
                
                /* get first token */
                line = string_token (line, &name, " ");

                /* get remainder of line */
                line = strtrim (line);
                string_token (line, &alias, "\n");
                
                if ((vlSym.Fetch (t, name) == nil) &&
                    (vlSym.Fetch (t, alias) == nil)) {
                    vlSym.Store (t, name, alias);
                    vlSym.Store (t, alias, name);
                }

                line = fgetline (fp);
            }
            
            fclose (fp);
        }
        
        fullpath = nil;

        /* Try to get another library that has the same alias as this one. */  
        if (libname != nil) {
            libpath = getnextlibpath (libname);
            if (libpath != nil)
                fullpath = libpath + slash + "data.des";
        }
    }

    t;

} /* function open_data_des */
