vcWv.MakeEnum ("InsPosVT", list ("Append", "InsertBefore"));

function _Create(data, region) {

    function ENABLE (button)
        if (!vcRegion.IsEnabled (button))
            vcRegion.Enable (button, TRUE, TRUE);
    
    function DISABLE (button)
        if (vcRegion.IsEnabled (button))
            vcRegion.Enable (button, FALSE, TRUE);
    
    initial := vcRegion.PrefGet (region, "initial");

    function build_visual(type, path, alias) {
        if (!null (alias) && !string_eq (alias, ""))
            str := sprintf("[%s] %s (%s)", type, path, alias);
        else
            str := sprintf("[%s] %s", type, path);
        
        str;
    }

    function LibIsDeleteable(str) {
        i := 0;
        found := FALSE;

        foreach(type, alias, path; vdraw.IniIterLibInfo ()) {
            v := build_visual (type, path, alias);
            
            if (string_eq (v, str)) {
                found = TRUE;
                break;
            }
            ++i;
        }
        
        if (found)
            found = vdraw.IniIsLibDeleteable (i);
        else
            found = TRUE;

        found;
    }

    list_of_strings := nil;

    foreach(type, alias, path; vdraw.IniIterLibInfo ()) {
        str := build_visual (type, path, alias);
        list_of_strings = cons (str, list_of_strings);
    }
    
    list_of_strings = reverse_destructive (list_of_strings);

    move_history := nil;

    function record_up () 
        data.history = cons (list ("Move", current_pick+0, current_pick-1),
                             data.history);
    function record_down () 
        data.history = cons (list ("Move", current_pick+0, current_pick+2),
                             data.history);

    down_arrow := vcWv.IconButtonCreate ("down", "arrow-dn", nil,
                    func (e) {
                        e1 := vcPicklist.Get (picklist, current_pick, 0);
                        e2 := vcPicklist.Get (picklist, current_pick+1, 0);
                        vcPicklist.Set (picklist, car (e1), cdr (e1),
                                        current_pick+1, 0);
                        vcPicklist.Set (picklist, car (e2), cdr (e2),
                                        current_pick, 0);
                        record_down ();
                        ++current_pick;
                        select_pick ("", TRUE, current_pick);
                    }, nil);
    
    up_arrow := vcWv.IconButtonCreate ("down", "arrow-up", nil,
                    func (e) {
                        e1 := vcPicklist.Get (picklist, current_pick, 0);
                        e2 := vcPicklist.Get (picklist, current_pick-1, 0);
                        vcPicklist.Set (picklist, car (e1), cdr (e1),
                                        current_pick-1, 0);
                        vcPicklist.Set (picklist, car (e2), cdr (e2),
                                        current_pick, 0);
                        record_up ();
                        --current_pick;
                        select_pick ("", TRUE, current_pick);
                    }, nil);

    function delete(e) {
        vcPicklist.DeleteRows (picklist, current_pick, 1);

	data.history = cons (list ("Delete", current_pick+0, nil),
			     data.history);
	select_pick ("", TRUE, current_pick);
    }

    function get_mega_alias(path) {
        alias := nil;
        cfg := vsMerge ("", path, "lib", "cfg");
        if (fexists (cfg)) {
            catch (vlEx.FAILURE) {
                fp := fopen(cfg, "r");
                if (!null (fp)) {
                    declare name, value;
                    
                    line := string_trim (fgetline (fp));
                    line = string_token (line, &name, " ");
                    line = string_token (line, &value, " ");
                    if (string_eqi (name, "alias"))
                        alias = string_trim (value);
                    
                    fclose(fp);
                }
            } exception
                ;
        }

        alias;
    }

    function check_for_mega(path) {

        ismega := fexists (vsMerge ("", path, "sch", "lib"));
        if (!ismega)
            ismega := fexists (vsMerge ("", path, "sym", "lib"));

        ismega;
    }


    function popup_add_dialog(e) {
        declare rows, cols;

        parent_window := vcEvent.Window (e);
        cmdline := vcRegion.Create ("cmdline", vcCmdline.fac, "", width <- 50);

        dtype := vcRegion.Create ("type", vcPickEnum.fac,
                                  vdraw.vqLibType.ReadOnly);

        aliasline := vcRegion.Create ("cmdline", vcCmdline.fac, "",
                                      width <- 15, max_width <- 32,
                                      max_size <- 32);

        pos := vcRegion.Create ("pos", vcPickEnum.fac, InsPos.Append);

        vcCmdline.CallOnReturn (cmdline,
                                func (e, str) {
                                    if (check_for_mega (str)) {
                                        alname := get_mega_alias (str);
                                        if (!null (alname))
                                            vcCmdline.Set (aliasline, alname);
                                        vcPickEnum.Set (dtype,
                                              vdraw.vqLibType.MegaFile);
                                    }
                                });

        cmdprompt := vcRegion.Create ("", vcLabel.fac, "Path");
        aliasprompt := vcRegion.Create ("", vcLabel.fac, "Alias");
        typeprompt := vcRegion.Create ("", vcLabel.fac, "Library Type");
        posprompt := vcRegion.Create ("", vcLabel.fac, "Insertion Position");

        t1 := vcTable.Make ("", 1, 2, cmdprompt, cmdline);
        t2 := vcTable.Make ("", 1, 2, typeprompt, dtype);
        t3 := vcTable.Make ("", 1, 2, aliasprompt, aliasline);
        t4 := vcTable.Make ("", 1, 2, posprompt, pos);

        function pick_position(data, on_or_off, index) {
            vcWin.ClearStatus (win);
            if (on_or_off) {
                insert_before_index = index;
                if (index == 0) {
                    pos := vcRegion.Get (pos, InsPosVT);
                    if (pos == InsPos.InsertBefore) {
                        vcWin.AddStatus (win,
                         "Select Primary for Libtype to add a new primary");
                        vcPicklist.DeselectAll (pick);
                    } /* if */
                } /* if */
            } /* if */
        } /* function pick_position */

        pick := vcRegion.Create ("", vcPicklist.fac, nil, columns <- 1,
                                     will_scroll <- TRUE,
                                     enhanced_notify <- TRUE);
        
        vcPicklist.Notify (pick, pick_position);

        vcPicklist.Dimensions (picklist, &rows, &cols);
        for (i := 0; i < rows; ++i ) {
            e := vcPicklist.Get (picklist, i , 0);
            vcPicklist.Add (pick, car (e), cdr (e));
        } /* foreach */

        entries := rows;

        insert_before_index := i;

        scrollp := vcPicklist.Scroll ("", pick, 1, 10, 0);

        function update_choice(choice) 
            if (choice == InsPos.InsertBefore && entries > 1)
                ENABLE (pick);
            else
                DISABLE (pick);
        
        DISABLE (pick);

        vcPickEnum.CallOnChange (pos, update_choice);
        vcPickEnum.CallOnChange (dtype,
                                 func (choice) {
                                     if (choice == vdraw.vqLibType.Primary) {
                                         DISABLE (pick);
                                         DISABLE (posprompt);
                                         DISABLE (pos);
                                     } else {
                                         ENABLE (posprompt);
                                         ENABLE (pos);
                                         pos := vcRegion.Get (pos, InsPosVT);
                                         if (pos == InsPos.Append)
                                             DISABLE (pick);
                                         else
                                             ENABLE (pick); 
                                     }
                                 });

        addtable := vcTable.Make ("", 5, 1, t1, t2, t3, t4, scrollp);

        function accept_f(e) {
            type := vcRegion.Get (dtype, vdraw.vqLibTypeVT);
            path := string_trim (vcCmdline.Get (cmdline));
            alias := string_trim (vcCmdline.Get (aliasline));
        
            /*
             * Check for embedded spaces
             */
            if (lindex (alias, ' ') != FALSE)
                vlEx.Throw2 (vlEx.FAILURE, "vdraw", vlErrSev.Error,
                             "Alias names may not contain embedded spaces");
            
            lp := string_search (alias, "(");
            rp := string_search (alias, ")");
            
            if (!null (lp) || !null (rp)) 
                vlEx.Throw2 (vlEx.FAILURE, "vdraw", vlErrSev.Error,
                             "Alias names must be specified WITHOUT parentheses");

            if (!string_eq (alias, "") && alias[0] >= '0' && alias[0] <= '9')
                vlEx.Throw2 (vlEx.FAILURE, "vdraw", vlErrSev.Error,
                             "Alias names may not start with digits 0-9");
            
            is_megafile := check_for_mega (path);

            if (type == vdraw.vqLibType.MegaFile && is_megafile) {
                alname := get_mega_alias (path);
                if (!null (alname))
                    alias = alname;
            } else if (type == vdraw.vqLibType.MegaFile && !is_megafile) {
                type = vdraw.vqLibType.ReadOnly;

                vlErr.Out (nil,
                           vlErr.Create ("", "vdraw", 6061, vlErrSev.Warning,
                      "Library is not a MegaFile, type changed to ReadOnly"));
                
            } else if (type != vdraw.vqLibType.MegaFile && is_megafile) {
                type = vdraw.vqLibType.MegaFile;
                alname := get_mega_alias (path);
                if (!null (alname))
                    alias = alname;
                
                vlErr.Out (nil,
                           vlErr.Create ("", "vdraw", 6060, vlErrSev.Warning,
                     "Library is a MegaFile, type changed to MegaFile"));
            }
            
            if (string_eq (path, ""))
                vlEx.Throw2 (vlEx.FAILURE, "vdraw", vlErrSev.Error,
                             "Unable to add library - Empty path string");
            
            
            /*
             * Re-build picklist
             */
            vcPicklist.Dimensions (picklist, &rows, &cols);
            
            newpick := vcRegion.Create ("", vcPicklist.fac, nil, columns <- 1,
                                        will_scroll <- TRUE,
                                        enhanced_notify <- TRUE);
            
            vcPicklist.Notify (newpick, select_pick);
            
            function convert_type(type) {
                declare str;
                
                if (type == vdraw.vqLibType.Writable)
                    str = "w";
                else if (type == vdraw.vqLibType.ReadOnly)
                    str = "r";
                else if (type == vdraw.vqLibType.MegaFile)
                    str = "rm";
                else if (type == vdraw.vqLibType.Primary)
                    str = "pw";
            }
            
            visual := build_visual (convert_type (type), path, alias);
            
            poschoice := vcRegion.Get (pos, InsPosVT);
            if (poschoice == InsPos.Append)
                insert_before_index = rows;
            
            if (type == vdraw.vqLibType.Primary)
                insert_before_index = 0;
            
            i := 0;
            for (i = 0; i < rows; ++i ) {
                e := vcPicklist.Get (picklist, i , 0);
                if (i == insert_before_index)
                    vcPicklist.Add (newpick, visual, visual);
                
/*
  If this is a new primary, then the current primary must have
  it's visual modified to change the [pw] to a [w]
  */
                if (i == 0 && i == insert_before_index) {
                    oldp := car (e);
                    brp := string_search (oldp, "[pw]");
                    if (!null (brp)) 
                        oldp = substring (oldp, 0, brp) + "[w]" + 
                            substring (oldp, brp+4, 0);
                    vcPicklist.Add (newpick, oldp, cdr (e));
                } else
                    vcPicklist.Add (newpick, car (e), cdr (e));
            }
            
            if (i == insert_before_index)
                vcPicklist.Add (newpick, visual, visual);
            
            scrollp = vcPicklist.Scroll ("", newpick, 1, 10, 0);
            picklist = newpick;
            
            vcTable.Set (table, 0, 1, scrollp);
            
            select_pick ("", TRUE, insert_before_index);

            data.history = cons (list ("Add", insert_before_index+0,
                                       list (type, path, alias)),
                                 data.history);

            vcWin.Destroy (win);
        }
        
        function cancel_f(e)
            ;
        
        
        buttons := vcRegion.Create ("", vcButnRow.fac,
                                    list (list ("yes", "Accept",
                                                accept_f, nil),
                                          list ("cancel", "Cancel",
                                                cancel_f,
                                                list (close_window <- TRUE))));

        vcTable.Col (addtable, -1, vcTable.Border.Visible, 2, 3, 5,
                     vgAlign.MiddleLeft);
        
        win := vcWin.Create (parent_window,
                             vcTable.Make ("Add Library",
                                           2, 1, addtable, buttons));

        vcWin.Forward (parent_window, win);
    } /* function popup_add_dialog */

    delete_button := vcWv.LabelButtonCreate ("", "Delete", nil, delete, nil);
    add_button  := vcWv.LabelButtonCreate ("", "Add Library...", nil,
                                           popup_add_dialog, nil);

    buttons := vcTable.Make ("", 4, 1, up_arrow, add_button,
                             delete_button, down_arrow);
    current_pick := 0;

    function select_pick (data, on_or_off, index) {
        if (on_or_off) {
	    declare rows, cols;

	    w := vcRegion.Window (region);

            current_pick = index;

	    vcPicklist.Dimensions (picklist, &rows, &cols);

            if (current_pick > 0 && LibIsDeleteable (data))
                vcRegion.Enable (delete_button, TRUE, TRUE);
            else
                vcRegion.Enable (delete_button, FALSE, TRUE);

            
            libs := rows - 1;

            if (current_pick == 0 || libs == 1) {
                DISABLE (up_arrow);
                DISABLE (down_arrow);
            } else if (current_pick == 1) {
                DISABLE (up_arrow);
                ENABLE (down_arrow);
            } else if (current_pick > 1 && current_pick < rows-1) {
                ENABLE (up_arrow);
                ENABLE (down_arrow);
            } else {
                ENABLE (up_arrow);
                DISABLE (down_arrow);
            }
            
            vcPicklist.SetPick (picklist, current_pick);
        } /* if */
    } /* function select_pick */

    picklist := vcRegion.Create ("", vcPicklist.fac, nil, columns <- 1,
                                 will_scroll <- TRUE, enhanced_notify <- TRUE);

    vcPicklist.Notify (picklist, select_pick);

    foreach(s; vscript.list_iter (list_of_strings)) 
        vcPicklist.Add (picklist, s, s);
    
    select_pick ("", TRUE, 0);

    scrollp := vcPicklist.Scroll ("", picklist, 1, 10, 0);
    
    table := vcTable.Make ("", 1, 2, buttons, scrollp);

    vcRegion.AddChild (region, table);

    data := struct {
        history = move_history;
        initial = initial;
        picklist = picklist
    };

    data;
}

vlPref.Replace (prefs, "initial", vlIntVT,
                "Initial Selection index", 0);


function _Get(data, region, ts) {
    declare ret_data;

    ret_data := nil;

    reverse_destructive (data.history);
} /* function _Get */

vcBind.Set (bindings, '\leftdbl', vcWv.Cmd (fac, func (d, e);, ""));



