/* ------------------------------------------------------------------------ *\
 * $Revision: 1.48 $   $Date: 1995/05/10 15:44:30 $
 *
 * Copyright (c) 1992 by Viewlogic Systems, Inc.
\* ------------------------------------------------------------------------ */
function DEBUG(args...);
DEBUG := printf; 

/*
**  This function doubles $ signs.  Before you send a command to Viewdraw,
**  you have to double these so that the the string will not be interpreted
**  as an env variable.
*/
function doubledollar(str)
{
   /*DEBUG("doubledollar, str = %s\n", str);*/

   if ( i := lindex(str, '$'))
        substring(str, 0, i+1) + "$" + 
           doubledollar(substring(str, i+1, 0));
        else
           str;
} /* doubledollar */


/*
** bogus path is returned when we can't find out our path, so that viewdraw
** will do a pop 99 and get to the top of the design.
*/
for (bogus_path := nil, i := 0; i < 100; ++i)
    bogus_path = cons ("", bogus_path);

function fallback_vdraw_path() 
{
    DEBUG ("-------- fallback_vdraw_path -------\nWBLOCK = %s\nWDESIGN = %s\n",
            vscript.getenv ("WBLOCK"),
            vscript.getenv ("WDESIGN"));

    if (string_eq (vscript.getenv ("WBLOCK"), vscript.getenv ("WDESIGN")))
        nil;
    else
        bogus_path;
} /* function fallback_vdraw_path */

function String2ReversePath(str)
{
    declare token;
    path := nil;
    DEBUG ("--------- String2ReversePath --------\nstr = %s\n", str);
    
    for (str = string_token(str, &token, "\\ \n");
         str != nil && token != nil && string_ref(token, 0) != '(';
         str = string_token(str, &token, "\\ \n"))
        path = cons (token, path);

    DEBUG ("path = %s\n", path);
    path;
} /* function String2ReversePath */

/*
** This function returns true if this is a named path
** false otherwise.
*/
function IsNamePath(path)
{
    TheLength := string_length(path);
    n := 0;
    GotAName := FALSE;
    while(TRUE) {
        NotUID := (path[n] != '$');
        while(++n < TheLength)
            if (path[n] == '\\') {
                GotAName = NotUID;
                ++n;
                break;
            } /* if */
        if (GotAName || n >= TheLength)
            break;
    } /* while(TRUE) */
    GotAName;
} /* IsNamePath */

/*
 * spinname_works is checked by vdrc.vs.  It existed but did not work
 * prior to 5.2.  Also, starting at 5.2, there was a function vcWv.Version,
 * which returns the Core version number, so we see if we're at 5.2 or
 * greater just by checking to see that the symbol exists.
 */
    if (isbound (`CompatibleVersion, vcWv))    /* Core 5.2 or greater... */
    {   
        spinname_works := TRUE;
        call_on_refresh := vcWin.CallOnRefresh;
    } /* if */
    else 
    {
       spinname_works := FALSE;
       define_enum := vcWv.MakeEnum;

       /*
        * ACK!  Viewdraw redefines "fexists", and redefines it wrong!
        */
       fexists = vscript.fexists;
          
       function vsUnique(path) 
       {
           declare pathname;
           keep_going := TRUE;

           for (i := 0; keep_going; ++i) 
           {
               pathname = vscript.vsMerge ("", path, "vdrctmp", i);
               keep_going = fexists (pathname);
               printf ("fexists(%s)==%s\n", pathname, keep_going);
           } /* for */
           pathname;
       } /* function vsUnique */
   
       function call_on_refresh(w, f) 
       {
           declare timer;

           function poll() 
           {
               if (vcWin.EverRefreshed (w)) 
               {
                   vcWv.CancelTimer (timer);
                   f ();
               } /* if */
           } /* function poll */
           timer := vcWv.Timer (wv, 200, poll, TRUE);
       } /* function call_on_refresh */
   } /* else */

/*
 * Special case sname here until Viewdraw fixes a bug. sfastname
 * cannot find GND or B1 (as in B[0:4]) but sname can.  Sname is
 * also slower.  So we do a special check here to determine which
 * search to use.
 *
 * This routine just turns a component (e.g. "$1N5") into "sname $1N5",
 * or "sfastname $1N5", as appropriate.
 */
function do_sname(d, e, str) {
if (IsNamePath (str))
   {
      DEBUG ("sfastname= %s\n", str);
      
        sfastname (d, e, str);
   }
    else
   {
      DEBUG ("sname= %s\n", str);
        sname (d, e, str);
   }
    
} /* function do_sname */

function NamePath2UIDPath(design, str)
{
    result := str;

    DEBUG ("IsNamePath(\"%s\") = %s\n", str, IsNamePath(str));

    if ( IsNamePath(str)) {
        DEBUG ("NamePath2UIDPath\n");
        DEBUG ("design = %s\n", design);
        DEBUG ("str = %s\n", str);

        NavContext := vipc.CntxCreate(vipc.GidANY, vipc.UidANY,
                                      vipc.FidDFLT, vipc.TidNAVIGATOR /*28*/,
                                      vipc.HnmANY, vipc.PidANY,
                                      vipc.CidANY);

        function handler(mtype, src, dest, dlen, dtype, data) {
            DEBUG ("Ipc Message:\n");
            DEBUG ("Mtype: %d\n", mtype);
            DEBUG ("Src:   %s\n", src);
            DEBUG ("Dest:  %s\n", dest);
            DEBUG ("Data:  %s\n", data);

            result = data;
        } /* handler */
        Message := vipc.Call(131, /*vipc.MsgNAMEPATH2UIDPATH*/
                             vipc.CntxDflt, NavContext,
                             design + " " + str,
                             handler, nil,
                             vipc.FlgDIRECT);

        DEBUG ("Message = %s\n", Message);

        /*declare data;*/
        /*if ((data = vipc.MsgGetData(Message)) != vipc.MsgDataNULL)*/
            /*result = data;*/

       /* DEBUG ("result = %s\n", result);*/

    } /* if */
    result;
} /* NamePath2UIDPath */

function vdraw_current_path(display, event, design) {
    /*
    ** This routine will write the temporary nodes file to a unique file
    ** placed in the directory that is pointed to by the TEMP environment
    ** variable.  If the variable is not defined or the directory does not
    ** exist the file will be written to the working directory.
    */
    DEBUG ("------vdraw_current_path------\n");
    pathname := vscript.getenv ("TEMP");

    if (pathname == nil) 
    {
        pathname = vscript.getenv ("PWD");
        if (pathname == nil)
            if (vsInfo ("/tmp") != nil)
                temp := "/tmp/";
            else
                temp := "C:\\";
    } /* if */

    /*
    **  The command below is for 5.2 and higher
    ** enable it when vdraw can build that version.
    */
    /* fname := vlPath.Unique(pathname);*/
    if (pathname == nil)
       fname := vsUnique(temp);
    else
       fname := vsUnique(pathname);

    DEBUG ("Unique returned %s\n", fname);
    wrnodes (display, event, fname, vdraw.vqOpenOutputMode.Overwrite);
    f := fopen (fname, "r");
    firstline := fgetline (f);
    DEBUG ("f = %s, firstline = %s\n", f, firstline);

    if (f != nil && firstline == nil) 
    {
        fclose (f);
        /*
        ** If we've gotten a file but it's empty it means nothing's selected 
        ** so we try to be more selective and perform the select again.
        */
        sfastname (display, event, "$*I*");
        wrnodes (display, event, fname, vdraw.vqOpenOutputMode.Overwrite);
        f = fopen (fname, "r");
        firstline := fgetline (f);
        DEBUG ("f = %s, firstline = %s\n", f, firstline);
    } /* if */

    path := nil;

    if (firstline == nil)
        path = fallback_vdraw_path ();
    else
    {
        item := nil;
        string_token(firstline, &item, " \n");
        PathStr := NamePath2UIDPath(design, item);
        path = String2ReversePath(PathStr);
        DEBUG("firstline/string_token = %s\nitem = %s\nPathStr=%s\nPath=%s\n",
               firstline, item, PathStr, path);    
    }

    DEBUG ("reversed current path plus = %s\n", path);
    fclose (f);
    vscript.vsDelete (fname, vscript.vsFILE);

    /* Get rid of the last component and reverse */
    /*path = reverse_destructive(cdr(path));*/

    /* modified to just reverse */
    path = reverse_destructive(path);
} /* function vdraw_current_path */

iterator windows() 
{
    foreach (w; vcWin.Loop ())
        if (vcWin.Parent (w) == nil)  /* no context windows */
        {
            r := vcWin.OwnerRegion (w);
            if (vcRegion.Fac (r) == vdraw.fac)
                iterate (w, r);
        } /* if */
} /* iterator vdraw_windows */

function with_block(f, test_region, open_window)  
{
    function call_user_function(w) 
    {
        r := vcRegion.FindFac (vcWin.OwnerRegion (w), vdraw.fac);
        function run_cmd(str) {
            DEBUG ("-------with block------\n");
            DEBUG ("Command str: %s\n", str);            
            str = doubledollar(str);
            DEBUG ("After doubledollar, Command str: %s\n", str);            
            vcWin.RunCmdline (w, str,
                              vcEvent.Create (0, 0, '0', str, w,
                                              vcWin.OwnerRegion (w), nil));
        } /* function run_cmd */
        d := vcRegion.Data (r, vcFac.Typespec (vdraw.fac));
        e := vcEvent.Create (0, 0, '0', "", w, r, nil);
        f (run_cmd, w, d, e);
        enable_cmds (d);
    } /* function call_user_function */

    hidden_window := nil;

    foreach (w, r; windows ())
        if (test_region (r)) 
        {
            if (vcWin.IsIconified (w)) 
            {
                if (hidden_window == nil)
                    hidden_window = w;
            } /* if */
            else 
            {
                call_user_function (w);
                return (nil);
            } /* else */
        } /* if */

    /*
     * If there is no visible window with the design, but there was
     * an iconified one, show it.  Otherwise, open a new window with
     * the supplied function.
     */
    if (hidden_window != nil) 
    {
        vcWin.Show (hidden_window);
        call_user_function (hidden_window);
    } /* if */

    /* If there is still no window, try to create one. */
    else 
    {
        w := open_window ();
        if (w == nil)
            error ("Unable to open Viewdraw window", nil);
        call_on_refresh (w, func(args...) call_user_function (w););
    } /* if */
} /* function with_block */

/*
 * Parses the title string of a window to determine if the window
 * contains the specified symbol or schematic.
 */
function match_window(r, the_type, the_lib, the_name) 
{
    declare vdraw_name, version, type, lib, name, rest;

    title := vcWin.Title (vcRegion.Window (r));
    if (((rest = string_token (title, &vdraw_name, " ")) == nil) ||
        !string_eqi (vdraw_name, "Viewdraw") ||
        ((rest = string_token (rest, &version, " ")) == nil) ||
        /* !string_eq (version, "5.1") || */
        ((rest = string_token (rest, &name, " (")) == nil) ||
        ((rest = string_token (rest, &type, "()- ")) == nil))
        error ("Unexpected Viewdraw window title", title);

    (((the_type == nil) || string_eqi (the_type, type)) &&
     ((the_lib == nil) || string_eqi (the_lib, getdesignlib (r))) &&
     ((the_name == nil) || string_eqi (the_name, name)));
} /* function is_correct_window */

/*
 * Call a function with the specified symbol as the "current" window.
 * The function is passed a function object which it can apply to
 * run commands on the desired symbol window.
 */
function with_symbol(the_symbol, f) 
{
    the_lib := nil;
    colon := lindex (the_symbol, ':');
    if (colon != FALSE) 
    {
        the_lib = substring (the_symbol, 0, colon);
        the_symbol = substring (the_symbol, colon + 1, 0);
    } /* if */
    if (lindex (the_symbol, '.') == FALSE)
        the_symbol += ".1";

    with_block (f,
                func(r) match_window (r, "SYM", the_lib, the_symbol);,
                func() symbol (the_symbol););
} /* function with_symbol */

/*
 * Searches for search_string in big_string, returning either FALSE
 * for a failed search, or the character index of start of search_string
 * in big_string.
 */
function vnstring_search(big_string, search_string) 
{
    search_len := string_length (search_string);
    function loop(big_string, big_len, offset) 
    {
        start := lindex (big_string, search_string[0]);
        if (start) 
        {
            if ((start + search_len <= big_len) &&
                string_eq (substring (big_string, start, start + search_len),
                           search_string))
                offset + start;
            else
                loop (substring (big_string, start + 1, big_len),
                      big_len - start - 1, offset + start + 1);
        } /* if */
        else
            FALSE;
    } /* function loop */

    loop (big_string, string_length (big_string), 0);
} /* function vnstring_search */

/*
 * Call a function with the specified schematic as the "current" window.
 * The function is passed a function object which it can apply to
 * run commands on the desired schematic window.
 */
function with_schematic(sch, f) 
{
    function call_user(c, w, d, e)
    {
        pop_if_unmodified (d, e, 100);
        f (c, w, d, e);
    } /* function call_user */
    with_block (call_user,
                func(r) (string_eqi (vdraw.getdesignname (r), sch) &&
                         match_window (r, "SCH", nil, nil));,
                func() schematic (sch););
} /* function with_schematic */

function with_design(design, sheet_str, user_function) {
    DEBUG ("-------Inside with_design-------\n");
    colon := rindex (design, ':');
    if (colon)
        design = substring (design, colon + 1, 0);

    DEBUG ("Design=%s\n",design);
    DEBUG ("sheet_str=%s\n", sheet_str);
    DEBUG ("user_function=%s\n", user_function);
 
    design_sch_title := string_upper (design) + ".1(SCH)";
    design_sch_title_for_sheet :=
        sprintf ("%s.%s(SCH)", string_upper (design), sheet_str);

    function test_region(r)
         (string_eqi (getdesignname (r), design) &&
         (vnstring_search (vcWin.Title (vcRegion.Window (r)),
                         design_sch_title) ||
          vnstring_search (vcWin.Title (vcRegion.Window (r)),
                         design_sch_title_for_sheet)));

    function call_user(cmd, window, d, e) 
    {
        while (getblockdatatype (d) == vqDataType.Symbol)
       {
            DEBUG ("POP to 1?\n"); 
            pop_if_unmodified (d, e, 1);
         };
                
        sheet := convert (vlIntVT, sheet_str) + 0;

        while (getblksheet (d) > sheet)
       {
           DEBUG ("POP to 1 num 2?\n");
           pop_if_unmodified (d, e, 1);
        };
        
        DEBUG ("with_design found window `%s'\n with design `%s', type `%s'\n",
               vcWin.Title (window), getdesignname (d), getblockdatatype (d));
        DEBUG ("Calling user_function with cmd:`%s'\n", cmd);
        DEBUG ("DataBlockType = %s\n", getblockdatatype (d));
        
        user_function (cmd, window, d, e);
    } /* function call_user_with_design */

    with_block (call_user, test_region, func() schematic (design););
} /* function with_design */

function pop_if_unmodified(d, e, count) 
{
    /* Bob D 11/17
     * Add check for modifed data.
     * If find some -- abort with an error.
     * Fixes RTM stopper wrt the Ok->Pop festival.
     */
    /*
     * error2 really should be used, but it doesn't seem to work
     * as advertised -- it doesn't warp out of the Viewscript
     * evaluation, so it causes major problems.  We have to use
     * the uglier but more functional "error", instead.
     * error2 ("vdraw", vlErrSev.Error, 0, nil, 
     *         "Modified schematics exist -- save before navigating");
     */
    if (modified (d, e, vqDataType.Schematic) == 1) 
        error ("\nModified schematics exist", "save before navigating");
    else
       popblock (d, count);
    
} /* function pop_if_unmodified */

function push_to_sheet(display, event, sheet) 
{
    if (!string_eq (convert (vlStringVT, getblksheet (display)), sheet))
        push_sheet (display, event, sheet);
} /* function push_to_sheet */

/*
 * Parse a partial location string in the form "path - name...", where
 * the path or the - can be missing, in which case path gets nil, and 
 * name gets the whole string.
 */
function parse_optional_path(str, path, name) 
{
    dash := lindex (str, '-');

    if (dash) 
    {
        *path = string_trim (substring (str, 0, dash));
        *name = string_trim (substring (str, dash + 1, 0));
    } /* if */
    else 
    {
        *path = nil;
        *name = string_trim (str);
    } /* else */
} /* function parse_optional_path */


define_enum ("vqObjTypeVT", list ("sch", "sym", "spn", "net", "pin", "com",
                                  "psch", "psym", "pspn"));

/*
 * Use an iterator that iterates once to do multiple-return-values!
 */
iterator parse_location(location) 
{
    declare design, obj_type, path, rest, obj, name, sheet, pin_name;

    rest = string_token (location, &design, " ");
    rest = string_token (rest, &obj_type, " ");
    obj_type = string_lower (obj_type);

    if (string_eq (obj_type, "s"))
        obj_type = vqObjType.sch;
    else
        obj_type = convert (vqObjTypeVT, obj_type);

    DEBUG("VVVVVVVVVVVVVVVVVVVVVVVVVVVVVV\nObject_type %s\nVVVVVVVVVVVVVVVVVVVVVVVVV\n", obj_type);
    
    /*
     *  - sym SymbolName[.Sheet]                (april 93 spec)
     *
     *  sym - SymbolName[.Sheet]     (old style for compatibility)
     */
    if (obj_type == vqObjType.sym) {
        if (string_eq (design, "-")) {                  /* April 1, 1993 */
            design = "";
            obj = rest;
        } /* if */
        else                                            /* Old-style */
            parse_optional_path (rest, &path, &obj);
    } /* if */

    /*
     * Design psym Path[.Sheet]                 (april 93 spec)
     */
    else if (obj_type == vqObjType.psym)
        sheet = string_token (rest, &path, ".");
        
    /*
     *  - spn SymbolName[.Sheet].PinName        (april 93 spec)
     *
     *  SymbolName sym - SymbolName[.Sheet]     (old style for compatibility)
     */
    else if (obj_type == vqObjType.spn) 
         {
             if (string_eq (design, "-"))                   /* April 93 */
             {
                 dot := rindex (rest, '.');
                 design = "";

                 if (!dot)
                     error ("Unable to parse symbol pin name", location);

                  obj = substring (rest, 0, dot);
                  pin_name = substring (rest, dot + 1, 0);
              } /* if */
              else 
              {                                          /* Old style */
                  parse_optional_path (rest, &path, &name);

                  if ((rest = string_token (name, &obj, ".")) == nil)
                       error ("Couldn't find symbol name in location string",
                               location);
                  else 
                  if ((rest = string_token (rest, &sheet, ".")) == nil)
                      error ("Couldn't find pin name in location string", location);
                  else 
                      if ((rest = string_token (rest, &pin_name, ".")) == nil) 
                      {
                          pin_name = sheet;
                          sheet = nil;
                       } /* else if */
               } /* else */
           } /* else if */

    /*
     * Design pspn Path[.Sheet].PinName         (april 93 spec)
     */
    else if (obj_type == vqObjType.pspn) {
        dot := rindex (rest, '.');
        if (!dot)
            error ("Unable to parse symbol pin name", location);
        pin_name = substring (rest, dot + 1, 0);
        rest = substring (rest, 0, dot);
        sheet = string_token (rest, &path, ".");
    } /* else if */

    /*
     * - sch SchematicName[.Sheet]              (april 93 spec)
     * design s - schematic[.Sheet]             (old)
     * design sch - schematic[.Sheet]           (old)
     * design sch schematic[.Sheet]             (old)
     */
    else if (obj_type == vqObjType.sch) {
        if (string_eq (design, "-")){
            design = "";
            obj = rest;
        } /* if */
        else {
            parse_optional_path (rest, &path, &name);

            if ((rest = string_token (name, &obj, ".")) == nil)
                error ("Couldn't find schematic name in location string",
                       location);
            else if ((rest = string_token (rest, &sheet, ".")) == nil)
                sheet = "1";
        } /* else */
    } /* else if */

    else if (obj_type == vqObjType.psch)
        sheet = string_token (rest, &path, ".");

    /*
     * Design net Path
     * Design com Path
     * Design pin Path.PinName
     */
    else {
        pin_name = string_token (rest, &path, ".");
        if ((pin_name != nil) && (string_length (pin_name) > 0) &&
            (pin_name[0] == '.'))
            pin_name = substring (pin_name, 1, 0); /* chop off . */
        path = string_trim (path);
        parse_path_element (path, &sheet);
        DEBUG ("Parsed path=%s, sheet=%s, pin_name=%s\n", path, sheet,
               pin_name);
    } /* else */

    if ((sheet == nil) && (obj != nil))
        sheet = string_token (string_copy (obj), &obj, ".");
    if ((sheet == nil) || string_eq (sheet, ""))
        sheet = "1";
    else if (sheet[0] == '.')
        sheet = substring (sheet, 1, 0);
        
    function tr(x) x == nil ? "" : string_trim (x);
    iterate (obj_type, tr (design), tr (path), tr (obj) , tr (sheet),
             tr (pin_name));
} /* iterator parse_location */

/*
 * Here are some regression tests for the location-parser.
 */
if (FALSE) {
    function seq(s1, s2) (s1==s2) || string_eq (s1, s2);

    function tst(str, obj_type, design, path, obj, sheet, pin_name) {
        foreach (ot, d, p, o, s, pn; parse_location (str)) {
            if (ot != obj_type) error (sprintf ("Object type %s != %s",
                                                obj_type, ot),
                                       str);
            if (!seq (design, d)) error (sprintf ("Design %s != %s",
                                                  design, d),
                                         str);
            if (!seq (path, p)) error (sprintf ("Path %s != %s", path, p),
                                       str);
            if (!seq (obj, o)) error (sprintf ("Obj %s != %s", obj, o), str);
            if (!seq (sheet, s)) error (sprintf ("Sheet %s != %s", sheet, s),
                                        str);
            if (!seq (pin_name, pn)) error (sprintf ("Pin_Name %s != %s",
                                                     pin_name, pn),
                                            str);
            printf ("`%s' parsed successfully\n", str);
        } /* foreach */
    } /* function tst */

    tst ("design net path", vqObjType.net, "design", "path", "", "1", "");
    tst ("design com path", vqObjType.com, "design", "path", "", "1", "");

    tst ("- sch Schem.Sht", vqObjType.sch, "", "", "Schem", "Sht", "");
    tst ("- sch Schem", vqObjType.sch, "", "", "Schem", "1", "");
    tst ("design s - sch.2", vqObjType.sch, "design", "", "sch", "2", "");
    tst ("design s - sch", vqObjType.sch, "design", "", "sch", "1", "");
    tst ("design psch path", vqObjType.psch, "design", "path", "", "1", "");
    tst ("design psch path.5", vqObjType.psch, "design", "path", "", "5", "");

    tst ("- sym Symbol.Sht", vqObjType.sym, "", "", "Symbol", "Sht", "");
    tst ("- sym Symbol", vqObjType.sym, "", "", "Symbol", "1", "");
    tst ("design sym - sym.8", vqObjType.sym, "design", "", "sym", "8", "");
    tst ("design sym - sym", vqObjType.sym, "design", "", "sym", "1", "");
    tst ("design psym path", vqObjType.psym, "design", "path", "", "1", "");
    tst ("design psym path.5", vqObjType.psym, "design", "path", "", "5", "");

    tst ("- spn Symbol.8.Pin", vqObjType.spn, "", "", "Symbol", "8", "Pin");
    tst ("- spn Symbol.Pin", vqObjType.spn, "", "", "Symbol", "1", "Pin");
    tst ("design spn - spn.6.pin", vqObjType.spn, "design", "", "spn", "6",
         "pin");
    tst ("design spn - spn.pin", vqObjType.spn, "design", "", "spn", "1",
         "pin");
    tst ("design pspn path.pin", vqObjType.pspn, "design", "path", "", "1",
         "pin");
    tst ("design pspn path.5.pin", vqObjType.pspn, "design", "path", "", "5",
         "pin");
} /* if */

/*
 * Select the object specified in the error "Location string".  This
 * may require popping thru hierarchy or bringing up a new window.
 */
function SelectLocation(location, more_stuff) 
{
   DEBUG ("SelectLocation location=%s, more_stuff=%s\n", location, more_stuff);
   
   foreach (obj_type, design, path, obj, sheet, pin_name;
             parse_location (location))
    {

        if (string_eq (path, "") && string_eq (obj, ""))
            obj = design;

        DEBUG ("SelectLocation(%s), path=%s, sheet=%s, obj=%s, pin=%s\n",
               location, path, sheet, obj, pin_name);

        function finish(cmd, w) {
           DEBUG ("Inside finish cmd=%s\n",cmd);
           
            if (more_stuff != nil)
                more_stuff (cmd, w);
        } /* function finish */

        function goto_block(with) 
        {
            if (string_eq (pin_name, ""))
                pin_name = "___select_nothing___";

            function goto_sheet(d, e, w, cmd) 
            {
                push_to_sheet (d, e, sheet);
                DEBUG ("LOOKING FOR PIN NAMESPPPPPPPPPPPPPPPPPPPPP\n");
                dot := lindex (pin_name, '.');
                DEBUG ("dot=%s\n",dot);
                
                if (dot) 
                    if (spinname_works)
                        spinname (d, e, pin_name);
                    else
                        do_sname (d, e, substring (pin_name, 0, dot));

                finish (cmd, w);
            } /* function goto_sheet */

            if (string_eq (path, ""))
            {
                DEBUG ("About to call with...\n");
                with (obj, func(cmd, window, d, e) 
                {
                       goto_sheet (d, e, window, cmd); 
                });
            }
            else
            {
                DEBUG ("Entering with_desing path...\n");
              
                with_design (design, sheet, func(cmd, window, d, e) 
                {
                    select_path (d, e, design, path);
                    pushblock (d, (with == with_symbol) ? "Symbol" :
                               "Schematic");
                    goto_sheet (d, e, window, cmd);
                });
             }
        } /* function goto_block */

        if ((obj_type == vqObjType.sch) || (obj_type == vqObjType.psch))
        {
            DEBUG ("with_schematic=%s\n", with_schematic);  
            goto_block (with_schematic);
        }
        else if ((obj_type == vqObjType.sym) || (obj_type == vqObjType.psym) ||
                 (obj_type == vqObjType.spn) || (obj_type == vqObjType.pspn))
        {
            DEBUG ("with_symbol=%s\n", with_symbol);
            goto_block (with_symbol);
        }
        else
        {
            DEBUG ("with_design=%s\n", with_design);    
            with_design (design, sheet, func(cmd, window, d, e) 
            {
                push_to_sheet (d, e, sheet);
                if (!string_eq (pin_name, ""))
                    path += "." + pin_name;
                DEBUG ("Calling select_path with design=%s and path=%s\n",design,path);
                
                select_path (d, e, design, path);
                finish (cmd, window);
            });
         }
        
    } /* foreach */
} /* function SelectLocation */
    

function locate(vdraw, event, design, obj_type, path) 
{
   DEBUG ("**** Inside locate command ****\n");
   obj_type = convert (vlStringVT, obj_type);
   DEBUG ("Calling SelectLocation with" + design + " " + obj_type + " " + convert (vlStringVT, path)); 
   SelectLocation (design + " " + obj_type + " " + convert (vlStringVT, path),
                    nil);
} /* function locate */

locate_cmd :=
    vcWv.Cmd (fac, locate, "locate", "Locate Object",
              vcWv.Param ("Design", vlStringVT, vcWv.Dynamic, getdesignname),
              vcWv.Param ("Object", vqObjTypeVT, vcWv.Sticky, vqObjType.net),
              vcWv.Param ("Path...", vlStringRestOfLineVT, vcWv.Sticky, "",
                          "Object-dependent path specification"));
vcCmd.EnableApply (locate_cmd, TRUE);


/* The following command to implements the i/f
 *     IpcCmd <design-name> <rest-of-line>
 * on top of the above functions.
 */
IpcRunCmd := vcWv.Cmd(fac, func(d, e, DesignName, CmdName)
                      {
                          DEBUG ("------In IpcRunCmd-------\n");
                          string_lower(DesignName);
                          DEBUG ("DesginName: %s, CmdName: %s\n", DesignName, CmdName);
                          

                          /*
                          ** Below we use the sheet that we are currently on so that commands
                          ** like ZSEL don't get trashed by having to go back to sheet
                          ** 1. If we are on the wrong sheet the search software above will 
                          ** take care of the problem.
                          */
                          with_design (DesignName, vscript.getenv("WSHEET"),
                                        func(f, w, d, e) f(CmdName);
                                       );
                      },
                      "IpcRunCmd", "IpcRunCmd",
                      vcWv.Param("Design", vlStringVT, vcWv.Sticky, ""),
                      vcWv.Param("CmdLine", vlStringRestOfLineVT,
                                 vcWv.Sticky, ""));
vcCmd.EnableMacro(IpcRunCmd, vlOnOff.On);

/*
 * This function implements the Navigator spec of April 1, 1993, which
 * states that path elements look like this:
 *
 *      $sheetInumber           Component UID with sheet
 *      $sheetNnumber           Net UID with sheet
 *      $sheet_name             Component name with sheet
 *      $sheet:name             Net name with sheet
 *
 * we must also handle the old case, which is that there is a net or
 * compononent name without a sheet number, in which case we must search!
 */
function parse_path_element(element, sheet)
{
    declare s;
    if (string_eq (element, "") ||
        (element[0] != '$') ||
        ((s = string_token (substring (element, 1, 0), sheet, "NI_:")) == nil))
    {
        *sheet = nil;  /* This forces a search to start at 1 */
        DEBUG ("Inside parse path element return val=%s, (search to one)\n",
                element);
        element;
    } /* if */
    else 
    {
        char_after_sheet := element[1 + string_length (*sheet)];

        if ((char_after_sheet == 'N') || (char_after_sheet == 'I'))
        {
            DEBUG ("Inside parse path element return val=%s\n", element);
            element;
        }
        else
        {
            DEBUG ("Inside parse path substring return val=%s\n", 
                    substring (s, 1, 0));
            substring (s, 1, 0);
        }
    } /* else */
} /* function parse_path_element */

/* This function will provide a lower bound for the number of pops needed.
 * It will be right if all parts are listed as UID's, but this will not be
 * true for named components or nets.
 */
function NumberOfPopsEstimate(path)
{
    declare sheet;
    n := -1;
    while (path != nil) 
    {
        parse_path_element (car (path), &sheet);
        if ((sheet != nil) && !string_eq (sheet, "1"))
            ++n;
        path = cdr (path);
        if (path != nil)
            ++n;
    } /* while */
    n;
} /* function NumberOfPopsEstimate */

/* pathA and pathB should contain one non-nil element,
 * the last item in the paths.
 */
function AreWeOnTheRightSheet(PathA, PathB)
{
    DEBUG ("--------\n Inside AreWeOnTheRightSheet\n---------\n");
    CompA := car (PathA);
    CompB := car (PathB);
    DEBUG ("CompA = %s\n CompB = %s\n", CompA, CompB);
   
    if (string_eq (CompA, CompB))
        TRUE;
    else
    {
        declare SheetA, SheetB;

        parse_path_element (CompA, &SheetA);
        parse_path_element (CompB, &SheetB);
        DEBUG ("SheetA = %s\n SheetB = %s\n", SheetA, SheetB);
        DEBUG ("Answer= %s\n", (SheetA != nil) && (SheetB != nil) && string_eq (SheetA, SheetB));
        (SheetA != nil) && (SheetB != nil) && string_eq (SheetA, SheetB);
    } /* else */
} /* function AreWeOnTheRightSheet */

/*
 * Figure out how to efficiently get viewdraw from it's current
 * position in the hierarchy to where we want to be, given the ability
 * to pop any number of levels at once, and push one schematic or
 * sheet at a time.  Both the desired_path and current_path are lists
 * of strings, starting at the roots of the path.
 *
 * Iterative version uses "pop n" optimization to avoid lots of expensive
 * individual pops.
 */
function vdraw_select_path(display, event, design, desired_path, current_path)
{
    declare comp, csheet;
    junk_sheet := 0;
    PopTo := nil;
    
    DEBUG ("-------\nInside vdraw_select_path\n--------\ndesired_path = %s\n",
            desired_path);
    DEBUG ("current_path = %s\n", current_path);

    /* The leading elements of the paths are added to the "PopTo" path,
     * which indicates the location after all pops have been done.
     *
     * This does not include the last element of the either path, which is
     * the selected or to-be-selected item (not pushed-into).
     *
     * Note that desired_path and current_path are also changed in this
     * loop to their respective paths wrt PopTo (rather than the top).
     * Also, they DO include the last element, the item selected or to be
     * selected.
     */
    while (   (cdr (desired_path) != nil)
           && (cdr (current_path) != nil)
           && string_eq (car (desired_path), car (current_path))) 
    {
        PopTo = cons (car (desired_path), PopTo);
        desired_path = cdr(desired_path);
        current_path = cdr(current_path);
    } /* while */

    /* PopTo built in reverse order, so fix that */
    PopTo = reverse_destructive (PopTo);

    DEBUG ("PopTo = %s\n", PopTo);
    DEBUG ("desired_path = %s\n", desired_path);
    DEBUG ("current_path = %s\n", current_path);

    /* Now do some popping if necessary.
     *
     * If we are on the same page, the cdr's of current & desired paths
     * will be nil, and the sheets of the car's will be the same.  In
     * this case we won't pop at all.  Otherwise we do the following.
     *
     * First we'll "pop n" based on the best estimate we can get, which
     * is a lower bound defined by the number of elements in the path
     * plus one for every component in the path on a sheet other than sheet
     * one (see definition of this function above). This will be exactly
     * correct most of the time -- exceptions include when any comp in path
     * is labeled and not on sheet one.
     */
    if (current_path != nil) 
    {
        declare NPops;
        /* Check if we just have the tail items of both paths and if
         * we definitely know we're at the right sch & sheet already.
         * In that case we won't pop anything.
         */

        DEBUG ("cdr(current_path) = %s\n cdr(desired_path)= %s\n", 
                           cdr(current_path), cdr(desired_path));
        DEBUG ("current_path = %s\ndesired_path= %s\n",
                current_path, desired_path);
        
        if (   cdr(current_path) == nil
            && desired_path != nil
            && cdr(desired_path) == nil
            && AreWeOnTheRightSheet(current_path, desired_path))
        {
            NPops = 0;
            DEBUG ("number of pops set to 0.\n");
        }  
        else 
        {
            /* Here we have different paths or sheets, so we do
             * the normal estimate.
             */
            NPops = NumberOfPopsEstimate (current_path); 
            DEBUG ("number of pops = %s\n", NPops);

            if (NPops > 0)
            {
                pop_if_unmodified (display, event, NPops);

               /* Here we check where we actually are now, to see that we did in
                * fact get to the PopTo location.  If not, we employ the
                * sure-to-work method of popping to sheet 1 of each schematic,
                * then popping the schematic level.
                */
                WhereWeAre := vdraw_current_path (display, event, design);
                DEBUG ("After pop <n>, WhereWeAre = %s\n", WhereWeAre);
            }
            else
               WhereWeAre := vdraw_current_path(display,event, design);

            /* remove equal leading components of PopTo and WhereWeAre. */
            while (   PopTo != nil
                   && cdr(WhereWeAre) != nil
                   && string_eq (car (PopTo), car (WhereWeAre))) 
            {
                DEBUG ("Yo\n");
                PopTo = cdr(PopTo);
                WhereWeAre = cdr(WhereWeAre);
                DEBUG ("PopTo Modified to %s WhereWeAre Modified to %s\n", PopTo, WhereWeAre);
            } 

            DEBUG ("WhereWeAre = %s\n", WhereWeAre);
            DEBUG ("current_path Count = %s; PopTo Count = %s, desired_path count = %s\n",
                   length(current_path), length (PopTo), length(desired_path));
            
            if (WhereWeAre != nil) 
            {
                /* number of path comps to deal with */
                npath := length (WhereWeAre)  - 1;
                DEBUG ("npath = %d\n", npath);
                /* We'll loop one extra time so the last item
                 * is on the right page.
                 */
                n := 0;
                DEBUG ("Wsheet = %s\n", vscript.getenv("WSHEET"));
                temp_path := desired_path;
                DEBUG ("car desired_path = %s\n", car(temp_path));
                comp = parse_path_element(car(temp_path), &csheet);
                DEBUG ("comp = %s, csheet = %s\n",comp, csheet);
                SheetStr := vscript.getenv("WSHEET");
                PrevSheetStr := vscript.getenv("WSHEET");
                
                while(1)
                {
                       /* pop until we're on sheet 1 */
                       SheetStr = vscript.getenv("WSHEET");
                       DEBUG ("n = %d, ", n);
                       DEBUG ("SheetStr = %s\n", SheetStr);

                       while (!string_eq(SheetStr, "1"))
                       {
                          DEBUG ("********Forcing pop to one!! ***********\n");
                          pop_if_unmodified (display, event, 1);
                          PrevSheetStr = SheetStr;
                          SheetStr = vscript.getenv("WSHEET");
                          DEBUG ("SheetStr = %s", SheetStr);
                          DEBUG (", PrevSheetStr = %s\n", PrevSheetStr);
                          if (string_eq (SheetStr, PrevSheetStr)){
                             /* 1st sheet on level doesn't necessarily have to
                              * be sheet 1.  Therefore, if we see the same
                              * sheet twice here, we are on the 1st sheet at
                              * the top level TGF 2/12/93 */
                              DEBUG ("Must be first sheet at top level\n");
                              break;
                          } /* if */
                       } /* while */
                       DEBUG ("\n");
                       /* pop schematic level */
                     
                       DEBUG ("Before popifmod sheet = %s\n", vscript.getenv("WSHEET"));
                       if (n < npath)
                          pop_if_unmodified (display, event, 1);
                       else
                          break;
                       DEBUG ("After pop sheet = %s\n", vscript.getenv("WSHEET"));
                       ++n;
                    } /* while(1) */

             } /* if */
        } /* if */
    } /* if */
 
    
    /* Now we just push down to get to the bottom of the desired path.
     * Note that "comp" is a component on most levels, but could also be
     * a net for the last part of the destination path.  This is convenient
     * since the viewdraw "sfastname" command handles either case.
     */
    DEBUG ("Current sheet = %s\n", vscript.getenv("WSHEET"));
    DEBUG ("Desired path = %s\n", desired_path);
    
    while ((desired_path != nil))
    {
        comp = parse_path_element (car (desired_path), &csheet);
        desired_path = cdr (desired_path);
        DEBUG ("Wsheet = %s\n", vscript.getenv("WSHEET"));
        DEBUG ("comp = %s, rest = %s, parsed_sheet =%s\n",
                comp, desired_path, csheet);

       if (csheet != nil) 
       {  
            DEBUG ("Calling push to sheet \n");
            push_to_sheet (display, event, csheet);
            DEBUG ("LOOKING FOR COMP OR NET *******************************\n");
            dot := lindex(comp, '.');
            /* 
            ** Now check for presence of a dot in the comp name
            ** If there is one, it means that we have a pin
            ** and need to find the component first
            */
            if (dot)
            {
              if (spinname_works)
                 spinname (display,event, comp);
              else 
                 do_sname (display, event, substring(comp, 0,dot));
            }
            else             
              do_sname (display, event, comp);
        } /* if */
        else 
        {                  /* if comp is a label */
            DEBUG ("pushing til we get something\n");
            BlockName := vscript.getenv("WBLOCK");

            for (n := string2number(vscript.getenv("WSHEET")); n < 1000; ++n) 
            {
                DEBUG ("looking on sheet ...%d\n",n);
                DEBUG ("LOOKING FOR LABEL *******************************\n");

            dot := lindex(comp, '.');
            /* 
            ** Now check for presence of a dot in the comp name
            ** If there is one, it means that we have a pin
            ** and need to find the component first
            */
            if (dot)
            {
              if (spinname_works)
                 spinname (display,event, comp);
              else 
                 do_sname (display, event, substring(comp, 0,dot));
            }
            else             
              do_sname (display, event, comp);

                DEBUG ("nosel(display=%s\n", nosel(display));
                /*
                ** nosel returns true if nothing selected false if we have
                ** something.
                */
                if (nosel (display) == 0)
                   break;

                DEBUG ("on sheet %s, Does next sheet %s exist? %s\n",
                       n,
                       BlockName + "." + convert(vlStringVT, n+1),
                       vqfexists(display, vqDataType.Schematic,
                                 BlockName + "." + convert(vlStringVT, n+1)));

                /* 
                ** if the next sheet doesn't exist, don't push any more! 
                */
                if (0 == vqfexists(display, vqDataType.Schematic,
                                   BlockName + "." + convert(vlStringVT, n+1)))
                    break;

                DEBUG ("pushing time number: %d\n", n);
                push_sheet (display, event, "");
            } /* for */
        } /* else */

        if (desired_path != nil)
            pushblock (display, "Schematic");
    } /* while */
} /* function vdraw_select_path */

function select_path(d, e, design, path_str)
{
    DEBUG("****------ select_path -------\npath_str=%s\n", path_str);
    path := reverse_destructive (String2ReversePath (string_upper (path_str)));
    DEBUG("****path after reverse = %s\n", path);
    current_path := vdraw_current_path (d, e, design);
    DEBUG("****current_path returned = %s\n", current_path);
    vdraw_select_path (d, e, design, path, current_path);
} /* function select_path */

function user_select_path(DrawData, Event, DesignName, PathStr) 
{
   /*
   ** This function has been upgraded to handle extra commands that 
   ** have been added to the end of the PathStr after colons.
   ** PathStr could be GND:ZSEL for example.
   ** This was done to close a window that permitted the wrong item
   ** to be zoom selected during long searches.
   */
   declare postcmds, postcmd;

   endcmd := lindex(PathStr, ':'); 
   rbracket := lindex(PathStr, ']');
   lbracket := lindex(PathStr, '[');
   /*
   ** If we found evidence of buss notation check to see if the :
   ** is inside the brackets and search for the next one as the
   ** command terminator.
   */ 
   if (endcmd)
   {
       if (rbracket && endcmd && lbracket)
          if ((lbracket < endcmd) && (endcmd < rbracket))
             endcmd = endcmd + 1 + lindex(substring(PathStr, endcmd+1, 0), ':');
       
       if (endcmd)
       {
          path := substring(PathStr, 0, endcmd);
          postcmds = substring(PathStr, endcmd+1, 0);
          endcmd = lindex(postcmds, ':');
       }
       else
       {
          postcmds = "";
          path := PathStr;
       }
   }
   else
   {
       postcmds = "";
       path := PathStr;
   }
   
   /*
   ** Perform the select path here
   */ 
   with_design (string_lower (DesignName), /*vscript.getenv("WSHEET"),*/ "1",
                 func(vdraw_cmd, window, d, e) {
                     select_path (DrawData, Event, DesignName, path);
                 });

   /*
   ** Now do all the rest --- If postcmds has some length than it is assumed
   ** there is at least one command there.  We strip each command that
   ** preceeds a ':' and shorten postcmds as we go. When there are no more 
   ** ;'s we just execute postcmds itself.
   */
   if (length(postcmds) > 0)
       while (1)
       {
          if (!endcmd)
          {
              with_design (DesignName, vscript.getenv("WSHEET"),
                                        func(f, w, d, e) f(postcmds);
                                       );
              break;
          }
          else
          {
              postcmd  = substring(postcmds, 0, endcmd);
              postcmds = substring(postcmds, endcmd+1, 0);
  
              with_design (DesignName, vscript.getenv("WSHEET"),
                                        func(f, w, d, e) f(postcmd);
                                       );
              endcmd = lindex(postcmds, ':');     
              rbracket = lindex(postcmds, ']');
              lbracket = lindex(postcmds, '[');

              if (rbracket && lbracket && endcmd)
                  if ((endcmd > lbracket) && (endcmd < rbracket))
                     endcmd = endcmd + 1 + lindex(substring(postcmds,endcmd+1,0));     
         }
      };
} /* function user_select_path */

SelectPathCmd := vcWv.Cmd(fac, user_select_path, "SelectPath", "SelectPath",
                          vcWv.Param("Design", vlStringVT, vcWv.Sticky, ""),
                          vcWv.Param("Path String", vlStringVT, vcWv.Sticky, ""));
vcCmd.EnableMacro(SelectPathCmd, vlOnOff.On);
