/*
 * $RCSfile: vdr_vdb.vs,v $ $Revision: 1.30 $
 *
 * Functional Description:
 *     This file should be loaded by ViewDraw.  It provides the
 *     commands and functions required by ViewDatabook II.
 *
 * Copyright (c) 1994 by Viewlogic Systems, Inc.
 */

load ("vdbde.vs");
load ("vapi.vs");
load ("vdbAnno.vs");

vdb_prefs := vlPref.Create("vdr_vdb");
vlPref.Replace (vdb_prefs, "debug", vlYesNoVT,
                "Print messages to enable ViewDatabook debugging",
                vlYesNo.No);
/* must call vlPref.Parse for any settings in powerview.ini to take effect */
vlPref.Parse ("powerview", wv);
if (vlPref.Val (vdb_prefs, "debug"))
{
    set_stdout (TRUE);
    DEBUG := printf;
}
else
    DEBUG := func(args...);;
function vdbDebug()
    vlPref.Val (vdb_prefs, "debug") == vlYesNo.Yes;

/*
 * list of attr/value/vis/oat elements, that gets passed between vdb's
 * add comp w/ attrs command, and vdraw's CallOnCreate action function.
 */
declare vdb_CompAttrList;
/*
 * TRUE if we're executing the "comp" command from vdb's
 * AddCompWithAttrs command.
 */
vdb_WeBeAddingComp := FALSE;
/*
 * Special context with specific name field, used only by vdraw/vdb.
 */
vdraw_cntx := vitc.SrcCntxCreate (vitc.TidVDRAW, sVdrawNameField);
/*
 * the VDB's to which we'll send msgs, and from which we'll receive
 * msgs.
 */
vdbcntx := GenericCntxCreate (tid <- vitcTOOLID_VDB);

vdbUseFileBasedITC := FALSE;


declare vdbCurrentWindow;
vdbCommandPending := FALSE;

/* throw exception */
function vdbThrow(num, args...)
    error2 ("vdr_vdb", vlErrSev.Error, num, nil, apply (sprintf, args));

/* report error, but don't throw exception */
function vdbAlert2(args...)
{
    <sLocation, eSeverity> := args;
    /* HACK_ALERT: location hard-coded */
    sLocation = "demo " + sLocation;
    args = cddr (args);
    vcErr.Report (vdraw.getcurrentwindow (), nil,
                  vlErr.Create (sLocation, "vdb_vdraw", 0, eSeverity,
                                apply (sprintf, args)));
}


/* 
 * The vdbPidList variable stores a list of the process IDs of the
 * current ViewDatabooks running.  (Although, there should only be 
 * one ViewDatabook running at a time, using a list handles the
 * degenerate case.)
 * 
 * When the list is empty, we know that we need to invoke ViewDatabook
 * before pinging.  If the list is non-empty, we know a vdb is already
 * running.
 */
declare vdbPidList;

function vdbItcRecordPid(src_cntx)
{
    pid := vipc.CntxGetPid (src_cntx);
    DEBUG("vdb2 started (pid %d)\n", pid);
    if (memv (pid, vdbPidList) == FALSE)
        vdbPidList = cons (pid, vdbPidList);

}

function vdbItcForgetPid(src_cntx)
{
    pid := vipc.CntxGetPid (src_cntx);
    DEBUG("vdb2 stopped (pid %d)\n", pid);
    vdbPidList =
        vscript.delete_list_element (vdbPidList, pid);
}

/* --------------------------------------------------------------
 * vdbIsVdbRunning
 *   ViewDatabook is running if the process ID list is not empty.
 */
function vdbIsVdbRunning()
    ((vdbUseFileBasedITC == TRUE) || (length(vdbPidList) > 0));


/*
 * Structure describing VDB configuration, as it relates to this modules.
 * This is a subset of the fields in the vdbCfg struct as kept by VDB
 * itself.
 */
vdbCfg := struct
{
    lExcludeSymsWhenLoading;    /* list of symbols excluded when selecting */
    lExcludeAttrsWhenLoading;   /* list of attrs excluded when loading */
    bMaintainAttrVis;           /* maintain attr vis from placeholder? */
    bSymAttrsGetLoaded;         /* look for symbol attributes when loading? */
    bOatsEnabled;               /* make sure oats mode is turned on? */
    bOatsGetLoaded              /* look for oats when loading? */
};

function vdbItcHandler(mtype, src_cntx, mtch_cntx, data)
{
    DEBUG ("vdr_vdb: Received ITC msg %s: \"%s\"\n",mtype,data);

/*    if (! string_eq (vitc.CntxGetTid (src_cntx), vitcTOOLID_VDB))
        vdbThrow ("Hey, this ITC message is not from vdb -- `%s'", data); */

    if (string_eq (mtype, sMsgADD_COMP))
    {
        win := vdraw.getcurrentwindow();
        rgn := vcWin.Region(win);
        event := vdbDummyEvent (win, rgn);

        vcWin.RunCmdline(win,data,event);
    }
    else if (string_eq (mtype, sMsgTRACK_COMPS))
        /*
         * `data' is the string rep of a list of component labels.
         */
        vdbTrackComps (eval_string (data, vdraw));
    else if (string_eq (mtype, sMsgBACK_ANNO))
    {
        vdbBackAnnotate (data);
    }
    else if (string_eq (mtype, sMsgREQ_SEL_COMP))
    {
        /*
         * For some reason, here vdraw.getcurrentwindow() sometimes returns
         * a window that isn't mapped (the window of a symbol block that's
         * selected).  Verify that we have a schematic window here!
         */
        win := vdraw.getcurrentwindow();
        display := vdraw.wingetdisplay (win);
        block := vdraw.DisplayGetBlock (display);
        if (vdraw.BlockGetDataType(block) == vqDataType.Schematic)
        {
            vdbCurrentWindow = win;
            /*DEBUG("*** In ITC handler, setting vdbCurrentWindow=%s\n",vdbCurrentWindow);*/
        }
        else if (null (vdbCurrentWindow))
            vdbThrow (1025, "Can't load values unless viewing a schematic sheet");
        vdbCommandPending = TRUE;
        vdb_issuePendingCmds();
    }
    else if (string_eq (mtype, sMsgREADY))
    {
        /*
         * `data' consists of ViewScript statements to initialize the user
         * configuration struct `vdbCfg'.
         */
        eval_string (data, vdraw);
        vdb_issuePendingCmds();  /* if any */
    }
    else if (string_eq (mtype, sMsgCMD))
        vcWin.RunCmdline (vdbCurrentWindow, data,
                          makeEvent (vdbCurrentWindow));
    else if (string_eq (mtype, sMsgPING))
        vdbSendToVdb(sMsgTOOLSTARTED, "");
    else if (string_eq (mtype, sMsgTOOL_STARTED))
    {
        /*
         * ViewDatabook is running.  Record this fact (so that we don't try
         * to start another one later).  Send a sMsgGET_READY message to
         * synchronize communication.  ViewDatabook will respond with
         * sMsgREADY.
         */
        vdbItcRecordPid (src_cntx);
        vdbSendToVdb (sMsgGET_READY, "");
        if (!null (vdbCurrentWindow))
        {
            vcWin.Busy (vdbCurrentWindow, FALSE);
            vdbPrintStatus (vdbCurrentWindow, nil); /* clear starting msg */
        }
    }
    else if (string_eq (mtype, sMsgTOOL_STOPPED))
    {
        vdbItcForgetPid (src_cntx);
        vdbCurrentWindow = nil;
    }
    else
        vdbThrow (1026, "Received unknown message type from vdb -- `%s'", mtype);

} /* vdbItcHandler */


/* HACK_ALERT: storing all the registration objs like this makes it */
/* easier to debug the vdraw stuff...then we can unregister the itc */
/* handlers and reload this file, and reregister the new ones */
if (isbound (`vdbHandlers, current_package ()))
    foreach (handler; vscript.list_iter (vdbHandlers))
        vitc.Unregister (handler);
vdbHandlers := nil;


function vdbRegister(mtype)
{
    /*DEBUG("Registering to receive mtype: \"%s\"\nsrc_cntx:   %s\nmatch_cntx: %s\n", mtype, vdraw_cntx, vdbcntx);*/
    handler := vitc.Register (mtype, vdraw_cntx, vdbcntx, vdbItcHandler);
    vdbHandlers = cons (handler, vdbHandlers);
}

vdbRegister (sMsgADD_COMP);
vdbRegister (sMsgTRACK_COMPS);
vdbRegister (sMsgBACK_ANNO);
vdbRegister (sMsgREQ_SEL_COMP);
vdbRegister (sMsgREADY);
vdbRegister (sMsgCMD);
vdbRegister (sMsgPING);
vdbRegister (sMsgTOOL_STARTED);
vdbRegister (sMsgTOOL_STOPPED);

function vdb_HandleCreate(block, obj)
{
    if (vdraw.ObjGetType(block, obj) == vqType.Component
        && vdb_WeBeAddingComp)
    {
        /* Save current selected list */
        selected_objects := BlockSaveSelected(block);

        FinderSetup ();
        new_attrs := FinderCompAddAttrs(nil, nil, block, obj, vdb_CompAttrList);
        FinderCleanup ();
        /*DEBUG("new_attrs=%s\n", new_attrs);*/

        /* restore the selected list */
        vdraw.BlockRestoreSelected(block, selected_objects);

        /*
         * It is necessary to add all newly created attributes
         * to the selected list before exiting this function. If you do 
         * this, then viewdraw will align them nicely for you starting 
         * at origin of the symbol block otherwise...
         */
        foreach(attr; vscript.list_iter(new_attrs))
            BlockSelectObj(block, attr);

        vdb_WeBeAddingComp = FALSE;
    }

} /* vdb_HandleCreate */


/*\BeginDebugVS
if (isbound (`vdb_AddCompWithAttrs, vdraw))
    printf ("Warning: not reregistering CallOnCreate handler\n");
else
\EndDebugVS*/
vdraw.CallOnCreate (vqCreateMode.Before, vdb_HandleCreate);


function vdbTrackComps(lComps)
{
    /*DEBUG("*** vdbTrackComps: vdbCurrentWindow = %s\n", vdbCurrentWindow);*/
    display := vdraw.wingetdisplay (vdbCurrentWindow);
    block := vdraw.DisplayGetBlock (display);
    symUidToBool := vlSym.StrCreate (13); /* special for friday, 13 jan 1995 */
    foreach (sUid; vscript.list_iter (lComps))
        vlSym.Store (symUidToBool, sUid, TRUE);
    vdraw.BlockDeselectAll (block);
    foreach (comp; vdraw.BlockIterObj (block, vqFlag.Component,
                                       vqAllOrSel.All))
        if (!null (vlSym.Fetch (symUidToBool, vdraw.ObjGetUID (block, comp))))
            vdraw.BlockSelectObj (block, comp);
    vdraw.DisplayRefresh (display);

} /* vdbTrackComps */


function vdb_AddCompWithAttrs(d,e,symname,attrlist_str)
{
    if(! (vapi.LibDataObjExists(string_trim(symname),"Symbol")))
        vdbThrow(1027, "Symbol %s cannot be located in ViewDraw search order",
                 symname);

    vdb_CompAttrList = convert(consVT,attrlist_str);
    vdb_CompAttrList = vscript.Sort(vdb_CompAttrList,
                                     func(a,b) string_le(car(a),car(b)););
    vdb_WeBeAddingComp = TRUE;
    vcCmd.Run(comp_cmd,d,e,list(symname),FALSE);
}


function vdbSendToVdb(sMsgType, sData)
{
    /*DEBUG ("vdraw: Sending msg %s: \"%s\"\n", sMsgType, sData);*/
    if (vitc.Send (sMsgType, vdraw_cntx, vdbcntx,
                   sData, vipc.FlgMULTI) != vitc.Success)
    {
        e := vlErr.Create (nil, "vdr_vdb", 1028, vlErrSev.Error,
          sprintf("Error sending message to ViewDatabook.  Message was \"%s\"",
                  sMsgType));
        e = vlErr.Append (e, vitc.ErrGet ());
        vlEx.Throw (vlEx.FAILURE, e);
    }
}


/*
 * Send a Ping message to ViewDatabook (to see if any were
 * already running before Viewdraw started.)  If one is already
 * running it will respond with a TOOL_START message.
 */
vdbSendToVdb (sMsgPING, "");

function vdbDummyEvent(win, rgn)
    vcEvent.Create (0, 0, '0', nil, win, rgn, nil);

function makeEvent(window)
    vcEvent.Create(0, 0, '0', nil, window, vcWin.OwnerRegion(window), nil);

function componentsAreSelected(window)
    vdraw.numselobj(vcWin.Region(window), vqObject.Component) > 0;
   
/* --------------------------------------------------------------
 * vdb_confirmProcessAllComponents
 *   This routine gets called when no components are selected. 
 *   It asks the user if all components should be processed.
 */
function vdb_confirmProcessAllComponents(window)
{
    display := vdraw.wingetdisplay(window);

    msg := sprintf("No components on sheet %s.%d are selected.  Do you\n" +
                   "want to process all the components on this sheet?",
                   string_upper(vdraw.getblkname(display)), 
                   vdraw.getblksheet(display));

    function acceptFunc(data, event) {
        /* Close the alert box window */
        vdbCloseAlertBox(data, event);

        /* Do it */
        vdb_doVDB(window, TRUE);
    }

    function cancelFunc(data, event) {
        /* Close the alert box window */
        vdbCloseAlertBox(data, event);
    }

    buttons := vdbMakeButtonRow(list(cons("Accept", acceptFunc),
                                     cons("Cancel", cancelFunc)));

    vdbAlert(nil, window, msg, vlErrSev.Status, buttons);
} /* vdb_confirmProcessAllComponents */

/* --------------------------------------------------------------
 * vdb_startViewDatabook
 *   If not already running, invoke ViewDatabook using a system
 *   command.
 */
function vdb_startViewDatabook(window)
{
    if (! vdbUseFileBasedITC && ! vdbIsVdbRunning()) {
        /* Reset the output stream */
        stream := nil;

        /* Start up can take a while */
        vdbPrintStatus(window, "Starting ViewDatabook II...please wait");
        vcWin.Busy(window, TRUE);

        /* Normal startup message go to terminal window */
        function reportStdout(msg) printf("%s", msg);

        declare error_string;

        /* Save error messages, but don't put them in an alert box */
        /* unless vdb exits abnormally */
        function recordStderr(msg)
        {
            if (null (error_string))
                error_string = ":\n";
            error_string += msg;
            printf("%s", msg);
        }

        function handleExit(mode, code)
        {
            /* unbusy the vdraw window & clear status line */
            vcWin.Busy (window, FALSE);
            vdbPrintStatus (window, nil);
            if ((mode != vcSysMode.Exited) || (code != 0))
                vcWv.Delay (wv, func() {
                    vlEx.Throw2 (vlEx.FAILURE, "vdraw",
                                 vlErrSev.Error,
                                 sprintf ("vdb2 %s %d%s",
                                          convert (vlStringVT, mode), code,
                                          (null (error_string)
                                           ? ""
                                           : error_string)));
                });
        }

        /* Invoke the interactive ViewDatabook process */
        vcSys.Command("vdb2",           /* ViewDatabook executable */
                      nil,              /* Use existing environment */
                      reportStdout,     /* output handler */
                      recordStderr,     /* error handler */
                      handleExit,       /* exit handler */
                      nil);             /* host */
    }
} /* vdb_startViewDatabook */

/* --------------------------------------------------------------
 * vdb_doVDB
 *   Make sure components are selected and start up ViewDatabook
 *   if its not already running.  
 */
function vdb_doVDB(window, select_all)
{
    if (select_all) {
        vcWin.RunCmdline(window, "_scomp * Select", makeEvent(window));
        if (! componentsAreSelected(window)) {
            vdbAlert(nil, window, "This sheet does not have any components.", 
                     vlErrSev.Error, nil);
            return (nil);
        }
    }

    vdb_startViewDatabook(window);
    vdbCurrentWindow = window;
    vdbCommandPending = componentsAreSelected(window);

    if (vdbUseFileBasedITC)
    {
        vdbPrintStatus(window, "Sending Load request to ViewDatabook.");

        /* Switch the ITC polling routine to active mode */
        vdbSwitchTimer(TRUE);
    }

    /* Tell ViewDatabook that we have a load request. */
    vdbSendToVdb(sMsgGET_READY, "");

} /* vdb_doVDB */

/* --------------------------------------------------------------
 * vdb_runViewDatabook
 *   Before re-invoking/re-initializing ViewDatabook, make sure 
 *   that we haven't already sent values to it which haven't been 
 *   backannotated yet.
 */
function vdb_runViewDatabook(data, event)
{
    function do_it(window)
        vdb_doVDB(window, FALSE);

    window := vcEvent.Window (event);

    if ((vdbUseFileBasedITC == FALSE) && 
        vdbIsVdbRunning() && (vdbCurrentWindow != nil))
    {

        msg := "ViewDatabook is already running, and has values which\n"
            +  "have not been written back to the schematic.  Do you\n"
            +  "want to discard these values and re-initialize ViewDatabook?";

        function acceptFunc(data, event) {
            /* Close the alert box window */
            vdbCloseAlertBox(data, event);

            /* Do it */
            do_it(window);
        }

        function cancelFunc(data, event) {
            /* Close the alert box window */
            vdbCloseAlertBox(data, event);
        }

        buttons := vdbMakeButtonRow(list(cons("Accept", acceptFunc),
                                         cons("Cancel", cancelFunc)));

        vdbAlert(nil, window, msg, vlErrSev.Warning, buttons);
    }
    else
        do_it(window);
} /* vdb_runViewDatabook */

/*
 * DESCRIPTION
 *      Given symbol, return name with sheet number, e.g. "resistor.1".
 */
function vdbSymGetNameWithSheet(symbol)
    /*
     * HACK_ALERT: can only handle ".1" symbols here, because I 
     * tickle a fatal error in 5.2 viewdraw API with this code
     * sSymbol := vdraw.ObjGetName (block, symbol, FALSE)
     *     + "." + vdraw.BlockGetSheet (symbol);
     *
     * This *used* to tickle a fatal 5.2 viewdraw bug....let's see....
     *
     * Don't want the sheet name any more.  This causes an unnecessary
     * distinction between sym.1 and sym.2 that VCS/VDB didn't have.
     */
    vdraw.ObjGetName (symbol, symbol, FALSE);
    /*+ "." + number2string (vdraw.BlockGetSheet (symbol));*/ /*".1"*/

/*
 * DESCRIPTION
 *      Used to filter out symbols like GND.1 when selecting all components
 *      for processing by VDB.
 */
/*function vdbSymIgnored(sSymbol)
    member (sSymbol, vdbCfg.lExcludeSymsWhenLoading);*/

/*
 * DESCRIPTION
 *      Used to filter out attributes like REFDES when sending data to VDB.
 */
/*function vdbAttrIgnored(sAttrName)
    member (sAttrName, vdbCfg.lExcludeAttrsWhenLoading);*/

/*
 * DESCRIPTION
 *      Write data about selected objects to a temporary file, and send
 *      the name of that file to vdb.  We write it to a file instead of
 *      packing it up in an ITC message because I anticipate the size of
 *      the file to sometimes exceed 4k, the current limit of ITC
 *      messages.  In addition, the act of constructing a large string in
 *      ViewScript is resource-consuming; either the string is allocated all
 *      at once and filled in character by character with string_set(), or
 *      the string is reallocated and copied as it is built with the
 *      overloaded `+' operator.
 */
function vdb_SendObjectData(win)
{
    rgn := vcWin.OwnerRegion (win);
    display := vcRegion.GetData (rgn);
    block := vdraw.DisplayGetBlock (display);

    /* l := BlockSaveSelected (block); // just want selected components */
    lComps := nil;
    foreach (comp; vdraw.BlockIterObj (block, vqFlag.Component,
                                       vqAllOrSel.Selected))
        lComps = cons (comp, lComps);
    if (null (lComps))
    {
        /* semantics of this copied from VDB1 */
        vdb_confirmProcessAllComponents (win);
        return (nil);
    }

    /* group selected components together based on attrs */
    lCompAttrs := vdbGetAttrs (block, lComps);
    lGroupList := vdbFormGroups (lCompAttrs);

    /* write groups list to file, send message to VDB */
    sFilename := vdbTempFilename ();
    f := fopen (sFilename, "w");
    fputline (f, convert (vlStringVT, lGroupList));
    fputline (f, ";\n");
    fclose (f);
    vdbSendToVdb (sMsgSEL_COMP, sFilename);

} /* vdb_SendObjectData */


/*
 * DESCRIPTION
 *      Given list of components, return big list o' attributes &
 *      values, from which groups will be formed.
 */
function vdbGetAttrs(block, lComps)
{
    /* as we process attrs, throw them in this hash table.  Do them in */
    /* order of increasing precedence, so that the last values win: sym */
    /* attrs then comp attrs then oats. */
    symAttrNameToList := vlSym.StrCreate (29);

    /*
     * DESCRIPTION
     *      If the attribute isn't ignored by name, and it has a
     *      non-null value, then add the following list
     *
     *          `(attr_name attr_value oat_flag)
     *
     *      to the hash table `symAttrNameToList'.
     */
    function MaybeStoreAttr(block, attr, bOatFlag)
    {
        vRetVal := nil;
        sName := vdraw.AttrGetName (block, attr);
        /* attr may be ignored by name */
        if (!member (sName, vdbCfg.lExcludeAttrsWhenLoading))
        {
            sValue := vdraw.AttrGetValue (block, attr);
            /* attr may be ignored because of empty value */
            if (!string_eq ("", sValue))
                vlSym.Store (symAttrNameToList, sName,
                             list (sName, sValue, bOatFlag));
        }

    }

    declare lSymAttrs, lCompAttrs;

/*\BeginDebugVS
    print_list("vdbGetAttrs: lComps", lComps);
\EndDebugVS*/

    foreach (comp; vscript.list_iter (lComps))
    {
        symbol := vdraw.CompGetBlock (block, comp);
        sSymbol := vdbSymGetNameWithSheet (symbol);

/*\BeginDebugVS
        printf("vdbGetAttrs: sSymbol = %s\n", sSymbol);
\EndDebugVS*/

        /* symbols may be ignored by name */
        if (! member (sSymbol, vdbCfg.lExcludeSymsWhenLoading))
        {
            sUid := vdraw.ObjGetUID (block, comp);

            vlSym.Clear (symAttrNameToList);
            vlSym.Store (symAttrNameToList, "SYMBOL",
                         list ("SYMBOL", sSymbol, FALSE));

            /* first symbol-level attrs, if used */
            if (vdbCfg.bSymAttrsGetLoaded)
                foreach (attr; vdraw.ObjIterAttr (symbol, symbol))
                    MaybeStoreAttr (symbol, attr, FALSE);

            /* second component-level attrs */
            foreach (attr; vdraw.ObjIterAttr (block, comp))
                MaybeStoreAttr (block, attr, FALSE);

            /* finally occurrence attrs, if used */
            if (vdbCfg.bOatsEnabled)
                foreach (attr; vdraw.ObjIterOat (block, comp))
                    MaybeStoreAttr (block, attr, TRUE);

            /* component may be excluded by use of boolean VDBEXC attr */
            lAttrVal := vlSym.Fetch (symAttrNameToList, "VDBEXC");
            if (null (lAttrVal)
                || !string_eqi (cadr (lAttrVal), "ON"))
            {
                /* now loop over attrs stored in hash table, and build */
                /* list of attrs for this comp */
                lAttrs := nil;
                foreach (sAttrName, l; vlSym.Loop (symAttrNameToList))
                    lAttrs = cons (l, lAttrs);

                /* add to bigger list of attrs for all comps */
                lCompAttrs = cons (list (sSymbol, sUid, lAttrs),
                                   lCompAttrs);
            }
        }
    }

    /* return value */
    lCompAttrs;

} /* vdbGetAttrs */


/*
 * INPUT
 *      lCompAttrs looks like:
 *
 *          `((symbol_name comp_label attr_list)
 *            (symbol_name comp_label attr_list)
 *            ...)
 *
 *      `attr_list' is a list of attributes:
 *
 *          (name value is_oat)
 *
 *      E.g.
 *          (("00.1" "$1I20" (("SYMBOL" "00.1" FALSE)
 *                            ("DEVICE" "00" FALSE)
 *                            ("LEVEL" "STD" FALSE)
 *                            ("CLASS" "IC" FALSE)))
 *           ("res_smt.1" "$1I3" (("SYMBOL" "res_smt.1" FALSE)
 *                                ("DEVICE" "RES_SMT" FALSE)
 *                                ("LEVEL" "STD" FALSE)
 *                                ("POWER" "5W" FALSE)
 *                                ("CLASS" "DISCRETE" FALSE)))
 *           ("res_smt.1" "$1I4" (("SYMBOL" "res_smt.1" FALSE)
 *                                ("DEVICE" "RES_SMT" FALSE)
 *                                ("LEVEL" "STD" FALSE)
 *                                ("POWER" "5W" FALSE)
 *                                ("CLASS" "DISCRETE" FALSE))))
 *
 * RETURNS
 *      vdbGroupListID of components, grouped according to symbol name and
 *      common attributes.
 *
 *      E.g.
 *          ((("$1I20") (("SYMBOL" "00.1" FALSE)
 *                       ("DEVICE" "00" FALSE)
 *                       ("LEVEL" "STD" FALSE)
 *                       ("CLASS" "IC" FALSE)))
 *           (("$1I3" "$1I4") (("SYMBOL" "res_smt.1" FALSE)
 *                             ("DEVICE" "RES_SMT" FALSE)
 *                             ("LEVEL" "STD" FALSE)
 *                             ("POWER" "5W" FALSE)
 *                             ("CLASS" "DISCRETE" FALSE))))
 *
 * GIVEN:
 *     `scmCompAttrList' contains `n' comps w/ avg. of `m' attrs each.
 *     `scmSymAttrList' contains `p' syms w/ avg. of `q' attrs each.
 *     c = constant time
 *
 * ALGORITHM:
 * HACK_ALERT: this description is out of date
 *
 * // on vdraw side
 * foreach selected comp                                        O(n)
 *     if (excluded (sym))                                      c
 *         break
 *     get sym from comp                                        c
 *     if fetch sym attrs from symbol->attrList hash == NULL    c
 *         get sym attrs                                        O(q)
 *         store in symbol->attrList hash                       c
 *     get comp attrs, write to file                            O(m)
 * foreach sym in symbol->attrList hash                         O(p)
 *     write to file                                            c
 *
 * --> O(n*(m+q)) + O(p)
 *
 * // in this function
 * foreach comp in scmCompAttrList                              O(n)
 *     if fetch groupList from symbol->groupList hash == NULL   c
 *         create new group (groupList of 1 elem)               c
 *         store in symbol->groupList hash                      c
 *     else
 *         foreach group in groupList                           c
 *             compare comp->attrList with  groupList->attrList O(m*m)
 *             if (equal)
 *                 add comp to group                            c
 *                 break
 *         create new group                                     c
 *         add to groupList                                     c
 * foreach sym in scmCompAttrList                               O(p)
 *     fetch groupList from symbol->groupList hash              c
 *     (assert not NULL)
 *     foreach group in groupList                               c
 *         foreach attr in scmSymAttrList                       O(q)
 *             if group does not have attr                      O(m)
 *                add attr to group                             c
 * make list from symbol->groupList hash and return it
 *
 * --> O(n*m*m) + O(q*m*m)
 *
 * ALGORITHM 2:
 *
 * // on vdraw side
 * foreach selected comp                                        O(n)
 *     get sym from comp                                        c
 *     if (ignored (sym))                                       c
 *         break
 *     if fetch sym attrs from symbol->attrList hash == NULL    c
 *         get sym attrs                                        O(q)
 *         sort sym attrs                                       O(q*log(q))
 *         store in symbol->attrList hash                       c
 *     get comp attrs                                           O(m)
 *     sort comp attrs                                          O(m*log(m))
 *     merge comp attrs w/ sym attrs into one sorted list       ...
 * 
 * // Hmm.  I was thinking that it would be beneficial to sort the comp
 * // + sym attrs before giving them to this function.  I now think this
 * // might not be so good.
 * // On 2nd thought, yeah, we should do it, to allow the use of equal() to
 * // compare attr lists.
 */
function vdbFormGroups(lCompAttrs)
{
    /* maps symbol name -> list of groups using that symbol */
    symSymbolToGroupList := vlSym.StrCreate (67);

    foreach (lThisComp; vscript.list_iter (lCompAttrs))
    {
        sSymbol := car (lThisComp);
        sUid := cadr (lThisComp);
        lAttrs := caddr (lThisComp);
        /* sort attribute lists so that we can compare them against each */
        /* other later by using equal() */
        lAttrs = vscript.Sort (lAttrs, func(a,b) string_le(car(a),car(b)););
        if (null (lGroupList := vlSym.Fetch (symSymbolToGroupList, sSymbol)))
            vlSym.Store (symSymbolToGroupList, sSymbol,
                         list (list (list (sUid), lAttrs)));
        else
        {
            bAddedToGroup := FALSE;
            foreach (lThisGroup; vscript.list_iter (lGroupList))
            {
                lTheseAttrs := cadr (lThisGroup);
                if (equal (lAttrs, lTheseAttrs))
                {
                    lTheseUids := car (lThisGroup);
                    lTheseUids = cons (sUid, lTheseUids);
                    /* we must insert the new Uid list into this group destructively */
                    set_car (lThisGroup, lTheseUids);
                    bAddedToGroup = TRUE;
                    break; /* break out of `foreach' */
                }
            }
            if (!bAddedToGroup)
            {
                /* Add new group to lGroupList; have to re-store in hash */
                /* table, because we've modified the list by cons'ing */
                /* onto it. */
                lGroupList = cons (list (list (sUid), lAttrs),
                                   lGroupList);
                vlSym.Store (symSymbolToGroupList, sSymbol, lGroupList);
            }
        }
    }

/*\BeginDebugVS
    DEBUG("FormGroups: intermediate group lists\n");
    foreach (sSymbol, lGroupList; vlSym.Loop (symSymbolToGroupList))
        print_list("list of groups using symbol "+sSymbol, lGroupList);
\EndDebugVS*/

    /* now collect list of all groups */
    lAllGroups := nil;
    foreach (sSymbol, lGroupList; vlSym.Loop (symSymbolToGroupList))
        foreach (lThisGroup; vscript.list_iter (lGroupList))
            lAllGroups = cons (lThisGroup, lAllGroups);

/*\BeginDebugVS
    print_list ("FormGroups: final group list", lAllGroups);
\EndDebugVS*/

    /* return value */
    lAllGroups;

} /* vdbFormGroups */


/* --------------------------------------------------------------
 * vdb_issuePendingCmds
 *   If a Load command is pending, issue it now.  (Commands have
 *   to be queued until ViewDatabook is finished initializing.)
 */
function vdb_issuePendingCmds()
{
    if (vdbCommandPending && (vdbCurrentWindow != nil))
    {
        vdbCommandPending = FALSE;
        vdb_SendObjectData (vdbCurrentWindow);
    }

} /* vdb_issuePendingCmds */

/*
 * Commands -- don't rename these commands!  Their existence has been
 * leaked...
 */
vdb_AddCompWithAttrs_cmd :=
    vcWv.Cmd(vdraw.fac, vdb_AddCompWithAttrs,
             sAddCompCmdName, "Add Component",
             vcWv.Param("_symbol", vlStringVT, vcWv.Constant, ""),
             vcWv.Param("_attrlist", vlStringRestOfLineVT, vcWv.Constant, ""));
vdb_cmd :=
    vcWv.Cmd(vdraw.fac, vdb_runViewDatabook, "viewdatabook");
vcCmd.SetHelp (vdb_cmd, "Start ViewDatabook II");

vcMenu.AppendEntry (util_menu, "ViewDatabook II", vdb_cmd);
