/* This file contains the Actel specific ViewFPGA information */


/*** Global declarations ***/


function fl_actfam () { list (list (
   cons ("Act-1", "act1"),
   cons ("Act-2", "act2"),
   cons ("Act-3", "act3"))); }

function fl_pads_actel () { list (
   list ("BBDLHS",   "PAD"), list ("BBHS",     "PAD"), list ("BIBUF",    "PAD"),
   list ("CLKBIBUF", "PAD"), list ("CLKBUF",   "PAD"), list ("IBDL",     "PAD"),
   list ("INBUF",    "PAD"), list ("OBDLHS",   "PAD"), list ("OBHS",     "PAD"),
   list ("ORTH",     "PAD"), list ("OUTBUF",   "PAD"), list ("TBDLHS",   "PAD"),
   list ("TBHS",     "PAD"), list ("TRIBUFF",  "PAD")); }

vcWv.MakeEnum ("vfpgaActStrengthVT", list ("None","Weak","Moderate","Strong"));
vcWv.MakeEnum ("vfpgaActGradeVT", list ("Comm", "Ind", "Mil"));
vcWv.MakeEnum ("vfpgaActProgFmtVT", list ("Activator", "DataIO"));


/*** Initialization functions ***/


function actGetALS (startup) {
   path := vscript.getenv ("ALSDIR");
   if (!null (path))
      if (!fexists (vscript.vsConvPath (path + "/data/system.def"))) path = nil;
   if (null (path)) {
      path = vscript.vsSrchdirs ("als", vscript.vsDIR);
      if ((!null (path)) &&
         (fexists (vscript.vsConvPath (path + "/data/system.def")))) {
            if (startup) vscript.putenv ("ALSDIR", path);
            cb_mainDbg (1, "setenv ALSDIR " + path); }
      else path = nil; }
   path; }


function actSetup () {
   declare typelist;

   /* Make sure all three Actel library aliases are present */
   setup_ok := TRUE;
   if (null (actGetALS (TRUE))) {
      cb_mainErr (1501, "Error", nil, "Unable to set up Actel environment");
      setup_ok = FALSE; }
   if (!cb_loadIfalias ("actelcells", TRUE))  setup_ok = FALSE;
   if (!cb_loadIfalias ("actelmodels", TRUE)) setup_ok = FALSE;
   if (!cb_loadIfalias ("actelsim", TRUE))    setup_ok = FALSE;
   if (setup_ok == TRUE)
      typelist = list ("ABEL", "VHDL", "JEDEC", "Actel", "LPM");
   else typelist = nil;

   menuValidTypes ("Actel", typelist,
      list (c_act_wir, c_act_val, c_act_routed, c_opt_actsyn,
         c_opt_actpkg, c_opt_actimp, c_act_pin));  }


function actDefallatts (prefdb, startup) {
   prefDefoneatt ("act_incplace", vfpgaActStrengthVT,
      "ACTEL: incremental placement strength", vfpgaActStrength.None,
      prefdb, startup);
   prefDefoneatt ("act_clkbal", vfpgaActStrengthVT,
      "ACTEL: clock balancing strength", vfpgaActStrength.None, prefdb,startup);
   prefDefoneatt ("act_timtemp", vfpgaActGradeVT,
      "ACTEL: timing model temperature range", vfpgaActGrade.Comm, prefdb,
      startup);
   prefDefoneatt ("act_timvolt", vfpgaActGradeVT,
      "ACTEL: timing model voltage range", vfpgaActGrade.Comm, prefdb, startup);
   prefDefoneatt ("act_progfmt", vfpgaActProgFmtVT,
      "ACTEL: programming file format", vfpgaActProgFmt.Activator, prefdb,
      startup);
   prefDefoneatt ("act_silsig", vlStringVT, "ACTEL: silicon signature", "",
      prefdb, startup);
   prefDefoneatt ("act_progfile", vlYesNoVT, "ACTEL: create programming file",
      vlYesNo.Yes, prefdb, startup);
   prefDefoneatt ("act_timmod", vlYesNoVT,
      "ACTEL: create timing model", vlYesNo.Yes, prefdb, startup); }


/*** File, tool and chain definitions ***/


cb_cfgAddFile ("actadl",     list (list ("%dot", "%base", "adl")));
cb_cfgAddFile ("actipf",     list (list ("%dot", "%base", "ipf")));
cb_cfgAddFile ("actcrt",     list (list ("%dot", "%base", "crt")));
cb_cfgAddFile ("actval",     list (list ("%dot", "%base", "val")));
cb_cfgAddFile ("actdef",     list (list ("%dot", "%base", "def")));
cb_cfgAddFile ("actplaced",  list (list ("%dot", "%base", "pli")));
cb_cfgAddFile ("actpin",     list (list ("%dot", "%base", "pin")));
cb_cfgAddFile ("actrouted",  list (list ("%dot", "%base", "rti")));
cb_cfgAddFile ("actfused",   list (list ("%dot", "%base", "fus")));
cb_cfgAddFile ("actfusdio",  list (list ("%dot", "%base", "dio")));
cb_cfgAddFile ("actexted",   list (list ("%dot", "%base", "del")));
cb_cfgAddFile ("actdtb",     list (list ("%dot", "%base", "dtb")));

cb_cfgAddTool ("actsetdie",  nil, list ("actdef"), list ("fpga_die"));
cb_cfgAddTool ("actsetpkg",  nil, list ("actdef"), list ("fpga_package"));
cb_cfgAddTool ("actsetsig",  nil, list ("actdef"), list ("act_silsig"));
cb_cfgAddTool ("actsetclk",  nil, list ("actdef"), list ("act_clkbal"));
cb_cfgAddTool ("wir2adl",    list ("wir"), list ("actadl"), nil);
cb_cfgAddTool ("actvalid",   list ("actadl", "actipf"), list ("actval"), nil);
/* Note: timestamp of "val" file is changed when place is finished, which
   makes the chainer think placement failed. "vld", which is the log file,
   does not get rewritten.  However, since incremental placement requires
   incremental validation, I have made placement depend on netlist rather
   than validation.*/
cb_cfgAddTool ("actplacer",  list ("actadl", "actcrt"),
   list ("actplaced", "actpin"), list ("act_clkbal"));
cb_cfgAddTool ("actrouter",  list ("actplaced"), list ("actrouted"), nil);
cb_cfgAddTool ("actextract", list ("actrouted"), list ("actexted"), nil);
cb_cfgAddTool ("actdel2vl",  list ("actexted"), list ("actdtb"),
   list ("act_timvolt", "act_timtemp", "fpga_speed"));
cb_cfgAddTool ("actfuser",   list ("actrouted"), list ("actfused"), nil);
cb_cfgAddTool ("actfus2dio", list ("actfused"), list ("actfusdio"),
   list ("act_progfmt"));

cb_cfgAddTgtname ("actvalid", "Actel validator report");
cb_cfgAddTgtname ("actroute", "Actel routed design");
cb_cfgAddTgtname ("actimod", "Actel timing model");

cb_cfgAddChain ("actvalid", "sch", list ("savesrc", "%fpganet",
   "wir2adl", "actsetdie", "actsetpkg", "actvalid"));
cb_cfgAddChain ("actroute", "sch", list ("savesrc", "%fpganet",
   "wir2adl", "actsetdie", "actsetpkg", "actvalid", "actsetclk",
   "actplacer", "actrouter", "actsetsig", "actfuser", "actfus2dio"));
/* Almost identical chain for timing model, which loads vsm file at the end */
cb_cfgAddChain ("actimod", "sch", list ("savesrc", "%fpganet",
   "wir2adl", "actsetdie", "actsetpkg", "actvalid", "actsetclk",
   "actplacer", "actrouter", "actsetsig", "actfuser", "actfus2dio",
   "actextract", "actdel2vl", "loadvsm"));


/*** Action functions and error routines ***/


function a_actsetdie (name) {
   "als -set die " + string_lower (cb_prefGet (name, "fpga_die")) +" "+name; }

function a_actsetpkg (name) {
   "als -set package " + string_lower (cb_prefGet (name, "fpga_package"))
      + " " + name; }

function a_actsetsig (name) {
   if (cb_prefGet (name, "act_progfile") == vlYesNo.Yes) {
      s := cb_prefGet (name, "act_silsig");
      if (!string_eq (s, ""))
         return ("als -set sig " + cb_prefGet (name, "act_silsig")+" "+name); }
   nil; }

function a_actsetclk (name) {
   v := cb_prefGet (name, "act_clkbal");
   if (!string_eq (cb_prefGet (name, "technology"), "act1")) nil;
   else {
      if      (v == vfpgaActStrength.None)     s := "0";
      else if (v == vfpgaActStrength.Weak)     s := "3";
      else if (v == vfpgaActStrength.Moderate) s := "6";
      else if (v == vfpgaActStrength.Strong)   s := "10";
      "als -set placeclockbalancestrength " + s; } }

function a_wir2adl (name) {
   str := "cae2adl cae:wv41 fam:" + cb_prefGet (name, "technology");
   str = str + " " + name; }

function e_wir2adl (name) { errWrap (1502, "Actel netlist translator", name); }

function a_actvalid (name) { "als -validate " + name; }

/* Provides error tracking by putting error file (.vld) into "visit" button */
function e_actvalid (name) {
   cb_mainErr (1503, "Error", "file " + name + ".vld",
      "Actel netlist validator failed"); }

function a_actplacer (name) { "als -place " + name; }

function e_actplacer (name) { errWrap (1504, "Actel placement", name); }

function a_actrouter (name) { "als -route " + name; }

function e_actrouter (name) { errWrap (1505, "Actel routing", name); }

function a_actextract (name) { "als -extract " + name; }

function e_actextract (name) { errWrap (1506, "Actel delay extracter", name); }

function a_actdel2vl (name) {
   declare str;
   function addrange (title, val) {
      if (val == vfpgaActGrade.Comm) str += title + "com";
      else if (val == vfpgaActGrade.Ind) str += title + "ind";
      else if (val == vfpgaActGrade.Mil) str += title + "mil"; }

   str = "del2vl";
   addrange (" tempr:", cb_prefGet (g_selel, "act_timtemp"));
   addrange (" voltr:", cb_prefGet (g_selel, "act_timvolt"));
   str + " speed_range:" + cb_prefGet (g_selel, "fpga_speed") + " " + name; }

function e_actdel2vl (name) {
   errWrap (1507, "Actel delay to Viewlogic translator", name); }

function a_actfuser (name) {
   if (cb_prefGet (name, "act_progfile") == vlYesNo.Yes)
      "als -fuse " + name;
   else nil; }

function e_actfuser (name) { errWrap (1508, "Actel fusemap writer", name); }

function a_actfus2dio (name) {
   if ((cb_prefGet (g_selel, "act_progfile") == vlYesNo.Yes) &&
      (cb_prefGet (g_selel, "act_progfmt") == vfpgaActProgFmt.DataIO))
         "als -fus2dio " + name;
   else nil; }

function e_actfus2dio (name) {
   errWrap (1509, "Actel Data I/O fusemap writer", name); }


/*** Pin editor ***/


function f_act_pin (d, e) {

   declare r_pins;

   function writeIpf () {
      declare rows, cols, name, pinno, fixed, line, ipfFile;

      function printIpfLine (s) fputline (ipfFile, sprintf ("%s.\n", s));

      ipfFile = fopen (sprintf ("%s.ipf", g_selel), "w");
      if (null (ipfFile)) cb_mainErr (1510, "Error", nil,
         "Unable to write pin file " + g_selel + ".ipf");
      else {

         printIpfLine (sprintf ("DEF %s", string_upper (g_selel)));
         vcTable.Dimensions (r_pins, &rows, &cols);
         for (i:=0; i<rows; ++i) {
            name  = vcRegion.Get (vcTable.Get (vcTable.Get (r_pins,i,0),0,0),
               vlStringVT);
            fixed = vcRegion.Get (vcTable.Get (vcTable.Get (r_pins,i,0),0,1),
               vscmVT);
            pinno = vcRegion.Get (vcTable.Get (vcTable.Get (r_pins,i,0),0,2),
               vlStringVT);
            if (!string_eq (pinno, "")) {
               line = sprintf ("NET %s; ; PIN:%s", name, pinno);
               if (fixed) line += ",FIX:''";
               printIpfLine (line); } }
         printIpfLine ("END");
         fclose (ipfFile);
         cb_mainErr (0, "Status", nil,
            "Pin location file " + g_selel + ".ipf written"); } }

   function setall (dest, val) {
      declare rows, cols;
      vcTable.Dimensions (r_pins, &rows, &cols);
      for (i:=0; i<rows; ++i)
         vcRegion.Set (vcTable.Get (vcTable.Get (r_pins, i, 0),0,dest), val); }

   /* Sample IPF lines: NET B; ; PIN:13,FIX:''.  NET C0; ; PIN:14.
      Lines in PIN file are similar but with newlines mixed in.  Read lines;
      once "NET" is found, continue to add to the line until a line ending
      with . is found; then process the line. */
   function readIpfNetLine (fp) {
      declare line2; found := FALSE; line := nil;
      while (!found) {
         if (null (line = fgetline (fp))) found = TRUE;
         else if (string_eq (substring (line, 0, 3), "NET")) {
            if (string_ref (line, string_length (line) - 2) == '.')
               found = TRUE;
            else while (!found) {
               if (null (line2 = fgetline (fp))) return (nil);
               line += line2;
               if (string_ref (line, string_length (line) - 2) == '.')
                  found = TRUE; } } }
      line; }

   function readIpf (h, isipf) {
      declare line, name, pinno, fixed, i;

      name := g_selel + (isipf ? ".ipf" : ".pin");
      if (!fexists (name)) return (FALSE);
      ipfFile := fopen (name, "r");
      while (!null (line = readIpfNetLine (ipfFile))) {
         fixed = FALSE; pinno = ""; name = ""; i = 0;
         foreach (token; string_loop (line, " ;,.:\n")) {
            if    (++i == 2) name  = token;
            else if ((i == 5) && string_eq (token, "FIX")) fixed = TRUE;
            else if (i == 4) pinno = token; }
         vlSym.Store (h, name, cons (pinno, fixed)); }
      fclose (ipfFile); }

   function makewindow (isipf) {
      r_title := vcTable.Make ("r_title", 1, 3,
         vcRegion.Create ("", vcLabel.fac, "Signal   "),
         vcRegion.Create ("", vcLabel.fac, "Fixed"),
         vcRegion.Create ("", vcLabel.fac, "Pin number"));

      pinlist := vscript.Sort (cb_loadPinlist (), string_lt);
      h_ipfdata := vlSym.StrCreate (101);
      readIpf (h_ipfdata, isipf);
      r_pins = vcTable.Create ("r_pins", length (pinlist), 1);
      l_allrows := cons (r_title, nil);

      declare pinno, fixed, data, r_label, r_onerow;
      for (l:=pinlist; !null (l); l=cdr(l)) {
         r_onerow = vcTable.Create ("", 1, 3);
         l_allrows = cons (r_onerow, l_allrows);
         r_label = vcRegion.Create ("", vcLabel.fac, car (l));
         vcTable.Add (r_onerow, r_label);
         vcRegion.Align (r_label, vgAlign.MiddleLeft);

         pinno = ""; fixed = FALSE;
         data = vlSym.Fetch (h_ipfdata, car (l));
         if (!null (data)) { pinno = car (data); fixed = cdr (data); }

         vcTable.Add (r_onerow,
            vcRegion.Create ("", vcToggleButton.fac, fixed));
         vcTable.Add (r_onerow,
            vcRegion.Create ("", vcCmdline.fac, pinno, width <- 5));
         vcTable.Add (r_pins, r_onerow); }

      vcTable.AlignColumns (l_allrows);
      r_scrollpick := vcTable.Scroll ("r_scrollpick", r_pins, r_pins, 10,10,10);

      r_button1 := vcTable.Make ("", 1, 7, 
         vcRubber.Create ("", 0, 0),
         vcWv.LabelButtonCreate ("", "Fix All", nil,
            vcWv.Cmd (fac, func (d,e) setall (1, TRUE);, ""), nil),
         vcRubber.Create ("", 0, 0),
         vcWv.LabelButtonCreate ("", "Unfix All", nil,
            vcWv.Cmd (fac, func (d,e) setall (1, FALSE);, ""), nil),
         vcRubber.Create ("", 0, 0),
         vcWv.LabelButtonCreate ("", "Clear All", nil,
            vcWv.Cmd (fac,
               func (d,e) { setall (1, FALSE); setall (2, ""); }, ""), nil),
         vcRubber.Create ("", 0, 0));
      r_button2 := vcRegion.Create ("r_buttons", vcButnRow.fac, list (
         list ("Accept","Accept", writeIpf, list (close_window <- TRUE)),
         list ("Cancel","Cancel", func ();, list (close_window <- TRUE)),
         list ("Help", "Help",
            func () vcAlert.SpawnHelp (vcEvent.Window (e),  "vfpga",
               "ActelPinAssign");,
            list (close_window <- FALSE))));
      r_root := vcTable.Make ("Assign pins", 4, 1, r_title, r_scrollpick,
         r_button1, r_button2);

      /* Display window and set up forwarding and notification */
      w_parent := vcEvent.Window (e);
      vcWin.Forward (w_parent, vcWin.Popup (w_parent, r_root, r_scrollpick,
         vcWin.NoMacros + vcWin.NoStatus));
      vcRegion.SetFocus (r_root, r_scrollpick); }

   /* Finally, the main function f_act_pin */
   name := g_selel + ".pin";
   if (fexists (name)) vfpgaConfirm ("ActPinAndIpf",
      "OK to read in automatic pin placement information?",
      func () makewindow (FALSE);, func () makewindow (TRUE););
   else makewindow (TRUE); }


/*** Create commands ***/


c_act_wir := cg_vdes ("ActelWirFile", "Create Actel WIR file", "fl_actfam",
                      "fpganet", TRUE);


c_act_val := vcWv.Cmd (fac,
   func (d, e, selel, fam, dev) {
      if (!string_eq (fam, "nil"))
         cb_prefPut (g_selel, "technology", fam, FALSE);
      partPutpref (dev);
      cb_makeDochain ("actvalid", g_selel, nil); },

   "ActelValid", "Validate Actel design",
   p_selected (),
   p_family ("fl_actfam"),
   p_fpgapart ("vfpgaact.ptl"));


vcCmd.VerifyParams (c_act_val, func (d, e, selel, fam, dev)
   partIsSufficient ("vfpgaact.ptl", dev, TRUE, TRUE, FALSE););


c_act_routed := vcWv.Cmd (fac,
   func (d, e, selel, fam, dev, pfile, tmod) {

      /* Put preferences */
      if (!string_eq (fam, "nil"))
         cb_prefPut (g_selel, "technology", fam, FALSE);
      cb_prefPut (g_selel, "act_progfile", pfile, FALSE);
      cb_prefPut (g_selel, "act_timmod", tmod, FALSE);
      partPutpref (dev);

      /* Execute chain */
      if (tmod == vlYesNo.Yes)
         cb_makeDochain ("actimod", g_selel, nil);
      else cb_makeDochain ("actroute", g_selel, nil); },

   "ActelRouted", "Create Actel routed design",
   p_selected (),
   p_family ("fl_actfam"),
   p_fpgapart ("vfpgaact.ptl"),
   vcWv.Param ("Programming File", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "act_progfile");,
      vcPickEnum.fac, "Create device programming file"),
   vcWv.Param ("Timing model", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "act_timmod");,
      vcPickEnum.fac, "Create simulation timing model after place and route"));


vcCmd.VerifyParams (c_act_routed,
   func (d, e, selel, fam, dev, pfile, tmod)
      partIsSufficient ("vfpgaact.ptl", dev, TRUE, TRUE,
         (tmod == vlYesNo.Yes)); );


/*** Option commands ***/


c_opt_actsyn := cg_vdes ("ActelSynthesis", "Actel synthesis options",
   "fl_actfam", "fpganet", FALSE);


c_opt_actpkg := vcWv.Cmd (fac,
   func (d, e, selel, fam, dev) {
      if (!string_eq (fam, "nil"))
         cb_prefPut (g_selel, "technology", fam, FALSE);
      partPutpref (dev); },
   "ActelPackage", "Actel package options",
   p_selected (), p_family ("fl_actfam"),
   p_fpgapart ("vfpgaact.ptl"));


c_opt_actimp := vcWv.Cmd (fac,
   func (d, e, selel, incplace, clkbal, timtemp, timvolt, silsig, progfmt) {
      cb_prefPut (g_selel, "act_incplace", incplace, FALSE);
      cb_prefPut (g_selel, "act_clkbal", clkbal, FALSE);
      cb_prefPut (g_selel, "act_timtemp", timtemp, FALSE);
      cb_prefPut (g_selel, "act_timvolt", timvolt, FALSE);
      cb_prefPut (g_selel, "act_silsig", silsig, FALSE);
      cb_prefPut (g_selel, "act_progfmt", progfmt, TRUE); },
   "ActelImplementation", "Actel implementation options",
   p_selected (),
   vcWv.Param ("Incremental placement", vfpgaActStrengthVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "act_incplace");,
      "Effect of previous placement upon current"),
   vcWv.Param ("Clock balancing", vfpgaActStrengthVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "act_clkbal");,
      "Effort to reduce clock skew (Act-1 only)"),
   vcWv.Param ("Timing model temperature range", vfpgaActGradeVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "act_timtemp");,
      "Select temperature range for timing model parameters"),
   vcWv.Param ("Timing model voltage range", vfpgaActGradeVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "act_timvolt");,
      "Select voltage range for timing model parameters"),
   vcWv.Param ("Silicon signature", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "act_silsig");,
      "Enter hex value to customize silicon (optional)"),
   vcWv.Param ("Programmer file format", vfpgaActProgFmtVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "act_progfmt");,
      "Select device programmer format"));


c_act_pin := vcWv.Cmd (fac, func (d, e) f_act_pin (d, e);,
   "ActelPinAssign", "Actel pin assignment");


/*** Help messages ***/


vcCmd.SetHelp (c_act_wir,    "Create WIR file for Actel design");
vcCmd.SetHelp (c_act_val,    "Run Actel Validator (DRC) on design");
vcCmd.SetHelp (c_act_routed, "Place and route Actel design");
vcCmd.SetHelp (c_opt_actsyn, "Set options for Actel synthesis");
vcCmd.SetHelp (c_opt_actpkg, "Set options for Actel package");
vcCmd.SetHelp (c_opt_actimp, "Set options for Actel implementation");
vcCmd.SetHelp (c_act_pin,    "Assign pin numbers for Actel implementation");


/* Disable all options but synthesis when either:
   - selel is top-level and not schematic type
   - selel is not top level */
function actSettype (type) {
   somecmds := list (c_act_routed, c_act_val, c_opt_actpkg, c_opt_actimp,
      c_act_pin);
   istop := cb_loadIstop (g_selel);
   if (!istop)
      menuEnableVenlist ("Actel", vlOnOff.Off,
         "Only permitted on top level file", somecmds);
   else if (((string_eq (type, "ABEL")) || (string_eq (type, "PLA")) ||
      (string_eq (type, "JEDEC")) || (string_eq (type, "VHDL"))) && istop)
         menuEnableVenlist ("Actel", vlOnOff.Off, 
            "Only permitted on top level schematic file", somecmds);
   else menuEnableVenlist ("Actel", vlOnOff.On, "", somecmds); }


/*** Partlist building functions ***/


function actGetSigHash (fam, die, pkg, alspath) {
   declare dir, line, i; h := nil; in_bonding := FALSE;
   if (fam[3] == '1') dir = "a1000";
   else if (fam[3] == '2') dir = "a1200";
   else if (fam[3] == '3') dir = "a1400";

   path := string_lower (sprintf ("%s/data/%s/a%s/%s.ddf", alspath, dir,
      die, pkg));
   if (fexists (path)) {
      file := fopen (path, "r");
      h = vlSym.StrCreate (101);
      while (1==1) {
         if (null (line = fgetline (file))) break;
         if (!null (string_search (line, "BONDING."))) in_bonding = TRUE;
         else if (!null (string_search (line, "End."))) in_bonding = FALSE;
         else if (in_bonding && (line[0] != ';')) {
            i = 0;
            foreach (token; string_loop (line, " '\t\n"))
               if (++i == 1) { vlSym.Store (h, token, nil); } } }
      fclose (file); }
   return (h); }


function actParsePart (name, pins, pkg_type, outfile, alspath) {
   i := 0; j := 0; declare fam, pkg, die, spds;
   foreach (token; string_loop (name, "_")) {
      if (++i == 1) die = token;
      else if (i == 2) pkg = token; }
   if (name[1] == '0') { fam =  "ACT1"; spds = "STD -1 -2"; }
   else if (name[1] == '2') { fam =  "ACT2"; spds = "STD -1"; }
   else if (name[1] == '4') { fam =  "ACT3"; spds = "STD -1"; }
   if (null (sighash := actGetSigHash (fam, die, pkg, alspath)))
      cb_mainErr (1511, "Warning", nil,
         sprintf ("Unable to find partlist file for %s %s %s",
            fam, die, pkg));
   else {
      fputline (outfile, sprintf ("DEV %s %s %s %s", fam, die, pkg, pins));
      fputline (outfile, sprintf (" %s\nSPD %s\n", pkg_type, spds));
      foreach (sym, val; vlSym.Loop (sighash)) {
         if (j == 0) fputline (outfile, "SIG");
         fputline (outfile, " ");
         fputline (outfile, sym);
         if (++j == 18) { j = 0; fputline (outfile, "\n"); } }
      if (j != 0) fputline (outfile, "\n");
      fputline (outfile, "\n"); } }


function actReadPart (alspath, outfile) {
   line := ""; in_data := FALSE; declare i, x, ofs, keep_name, keep_pins;
   path := alspath + "/data/system.def";
   if (!fexists (path))
      cb_mainErr (1512, "Error", nil, "Unable to find system.def file");
   else {
      file := fopen (path, "r");
      while (1==1) {
         if (null (line = fgetline (file))) break;
         if (string_length (line) > 4)
            if (string_eq (substring (line, 0, 4), "data")) {
               i = 0; keep_name = nil;
               foreach (token; string_loop (line, " '\t\n")) {
                  if (++i == 2) {
                     if (string_eq (token, "tmpFamily_Info")) {
                        in_data = FALSE; break; }
                     if (in_data) {
                        ofs = string_search (token, "_");
                        if (null (ofs)) break;
                        x = substring (token, ofs+1, 0);
                        if (!null (string_search (x, "_"))) keep_name = token; }
                     if (string_eq (token, "tmpPACKAGE_Ext")) {
                        in_data = TRUE; break; } }
                  else if (i == 3) keep_pins = token;
                  else if ((i == 4) && (!null (keep_name)))
                     actParsePart (keep_name, keep_pins, token, outfile,
                        alspath); } } }
      fclose (file); } }


c_util_actptl := vcWv.Cmd (fac,
   func (d, e) vcWin.WithBusy (vcRegion.Window (gr_log), func () {
      file := fopen ("vfpgaact.ptl", "w");
      alspath := actGetALS (FALSE);
      fputline (file,
         "| Automatically generated partlist file for Actel\n\n");
      actReadPart (alspath, file);
      cb_mainErr (0, "Status", nil, "Partlist file vfpgaact.ptl created");
      fclose (file); });,
   "ActelPTL", "Create new partlist file for Actel");


/*** Utility commands ***/


c_util_act2tut := vcWv.Cmd (fac,
   func (d, e) menuCopyTutFiles (list ("actelcells", "actelmodels", "actelsim"),
      list ("fsm_case.vhd", "accum.abl", "reg5.abl", "sym/accum.1",
         "sym/reg5.1", "sch/muxp_a2.1"),
      list ("accum.bl0", "accum.xrf", "accum.tt2", "accum.dmc",
            "accum.vhd", "sym/accum.pin", "wir/accum.1", "sch/accum.1",
            "reg5.bl0", "reg5.xrf", "reg5.tt2", "reg5.dmc", "reg5.vhd",
            "sym/reg5.pin", "wir/reg5.1",
         "sch/reg5.1", "wir/fsm_case.1", "sch/fsm_case.1"));,
   "CopyAct2Tut", "Copy files for online Actel Act-2 tutorial");


function fl_actdel () { list (
   list (cons ("AAL", "aal"), cons ("ADL", "adl"), cons ("AFL", "afl")),
   list (cons ("ALS", "als"), cons ("BL0", "bl0"), cons ("COB", "cob")),
   list (cons ("CRT", "crt"), cons ("DEF", "def"), cons ("DEL", "del")),
   list (cons ("DFR", "dfr"), cons ("DMC", "dmc"), cons ("DTB", "dtb")),
   list (cons ("FUS", "fus"), cons ("HPF", "hpf"), cons ("IPF", "ipf")),
   list (cons ("LIS", "lis"), cons ("LOC", "loc"), cons ("OFL", "ofl")),
   list (cons ("OLC", "olc"), cons ("ORD", "ord"), cons ("PIN", "pin")),
   list (cons ("PLI", "pli"), cons ("RTI", "rti"), cons ("SEG", "seg")),
   list (cons ("TIM", "tim"), cons ("TMV", "tmv"), cons ("TT2", "tt2")),
   list (cons ("VAL", "val"), cons ("VLD", "vld"), cons ("XRF", "xrf"))); }


c_util_actdel := cg_util_delint ("DelActIntFiles",
   "Delete Actel intermediate files", "fl_actdel",
   "aal bl0 cob dfr dmc hpf lis ofl olc ord tmv tt2 xrf");
