vcWv.MakeEnum ("vfpgaDefYesNoVT", list("Default","Yes","No"));
vcWv.MakeEnum ("vfpgaAltOptVT", list("Default","Area","Delay","Routing"));
vcWv.MakeEnum ("vfpgaAltMinVT", list("Default","Full","Partial","None"));
vcWv.MakeEnum ("vfpgaAltChnVT", list("Default","Auto","Manual","Ignore"));
vcWv.MakeMask ("vfpgaAltRptVT", list("PinAssignments", 
                                     "FileHierarchy", 
                                     "MacroConnections", 
                                     "Equations")); 

/*** Global declarations ***/
declare g_altera_dir;
PartFile := "vfpgaalt.ptl";
function fl_altlist () { 
    list (list (cons ("EP330",    "EP330"),
                cons ("CLASSIC",  "Classic"),
                cons ("EPS464",   "EPS464"),
                cons ("MAX5000",  "Max5000"),
                cons ("MAX7000",  "Max7000"),
                cons ("MAX7000E", "Max7000E"),
                cons ("Flex8000", "Flex8000"))); 
}

g_altsetup := FALSE;
function altSetup () {
    declare typelist;

    /* Make sure maxplus2 is in the path..., set g_altera_dir there */
    mp2_path := cb_FindExeInPath("maxplus2");
    if (!null (mp2_path)) 
        g_altera_dir = vscript.vsUppath(cadr(vscript.vsParse(mp2_path)));
    else {
        g_altera_dir = vscript.vsSrchdirs("maxplus2", vscript.vsDIR);
        if (!null (g_altera_dir))
            if (!fexists(vscript.vsConvPath(g_altera_dir + "/bin/maxplus2"))) 
                g_altera_dir = nil;
            else {
                newpath := vscript.getenv("PATH") + ":" + 
                    vscript.vsDnpath(g_altera_dir, "bin");
                vscript.putenv("PATH", newpath);
                cb_mainDbg (1, "setenv PATH " + newpath);
            }
    }
    setup_ok := TRUE;
    if (null(g_altera_dir)) {
        cb_mainErr (1601, "Error", nil, "Unable to set up Altera environment");
        setup_ok = FALSE;
    }
    
    /* Make sure libraries are in viewdraw.ini */
    if (!cb_loadIfalias("altera", TRUE)) setup_ok = FALSE;
    if (!cb_loadIfalias("max2sim", TRUE)) setup_ok = FALSE;

    if (setup_ok == TRUE) {
        g_altsetup = TRUE;
        typelist = list ("ABEL", "VHDL", "JEDEC", "AlteraChip", "TDF", 
                         "DIO", "LPM");
    }
    else typelist = nil;

    menuValidTypes ("Altera", typelist,
                    list (c_alt_wir, c_alt_routed, c_opt_altsyn, 
                          c_opt_altopt, c_opt_altimp));
                   

    /* Make sure ~/maxplus2.ini has correct LMF, EDC files */
    if (!null(g_altera_dir)) {
        topdir := vscript.getenv("HOME");
        mp2ini :=vscript.vsConvPath(topdir + "/maxplus2.ini");
        AltIniRead(mp2ini);
        AltIniSetVal("COMPILER", "EDIF_LMF1", 
                     vsConvPath(g_altera_dir + "/lmf/vwl_bas.lmf"));
        AltIniSetVal("COMPILER", "EDIF_LMF2", 
                     vsConvPath(g_altera_dir + "/lmf/vwl_dio.lmf"));
        AltIniSetVal("COMPILER", "EDIF_EDC", 
                     vsConvPath(g_altera_dir + "/edc/vwl.edc"));
        lib1 :=  vsConvPath(g_altera_dir + "/max2lib/mf");
        lib2 :=  vsConvPath(g_altera_dir + "/max2inc");
        AltIniSetVal("system", "MAXPLUS_LIB", lib1 + ";" + lib2);
        AltIniSetVal("system", "USER_LIB", vscript.vsGetdef());
        AltIniWrite(mp2ini);
    }
}

function altDefallatts (prefdb, startup) {
    prefDefoneatt ("alt_out", vlStringVT, 
                   "ALTERA: Timing Simulation Model Name", "",
                   prefdb, startup);
    prefDefoneatt ("alt_vgen", vlYesNoVT, "ALTERA: Run Viewgen", vlYesNo.No,
                   prefdb, startup);
    prefDefoneatt ("alt_mult_dev", vlYesNoVT, "ALTERA: Allow Partitioning",
                   vlYesNo.No, prefdb, startup);
    prefDefoneatt ("alt_opt", vfpgaAltOptVT, 
                   "ALTERA: Priority for Optimization", 
                   vfpgaAltOpt.Default, prefdb, startup);
    prefDefoneatt ("alt_min", vfpgaAltMinVT, 
                   "ALTERA: Logic Minimization Routine", 
                   vfpgaAltMin.Default, prefdb, startup);
    prefDefoneatt ("alt_slew", vfpgaDefYesNoVT, 
                   "ALTERA: Slow Output Transition", 
                   vfpgaDefYesNo.Default, prefdb, startup);
    prefDefoneatt ("alt_xor", vfpgaDefYesNoVT, "ALTERA: XOR Synthesis", 
                   vfpgaDefYesNo.Default, prefdb, startup);
    prefDefoneatt ("alt_turbo", vfpgaDefYesNoVT, "ALTERA: Turbo Bit", 
                   vfpgaDefYesNo.Default, prefdb, startup);
    prefDefoneatt ("alt_par_exp", vfpgaDefYesNoVT, 
                   "ALTERA: Parallel Expanders", 
                   vfpgaDefYesNo.Default, prefdb, startup);
    prefDefoneatt ("alt_exp_fac", vfpgaDefYesNoVT, 
                   "ALTERA: Expander Factoring", 
                   vfpgaDefYesNo.Default, prefdb, startup);
    prefDefoneatt ("alt_cascade", vfpgaAltChnVT, "ALTERA: Cascade Chain", 
                   vfpgaAltChn.Default, prefdb, startup);
    prefDefoneatt ("alt_carry", vfpgaAltChnVT, "ALTERA: Carry Chain", 
                   vfpgaAltChn.Default, prefdb, startup);
    prefDefoneatt ("alt_drc", vlYesNoVT, "ALTERA: Run Design Doctor", 
                   vlYesNo.No, prefdb, startup);
    prefDefoneatt ("alt_progfile", vlYesNoVT, 
                   "ALTERA: Create programming file",
                   vlYesNo.Yes, prefdb, startup);
    prefDefoneatt ("alt_timmod", vlYesNoVT, "ALTERA: Create timing model",
                   vlYesNo.Yes, prefdb, startup);
    prefDefoneatt ("alt_prev_fit", vlYesNoVT, "ALTERA: Use previous fit",
                   vlYesNo.No, prefdb, startup);
    prefDefoneatt ("alt_reserve_cells", vlIntVT,
                   "ALTERA: Percent Logic cells reserved", 0, prefdb, startup);
    prefDefoneatt ("alt_reserve_pins", vlIntVT, 
                   "ALTERA: Percent user pins reserved", 0, prefdb, startup);
    prefDefoneatt ("alt_jedec", vlYesNoVT, "ALTERA: Generate Jedec", 
                   vlYesNo.No, prefdb, startup);
    prefDefoneatt ("alt_report", vfpgaAltRptVT, 
                   "ALTERA: Compiler Report Contents", 
                   vfpgaAltRpt.PinAssignments + vfpgaAltRpt.Equations, 
                   prefdb, startup); 
    prefDefoneatt ("alt_out_vhd", vlYesNoVT, 
                   "ALTERA: VHDL Simulation Model", 
                   vlYesNo.No, prefdb, startup);
}

/* Translations for synthesis library name */
vlSym.Store (g_vdes_alias_hash, "EP330", "altera");
vlSym.Store (g_vdes_alias_hash, "CLASSIC", "altera");
vlSym.Store (g_vdes_alias_hash, "EPS464", "altera");
vlSym.Store (g_vdes_alias_hash, "MAX5000", "altera");
vlSym.Store (g_vdes_alias_hash, "MAX7000", "altera");
vlSym.Store (g_vdes_alias_hash, "MAX7000E", "altera");
vlSym.Store (g_vdes_alias_hash, "FLEX8000", "altera");

/* File Definitions */
cb_cfgAddFile("altedf", list (list ("%dot", "alt_out", "edf")));
cb_cfgAddFile("altini", list (list ("%dot", "alt_out", "ini")));
cb_cfgAddFile("altedo", list (list ("%dot", "alt_out", "edo")));
cb_cfgAddFile("altvho", list (list ("%dot", "alt_out", "vho")));
cb_cfgAddFile("altpof", list (list ("%dot", "alt_out", "pof")));
cb_cfgAddFile("altfit", list (list ("%dot", "alt_out", "fit")));
cb_cfgAddFile("altord", list (list ("%dot", "alt_out", "ord")));
cb_cfgAddFile("altrpt", list (list ("%dot", "alt_out", "rpt")));
cb_cfgAddFile("altwir", list (list ("%wir", "alt_out", "1")));
cb_cfgAddFile("altsch", list (list ("%sch", "alt_out", "1")));
cb_cfgAddFile("altsym", list (list ("%sym", "alt_out", "1")));
cb_cfgAddFile("altvsm", list (list ("%dot", "alt_out", "vsm")));

/* Tool Definitions */
cb_cfgAddTool("p2v_altdio", list ("tt2"), list ("wir"), nil);
cb_cfgAddTool("altedifo", list ("wir"), list ("altedf"), 
   list ("alt_out"));
cb_cfgAddTool("maxplus",  list ("altedf"), list ("altfit", "altrpt"), 
   list ("alt_opt", "alt_min", "alt_slew", "alt_xor", "alt_turbo", 
         "alt_par_exp", "alt_exp_fac", "alt_cascade", "alt_carry",
         "alt_drc", "alt_progfile", "alt_timmod", "alt_prev_fit",
         "alt_reserve_cells", "alt_reserve_pins", "alt_jedec",
         "alt_report", "alt_out_vhd", "technology", "fpga_die",
         "fpga_package", "fpga_speed"));
cb_cfgAddTool("altedifi", list ("altedo"), list ("altwir"), nil);
cb_cfgAddTool("altdelp2", list ("altwir"), list ("altwir"), nil);
cb_cfgAddTool("altvgen",  list ("altwir"), list ("altsch"), 
   list ("alt_vgen"));
cb_cfgAddTool("altvsm",   list ("altwir"), list ("altvsm"), nil);
cb_cfgAddTool("altprtvsm",  list ("prtwir"), list ("prtvsm"), nil);
cb_cfgAddTool("p2v_altsym", list ("altord"), list ("altsym"), nil);
cb_cfgAddTool("altfit2sch", list ("altfit"), list ("prtsch"), nil);
cb_cfgAddTool("loadaltvsm", nil, nil, nil);

/* There are several targets defined in the Altera design flow:
 * 
 *   altera_gates  : Netlist of Altera or DIO gates
 *   altera_routed : MAX+Plus2 has been run, a fit file exists
 *   altera_onevsm : Post-route simulation model for single chip design
 *   altera_prtsch : Interconnection schematic of multi-chip design
 *   altera_prtwir : Wir file for one chip of multi-chip design
 *   altera_prtvsm : Post-route simulation model for multi-chip design
 * 
 * Chains are required for most combinations of source and targets,
 * but the altera_prtwir target is only accessed from AlteraChip source.
 */
cb_cfgAddTgtname ("altera_gates",  "Altera pre-route netlist");
cb_cfgAddTgtname ("altera_routed", "Altera routed design");
cb_cfgAddTgtname ("altera_onevsm", "Altera post-route timing model");
cb_cfgAddTgtname ("altera_prtsch", "Altera multi-chip schematic");
cb_cfgAddTgtname ("altera_prtwir", "Altera post-route timing model");
cb_cfgAddTgtname ("altera_prtvsm", "Altera multi-chip timing model");

/* From a schematic... */
cb_cfgAddChain("altera_gates",  "sch", list("savesrc","%altera_gates"));
cb_cfgAddChain("altera_routed", "sch", list("savesrc","%altera_gates", 
                                            "altedifo","maxplus"));
cb_cfgAddChain("altera_onevsm", "sch", list("p2v_altsym","altedifi", 
                                            "altdelp2","altvgen","altvsm",
                                            "loadaltvsm"));
cb_cfgAddChain("altera_prtsch", "sch", list("altfit2sch","prtcheck",
                                            "loadprt", "drawprt"));
cb_cfgAddChain("altera_prtvsm", "sch", list("%altera_prtwir", "altfit2sch",
                                            "altprtvsm", "loadprt",
                                            "loadprtvsm"));

/* From VHDL... */
cb_cfgAddChain("altera_gates", "lpm",  list("vdes_vhdl", "vdes_viewgen"));
cb_cfgAddChain("altera_gates", "vhdl", list("vdes_vhdl", "vdes_viewgen"));
cb_cfgAddChain("altera_routed","vhdl", list("vdes_vhdl","altedifo", 
                                            "maxplus"));
cb_cfgAddChain("altera_onevsm","vhdl", list("p2v_altsym","altedifi", 
                                            "altdelp2","altvgen","altvsm",
                                            "loadaltvsm"));
cb_cfgAddChain("altera_prtsch","vhdl", list("altfit2sch","prtcheck",
                                            "loadprt", "drawprt"));

/* From ABEL... */
cb_cfgAddChain("altera_gates", "abel", list("ahdl2pla","mergett","p2v_altdio",
                                            "vdes_viewgen"));
cb_cfgAddChain("altera_routed","abel", list("ahdl2pla","mergett","p2v_altdio",
                                            "altedifo","maxplus"));
cb_cfgAddChain("altera_onevsm","abel", list("p2v_altsym","altedifi", 
                                            "altdelp2","altvgen","altvsm",
                                            "loadaltvsm"));
cb_cfgAddChain("altera_prtsch","abel", list("altfit2sch","prtcheck",
                                            "loadprt", "drawprt"));

/* From PLA... */
cb_cfgAddChain("altera_gates",  "partition", list("p2v_altdio", 
                                                  "vdes_viewgen"));
cb_cfgAddChain("altera_routed", "partition", list("p2v_altdio","altedifo",
                                                  "maxplus"));
cb_cfgAddChain("altera_onevsm", "partition", list("p2v_altsym","altedifi", 
                                                  "altdelp2","altvgen",
                                                  "altvsm", "loadaltvsm"));
cb_cfgAddChain("altera_prtsch", "partition", list("altfit2sch","prtcheck",
                                                  "loadprt", "drawprt"));

/* From JEDEC... */
cb_cfgAddChain("altera_gates", "jedec",list("jed2ahdl", "ahdl2pla", "mergett",
                                            "p2v_altdio", "vdes_viewgen"));
cb_cfgAddChain("altera_routed","jedec",list("jed2ahdl", "ahdl2pla", "mergett",
                                            "p2v_altdio", "altedifo", 
                                            "maxplus"));
cb_cfgAddChain("altera_onevsm","jedec",list("p2v_altsym","altedifi", 
                                            "altdelp2","altvgen",
                                            "altvsm", "loadaltvsm"));
cb_cfgAddChain("altera_prtsch","jedec",list("altfit2sch","prtcheck",
                                            "loadprt", "drawprt"));

/* From Altera Text Design File... */
cb_cfgAddChain("altera_gates",  "tdf", nil);

/* From Altera Chip... */
cb_cfgAddChain("altera_prtwir","alterachip", list("altedifi", "p2v_altsym",
                                                  "altdelp2", "altvgen"));

/* This function returns a boolean which is true if either the preference
   value is vfpgaDefYesNo.Yes or the technology is a member of the
   matchlist, */
function resolveDefYesNo(prefname, tech, matchlist) {
    retval := FALSE;
    x := cb_prefGet(g_selel, prefname);
    if (x == vfpgaDefYesNo.Yes) 
        retval = TRUE;
    else if (x == vfpgaDefYesNo.Default) {
        for (l:=matchlist; !null (l); l=cdr(l)) 
            if (string_eqi(tech, car(l))) retval = TRUE;
    }
    retval;
}

function bitSetInMask(mask, bit) {
    retval := ((mask & bit) == bit) ? "1" : "0";
}

function writeLineToFile (fname, str) {
    fp := fopen(fname, "w");
    if (null (fp)) 
        cb_mainErr (1611, "Error", "nil",
                    "Unable to open file " + name + " for writing");
    else {
        fputline(fp, str + "\n");
        fclose(fp);
    }
}


function setAlteraIniValues(name) {
    AltIniRead(name + ".ini");

    /* Set constants */
    AltIniSetVal("project", "EDIF_VCC", "VDD");
    AltIniSetVal("project", "EDIF_VCC_OUTPUT", "VDD");
    AltIniSetVal("project", "EDIF_VERSION", "200");
    AltIniSetVal("project", "EDIF_USE_EDC", "1");
    AltIniSetVal("project", "EDIF_USE_LMF1", "1");
    AltIniSetVal("project", "EDIF_USE_LMF2", "1");
    AltIniSetVal("project", "EDIF_CHAR_MAPPING", "1");
    AltIniSetVal("project", "STYLE", "Normal");

    /* Set optimization preferences */
    tech := cb_prefGet (g_selel, "technology");
    if (string_eqi(tech, "classic") || string_eqi(tech, "ep330"))
        opt_section := "Normal.CLASSIC";
    else if (string_eqi(tech, "max5000") || string_eqi(tech, "eps464"))
        opt_section := "Normal.MAX5000";
    else if (string_eqi(tech, "max7000") || string_eqi(tech, "max7000e"))
        opt_section := "Normal.MAX7000";
    else
        opt_section := "Normal.FLEX8000";
    x := cb_prefGet(g_selel, "alt_opt");
    if (x == vfpgaAltOpt.Delay) AltIniSetVal(opt_section, "opt", "delay");
    else if (x == vfpgaAltOpt.Routing) 
        AltIniSetVal(opt_section, "opt", "routing");
    else AltIniSetVal(opt_section, "opt", "area");
    x := cb_prefGet(g_selel, "alt_min");
    if (x == vfpgaAltMin.Partial) AltIniSetVal(opt_section, "min", "part");
    else if (x == vfpgaAltMin.None) 
        AltIniSetVal(opt_section, "min", "none");
    else AltIniSetVal(opt_section, "min", "full");
    x = (cb_prefGet(g_selel, "alt_slew") == vfpgaDefYesNo.Yes) ? "on" : "off";
    AltIniSetVal(opt_section, "slow", x);
    x = resolveDefYesNo("alt_xor", tech, list ("max5000", "max7000"));
    AltIniSetVal(opt_section, "xor", (x) ? "on" : "off");
    x = resolveDefYesNo("alt_turbo", tech, list ("classic", "max7000"));
    AltIniSetVal(opt_section, "turbo", (x) ? "on" : "off");
    if (cb_prefGet(g_selel, "alt_par_exp") == vfpgaDefYesNo.Yes)
        AltIniSetVal(opt_section, "para", "on");
    else
        AltIniSetVal(opt_section, "para", "off");
    x = resolveDefYesNo("alt_exp_fac", tech, list ("max5000", "max7000"));
    AltIniSetVal(opt_section, "exp", (x) ? "on" : "off");
    if (string_eqi(tech, "flex8000")) {
        x = cb_prefGet(g_selel, "alt_cascade");
        if (x == vfpgaAltChn.Ignore) 
            AltIniSetVal(opt_section,"casc","ignore");
        else if (x == vfpgaAltChn.Manual) 
            AltIniSetVal(opt_section,"casc","manual");
        else AltIniSetVal(opt_section,"casc","auto");
        x = cb_prefGet(g_selel, "alt_carry");
        if (x == vfpgaAltChn.Ignore) 
            AltIniSetVal(opt_section,"carry","ignore");
        else if (x == vfpgaAltChn.Manual) 
            AltIniSetVal(opt_section,"carry","manual");
        else AltIniSetVal(opt_section,"carry","auto");
    }
    else {
        AltIniSetVal(opt_section, "casc", "ignore");
        AltIniSetVal(opt_section, "carry", "ignore");
    }


    /* Set project preferences */
    AltIniSetVal("project", "FAMILY", tech);
    x = (cb_prefGet (g_selel, "alt_mult_dev") == vlYesNo.No) ? "1" : "0";
    AltIniSetVal("project", "ASK_BEFORE_ADDING", x);
    x = (cb_prefGet (g_selel, "alt_prev_fit") == vlYesNo.No) ? "1" : "0";
    AltIniSetVal("project", "COMPILER_IGNORE_FIT", x);
    x = (cb_prefGet (g_selel, "alt_timmod") == vlYesNo.Yes) ? "1" : "0";
    AltIniSetVal("project", "COMPILER_SNF", x);
    AltIniSetVal("project", "COMPILER_SNF2EDF", x);
    x = (cb_prefGet (g_selel, "alt_drc") == vlYesNo.Yes) ? "1" : "0";
    AltIniSetVal("project", "DO_DRC", x);
    AltIniSetVal("project", "RESERVED_PERCENT",
                 cb_prefGet (g_selel, "alt_reserve_cells"));
    AltIniSetVal("project", "RESERVED_PINS_PERCENT",
                 cb_prefGet (g_selel, "alt_reserve_pins"));
    x = (cb_prefGet (g_selel, "alt_jedec") == vlYesNo.Yes) ? "1" : "0";
    AltIniSetVal("project", "GENERATE_JEDEC", x);
    x = cb_prefGet (g_selel, "alt_report");
    x1 := bitSetInMask(x, vfpgaAltRpt.PinAssignments);
    x2 := bitSetInMask(x, vfpgaAltRpt.FileHierarchy);
    x3 := bitSetInMask(x, vfpgaAltRpt.MacroConnections);
    x4 := bitSetInMask(x, vfpgaAltRpt.Equations);
    AltIniSetVal("project", "COMPILER_REPORT", 
                 x1 + "," + x2 + "," + x3 + "," + x4);

    /* Clean up old device selector data */
    numdev := AltIniGetVal("devices", "NUM_DEVICES");
    if (!null(numdev)) {
        for (i:=0; i < string2number(numdev); ++i) 
            AltIniRemove("devices", "DEVICE"+number2string(i+1));
    }
    
    /* Build device selector device list */
    die := cb_prefGet (g_selel, "fpga_die");
    pkg := cb_prefGet (g_selel, "fpga_package");
    spd := cb_prefGet (g_selel, "fpga_speed");
    plist := ListFpgaParts(PartFile);
    matches := getMatchList(string_upper(tech), die, pkg, spd, 
                            plist, "%s%s-%s");
    declare s;
    for (l:=matches, i:=1; !null(l); l = cdr(l), ++i) {
        s = sprintf ("-1,%s", car(car(l)));
        AltIniSetVal("devices", "DEVICE"+number2string(i), s);
    }
    AltIniSetVal("devices", "NUM_DEVICES", number2string(i-1));

    /* Set the device in prb file */
    if (cb_prefGet (g_selel, "alt_mult_dev"))
        vscript.vsDelete(name + ".prb", vscript.vsFILE);
    else {
        if ((string_eqi(die, "ANY")) ||
            (string_eqi(pkg, "ANY")) ||
            (string_eqi(spd, "ANY")))
            dev := "AUTO";
        else dev := die + pkg + "-" + spd;
        writeLineToFile(name + ".prb", "DEVICE\t\t" + name + "\t\t" + dev);
    }

    /* Write out ini file */
    AltIniWrite(name + ".ini");
}



/* Action Function Definitions */
function a_p2v_altdio (name) { "pla2view " + name + " -wir -bi_tri"; }

function a_altedifo (name) {
    outname := cb_prefGet (name, "alt_out");
    swapname := outname + ".eso";
    writeLineToFile(swapname, outname + "\\=" + name);
    str := "edifneto -F " + name + " " + outname + ".edf -n " + swapname;
    str = str + " -l dio -l std -l altera";
}

function a_maxplus (name) {
    outname := cb_prefGet (name, "alt_out");

    /* Load ini file, then write it out */
    setAlteraIniValues(outname);
    str := "maxplus2 -c " + outname; 
}

function a_altedifi (name) {
    outname := cb_prefGet (name, "alt_out");
    str := "edifneti " + outname + ".edo"; 
    if (fexists(outname + ".esi"))
        str = str + " -l " + outname + ".esi";
}

function a_altdelp2(name) {
    outname := cb_prefGet (name, "alt_out");
    str := "delports " + outname + " " + outname + " -c vdd,gnd"; 
}

function a_altvgen (name) {
    if (cb_prefGet (name, "alt_vgen")) {
        outname := cb_prefGet (name, "alt_out");
        str := "viewgen " + outname;
    }
    else str := "";
}

function a_altvsm(name) {
    outname := cb_prefGet (name, "alt_out");
    str := "vsm " + outname;
}

function a_altprtvsm(name) {
    outname := cb_prefGet (name, "part_out");
    str := "vsm " + outname;
}

function a_p2v_altsym(name) {
    outname := cb_prefGet (name, "alt_out");
    str := "pla2view " + outname + " -fpgadata vfpgaalt.ptl -pin";
    str = add_sym_prefs (name, str);
}

function a_altfit2sch(name) {
    outname := cb_prefGet (name, "alt_out");
    schname := cb_prefGet (name, "part_out");
    str := "prt2sch " + outname + ".fit -fit -nopla";
    if (!string_eq(schname, "")) {
        str = str + " -o " + schname;
        if (string_eq(schname, name))
            str = str + " -nosym";
    }
    str;
}

function a_loadaltvsm(name) {
    outname := cb_prefGet (name, "alt_out");
    if ((cb_itcSimActive ()) && (string_eqi (g_selel, name))) {
        if (vlPref.Val (prefs, "warnalerts"))
            vfpgaConfirm ("OverwriteSim",
                          "OK to load new netlist for " + outname + 
                          " into ViewSim?\n" +
                          "This will flush the existing simulation.",
                          func () cb_itcLoadVSM (outname);, func (););
        else cb_itcLoadVSM (outname); 
    } 
    str := "";
}

/* Error Function Definitions */
function e_altedifo (name) { errWrap (1602, "EDIF Netlist Writer", name); }
function e_maxplus (name) { errWrap (1603, "Altera Compiler", name); }
function e_altedifi (name) { errWrap (1604, "EDIF Netlist Reader", name); }
function e_altvgen (name) { 
    cb_mainErr (1605, "Error", nil,
                "Viewgen failed to create schematic for " + name);
}
function e_altvsm (name) { errWrap (1606, "ViewSim Netlister", name); }
function e_p2v_altsym(name) { errWrap (1607, "Symbol Generator", name); }
function e_altfit2sch(name) { errWrap (1608, "Altera Schematic", name); }

/* Temporary tools! */
function e_altdelp2 (name) { errWrap (1699, "Wir File Fixer", name); }


function move_files(oldname, newname, extlist) {
    cb_mainDbg (1, "Moving files: " + oldname + " to " + newname);
    declare ext, oldf, newf;
    for (a:=extlist; !null(a); a=cdr(a)) {
        ext = car(a);
        oldf = vscript.vsMerge("", "", oldname, ext);
        newf = vscript.vsMerge("", "", newname, ext);
        if (fexists(oldf)) vscript.vsRename(oldf, newf);
    }
}

/* This function parses the fit file, writing out an ord file for
   each chip.  If there is more than one chip, or one chip with a
   different name than the fit file, the name of the partitioning
   schematic is returned.  Otherwise, an empty string is returned 
   to indicate that partitioning design flow is not required.  If 
   chips in the netlist have upper-case names, this function will 
   copy the files to the lower-case named files. */
function fit2ord (name) {
    fitfile := fopen(name + ".fit", "r");
    if (fitfile == nil) return( "" );
    numchips := 0; 
    declare token, p, ordfp, esifp;
    /* Run through the fit file, counting the devices, and looking 
       for the intermediate node aliases. */
    alias_hash := vlSym.StrCreate (11);
    for (s := fgetline(fitfile); s != nil; s = fgetline (fitfile)) {
        p = string_token (s, &token, " \n");
        if (!null(token) && string_eqi(token, "device")) ++numchips; 
        else if (!null(token)) {
            signame := string_copy(token);
            declare index;
            if (!null(p)) index = string_search(p, "--");
            if (!null(index)) {
                p = substring(p, index + 2, 0);
                index = string_search(p, "--");
                if (!null(index)) {
                    p = substring(p, index + 2, 0);
                    string_token (p, &token, " \n");
                    vlSym.Store (alias_hash, signame, token);
                }
            }
        }
    }
    fclose(fitfile);
    fitfile := fopen(name + ".fit", "r");
    chipname := ""; schname := name;
    for (s := fgetline(fitfile); s != nil; s = fgetline (fitfile)) {
        p = string_token (s, &token, " \n");
        if (null(token)) ;
        else if (string_eqi(token, "device")) { 
            p = string_token (p, &token, " \"");
            chipname = string_lower(string_copy(token));
            if ((numchips == 1) && (string_eq(chipname, name))) 
                schname = "";
            if ((numchips > 1) && (string_eq(chipname, name)))
                schname = name + "_top";
            if (!string_eq(token, chipname)) 
                move_files(token, chipname, list("pof","snf","vho","edo"));

            /* Open the ord file and put in the device */
            p = string_token (p, &token, " \"");
            p = string_token (p, &token, " -\"");
            ordfp := fopen(chipname + ".ord", "w");
            if (null (ordfp))  cb_mainErr (1611, "Error", "nil",
                                           "Unable to open file " + 
                                           chipname + ".ord for writing");
            esifp := fopen(chipname + ".esi", "w");
            if (null (esifp))  cb_mainErr (1612, "Error", "nil",
                                           "Unable to open file " + 
                                           chipname + ".esi for writing");
            fputline(ordfp, token + "\n");
        }
        else if (string_eqi(token, "end;")) {
            if (!null(ordfp)) { 
                fclose(ordfp); ordfp = nil; 
                fclose(esifp); esifp = nil; 
            } 
        }
        else if ((!string_eqi(token, "begin")) &&
                 (!null(ordfp)) && (!null(esifp))) {
            val := vlSym.Fetch(alias_hash, token);
            if (!null(val)) {
                fputline(esifp, val + "\\=" + token + "\n");
                fputline(esifp, token + "\\=" + val + "\n");
            }
            /* make sure token does not have ~ */
            tilde := lindex(token, '~');
            if (tilde) 
                str := substring(token, 0, tilde) + "x" 
                    + substring(token, tilde + 1, 0) + " "; 
            else
                str := token + " "; 
            p = string_token (p, &token, " @");
            str = str + token + " ";
            p = string_token (p, &token, " :;");
            if (string_eqi(token,"INPUT")) fputline(ordfp, str + "1\n");
            else if (string_eqi(token,"OUTPUT")) fputline(ordfp, str + "0\n");
            else if (string_eqi(token,"BIDIR")) fputline(ordfp, str + "2\n");
        }
    }
    schname;
}

function altera_rest () {

    /* Run fit2ord to generate ord files and determine which 
       design flow to use. If fit2ord returns "", the fit file 
       only contains one part with the selected element name, 
       so run the altera_onevsm chain.  Otherwise, run the 
       altera_prtsch chain, load the schematic, run the 
       altera_prtvsm chain. */
    outname := cb_prefGet (g_selel, "alt_out");
    schname := fit2ord(outname);
    if (string_eq(schname, ""))
        cb_makeDochain ("altera_onevsm", g_selel, nil);
    else  {
        cb_prefPut(g_selel, "part_out", schname, FALSE);
        cb_makeDochain ("altera_prtsch", g_selel, func () {
            cb_prefPut(outname, "part_out", schname, FALSE);
            cb_prefPut(schname, "part_out", schname, FALSE);
            cb_prefPut(schname, "alt_out", outname, FALSE);
            for (l:=cb_loadPartlist(g_selel);!null(l);l=cdr(l)) 
                cb_prefPut(car(l), "alt_out", car(l), FALSE);
            cb_makeDochain ("altera_prtvsm", schname, nil);
        });
    }
}

/*** Create commands ***/

c_alt_wir := cg_vdes ("AlteraWirFile", "Create Altera WIR file", "fl_altlist",
                      "altera_gates", TRUE);

c_alt_routed :=
   vcWv.Cmd (fac, func (d, e, selel, fam, dev, out, mult, fit, pfile, tmod) {
       cb_prefPut (g_selel, "alt_out", out, FALSE);
       cb_prefPut (g_selel, "technology", fam, FALSE);
       cb_prefPut (g_selel, "alt_mult_dev", mult, FALSE);
       cb_prefPut (g_selel, "alt_prev_fit", fit, FALSE);
       cb_prefPut (g_selel, "alt_progfile", pfile, FALSE);
       cb_prefPut (g_selel, "alt_timmod", tmod, FALSE);
       partPutpref (dev);
       cb_makeDochain ("altera_routed", g_selel, altera_rest); },
             "AlteraRouted", "Create Altera routed design",
    p_selected (),
    p_family ("fl_altlist"),
    p_fpgapart (PartFile),             
    vcWv.Param ("Output filename", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_out");,
      vfpgaNotEmptyString.fac,
      "Enter name for Altera chip and timing netlist file"),
    vcWv.Param ("Use Multiple Devices", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_mult_dev");,
      vcPickEnum.fac, "Select Yes to use Altera design partitioner"),
    vcWv.Param ("Use Previous Fit", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_prev_fit");,
      vcPickEnum.fac, "Select Yes to maintain pinout from previous results"),
    vcWv.Param ("Programming File", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_progfile");,
      vcPickEnum.fac, "Create device programming file"),
    vcWv.Param ("Timing model", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_timmod");, vcPickEnum.fac,
      "Create simulation timing model after place and route"));

vcCmd.VerifyParams (c_alt_routed, 
                    func (d,e,selel,fam,dev,mult,fit,pfile,tmod,out)
                    partIsSufficient(PartFile,dev,FALSE,FALSE,FALSE););

/*** Option commands ***/


c_opt_altsyn := cg_vdes ("AlteraSynthesis", "Altera synthesis options",
   "fl_altlist", "altera_gates", FALSE);

c_opt_altopt := vcWv.Cmd (fac, 
   func (d, e, selel, opt, min, slew, xor, turbo, pexp, expf, cas, car) {
       cb_prefPut (g_selel, "alt_opt", opt, FALSE);
       cb_prefPut (g_selel, "alt_min", min, FALSE);
       cb_prefPut (g_selel, "alt_slew", slew, FALSE);
       cb_prefPut (g_selel, "alt_xor", xor, FALSE);
       cb_prefPut (g_selel, "alt_turbo", turbo, FALSE);
       cb_prefPut (g_selel, "alt_par_exp", pexp, FALSE);
       cb_prefPut (g_selel, "alt_exp_fac", expf, FALSE);
       cb_prefPut (g_selel, "alt_cascade", cas, FALSE);
       cb_prefPut (g_selel, "alt_carry", car, TRUE); },
                          "AlteraOptimization", "Altera optimization options",
    p_selected (),
    vcWv.Param ("Optimization", vfpgaAltOptVT, vcWv.Dynamic,
                func (d) cb_prefGet (g_selel, "alt_opt");,
                "Select optimization priority"),
    vcWv.Param ("Minimization", vfpgaAltMinVT, vcWv.Dynamic,
                func (d) cb_prefGet (g_selel, "alt_min");,
                "Select logic minimization routine"),
    vcWv.Param ("Slow Slew Rate", vfpgaDefYesNoVT, vcWv.Dynamic,
                func (d) cb_prefGet (g_selel, "alt_slew");,
                "Select Yes for slow output transition"),
    vcWv.Param ("XOR Synthesis", vfpgaDefYesNoVT, vcWv.Dynamic,
                func (d) cb_prefGet (g_selel, "alt_xor");,
                "Select Yes for XOR synthesis"),
    vcWv.Param ("Turbo Bit", vfpgaDefYesNoVT, vcWv.Dynamic,
                func (d) cb_prefGet (g_selel, "alt_turbo");,
                "Select Yes for Turbo bit"),
    vcWv.Param ("Parallel Expanders", vfpgaDefYesNoVT, vcWv.Dynamic,
                func (d) cb_prefGet (g_selel, "alt_par_exp");,
                "Select Yes to utilize parallel expanders"),
    vcWv.Param ("Expander Factoring", vfpgaDefYesNoVT, vcWv.Dynamic,
                func (d) cb_prefGet (g_selel, "alt_exp_fac");,
                "Select Yes to utilize expander factoring"),
    vcWv.Param ("Cascade Chain", vfpgaAltChnVT, vcWv.Dynamic,
                func (d) cb_prefGet (g_selel, "alt_cascade");,
                "Select cascade chain"),
    vcWv.Param ("Carry Chain", vfpgaAltChnVT, vcWv.Dynamic,
                func (d) cb_prefGet (g_selel, "alt_carry");,
                "Select carry chain"));

c_opt_altimp := vcWv.Cmd (fac, 
   func (d, e, selel, fam, dev, mult, fit, res_cells, res_pins, jed, rpt) {
       cb_prefPut (g_selel, "technology", fam, FALSE);
       cb_prefPut (g_selel, "alt_mult_dev", mult, FALSE);
       cb_prefPut (g_selel, "alt_prev_fit", fit, FALSE);
       cb_prefPut (g_selel, "alt_reserve_cells", res_cells, FALSE);
       cb_prefPut (g_selel, "alt_reserve_pins", res_pins, FALSE); 
       cb_prefPut (g_selel, "alt_jedec", jed, FALSE);
       cb_prefPut (g_selel, "alt_report", rpt, TRUE); },
    "AlteraImplementation", "Altera implementation options",
    p_selected (),
    p_family ("fl_altlist"),
    p_fpgapart (PartFile),             
    vcWv.Param ("Use Multiple Devices", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_mult_dev");,
      vcPickEnum.fac, "Select Yes to use Altera design partitioner"),
    vcWv.Param ("Use Previous Fit", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_prev_fit");,
      vcPickEnum.fac, "Select Yes to maintain pinout from previous results"),
    vcWv.Param ("Reserve Logic Cells (%)", vlIntVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_reserve_cells");,
      "Enter percentage of logic cells to reserve"),
    vcWv.Param ("Reserve User Pins (%)", vlIntVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_reserve_pins");,
      "Enter percentage of user pins to reserve"),
    vcWv.Param ("Generate Jedec", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_jedec");,
      vcPickEnum.fac, "Select Yes to generate JEDEC programming file"),
    vcWv.Param ("Compiler Report", vfpgaAltRptVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "alt_report");,
      vcPickMask.fac, "Select contents of compiler report", columns <- 1));

vcCmd.VerifyParams (c_opt_altimp, 
                    func(d,e,selel,fam,dev,mult,fit,res_cells,res_pins,jed,rpt)
                    partIsSufficient(PartFile,dev,FALSE,FALSE,FALSE););

/*** Help messages ***/

vcCmd.SetHelp (c_alt_wir,      "Create WIR file for Altera design");
vcCmd.SetHelp (c_alt_routed,   "Place and route Altera design");
vcCmd.SetHelp (c_opt_altsyn,   "Set options for Altera synthesis");
vcCmd.SetHelp (c_opt_altopt,   "Set options for Altera optimization");
vcCmd.SetHelp (c_opt_altimp,   "Set device for Altera implementation");

/* Disable all options but synthesis when not top level */
function altSettype (type) {
    if (g_altsetup == TRUE) {
        somecmds := list (c_alt_routed, c_opt_altopt, c_opt_altimp);
        istop := cb_loadIstop (g_selel);
        if (!istop)
            menuEnableList (vlOnOff.Off, "Only permitted on top level file",
                            somecmds);
        else menuEnableList (vlOnOff.On, "", somecmds); 
    }
}

/*** Utility commands ***/


c_util_alttut := vcWv.Cmd (fac,
   func (d, e) menuCopyTutFiles (list ("altera", "max2sim"),
      list ("fsm_case.vhd", "accum.abl", "reg5.abl", "sym/accum.1",
         "sym/reg5.1"),
      list ("accum.bl0", "accum.xrf", "accum.tt2", "accum.dmc",
            "accum.vhd", "sym/accum.pin", "wir/accum.1", "sch/accum.1",
            "reg5.bl0", "reg5.xrf", "reg5.tt2",
         "reg5.dmc", "reg5.vhd", "sym/reg5.pin", "wir/reg5.1",
         "sch/reg5.1", "wir/fsm_case.1", "sch/fsm_case.1"));,
   "CopyAltTut", "Copy files for online Altera tutorial");


function fl_altdel () { list (
   list (cons ("ABL", "abl"), cons ("BAK", "bak"), cons ("BL0", "bl0")),
   list (cons ("CHP", "chp"), cons ("CMD", "cmd"), cons ("CNF", "cnf")),
   list (cons ("DMC", "dmc"), cons ("EDF", "edf"), cons ("EDO", "edo")),
   list (cons ("ESI", "esi"), cons ("ESO", "eso"), cons ("HIF", "hif")),
   list (cons ("FIT", "fit"), cons ("INI", "ini"), cons ("LIS", "lis")),
   list (cons ("MTF", "mtf"), cons ("ORD", "ord"), cons ("PIN", "pin")),
   list (cons ("POF", "pof"), cons ("PRB", "prb"), cons ("SNF", "snf")),
   list (cons ("SOF", "sof"), cons ("TDF", "tdf"), cons ("TMV", "tmv")),
   list (cons ("TT2", "tt2"), cons ("RPT", "rpt"), cons ("XRF", "xrf"))); }


c_util_altdel := cg_util_delint ("DelAltIntFiles",
   "Delete Altera intermediate files", "fl_altdel",
   "bak bl0 chp cnf dmc esi eso hit lis mtf ord pin prb sel snf tmv tt2 xrf");
