/* This file contains the Xilinx specific ViewFPGA information */


/*** Global declarations ***/


function fl_xilfam () { list (list (
   cons ("Xilinx 2000", "xc2000"),
   cons ("Xilinx 3000", "xc3000"),
   cons ("Xilinx 4000", "xc4000"))); }

function fl_xil23fam () { list (list (
   cons ("Xilinx 2000", "xc2000"),
   cons ("Xilinx 3000", "xc3000"))); }

function fl_xil4bits () { list (
   cons ("None", "none"),
   cons ("C clock, No sync", "Cclk_Nosync"),
   cons ("C clock, No sync", "Cclk_Nosync"),
   cons ("U clock, Sync", "Cclk_Sync"),
   cons ("U clock, Sync", "Uclk_Sync")); }

function fl_xil23map () { list (
   cons ("Pin gain", "-c"),
   cons ("Direct input", "-i"),
   cons ("Signal combining", "-s"),
   cons ("More levels", "-r"),
   cons ("Ignore CLB maps", "-m"),
   cons ("Ignore location", "-n"),
   cons ("Ignore register ordering", "-o")); }


vcWv.MakeEnum ("vfpgaXilInlevVT", list ("TTL", "CMOS"));
vcWv.MakeEnum ("vfpgaXilReadbackVT", list ("OnCommand", "Once", "Never"));
vcWv.MakeEnum ("vfpgaXilTimeVT", list ("Early", "Late"));
vcWv.MakeEnum ("vfpgaXilXtalVT", list ("Disable", "Enable", "Div2"));
vcWv.MakeEnum ("vfpgaXilOutfmtVT", list ("Bitstream", "ASCII", "Mask"));
vcWv.MakeEnum ("vfpgaXilPhaseVT", list ("ALL", "PlaceOnly", "RouteOnly",
   "TimingImprove"));
vcWv.MakeEnum ("vfpgaXilRalgoVT", list ("One", "Two", "Three", "Four"));


/*** Initialization functions ***/


function xilGetXACT (startup) {
   path := vscript.getenv ("XACT");
   if (!null (path))
      if (!fexists (vscript.vsConvPath (path+"/data/partlist.xct"))) path = nil;
   if (null (path)) {
      path = vscript.vsSrchdirs ("xact", vscript.vsDIR);
      if ((!null (path)) && startup &&
         (fexists (vscript.vsConvPath (path + "/data/partlist.xct")))) {
            vscript.putenv ("XACT", path);
            cb_mainDbg (1, "setenv XACT " + path); }
      else path = nil; }
   path; }


function xilSetup () {
   if (null (xilGetXACT (TRUE)))
      cb_mainErr (1701, "Error", nil, "Unable to set up Xilinx environment");

   /* Check for xc2 or xc3 or xc4 alias */
   ok_alias := FALSE;
   if (cb_loadIfalias ("xc2000", FALSE)) ok_alias = TRUE;
   else if (cb_loadIfalias ("xc3000", FALSE)) ok_alias = TRUE;
   else if (cb_loadIfalias ("xc4000", FALSE)) ok_alias = TRUE;
   if (!ok_alias)
      cb_mainErr (1712, "Error", nil, "Unable to find Xilinx libraries");

   /* Create "xnf" subdirectory if not present */
   if (!fexists ("xnf")) vscript.vsCreate ("xnf", vsAttr.Dir);

   /* Register valid types */
   if (!ok_alias) typelist := nil;
   else typelist := list ("ABEL", "VHDL", "JEDEC", "LPM", "Xilinx");
   menuValidTypes ("Xilinx", typelist,
      list (c_xil_wir, c_xil23_routed, c_xil4_routed, c_opt_xilsyn,
         c_opt_xildev, c_opt_xilbits, c_opt_xil23apr)); }


function xilDefallatts (prefdb, startup) {
   prefDefoneatt ("xil_out", vlStringVT,
      "XILINX: filename for output LCA and timing model", "", prefdb, startup);
   prefDefoneatt ("xil_timmod", vlYesNoVT, "XILINX: create timing model",
      vlYesNo.Yes, prefdb, startup);
   prefDefoneatt ("xil_mapopt", vlStringVT,
      "XILINX xnfmap: packing options", "", prefdb, startup);
   prefDefoneatt ("xil_guide", vlStringVT,
      "XILINX: map/place/route guide filename", "", prefdb, startup);
   prefDefoneatt ("xil_constr", vlStringVT,
      "XILINX: constraint filename", "", prefdb, startup);
   prefDefoneatt ("xil_phase", vfpgaXilPhaseVT,
      "XILINX apr: phases for place and route", vfpgaXilPhase.ALL,
      prefdb, startup);
   prefDefoneatt ("xil_ralgo", vfpgaXilRalgoVT,
      "XILINX apr: routing algorithm", vfpgaXilRalgo.Four, prefdb, startup);
   prefDefoneatt ("xil_routry", vlIntVT, "XILINX apr: number of routing tries",
      3, prefdb, startup);
   prefDefoneatt ("xil_seed", vlStringVT, "XILINX apr: random seed for routing",
      "", prefdb, startup);
   prefDefoneatt ("xil_xblox", vlYesNoVT,
      "XILINX xblox: run XBLOX on design", vlYesNo.No, prefdb, startup);
   prefDefoneatt ("xil_tie", vlYesNoVT,
      "XILINX makebits: tie off unused nets",
      vlYesNo.Yes, prefdb, startup);
   prefDefoneatt ("xil_pullup", vlYesNoVT,
      "XILINX makebits: enable pullup on Done pad",
      vlYesNo.No, prefdb, startup);
   prefDefoneatt ("xil_inlev", vfpgaXilInlevVT,
      "XILINX makebits: input pad voltage level",
      vfpgaXilInlev.TTL, prefdb, startup);
   prefDefoneatt ("xil_done", vfpgaXilTimeVT,
      "XILINX makebits: done signal assertion time",
      vfpgaXilTime.Early, prefdb, startup);
   prefDefoneatt ("xil_reset", vfpgaXilTimeVT,
      "XILINX makebits: reset signal assertion time",
      vfpgaXilTime.Late, prefdb, startup);
   prefDefoneatt ("xil_read", vfpgaXilReadbackVT,
      "XILINX makebits: readback setup",
      vfpgaXilReadback.OnCommand, prefdb, startup);
   prefDefoneatt ("xil_xtal", vfpgaXilXtalVT,
      "XILINX makebits: enable for crystal oscillator",
      vfpgaXilXtal.Disable, prefdb, startup);
   prefDefoneatt ("xil_outfmt", vfpgaXilOutfmtVT,
      "XILINX makebits: format for bitstream output",
      vfpgaXilOutfmt.Bitstream, prefdb, startup);
   prefDefoneatt ("xil_cfg4k", vlStringVT,
      "XILINX makebits: 4k family configuration", "none", prefdb, startup);
   prefDefoneatt ("xil_bit4karg", vlStringVT,
      "XILINX makebits: additional arguments for 4k", "", prefdb, startup); }


/*** File, tool and chain definitions ***/


cb_cfgAddFile ("xnfin",     list (list ("xnf",  "%base",   "xnf")));
cb_cfgAddFile ("xff",       list (list ("%dot", "%base",   "xff")));
cb_cfgAddFile ("xtf",       list (list ("%dot", "%base",   "xtf")));
cb_cfgAddFile ("xtg",       list (list ("%dot", "%base",   "xtg")));
cb_cfgAddFile ("xg",        list (list ("%dot", "%base",   "xg")));
cb_cfgAddFile ("map",       list (list ("%dot", "%base",   "map")));
cb_cfgAddFile ("lcain",     list (list ("%dot", "%base",   "lca")));
cb_cfgAddFile ("lcaout",    list (list ("%dot", "xil_out", "lca")));
cb_cfgAddFile ("xnfout",    list (list ("%dot", "xil_out", "xnf")));
cb_cfgAddFile ("bitstream", list (list ("%dot", "xil_out", "bit")));
cb_cfgAddFile ("xilwir",    list (list ("%wir", "xil_out", "1")));
cb_cfgAddFile ("xilvsm",    list (list ("%dot", "xil_out", "vsm")));

cb_cfgAddTool ("wir2xnf",   list ("wir"), list ("xnfin"), nil);
cb_cfgAddTool ("xnfmerge",  list ("xnfin"), list ("xff"), nil);
cb_cfgAddTool ("xnfprep",   list ("xff"), list ("xtf"), nil);
cb_cfgAddTool ("xnfmap",    list ("xtf"), list ("map"),
                            list ("xil_guide", "xil_mapopt"));
cb_cfgAddTool ("map2lca",   list ("map"), list ("lcain"),
                            list ("fpga_die", "fpga_package"));
cb_cfgAddTool ("apr",       list ("lcain"), list ("lcaout"),
                            list ("xil_phase", "xil_ralgo", "xil_routry",
                               "xil_seed", "fpga_die", "fpga_package",
                               "xil_constr"));
cb_cfgAddTool ("ppr",       list ("xtf"), list ("lcaout"),
                            list ("xil_out", "xil_guide", "xil_constr"));
cb_cfgAddTool ("xdelay4k",  nil, list ("lcaout"), nil);
cb_cfgAddTool ("makebits",  list ("lcaout"), list ("bitstream"),
                            list ("xil_inlev", "xil_done", "xil_reset",
                               "xil_read", "xil_xtal", "xil_outfmt",
                               "xil_tie", "fpga_speed", "xil_bit4karg",
                               "xil_cfg4k"));
cb_cfgAddTool ("lca2xnf",   list ("lcaout"), list ("xnfout"), nil);
cb_cfgAddTool ("xnfba",     nil, list ("xnfout"), nil);
cb_cfgAddTool ("xnf2wir",   list ("xnfout"), list ("xilwir"), nil);
cb_cfgAddTool ("xilvsm",    list ("xilwir"), list ("xilvsm"), nil);
cb_cfgAddTool ("xilvsmup",  nil, list ("xilvsm"), nil);
cb_cfgAddTool ("xiloadvsm", nil, nil, nil);

/* Extra tools for XBLOX chains */
cb_cfgAddTool ("xnfprep1b", list ("xff"), list ("xtg"), nil);
cb_cfgAddTool ("xblox", list ("xtg"), list ("xg"), nil);
cb_cfgAddTool ("xnfprep2b", list ("xg"), list ("xtf"), nil);

xilBefore := list ("wir2xnf", "xnfmerge", "xnfprep");
xilAfter := list ("lca2xnf","xnfba","xnf2wir","xilvsm","xilvsmup","xiloadvsm");
xilAPR := list ("xnfmap", "map2lca", "apr", "makebits");
xilPPRtiming := list ("ppr", "makebits", "xdelay4k");
xil3kAL := list ("xnfmap", "ppr", "makebits", "xdelay4k");
xilBeforeBlox := list ("wir2xnf","xnfmerge","xnfprep1b","xblox","xnfprep2b");


/* Return true if 3k family part in sub-family "3000A" or "3000L" */
function xil3kIsAL (die) {
   lastchar := die[string_length(die)-1];
   ((lastchar == 'A') || (lastchar == 'L')); }

function xilBuildChain (inlist...) {
   final := nil; declare l;
   for (; !null(inlist); inlist=cdr(inlist)) {
      if (isstring (car (inlist))) final = cons (car (inlist), final);
      else for (l=car(inlist);!null(l);l=cdr(l)) final = cons(car (l),final); }
   reverse_destructive (final); }

/*** 3k tools and chains ***/

cb_cfgAddChain ("xiltimod", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBefore, xilAPR, xilAfter));
cb_cfgAddChain ("xiltimod", "abel", xilBuildChain ("ahdl2pla", "mergett",
   "p2v", "vdes_vhdl", "vdes_viewgen", xilBefore, xilAPR, xilAfter));
cb_cfgAddChain ("xiltimod", "vhdl", xilBuildChain ("vdes_vhdl", "vdes_viewgen",
   xilBefore, xilAPR, xilAfter));
cb_cfgAddChain ("xilbits", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBefore, xilAPR));
cb_cfgAddChain ("xilbits", "abel", xilBuildChain ("ahdl2pla", "mergett", "p2v",
   "vdes_vhdl", "vdes_viewgen", xilBefore, xilAPR));
cb_cfgAddChain ("xilbits", "vhdl", xilBuildChain ("vdes_vhdl", "vdes_viewgen",
   xilBefore, xilAPR));

cb_cfgAddTgtname ("xiltimod", "Xilinx 2k/3k timing model");
cb_cfgAddTgtname ("xilbits", "Xilinx 2k/3k bitstream");

/*** 4k tools and chains ***/

cb_cfgAddChain ("xiltimod4k", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBefore, xilPPRtiming, xilAfter));
cb_cfgAddChain ("xiltimod4k", "abel", xilBuildChain ("ahdl2pla", "mergett",
   "p2v", "vdes_vhdl", "vdes_viewgen", xilBefore, xilPPRtiming, xilAfter));
cb_cfgAddChain ("xiltimod4k", "vhdl", xilBuildChain ("vdes_vhdl",
   "vdes_viewgen", xilBefore, xilPPRtiming, xilAfter));
cb_cfgAddChain ("xilbits4k", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBefore, "ppr", "makebits"));
cb_cfgAddChain ("xilbits4k", "abel", xilBuildChain ("ahdl2pla", "mergett",
   "p2v", "vdes_vhdl", "vdes_viewgen", xilBefore, "ppr", "makebits"));
cb_cfgAddChain ("xilbits4k", "vhdl", xilBuildChain ("vdes_vhdl",
   "vdes_viewgen", xilBefore, "ppr", "makebits"));

cb_cfgAddTgtname ("xiltimod4k", "Xilinx 4k timing model");
cb_cfgAddTgtname ("xilbits4k", "Xilinx 4k bitstream");

cb_cfgAddChain ("xiltimod4kb", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBeforeBlox, xilPPRtiming, xilAfter));
cb_cfgAddChain ("xiltimod4kb", "abel", xilBuildChain ("ahdl2pla", "mergett",
   "p2v", "vdes_vhdl", "vdes_viewgen", xilBeforeBlox, xilPPRtiming, xilAfter));
cb_cfgAddChain ("xiltimod4kb", "vhdl", xilBuildChain ("vdes_vhdl",
   "vdes_viewgen", xilBeforeBlox, xilPPRtiming, xilAfter));
cb_cfgAddChain ("xilbits4kb", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBeforeBlox, "ppr", "makebits"));
cb_cfgAddChain ("xilbits4kb", "abel", xilBuildChain ("ahdl2pla", "mergett",
   "p2v", "vdes_vhdl", "vdes_viewgen", xilBeforeBlox, "ppr", "makebits"));
cb_cfgAddChain ("xilbits4kb", "vhdl", xilBuildChain ("vdes_vhdl",
   "vdes_viewgen", xilBeforeBlox, "ppr", "makebits"));

cb_cfgAddTgtname ("xiltimod4kb", "Xilinx 4k timing model");
cb_cfgAddTgtname ("xilbits4kb", "Xilinx 4k bitstream");

/*** 3k A,L chains ***/

cb_cfgAddChain ("xiltimod3k", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBefore, xil3kAL, xilAfter));
cb_cfgAddChain ("xiltimod3k", "abel", xilBuildChain ("ahdl2pla", "mergett",
   "p2v", "vdes_vhdl", "vdes_viewgen", xilBefore, xil3kAL, xilAfter));
cb_cfgAddChain ("xiltimod3k", "vhdl", xilBuildChain ("vdes_vhdl",
   "vdes_viewgen", xilBefore, xil3kAL, xilAfter));
cb_cfgAddChain ("xilbits3k", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBefore, xil3kAL));
cb_cfgAddChain ("xilbits3k", "abel", xilBuildChain ("ahdl2pla", "mergett",
   "p2v", "vdes_vhdl", "vdes_viewgen", xilBefore, xil3kAL));
cb_cfgAddChain ("xilbits3k", "vhdl", xilBuildChain ("vdes_vhdl",
   "vdes_viewgen", xilBefore, xil3kAL));

cb_cfgAddChain ("xiltimod3kb", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBeforeBlox, xil3kAL, xilAfter));
cb_cfgAddChain ("xiltimod3kb", "abel", xilBuildChain ("ahdl2pla", "mergett",
   "p2v", "vdes_vhdl", "vdes_viewgen", xilBeforeBlox, xil3kAL, xilAfter));
cb_cfgAddChain ("xiltimod3kb", "vhdl", xilBuildChain ("vdes_vhdl",
   "vdes_viewgen", xilBeforeBlox, xil3kAL, xilAfter));
cb_cfgAddChain ("xilbits3kb", "sch", xilBuildChain ("savesrc", "%fpganet",
   xilBeforeBlox, xil3kAL));
cb_cfgAddChain ("xilbits3kb", "abel", xilBuildChain ("ahdl2pla", "mergett",
   "p2v", "vdes_vhdl", "vdes_viewgen", xilBeforeBlox, xil3kAL));
cb_cfgAddChain ("xilbits3kb", "vhdl", xilBuildChain ("vdes_vhdl",
   "vdes_viewgen", xilBeforeBlox, xil3kAL));

cb_cfgAddTgtname ("xiltimod3k", "Xilinx 3k A,L timing model");
cb_cfgAddTgtname ("xilbits3k", "Xilinx 3k A,L bitstream");


/*** Action functions and error routines ***/


function a_wir2xnf (name) {
   "wir2xnf " + name + " -b -od xnf -p " + cb_prefGet (name, "fpga_die") +
   cb_prefGet (name, "fpga_package") + " " + name + ".xnf"; }

function e_wir2xnf (name) { errWrap (1702, "Xilinx XNF netlister", name); }


function a_xnfmerge (name) {
   "xnfmerge -a -d xnf xnf/" + name + ".xnf " + name + ".xff"; }

function e_xnfmerge (name) { errWrap (1717, "Xilinx XNF merger", name); }


function a_xnfprep (name) {
   "xnfprep " + name + ".xff " + name + ".xtf"; }

function e_xnfprep (name) { errWrap (1718,"Xilinx design rule checker",name); }


function a_xnfmap (name) {
   str := "xnfmap " + name + ".xtf " + name + ".map";
   str += " -p " + cb_prefGet (name, "fpga_die") +
      cb_prefGet (name, "fpga_package");
   x := cb_prefGet (name, "xil_guide");
   if (!string_eq (x, "")) str += " -k";
   str += " " + cb_prefGet (name, "xil_mapopt"); }

function e_xnfmap (name) { errWrap (1703, "Xilinx XNF packer", name); }


function a_map2lca (name) {
   "map2lca " + name + " -p " + cb_prefGet (name, "fpga_die") +
      cb_prefGet (name, "fpga_package"); }

function e_map2lca (name) {
   errWrap (1704, "Xilinx MAP to LCA translator", name); }


function a_apr (name) {
   str := "apr " + name + " " + cb_prefGet (name, "xil_out") + " -w";

   x := cb_prefGet (name, "xil_phase");
   if      (x == vfpgaXilPhase.ALL) {;}
   else if (x == vfpgaXilPhase.PlaceOnly)     str += " -j";
   else if (x == vfpgaXilPhase.RouteOnly)     str += " -l";
   else if (x == vfpgaXilPhase.TimingImprove) str += " -t";

   x = cb_prefGet (name, "xil_guide");
   if (!string_eq (x, "")) str += " -g " + x;
   x = cb_prefGet (name, "xil_constr");
   if (!string_eq (x, "")) str += " -c " + x;

   x = cb_prefGet (name, "xil_ralgo");
   if      (x == vfpgaXilRalgo.One)   str += " -r 1";
   else if (x == vfpgaXilRalgo.Two)   str += " -r 2";
   else if (x == vfpgaXilRalgo.Three) str += " -r 3";
   else if (x == vfpgaXilRalgo.Four)  {;} /*default*/

   x = cb_prefGet (name, "xil_routry");
   if (x != 3) str += sprintf (" -a %d", x);
   x = cb_prefGet (name, "xil_seed");
   if (!string_eq (x, "")) str += " -s" + x;

   str; }

function e_apr (name) {
   errWrap (1705, "Xilinx automatic place and route", name); }


function a_makebits (name) {
   str := "makebits ";

   /* Basic options */
   x := cb_prefGet (name, "xil_outfmt");
   if (x == vfpgaXilOutfmt.ASCII) str += " -b";
   else if (x == vfpgaXilOutfmt.Mask) str += " -m";
   if (cb_prefGet (name, "xil_tie") == vlYesNo.Yes) str += " -t";

   /* 2k and 3k options */
   if (!string_eq (g_family, "xc4000")) {
      if (cb_prefGet (name, "xil_inlev") == vfpgaXilInlev.CMOS) str += " -c";
      x = cb_prefGet (name, "xil_read");
      if (x == vfpgaXilReadback.Never) str += " -r0";
      else if (x == vfpgaXilReadback.Once) str += " -r1";
      if (cb_prefGet (name, "xil_pullup") == vlYesNo.No) str += " -p";  }

   /* 3k specific options */
   if (string_eq (g_family, "xc3000")) {
      x = cb_prefGet (name, "xil_xtal");
      if (x == vfpgaXilXtal.Enable) str += " -s1";
      else if (x == vfpgaXilXtal.Div2) str += " -s2";
      if (cb_prefGet (name, "xil_done")  == vfpgaXilTime.Early) str += " -x";
      if (cb_prefGet (name, "xil_reset") == vfpgaXilTime.Early) str += " -y"; }

   /* 4k specific options */
   if (string_eq (g_family, "xc4000")) {
      x = cb_prefGet (name, "xil_cfg4k");
      if (!string_eq (x, "none")) str += " -f " + x;
      x = cb_prefGet (name, "xil_bit4karg");
      if (!string_eq (x, "")) str += " " + x; }

   str += " " + cb_prefGet (name, "xil_out"); }

function e_makebits (name) {
   errWrap (1706, "Xilinx bitstream generator",
      cb_prefGet (name, "xil_out")); }


function a_lca2xnf (name) { "lca2xnf -w " + cb_prefGet (name, "xil_out"); }


function e_lca2xnf (name) {
   errWrap (1710, "Xilinx LCA to XNF translator",
      cb_prefGet (name, "xil_out")); }


function a_xnfba (name) {
   "xnfba " + name + " " + cb_prefGet (name, "xil_out"); }

function e_xnfba (name) {
   errWrap (1707, "Xilinx timing back annotator",
      cb_prefGet (name, "xil_out")); }


function a_xnf2wir (name) { "xnf2wir " + cb_prefGet (name, "xil_out"); }

function e_xnf2wir (name) {
   errWrap (1708, "Xilinx XNF to WIR netlister",
      cb_prefGet (name, "xil_out")); }


function a_xilvsmup (name) {
   "vsmupd " + name + " " + cb_prefGet (name, "xil_out"); }

function e_xilvsmup (name)
   errWrap (1719, "Xilinx vsm netlist updater", cb_prefGet (name, "xil_out"));


function a_xilvsm (name) { "vsm " + cb_prefGet (name, "xil_out"); }

function e_xilvsm (name)
   errWrap (1423, "Viewsim netlister", cb_prefGet (name, "xil_out"));


function a_xiloadvsm (name) { a_loadvsm (cb_prefGet (name, "xil_out")); }


function a_ppr (name) {
   str := "ppr " + name + " outfile=" + cb_prefGet (name, "xil_out");
   x := cb_prefGet (name, "xil_guide");
   if (!string_eq (x, "")) str += " guide=" + x;
   x = cb_prefGet (name, "xil_constr");
   if (!string_eq (x, "")) str += " cstfile=" + x;

   str; }

function e_ppr (name) errWrap (1711, "Xilinx automatic place and route", name);


function a_xdelay4k (name) "xdelay -d -w " + cb_prefGet (name, "xil_out");

function e_xdelay4k (name) errWrap (1720, "Xilinx 4k delay extractor ", name);


function a_xnfprep1b (name) "xnfprep " + name + ".xff " + name + ".xtg";

function e_xnfprep1b (name) e_xnfprep (name);


function a_xblox (name) "xblox " + name + ".xtg " + name + ".xg";

function e_xblox (name) errWrap (1721, "Xilinx XBLOX optimizer", name);


function a_xnfprep2b (name) "xnfprep " + name + ".xg " + name + ".xtf";

function e_xnfprep2b (name) e_xnfprep (name);


/*** Commands ***/


c_xil_wir := cg_vdes ("XilWirFile", "Create Xilinx WIR file", "fl_xilfam",
                      "fpganet", TRUE);


c_xil23_routed := vcWv.Cmd (fac,
   func (d, e, selel, fam, dev, outname, guide, constr, xblox, tie, tmod) {

      /* Put preferences */
      if (!string_eq (fam, "nil"))
         cb_prefPut (g_selel, "technology", fam, FALSE);
      cb_prefPut (g_selel, "xil_out", outname, FALSE);
      cb_prefPut (g_selel, "xil_guide", guide, FALSE);
      cb_prefPut (g_selel, "xil_constr", constr, FALSE);
      cb_prefPut (g_selel, "xil_xblox", xblox, FALSE);
      cb_prefPut (g_selel, "xil_tie", tie, FALSE);
      cb_prefPut (g_selel, "xil_timmod", tmod, FALSE);
      partPutpref (dev);

      /* Decide which chain to execute */
      if (tmod == vlYesNo.Yes) name := "xiltimod";
      else name := "xilbits";
      if (xil3kIsAL (cb_prefGet (selel, "fpga_die"))) {
         name += "3k";
         if (xblox == vlYesNo.Yes) name += "b"; }

      /* Execute it */
      cb_makeDochain (name, g_selel, nil); },

   "Xilinx23Routed", "Create Xilinx 2k/3k routed design",
   p_selected (),
   p_family ("fl_xil23fam"),
   p_fpgapart ("vfpgaxil.ptl"),
   vcWv.Param ("Output filename", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_out");,
      "Enter name for output LCA and netlist file"),
   vcWv.Param ("Guide filename", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_guide");,
      "Enter optional name for guide file"),
   vcWv.Param ("Constraint filename", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_constr");,
      "Enter optional name for constraint file"),
   vcWv.Param ("Run XBLOX", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_xblox");,
      vcPickEnum.fac, "Run XBLOX tool on design"),
   vcWv.Param ("Tie unused nets", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_tie");,
      vcPickEnum.fac, "Tie unused nets to known voltage"),
   vcWv.Param ("Timing model", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_timmod");,
      vcPickEnum.fac, "Create simulation timing model after place and route"));


vcCmd.VerifyParams (c_xil23_routed,
   func (d, e, selel, fam, dev, outname, guide, constr, xblox, tie, tmod) {
      if (tmod == vlYesNo.Yes) {
         if (string_eq (outname, "")) {
            s := "Output filename required for implementation";
            cb_mainErrLogonly (1722, "Error", nil, s);
            vlEx.Throw (vlEx.FAILURE, vlErr.Create (nil, "vfpga", 1722,
               vlErrSev.Error, s)); }
         partIsSufficient ("vfpgaxil.ptl", dev, TRUE, TRUE, TRUE); }

      /* For XBLOX flow, only 3k A and L parts are allowed */
      if (xblox == vlYesNo.Yes) {
         declare die;
         string_token (dev, &die,   " ");
         if (!xil3kIsAL (die)) {
            s := "XBLOX flow only supported for 3k A and L families";
            cb_mainErrLogonly (1723, "Error", nil, s);
            vlEx.Throw (vlEx.FAILURE, vlErr.Create (nil, "vfpga", 1723,
               vlErrSev.Error, s)); } } } );


c_xil4_routed := vcWv.Cmd (fac,
   func (d,e) {
      g_family = "xc4000"; vcCmd.Run (c_xil4_routsub, d, e, nil, FALSE); },
   "Xilinx4RoutInternal", "");

c_xil4_routsub := vcWv.Cmd (fac,
   func (d, e, selel, dev, outname, guide, constr, xblox, tie, tmod) {
      /* Put preferences */
      cb_prefPut (g_selel, "technology", "xc4000", FALSE);
      cb_prefPut (g_selel, "xil_out", outname, FALSE);
      cb_prefPut (g_selel, "xil_guide", guide, FALSE);
      cb_prefPut (g_selel, "xil_xblox", xblox, FALSE);
      cb_prefPut (g_selel, "xil_tie", tie, FALSE);
      cb_prefPut (g_selel, "xil_timmod", tmod, FALSE);
      partPutpref (dev);

      /* Execute chain */
      if (tmod == vlYesNo.Yes) name := "xiltimod4k";
      else name := "xilbits4k";
      if (xblox == vlYesNo.Yes) name += "b";
      cb_makeDochain (name, g_selel, nil); },

   "Xilinx4Routed", "Create Xilinx 4k routed design",
   p_selected (),
   p_fpgapart ("vfpgaxil.ptl"),
   vcWv.Param ("Output filename", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_out");,
      "Enter name for output LCA and netlist file"),
   vcWv.Param ("Guide filename", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_guide");,
      "Enter optional name for guide file"),
   vcWv.Param ("Constraint filename", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_constr");,
      "Enter optional name for constraint file"),
   vcWv.Param ("Run XBLOX", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_xblox");,
      vcPickEnum.fac, "Run XBLOX tool on design"),
   vcWv.Param ("Tie unused nets", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_tie");,
      vcPickEnum.fac, "Tie unused nets to known voltage"),
   vcWv.Param ("Timing model", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_timmod");,
      vcPickEnum.fac, "Create simulation timing model after place and route"));


vcCmd.VerifyParams (c_xil4_routsub,
   func (d, e, selel, dev, outname, guide, constr, xblox, tie, tmod) {
      if (tmod == vlYesNo.Yes) {
         if (string_eq (outname, "")) {
            s := "Output filename required for implementation";
            cb_mainErrLogonly (1722, "Error", nil, s);
            vlEx.Throw (vlEx.FAILURE, vlErr.Create (nil, "vfpga", 1722,
               vlErrSev.Error, s)); }
         partIsSufficient ("vfpgaxil.ptl", dev, TRUE, TRUE, TRUE); } } );


c_opt_xilsyn := cg_vdes ("XilSynthesis", "Xilinx synthesis options",
   "fl_xilfam", "fpganet", FALSE);


c_opt_xildev := vcWv.Cmd (fac,
   func (d, e, selel, fam, dev) {
      if (!string_eq (fam, "nil"))
         cb_prefPut (g_selel, "technology", fam, FALSE);
      partPutpref (dev); },
   "XilinxPackage", "Xilinx package options",
   p_selected (), p_family ("fl_xilfam"),
   p_fpgapart ("vfpgaxil.ptl"));


c_opt_xil23apr := vcWv.Cmd (fac,
   func (d, e, selel, mapopt, phase, ralgo, routry, seed) {
      cb_prefPut (g_selel, "xil_mapopt", mapopt, FALSE);
      cb_prefPut (g_selel, "xil_phase", phase, FALSE);
      cb_prefPut (g_selel, "xil_ralgo", ralgo, FALSE);
      cb_prefPut (g_selel, "xil_routry", routry, FALSE);
      cb_prefPut (g_selel, "xil_seed", seed, TRUE); },
   "Xilinx23Opts", "Xilinx 2k/3k implementation options",
   p_selected (),
   vcWv.Param ("Mapping options", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_mapopt");, vfpgaPopPick.fac,
      "Choose zero or more mapping (xnfmap) arguments",
      funclist <- "fl_xil23map", exclusive <- FALSE),
   vcWv.Param ("Execute phases", vfpgaXilPhaseVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_phase");,
      "Choose place and route phases to execute"),
   vcWv.Param ("Routing algorithm", vfpgaXilRalgoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_ralgo");,
      "Choose routing algorithm"),
   vcWv.Param ("Routing attempts", vlIntVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_routry");,
      "Enter number of attempts for routing"),
   vcWv.Param ("Random seed", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_seed");,
      "Seed for random numbers in routing"));


c_opt_xil2bits := vcWv.Cmd (fac,
   func (d, e, selel, pup, inlev, read, outfmt) {
      cb_prefPut (g_selel, "xil_pullup", pup,    FALSE);
      cb_prefPut (g_selel, "xil_inlev",  inlev,  FALSE);
      cb_prefPut (g_selel, "xil_read",   read,   FALSE);
      cb_prefPut (g_selel, "xil_outfmt", outfmt, TRUE); },
   "Xilinx2Bitstream", "Xilinx 2k bitstream options",
   p_selected (),
   vcWv.Param ("Done pullup", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_pullup");,
      vcPickEnum.fac, "Enable pullup on Done pad"),
   vcWv.Param ("Input level", vfpgaXilInlevVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_inlev");,
      "Select input level for pads"),
   vcWv.Param ("Readback", vfpgaXilReadbackVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_read");,
      "Select type of readback"),
   vcWv.Param ("Output format", vfpgaXilOutfmtVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_outfmt");,
      "Select output format for makebits"));


c_opt_xil3bits := vcWv.Cmd (fac,
   func (d, e, selel, pup, inlev, done, reset, xtal, read, outfmt) {
      cb_prefPut (g_selel, "xil_pullup", pup,    FALSE);
      cb_prefPut (g_selel, "xil_inlev",  inlev,  FALSE);
      cb_prefPut (g_selel, "xil_done",   done,   FALSE);
      cb_prefPut (g_selel, "xil_read",   read,   FALSE);
      cb_prefPut (g_selel, "xil_xtal",   xtal,   FALSE);
      cb_prefPut (g_selel, "xil_reset",  reset,  FALSE);
      cb_prefPut (g_selel, "xil_outfmt", outfmt, TRUE); },
   "Xilinx3Bitstream", "Xilinx 3k bitstream options",
   p_selected (),
   vcWv.Param ("Done pullup", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_pullup");,
      vcPickEnum.fac, "Enable pullup on Done pad"),
   vcWv.Param ("Input level", vfpgaXilInlevVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_inlev");,
      "Select input level for pads"),
   vcWv.Param ("Done signal", vfpgaXilTimeVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_done");,
      "Select Done signal transition time"),
   vcWv.Param ("Reset signal", vfpgaXilTimeVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_reset");,
      "Select Reset signal transition time"),
   vcWv.Param ("Crystal oscillator", vfpgaXilXtalVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_xtal");,
      "Select type of crystal oscillator"),
   vcWv.Param ("Readback", vfpgaXilReadbackVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_read");,
      "Select type of readback"),
   vcWv.Param ("Output format", vfpgaXilOutfmtVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_outfmt");,
      "Select output format for makebits"));


c_opt_xil4bits := vcWv.Cmd (fac,
   func (d, e, selel, cfg, outfmt, addl) {
      cb_prefPut (g_selel, "xil_cfg4k", cfg, FALSE);
      cb_prefPut (g_selel, "xil_bit4karg", addl, FALSE);
      cb_prefPut (g_selel, "xil_outfmt", outfmt, TRUE); },
   "Xilinx4Bitstream", "Xilinx 4k bitstream options",
   p_selected (),
   vcWv.Param ("Configuration", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_cfg4k");,
      vfpgaVertPick.fac, "Configuration format",
         funclist <- "fl_xil4bits"),
   vcWv.Param ("Output format", vfpgaXilOutfmtVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "xil_outfmt");,
      "Select output format for makebits"),
   p_additional ("xil_bit4karg"));


/* Caution: since this command merely decides which other command to run,
   it must not be macro recorded or the macro will execute the subcommand
   twice.  Code stolen from symbol creation. */
c_opt_xilbits := vcWv.Cmd (fac,
   func (d, e) {
      if (string_eqi (g_family, "xc2000"))
         vcCmd.Run (c_opt_xil2bits, d, e, nil, FALSE);
      else if (string_eqi (g_family, "xc3000"))
         vcCmd.Run (c_opt_xil3bits, d, e, nil, FALSE);
      else if (string_eqi (g_family, "xc4000"))
         vcCmd.Run (c_opt_xil4bits, d, e, nil, FALSE);
      else cb_mainErr (1709, "Error", nil, "No family selected"); },
   "XilBitstream", "Xilinx bitstream options");
vcCmd.EnableMacro (c_opt_xilbits, FALSE);


vcCmd.SetHelp (c_xil_wir,      "Create WIR file for Xilinx design");
vcCmd.SetHelp (c_xil23_routed, "Place and route Xilinx 2k/3k design");
vcCmd.SetHelp (c_xil4_routed,  "Place and route Xilinx 4k design");
vcCmd.SetHelp (c_opt_xilsyn,   "Set options for Xilinx synthesis");
vcCmd.SetHelp (c_opt_xildev,   "Set options for Xilinx package");
vcCmd.SetHelp (c_opt_xil23apr, "Set options for Xilinx 2k/3k implementation");
vcCmd.SetHelp (c_opt_xilbits,  "Set options for Xilinx bitstream");


/*** Menu greying functions ***/


/* Disable all options but synthesis when not top level */
function xilSettype (type) {
   somecmds := list (c_xil23_routed, c_xil4_routed, c_opt_xildev,
      c_opt_xil23apr, c_opt_xilbits);
   istop := cb_loadIstop (g_selel);
   if (!istop)
      menuEnableVenlist ("Xilinx", vlOnOff.Off,
         "Only permitted on top level file", somecmds);
   else menuEnableVenlist ("Xilinx", vlOnOff.On, "", somecmds); }


/*** Partlist building functions ***/


function xilOpenData (s, xactpath) {
   file := nil;
   name := xactpath + "/data/" + s;
   if (!fexists (name)) cb_mainErr (1715, "Warning", nil, 
      sprintf ("Unable to find partlist file %s", s));
   else file = fopen (xactpath + "/data/" + s, "r");
   file; }


function xilPutPart (part, pkgfile, die, strip_p, xactpath, speedhash,
   outfile, kept_name) {

   line := ""; declare i, j;
   infile := xilOpenData (pkgfile, xactpath);
   if (!null (infile)) {
      if (!xilPutSpeed (part, speedhash, die, outfile, kept_name)) continue;
      j = 0;
      while (1==1) {
         if (null (line = fgetline (infile))) break;
         if (string_length (line) > 4)
            if (string_eq (substring (line, 0, 3), "pin")) {
               i = 0;
               foreach (token; string_loop (line, " \n"))
                  if (++i == 3) {
                     if (j == 0) fputline (outfile, "SIG");
                     if (strip_p && ((token[0] == 'P') || token[0] == 'p'))
                        token = substring (token, 1, 0);
                     fputline (outfile, " ");
                     fputline (outfile, token);
                     if (++j == 18) {
                        j = 0; fputline (outfile, "\n"); } } } }
      fclose (infile); }
      if (j != 0) fputline (outfile, "\n");
      fputline (outfile, "\n"); }


function xilReadPart (xactpath, speedhash, outfile) {
   pkg_lookup := list (
      list ("CB", "CQFP", TRUE),  list ("CQ", "CQFP", TRUE),
      list ("MQ", "MQFP", TRUE),  list ("PC", "PLCC", TRUE),
      list ("PD", "PDIP", TRUE),  list ("PG", "CPGA", FALSE),
      list ("PP", "PPGA", FALSE), list ("PQ", "PQFP", TRUE),
      list ("VQ", "VQFP", TRUE),  list ("BG", "BGA",  TRUE),
      list ("TQ", "TQFP", TRUE));

   line := ""; declare i, s, die, ofs, len, l, strip_p, kept_name;
   file := xilOpenData ("partlist.xct", xactpath);
   while (1==1) {
      if (null (line = fgetline (file))) break;
      if (string_length (line) > 4)
         if (string_eq (substring (line, 0, 4), "part")) {
            if (string_eq (substring (line, 0, 5), "part7")) continue;
            i = 0; die = nil;
            foreach (token; string_loop (line, " \n")) {
               if (++i == 2) {
                  kept_name = token;
                  for (l=pkg_lookup; !null(l); l=cdr(l)) {
                     ofs = string_search (token, caar (l));
                     if (!null (ofs)) {
                        len = string_length (token);
                        die = substring (token, 0, ofs);
                        strip_p = caddar (l);
                        s = sprintf ("DEV XC%c000 %s ", token[0], die);
                        s += substring (token, ofs, len) + " ";
                        s += substring (token, ofs+2, len) + " ";
                        s += cadar (l) + " ";
                        break; } } }
               else if (i == 4) {
                  if (null (die)) cb_mainErr (1714, "Warning", nil, 
                     sprintf ("Unable to map pkg_type attribute for '%s'",
                        kept_name));
                  else xilPutPart (s, token, die, strip_p, xactpath,
                     speedhash, outfile, kept_name); } } } }
   fclose (file); }


function xilPutSpeed (part, speedhash, die, outfile, kept_name) {
   val := vlSym.Fetch (speedhash, die);
   if (null (val)) {
      cb_mainErr (1716, "Warning", nil, 
         sprintf ("No speed grades for part '%s'", kept_name));
      FALSE; }
   else {
      fputline (outfile, sprintf ("%s\nSPD %s\n", part, val));
      TRUE; } }


function xilReadSpeed (xactpath, speedhash) {
   /* Binary file added in XACT 5 */
   function storelist (pkglist, speedstr) {
      for (l:=pkglist; !null(l); l=cdr(l)) 
         vlSym.Store (speedhash, car (l), speedstr); }

   storelist (list ("2018", "2064"), "-130 -100 -70 -50 -30");
   storelist (list ("2018L", "2064L"), "-10");
   storelist (list ("3020","3030","3042","3064","3090"), "-125 -100 -70 -50");
   storelist (list ("3020A","3030A","3042A","3064A","3090A"),
      "-125 -100 -70 -50");
   storelist (list ("3020L","3030L","3042L","3064L","3090L"), "-8");
   storelist (list ("3120","3130","3142","3164","3190", "3195"), "-3 -4 -5");
   storelist (list ("3120A","3130A","3142A","3164A","3190A", "3195A"),
      "-3 -4 -5");
   storelist (list ("4003", "4006", "4008", "4013", "4003A", "4005A"),
      "-4 -5 -6");
   vlSym.Store (speedhash, "4005", "-4 -5 -6 -6B -10");
   storelist (list ("4010"), "-4 -5 -6 -10");
   storelist (list ("4010D", "4002A", "4004A", "4003H", "4005H"), "-5 -6"); }


c_util_xilptl := vcWv.Cmd (fac,
   func (d, e) vcWin.WithBusy (vcRegion.Window (gr_log), func () {
      file := fopen ("vfpgaxil.ptl", "w");
      xactpath := xilGetXACT (FALSE);
      fputline (file,
         "| Automatically generated partlist file for Xilinx\n\n");
      speedhash := vlSym.StrCreate (101);
      xilReadSpeed (xactpath, speedhash);
      xilReadPart (xactpath, speedhash, file);
      cb_mainErr (0, "Status", nil, "Partlist file vfpgaxil.ptl created");
      fclose (file); });,
   "XilinxPTL", "Create new partlist file for Xilinx");


/*** Utility commands ***/


c_util_x3tut := vcWv.Cmd (fac,
   func (d, e) menuCopyTutFiles (list ("xc3000"),
      list ("fsm_case.vhd", "accum.abl", "reg5.abl", "sym/accum.1",
         "sym/reg5.1", "sch/muxp_x3.1"),
      list ("accum.bl0", "accum.xrf", "accum.tt2", "accum.dmc",
            "accum.vhd", "sym/accum.pin", "wir/accum.1", "sch/accum.1",
            "reg5.bl0", "reg5.xrf", "reg5.tt2",
         "reg5.dmc", "reg5.vhd", "sym/reg5.pin", "wir/reg5.1",
         "sch/reg5.1", "wir/fsm_case.1", "sch/fsm_case.1"));,
   "CopyX3Tut", "Copy files for online Xilinx 3000 tutorial");


function fl_xildel () { list (
   list (cons ("ABL", "abl"), cons ("BAK", "bak"), cons ("BL0", "bl0")),
   list (cons ("CHP", "chp"), cons ("CMD", "cmd"), cons ("DMC", "dmc")),
   list (cons ("FIT", "fit"), cons ("JED", "jed"), cons ("LIS", "lis")),
   list (cons ("ORD", "ord"), cons ("PIN", "pin"), cons ("PR0", "pr0")),
   list (cons ("PR1", "pr1"), cons ("PRT", "prt"), cons ("SEL", "sel")),
   list (cons ("TIM", "tim"), cons ("TMV", "tmv"), cons ("TT2", "tt2")),
   list (cons ("TT3", "tt3"), cons ("VNT", "vnt"), cons ("XRF", "xrf"))); }


c_util_xildel := cg_util_delint ("DelXilIntFiles",
   "Delete Xilinx intermediate files", "fl_xildel",
   "bak bl0 chp dmc fit lis ord pin pr0 prt sel tim tmv tt2 tt3 vnt xrf");
