/*   V I E W S I M . H



This header file should be included in all C routines 
written using the C <-> Viewsim interface.

This file contains typedefs for various 
data structures, enumerated types and procedure definitions used by
the interface.



*/

#ifndef __viewsim_h__
#define __viewsim_h__

#ifdef vlCOMP_PROTO
#if (vlCOMP_PROTO)
#ifndef PROTO
#define PROTO 1  /* turn on prototypes */
#endif
#endif
#if (vlCOMP_GC)
#ifndef vscLONGLONG
#define vscLONGLONG 1   /* use gcc long longs */
#endif
#endif
#include "config.h"
#if (!CONFG_64_BIT_TIME)
#define NO_64_BIT_TIME 1   /* 32-bit time support only */
#endif
#endif

#ifndef vscLONGLONG
#ifdef __alpha
#define vscLONGLONG 1
#endif
#endif

#ifdef NO_64_BIT_TIME
#ifdef __alpha
typedef int vsc_time_64;
#else
typedef long vsc_time_64;
#endif
#else
#ifdef vscLONGLONG
typedef long long vsc_time_64;
#else
#if 1
typedef struct { long high; unsigned long low; } vsc_time_64;
#else
typedef struct { unsigned long low; long high } vsc_time_64;
#endif
#endif
#endif

/* The value of a pin can be one of the following 28
   enumerated state values */

typedef enum {
                D1,   D1W1, D1C1, D1C0, D1W0, D1D0, W1,
                W1C1, W1C0, W1W0, W1D0, C1,   C1C0, C1W0,
                C1D0, C0,   C0W0, C0D0, W0,   W0D0, D0,
                Z,    D1Z,  W1Z,  C1Z,  ZC0,  ZW0,  ZD0,
		GX, G0, G1
} vsc_pin_values;

/* For ease of computation, the 31 state value can be mapped
   into the following 4 state enumerated values */

typedef enum {
                LOW4,
                X4,
                HIGHZ4,
                HIGH4
} vsc_pin_4values;

/* Possible pin types */

typedef enum {  INPUT,
                OUTPUT,
                BIPUT
} vsc_pin_types;

/* Three possible delay types */

typedef enum  { MIN_DELAY,
                TYP_DELAY,
                MAX_DELAY
} vsc_delay_type;

/* Delay scheduling models */

typedef enum  { TRANSPORT_DELAY,
                INERTIAL_DELAY,
                INSERTION_DELAY,
                TRANSPORT_X_DELAY
} vsc_delay_model;

/* Flags for time handlers */

typedef enum { ONE_TIME,
               EVERY_TIME
} vsc_persistent;

/* values used as argument to vsc_display entry point routine */

typedef enum {
    DISPLAY,
    TRACE,
    UNTRACE
} vsc_display_type;

/* Status bits used by vscu_report_settings */

#define RPT_SPIKES     1
#define RPT_TIMING     2
#define RPT_STABILITY  4 
#define RPT_MEMERRS    8
#define RPT_HWMX      16

/* Status bits used by vscu_defaults_settings */

#define DFLT_CMDFILE   1
#define DFLT_WATCH     2
#define DFLT_TIME      4 
#define DFLT_FORCEX    8
#define DFLT_BIGNET   16
#define DFLT_PAGING   32
#define DFLT_FLTANNO  64

/* bits for capability registration - vscu_register_capability and
   vsbpu_register_capability */

#define CPB_DISPLAY 1
#define CPB_TRACE   2
#define CPB_FORCE   4

/* The following descibes the entry point structure and the function
   definition of the routines contained in the structure.  The structure
   is a list of function pointers (entry points).  First the function 
   definition. */

/*=========================== VSC_INIT =====================================*/

typedef void * (* vsc_init) (
#ifdef PROTO
                             void
#endif
                             );
/*=========================== VSC_EVENT ====================================*/

typedef void (* vsc_event) (
#ifdef PROTO
                            void *instance_id,
                            vsc_pin_values *pin_array
#endif
                            );
/*=========================== VSC_RESTART ==================================*/

typedef void (* vsc_restart) (
#ifdef PROTO
                              void * instance_id
#endif
                              );
/*=========================== VSC_SAVE =====================================*/

typedef void (* vsc_save) (
#ifdef PROTO
                           void *instance_id
#endif
                           );
/*=========================== VSC_RESTORE ==================================*/

typedef void (*vsc_restore) (
#ifdef PROTO
                             void *instance_id
#endif
                             );
/*=========================== VSC_INFO =====================================*/

typedef void (*vsc_info) (
#ifdef PROTO
                          void * instance_id
#endif
                          );
/*=========================== VSC_CLOSE ====================================*/

typedef void (*vsc_close) (
#ifdef PROTO
                           void * instance_id
#endif
                           );
/*=========================== VSC_DISPLAY ==================================*/

typedef void (*vsc_display) (
#ifdef PROTO
                            void *id, /* instance_id or sim_id */
                            char *name,
                            void *uid,
                            vsc_display_type type
#endif
                            );
/*=========================== VSC_FORCE ==================================*/

typedef void (*vsc_force) (
#ifdef PROTO
                          void *id, /* instance_id or sim_id */
                          char *name,
                          void *uid,
                          char *value
#endif
                          );
/*==========================================================================*/

/* The following is the entry structure that defines the points of entry 
   for a particular model */

typedef struct {
    vsc_init            inst_init;
    vsc_event           inst_eval;
    vsc_restart         inst_restart;
    vsc_info            inst_info;
    vsc_close           inst_close;
    vsc_save            inst_save;
    vsc_restore         inst_restore;
    vsc_display         inst_display;
    vsc_force           inst_force;
} vsc_entries_struct;

/* The following is the entry structure that defines the points of entry 
   for a particular simulator */

typedef struct {
    vsc_display      sim_display;
    vsc_force        sim_force;
} vsbp_entries_struct;

/* The following typedef defines a function that returns a pointer to 
   the vsc_entries_struct.  The function's argument is a character 
   string.  The utility routine vscu_set_type uses this typedef.  One 
   of the arguments to vcsu_set_type is a function pointer of type 
   vsc_entry_proc defined below: */

#ifndef PROTO
typedef vsc_entries_struct * (*vsc_entry_proc) ();
#else
typedef vsc_entries_struct * (*vsc_entry_proc) (char * model_name);
#endif

/* The following typedef defines a function pointer whose argument
   are an integer and a character string.  This is used by the utility 
   vscu_set_keyword.  This utility requires that the user supply
   Viewsim with a function pointer of this type - vcs_keyword_handler */

#ifndef PROTO 
typedef void (*vsc_keyword_handler) ();
#else
typedef void (*vsc_keyword_handler) (int num_args, char * args[]);
#endif
 
#ifndef PROTO
typedef char * (*vsc_command_handler)();
#else
typedef char * (*vsc_command_handler)(char *);
#endif

#ifndef PROTO
typedef void (*vsc_trace_handler)();
#else
typedef void (*vsc_trace_handler)(char *, char *, void *);
#endif

typedef void (*vsc_time_handler)();
typedef void (*vsc_dtime_handler)();

#ifndef PROTO
typedef void (*vsc_variable_handler) ();
#else
typedef void (*vsc_variable_handler) (char *name);
#endif

#ifndef PROTO
typedef int (*vsc_loop_var_proc) ();
#else
typedef int (*vsc_loop_var_proc) (char *name);
#endif

#ifndef PROTO
typedef void (*vsc_variable_iterator) ();
#else
typedef void (*vsc_variable_iterator) (vsc_loop_var_proc proc);
#endif

/* The following is the definition for all the UTILITY routines */

#ifndef PROTO

unsigned long   vscu_time();
vsc_time_64     vscu_time64();
double          vscu_timescale(); 
#ifndef vlHW_HP9K700
void            vscu_printf();
#endif
int             vscu_num_pins();
int             vscu_define_pin();
int             vscu_define_unresolved_pin();
int             vscu_set_type();
int             vscu_set_keyword();
int             vscu_reset_keyword();
void           *vscu_allocate();
void            vscu_deallocate();
void           *vscu_sched_wakeup();
int             vscu_cancel_wakeup();
void            vscu_sched_output();
int             vscu_pin_changed();
int             vscu_signal_value();
char           *vscu_pin_signal();
int             vscu_save_data();
int             vscu_restore_data();
int             vscu_get_inst_attr();
int             vscu_get_quoted_inst_attr();
void            vscu_break_sim();
vsc_delay_type  vscu_get_delay_type();
unsigned long   vscu_get_stepsize();
float           vscu_get_scale();
vsc_delay_model vscu_get_delay_model();
char           *vscu_get_inst_name();
char           *vscu_get_symbol_name();
char           *vscu_get_network_name();
unsigned int    vscu_report_settings();
unsigned int    vscu_defaults_settings();
int             vscu_last_sig_event();
int             vscu_last_sig_event64();
int             vscu_next_sig_event();
int             vscu_next_sig_event64();
int             vscu_signal_loads();
char           *vscu_time_string();
char           *vscu_time64_string();
int             vscu_string_time();
int             vscu_string_time64();
int             vscu_string_delay();
long            vscu_get_elapsed_time();
int             vscu_next_event();
int             vscu_next_event64();
char           *vscu_get_version();
void            vsc_initialization();
void            vsc_end_initialization();
void            vsc_terminate();
int             vscu_register_variable();
int             vscu_reg_variable();
int             vscu_unregister_variable();
int             vscu_variable_value();
void            *vsbpu_register_simulator();
int             vsbpu_register_capability();
int             vsbpu_register_variable();
int             vsbpu_reg_variable();
int             vsbpu_unregister_variable();
int             vsbpu_variable_value();
void            vscu_register_command_handler();
int             vscu_unregister_command_handler();
int             vscu_trace();
int             vscu_untrace();
void            vscu_command();
void            vscu_register_time_handler();
int             vscu_unregister_time_handler();
void            vscu_register_dtime_handler();
int             vscu_unregister_dtime_handler();
void            vscu_sensitize();
void            vscu_desensitize();
int             vscu_register_capability();
void            vscu_register_variable_handler();
void            vscu_register_variable_iterator();

#else

unsigned long   vscu_time(void);
vsc_time_64     vscu_time64(void);
double          vscu_timescale(void); 

#ifdef __STDC__
void            vscu_printf(char *, ...);
#else	/* not __STDC__ */
#ifdef __alpha
void            vscu_printf(char *, ...);
#else	/* not __alpha */
void            vscu_printf();
#endif	/* not __alpha */

#endif	/* not __STDC__ */

int             vscu_num_pins(void * instance_id);
int             vscu_define_pin(unsigned int pin_position,
                                vsc_pin_types pin_type);
int             vscu_define_unresolved_pin(unsigned int pin_position,
                                vsc_pin_types pin_type);
int             vscu_set_type(char * model_type,
                              vsc_entry_proc entry_procedure);
int             vscu_set_keyword(char * prefix_keyword,
                                 vsc_keyword_handler keyword_handler);
int             vscu_reset_keyword(char * prefix_keyword);
void           *vscu_allocate(unsigned int size);
void            vscu_deallocate(void * mem_block);
void           *vscu_sched_wakeup(void * instance_id, unsigned long delay);
int             vscu_cancel_wakeup(void * wakeup_id);
void            vscu_sched_output(void * instance_id, int pin_ordinal,
                                  vsc_pin_values pin_value,
                                  unsigned long delay,
                                  vsc_delay_model delay_model);
int             vscu_pin_changed(void * instance_id, int pin_ordinal);
int             vscu_signal_value(char * signal_name,
                                  vsc_pin_values * signal_value);
char           *vscu_pin_signal(void * instance_id, int pin_ordinal );
int             vscu_save_data(void * instance_id, void * data_item,
                          int data_length);
int             vscu_restore_data(void * instance_id, void * data_item,
                                  int data_length);
int             vscu_get_inst_attr(char ** attr_argv[]);
int             vscu_get_quoted_inst_attr(char *** attr_argv);
void            vscu_break_sim(unsigned long delay);
vsc_delay_type  vscu_get_delay_type(void);
unsigned long   vscu_get_stepsize(void);
float           vscu_get_scale(void);
vsc_delay_model vscu_get_delay_model(void);
char           *vscu_get_inst_name(void * instance_id);
char           *vscu_get_symbol_name(void * instance_id);
char           *vscu_get_network_name(void);
unsigned int    vscu_report_settings(void);
unsigned int    vscu_defaults_settings(void);
int             vscu_last_sig_event(char * signal_name, unsigned long * time,
                                    vsc_pin_values * signal_value);
int             vscu_last_sig_event64(char * signal_name, vsc_time_64 * time,
                                      vsc_pin_values * signal_value);
int             vscu_next_sig_event(char * signal_name, unsigned long * time,
                                    vsc_pin_values * signal_value);
int             vscu_next_sig_event64(char * signal_name, vsc_time_64 * time,
                                      vsc_pin_values * signal_value);
int             vscu_signal_loads(char * signal_name, unsigned int * load);
char           *vscu_time_string(long time);
char           *vscu_time64_string(vsc_time_64 time);
int             vscu_string_time(char * time_string, long * time);
int             vscu_string_time64(char * time_string, vsc_time_64 * time);
int             vscu_string_delay(char * delay_string, long * time);
long            vscu_get_elapsed_time(void);
int             vscu_next_event(unsigned long * time);
int             vscu_next_event64(vsc_time_64 * time);
char           *vscu_get_version(void);
void            vsc_initialization(void);
void            vsc_end_initialization(void);
void            vsc_terminate(void);
int             vscu_register_variable(void *instance_id, char *name, 
                                       void *uid);
int             vscu_reg_variable(void *instance_id, char *name, 
                                  void *uid, char *type);
int             vscu_unregister_variable(void *instance_id, char *name);
int             vscu_variable_value(void *instance_id, char *name, 
                                    char *value);
void            *vsbpu_register_simulator(char *name, 
                                          vsbp_entries_struct *entries);
int             vsbpu_register_capability(void *sim_id, unsigned int cap);
int             vsbpu_register_variable(void *sim_id, char *name, void *uid);
int             vsbpu_reg_variable(void *sim_id, char *name, void *uid,
                                   char *type);
int             vsbpu_unregister_variable(void *sim_id, char *name);
int             vsbpu_variable_value(void *sim_id, char *name, char *value);
void            vscu_register_variable_handler(vsc_variable_handler vhandler);
void          vscu_register_variable_iterator(vsc_variable_iterator ihandler);
void            vscu_register_command_handler(vsc_command_handler vhandler);
int             vscu_unregister_command_handler(vsc_command_handler vhandler);
void            vscu_command(char *command_string);
int             vscu_trace(vsc_trace_handler vhandler, char *signal_name, 
                           void *data);
int             vscu_untrace(vsc_trace_handler vhandler, char *signal_name);
void            vscu_register_time_handler(vsc_time_handler vhandler,
                                           vsc_persistent flag);
int             vscu_unregister_time_handler(vsc_time_handler vhandler);
void            vscu_register_dtime_handler(vsc_dtime_handler vhandler,
                                            vsc_persistent flag);
int             vscu_unregister_dtime_handler(vsc_dtime_handler vhandler);
void            vscu_sensitize(void *instance_id, int pin_ordinal);
void            vscu_desensitize(void *instance_id, int pin_ordinal);
int             vscu_register_capability(void *instance_id, unsigned int cap);
#endif

#if (vlCOMP_PROTO)
#undef PROTO
#endif

#endif
