/* -----------------------------------------------------------------------*\
/* Module -- visiterr.vs
 *
 *      Facility:      visiterrfac
 *
 *      Environment:   SUN4, Powerview 5.2
 *
 *      Authors:       Mani Alagappan,  Apr 20, 1994
 *                     Jeff Hare
 *
 *      Description:   ViewTime Visit Error command argument Editor Facility
 *       
 *
 *      Copyright (c) 1993 by Viewlogic Systems, Inc.
 *
\* ---------------------------------------------------------------------- */
/*\Section visiterr -- ViewTime Verification results editor facility
*/

function label(str) vcRegion.Create1(str,vcLabel.fac,str,nil);


/* -------------------------------------------------- *\
 *  The following definitions map timing constraints
 *  and edges to constraint & edge strings.  These
 *  strings are used to form the string returned by
 *  the _Get method.
\* -------------------------------------------------- */
ALL := "All";
SET := "Setup";
HLD := "Hold";
PW  := "Pulse";     /* Pulsewidth = hold check?*/
SKW := "Skew";     
LH  := "Rising";
HL  := "Falling";
LO  := "Low";       /* Pulsewidth low = Falling edge check? */
HI  := "High";      /* Pulsewidth hi  = Rising edge check? */

/* -------------------------------------------------- *\
 *  The following definitions map the Constraint and
 *  Edge types to the button strings in the picklist.
\* -------------------------------------------------- */
SETUPstr       := "Setup";
HOLDstr        := "Hold";
PULSEstr       := "Pulsewidth";
SKEWstr       := "Skew";


function Cr8Defaults()
{
   list (list ("Setup 1.25 LH D $1I51\\$1i52\\Mani",
               "Setup 1.25 HL D U4"),
         list ("Hold  1.25 LH D $1I51\\$1i52\\Jeff",
               "Hold  1.25 HL D U4"),
         list ("Pulse 1.25 LO D $1I51\\$1i52\\Brad",
               "Pulse  1.25 HI D U4"));
}

function MakeEvent(r)
{
   w := vcRegion.Window (r);
   e := vcEvent.Create (0, 0, ' ', "", w, r, nil);
   e;
}

/*---------------------------------------------------------------------*/
/* The code following the vcAlert.track() call, tests for a nil return */
/* status.  Non-nil indicates that the message was not successfully    */
/* delivered to VNAV.  Beware that vcAlert may change this behavior.   */
/*---------------------------------------------------------------------*/
track_cmd := vcWv.Cmd (vcAlert.fac, 
                       func(d,e,location_string) {
                          if (vcAlert.track("vtime", 
                                            convert (vlIntVT,vlErrSev.Error), 
                                            convert (vlIntVT, 0), 
                                            location_string,"Violation") != nil) {
                             vipc.ErrGetStr();
                             w := vcRegion.Window(vcEvent.Region(e));
                             vcWin.ClearStatus(w);
                             vcWin.AddStatus(w,"Navigator (vnav) is not running in this Framework");
                          }
                       },
                       "visit", "Visit Location",
                       vcWv.Param("location", vlStringRestOfLineVT, vcWv.Constant,""));
 

/*===========================================================================*/
/*==                       _ C r e a t e                                   ==*/
/*===========================================================================*/
/*\Function _Create  -- pathtrc facility data creation routine
  
  DESCRIPTION
      This facility creates the Visit Errors after a verification run of
      the design. Tbe user has the ability to either navigate each error
      or perform a "worst case" trace analysis of each error.

      The argument to this facility is a list of vtime data, vtime event,
      list of setup violations, list of hold violsations and a list of 
      all pulsewidth violations.
       
  RETURNS
       None.

  EXAMPLE
   function visiterrors(d,e)
      args := list (d,e,
                    list ("Setup 1.25 LH D $1I51\\$1i52\\Mani",
                          "Setup 1.25 HL D U4"),
                    list ("Hold  1.25 LH D $1I51\\$1i52\\Jeff",
                          "Hold  1.25 HL D U4"),
                    list ("Pulse 1.25 LO D $1I51\\$1i52\\Brad",
                          "Pulse  1.25 HI D U4"));
      rgn := vcRegion.Create("error", 
                             visiterr.fac,
                            args);
      vcWin.Popup (vcEvent.Window (e), rgn, rgn,vcWin.NoMenus);
       
  CAUTION
     This command is currently a "NOQueue" command, meaning the 
     "visiterror" command will not appear in the transcript window.
     But the "worstcase" command is queued. 

  SEE ALSO
       
*/
function _Create(Defaults, TopRegion)
{

   /*--------------------------------------------------*\
    *  The following region declarations are global to
    *  all the functions defined in _Create()
   \*--------------------------------------------------*/
   declare ViolationRgn;    /* violation picklist region */
   declare ConRgn;     /* Display picklist of setpu/hold/pw */
   declare MagnitudeRgn;   /* Scalebar region */
   declare ViolationList;


   /*====================================================*/
   /*==   B u i l d V e r i f y R e s u l t s G u i=    =*/
   /*====================================================*/
   /*\Function BuildVerifyResultsGui  -- Establish the Dialog box
     
     DESCRIPTION
          Builds the verification results GUI.
     RETURNS
     
   */
   function BuildVerifyResultsGui(s)      
   {  
      Title := vcRegion.Create("TDBorder", vcBorder.fac,
                               vcTable.Make("VisitErr", 1, 5, 
                                            label ("Constraint"),
                                            label ("Margin"),
                                            label ("Edge"),
                                            label ("Pin"),
                                            label ("Component")));
      
      /*------------------------------------------*/
      /*     Set up the Picklist region           */
      /*-------------------------------------------*/
      ViolationRgn     := vcRegion.Create("Violation Types Picklist", 
                                          vcPicklist.fac,
                                          nil /*ViolationList*/,
                                          columns <- 1,
                                          rows <- 4);
         
      /*----------------------------------------*\
             Set up the Violation Magnitude Region
      \*----------------------------------------*/
/*
      MagnitudeTitleRgn := vcRegion.Create("", 
                                           vcLabel.fac, 
                                           "Show violations Less Than(ns)");
         

      MagnitudeScaleRgn := vcRegion.Create("MagRgn", 
                                           vcScalebar.fac, 50,
                                           increment <- .5,
                                           min <- 1,
                                           max <- 50,
                                           ticks <- 10,
                                           clip <- FALSE);
      vcScalebar.CallOnChange(MagnitudeScaleRgn, set_magnitude);   
      MagnitudeRgn = vcRegion.Create("TDBorder", vcBorder.fac, 
                                     vcTable.Make("Magnitude", 1, 2, 
                                                  MagnitudeTitleRgn, MagnitudeScaleRgn));
*/
      /*label (" Less than (ns)")));*/

      /*-------------------------------------------*\
        *   Set up the Trace Constraints Region    *
      \*--------------------------------------------*/
      SETUP     := cons(SETUPstr,  cons(SET,reset_listing));
      HOLD      := cons(HOLDstr,   cons(HLD,reset_listing));
      PULSE     := cons(PULSEstr,  cons(PW, reset_listing));
      SKEW      := cons(SKEWstr,   cons(SKW,reset_listing));
      
      constraint_types := list(list(SETUP, HOLD, PULSE, SKEW ));
      ConTypeRgn       := vcRegion.Create("Constraints Picklist", 
                                          vcPicklist.fac,
                                          constraint_types,
                                          exclusive <- FALSE);
      vcPicklist.SetPick (ConTypeRgn,0);
      vcPicklist.SetPick (ConTypeRgn,1);
      vcPicklist.SetPick (ConTypeRgn,2);
      vcPicklist.SetPick (ConTypeRgn,3);
      vcPicklist.Notify( ConTypeRgn, Callonpick);

      ConTypeTitleRgn  := label ("Display Constraints");
      ConRgn      :=  vcTable.Make("Constraints", 1, 3, 
                                   ConTypeTitleRgn,ConTypeRgn);


      /*----------------------------------------*\
        *          Add a row   of buttons       *
      \*----------------------------------------*/
      function EnoughArgs()
      {
         code := TRUE;

         if (string_eqi (s.Part, "") == TRUE || string_eqi (s.Pin, "") == TRUE){
            vcWin.ClearStatus (vcRegion.Window (TopRegion));
            vcWin.AddStatus (vcRegion.Window (TopRegion),
                             "Please select an error from the list.");
            code = FALSE;
         }
         else 
            vcWin.ClearStatus (vcRegion.Window (TopRegion));

         code;
      }

      function Visit(){
         if (EnoughArgs () == FALSE)
            return (FALSE);

         /* Send a command to Navigator */
         cmd  := sprintf ("%s object %s", vtime.GetBlockName (s.ParentData), s.Part);
         vcCmd.Run (track_cmd, s,  MakeEvent (TopRegion), list (cmd), TRUE);
         vcPicklist.SetPick (s.PickList, s.CurPick+1);
      }
      
      function WC_Trace() {
         if (EnoughArgs () == FALSE)
            return (FALSE);
         if (string_eqi (s.Con, SET) || string_eqi (s.Con, HLD))
            vtq.Add (vtime.GetQueue (s.ParentData), s.ParentEv,                  
                     sprintf ("trace tw %s %s %s %s", 
                              s.Part,
                              s.Pin,
                              s.Con,
                              string_eqi (s.Edge, "R") ? LH : HL));
         else if (string_eqi (s.Con, PW))
            vtq.Add (vtime.GetQueue (s.ParentData), s.ParentEv,                  
                     sprintf ("trace tw %s %s %s %s", 
                              s.Part,
                              s.Pin,
                              s.Con,
                              string_eqi (s.Edge, "H") ? HI : LO));
         else {
            vcWin.ClearStatus (vcRegion.Window (TopRegion));
            vcWin.AddStatus (vcRegion.Window (TopRegion),
                             "Worst Case Trace is not available for Skew violations.");
         }
         vcPicklist.SetPick (s.PickList, s.CurPick+1);
      }

      function Path() {
         WC_Trace();
         vtq.Add (vtime.GetQueue (s.ParentData), s.ParentEv, "Path");                 
      }

      function Cancel() TRUE;

      buttons := vcRegion.Create("Buttons", vcButnRow.fac,
                                 list (list ("Visit",
                                             "Visit",
                                             Visit,
                                             nil),
                                       list ("Worst Case Trace",
                                             "Worst Case Trace",
                                             WC_Trace,
                                             nil),
                                       list ("Path Display",
                                             "Path Display",
                                             Path,
                                             nil),
                                       list ("Cancel",
                                             "Cancel",
                                             Cancel,
                                             list (close_window <- TRUE))));

      /*----------------------------------------*\
            Layout the Visit Errors  Window
      \*----------------------------------------*/
      LayoutRgn :=  vcRegion.Create("OkQuit", vcMargin.fac,
                                    s.Table = vcTable.Make("VisitErrEditor", 4, 1,
                                                           Title,
                                                           ViolationRgn,
                                                           ConRgn,
/*                                                           MagnitudeRgn,*/
                                                           buttons));

      vcTable.Row (s.Table,-1,vcTable.Border.Visible, 5,5,5,vgAlign.MiddleLeft);
      LayoutRgn;
   } /* BuildVerifyResultsGui */
   
   
   
   /*=====================================================*/
   /*==          C a l l O n P i c k 1                  ==*/
   /*=====================================================*/
   function Callonpick(data, on_or_off) {
      if (isprocedure(cdr(data))) {
         (cdr(data))(car (data),on_or_off);
      }
      else 
         vlEx.Throw2(vlEx.FAILURE,"vtime",vlErrSev.Error,
                     "No action specified for this pick in visiterr.BuildVerifyResultsGui()");
   } /* Callonpick */



   function NotifyViolation(r,onoff, index)
   {
      if (onoff == vlOnOff.On){
         table := vcRegion.GetData (r);
         vcTable.Dimensions (r, &n_rows, &n_cols);
         settings.Con  = vcRegion.GetData (vcTable.Get (r, 0,0));
         settings.Edge = vcRegion.GetData (vcTable.Get (r, 0,2));
         settings.Pin  = vcRegion.GetData (vcTable.Get (r, 0,3));
         settings.Part = vcRegion.GetData (vcTable.Get (r, 0,4));
         settings.CurPick = index;
         
         /*printf ("Con: %s, Edge: %s, Pin: %s, Part: %s\n",
                 settings.Con, settings.Edge, settings.Pin, settings.Part);*/
      }

   }

   /*=====================================================*/
   /*==               s e t _ d e p t h                 ==*/
   /*=====================================================*/
   function set_magnitude(val) {
      settings.Magnitude = val;
      vcScalebar.Set(MagnitudeRgn,settings.Magnitude);
      UpdatePicklist();
      
   }

   function UpdatePicklist()
   {
     TRUE;
   }

   function reset_listing(data,onoff)
   {
      if (onoff == vlOnOff.On){
         if (string_eqi (data, SET) == TRUE)
            settings.SetupType = vlYesNo.Yes;
         else if (string_eqi (data, HLD) == TRUE)
            settings.HoldType = vlYesNo.Yes;
         else if (string_eqi (data, PW) == TRUE)
            settings.PulseType = vlYesNo.Yes;
         else if (string_eqi (data, SKW) == TRUE)
            settings.SkewType = vlYesNo.Yes;
      }
      else {
         if (string_eqi (data, SET) == TRUE)
            settings.SetupType = vlYesNo.No;
         else if (string_eqi (data, HLD) == TRUE)
            settings.HoldType = vlYesNo.No;
         else if (string_eqi (data, PW) == TRUE)
            settings.PulseType = vlYesNo.No;
         else if (string_eqi (data, SKW) == TRUE)
            settings.SkewType = vlYesNo.No;
      }
      UpdateVerifyResultsGui (settings);
   }

   /*====================================================*/
   /*==  U p d a t e V e r i f y R e s u l t G u i     ==*/
   /*====================================================*/
   /*\Function UpdateVerifyResultsGui  -- Refresh using settings
     
     DESCRIPTION
     
   */
   function UpdateVerifyResultsGui(s) 
   {
      /*printf ("UpdateVerifyResultsGui: \n");*/

      pkdata := nil;

      function PickData2List(entries)
      {
         if (entries == nil)
            entries;
         else {
            pklist := nil;
            tables := nil;
            foreach (item;vscript.list_iter (entries)){
               row := vcTable.Create("",1,5);
               tables = cons (row, tables);
               foreach (token;string_loop (item, " "))
                  vcTable.Add (row,label (token));
               pklist = cons (row, pklist);
            }
            vcTable.AlignColumns (tables);
            reverse_destructive (pklist);
         }
      }

      function UpdateViolationPicklist(pklist)
      {
         /*printf ("UpdateViolationPicklist: %s\n", pklist);*/

         len := 1;
         declare rows;
         declare cols;

         if (pklist != nil)
            s.Size = length (pklist);

         ViolationList     := vcRegion.Create("Violation Types Picklist", 
                                             vcPicklist.fac,
                                             nil,
                                             columns <- 1,
                                             rows <- s.Size,
                                              enhanced_notify <- TRUE);
         if (pklist != nil)
            foreach (i;vscript.list_iter (pklist)) 
               vcPicklist.Add (ViolationList, i, i); 
          
         
         s.PickList =  vcPicklist.Scroll ("", ViolationList, 3, 5, 0);
         vcPicklist.Notify(s.PickList, NotifyViolation);
         vcTable.Set (s.Table, 1, 0, s.PickList);
         
         vcPicklist.Dimensions(s.PickList,&rows,&cols);
         if (rows > 0) vcPicklist.SetPick(s.PickList,0);
      }

      if (s.SetupType == vlYesNo.Yes)
         if (pkdata == nil)
            pkdata = s.SetupList;
         else
            pkdata = append (pkdata, s.SetupList);

      if (s.HoldType == vlYesNo.Yes)
         if (pkdata == nil)
            pkdata = s.HoldList;
         else
            pkdata = append (pkdata, s.HoldList);

      if (s.PulseType == vlYesNo.Yes)
         if (pkdata == nil)
            pkdata = s.PulseList;
         else
            pkdata = append (pkdata, s.PulseList);

      if (s.SkewType == vlYesNo.Yes)
         if (pkdata == nil)
            pkdata = s.SkewList;
         else
            pkdata = append (pkdata, s.SkewList);

      s.PickData = pkdata;
      pklist := PickData2List(s.PickData);
      UpdateViolationPicklist (pklist);

   } /* UpdateVerifyResultsGui */
   

   if (Defaults == nil)
      settings := GetDefaultSettings();

   else 
      settings := Str2Data(Defaults);

   DialogRgn := BuildVerifyResultsGui(settings);
   UpdateVerifyResultsGui(settings);

   vcRegion.AddChild(TopRegion,DialogRgn);
   settings;  /* Data is a structure */
}


/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function Data2Str  -- Convert a settings structure to a string
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function Data2Str(s,r)
{
   declare str;

   str;
} /* Data2Str */



/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function Str2Data  -- Convert a string to a settings structure
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function Str2Data(defaults)
{
   /**printf ("Str2Data: %s\n",defaults);*/
   s := GetDefaultSettings();
   s.ParentData = car(defaults); defaults = cdr(defaults);
   s.ParentEv   = car(defaults); defaults = cdr(defaults);
   s.SetupList  = car(defaults); defaults = cdr(defaults);
   s.HoldList   = car(defaults); defaults = cdr(defaults);
   s.PulseList  = car(defaults); defaults = cdr(defaults);
   s.SkewList   = car(defaults); 
   
   s;
} /* Str2Data */

/*===========================================================================*/
/*==                      _ G e t                                          ==*/
/*===========================================================================*/
/*\Function _Get  -- The _Get data method for this facility
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function _Get(d,r,type) {      

	/*foo := Data2Str(d,r);   */
   "";
} /* _Get */


/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function GetDefaultSettings  -- 
  
  DESCRIPTION
       Get the default settings of the facility dialog box. 
       
  RETURNS
      The  default settings are returned in a viewscript structure.
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function GetDefaultSettings() {
   /*printf ("GetDefaultSettings\n");*/

   struct {
      ParentData   = nil;
      ParentEv     = nil;

      Table        = nil;
      SetupList    = nil;
      HoldList     = nil;
      PulseList    = nil;
      SkewList     = nil;

      PickData     = nil;
      PickList     = nil;
      Size         = 0;
      CurPick      = 0;
      
      SetupType    = vlYesNo.Yes;
      HoldType     = vlYesNo.Yes;
      PulseType    = vlYesNo.Yes;
      SkewType     = vlYesNo.Yes;
      
      Con          = "";
      Edge         = "";
      Part         = "";
      Pin          = "";

      Magnitude   = 100
   } settings;
} /* GetDefaultSettings */





