/*
 * Name:         vlinstall.vs 
 * Facility:     Vlinstall
 * Description:  Main viewscript file for Vlinstall
 * Enivornment:  SunOS 4.1 & CORE 5.0
 * Author:       John Dube
 *
 * Modified by:
 * Who           Version   When      What
 * JAD           1.0       01/21/92  Initial Revision
 * JAD           1.1       06/09/92  Moved UI functionality from
 *                                   c file to viewscript for flexibility
 * RCC           1.2       05/04/93  Changed names of a couple of buttons
 *                                   Cleaned up some of the refreshing stuff
 */

/* Available and necessary space integers and strings */
available_space:=0;  necessary_space:=0;
s_availspace:=nil;   s_necspace:=nil;
s_rootpath:=nil;

/* Various windows and regions */
w_main:=nil;         t_list:=nil;         shell:=nil;  
HeadEntry:=nil;

/* Defines */
NUMBLANKS := 10;

/* Global Variable */
PopSubWindows := TRUE;

/*********************** Function definitions ******************************/

/* print_to_shell: Makes sure the shell exists, then prints a string to    */
/*                 that shell.                                             */
function print_to_shell(str) {
    if (shell != nil)
        vcTxShell.PrintString(shell, str);
}

/* update_avail_space: Calls GetAvailSpace callback to determine integer   */
/*                     number of kilobytes available for that path, then   */
/*                     updates the static text region with that info.      */
function update_avail_space(path) {
    available_space = GetAvailSpace(path);
    spacestr := sprintf("%d", available_space);
    vcStaticText.Clear(s_availspace);
    vcStaticText.Add(s_availspace, 
                     "Available Disk Space = " + spacestr + " kbytes");
}

/* update_nec_space: Increments (or decrements) the necessary_space        */
/*                   variable, then updates the static text region with    */
/*                   that info.                                            */
function update_nec_space(diff) {
    necessary_space = necessary_space + diff;
    spacestr := sprintf("%d", necessary_space);
    vcStaticText.Clear(s_necspace);
    vcStaticText.Add(s_necspace, 
                     "Necessary Disk Space = " + spacestr + " kbytes");
}
    
/* update_rootpath: Updates the "Install Directory" static text region.    */
function update_rootpath(path) {
    vcStaticText.Clear(s_rootpath);
    vcStaticText.Add(s_rootpath, "Install Directory: " + path);
}
    
/* win_status: Adds a message to the main window's status region           */
function win_status(entry, on_off, index) {
    if (on_off == vlOnOff.On) {
        msg := GetEntryHelp(entry);
        if (msg != nil) {
            vcWin.ClearStatus(w_main);
            vcWin.AddStatus(w_main, msg);
        }
    }
}

/* build_menu_window: Creates a window around a picklist built from        */
/*                    the entry arg, makes it a child of the window        */
/*                    of the par_menu arg, and creates the accept          */
/*                    and cancel buttons with action routines.             */
function build_menu_window(par_menu, title_str, entry) {
    declare w_menu, r_picklist;

    function accept(menuwin, picklist) {
        vcWin.Destroy (menuwin);
        vcPicklist.SetPick(GetEntryList(entry), GetEntryIndex(entry));
        vcWin.ClearStatus(w_main);
    }
    
    function select_sub(data,flag) {
        declare rows, cols, picklist_data;

        picklist := MakeSubPicklist(cdr(data));
        vcPicklist.Notify(picklist, f_picked);
        vcPicklist.Dimensions (picklist, &rows, &cols);
        for (i:=0; i<rows; ++i) {
            picklist_data = vcPicklist.Get(picklist,i,0);
            if (HasSubMenu(cdr(picklist_data))) {
                vcPicklist.SetPick(picklist, i);
                select_sub(picklist_data,flag);
            }
            else if (vcPicklist.IsSet(picklist, i) == flag) {
                vcPicklist.SetPick(picklist, i);
            }
        }
    }

    function selectall(menuwin, picklist,flag) {
        declare rows, cols, picklist_data;

        /* Disable the poping of subwindows temporarlly while vlinstall
           loops down through all of the subwindows.  Don't want to annoy
           the users by poping lot's o' windows as a result of select all */
        PopSubWindows = FALSE;
        vcWin.Busy(menuwin, TRUE);
        
        vcPicklist.Dimensions (picklist, &rows, &cols);
        for (i:=0; i<rows; ++i) {
            picklist_data = vcPicklist.Get(picklist,i,0);
            if (HasSubMenu(cdr(picklist_data))) {
                vcPicklist.SetPick(picklist, i);
                select_sub(picklist_data,flag);
            }
            else if (vcPicklist.IsSet(picklist, i) == flag) {
                vcPicklist.SetPick(picklist, i);
            }
        }
        PopSubWindows = TRUE;
        vcWin.Busy(menuwin, FALSE);

        vcWin.Destroy (menuwin);
        vcPicklist.SetPick(GetEntryList(entry), GetEntryIndex(entry));
        vcWin.ClearStatus(w_main);
    }

    /* Use MakeSubPicklist to generate picklist region                     */
    r_picklist = MakeSubPicklist(entry);
    vcPicklist.Notify(r_picklist, f_picked);
    vcPicklist.HighlightNotify(r_picklist, win_status);

    /* Create accept and cancel buttons                                    */
    r_buttons := vcRegion.Create ("r_buttons", vcButnRow.fac,
                        list (list ("Accept", "Accept",
                                    func () {
                                        accept(w_menu, r_picklist);},
                                    list (close_window <- FALSE)),
                              list ("Select All", "Select All",
                                    func () {
                                          selectall(w_menu, r_picklist,FALSE);},
                                    list (close_window <- FALSE)),
                              list ("Unselect All", "Unselect All",
                                    func() {
                                          selectall(w_menu, r_picklist,TRUE);},
                                    list (close_window <- FALSE))));

    /* Create overall table region                                         */
    t_menu := vcTable.Create ("t_menu", 3, 1);
    vcTable.Add (t_menu, vcRegion.Create1(title_str, vcStaticText.fac, 
                                          title_str, nil));
    vcTable.Add (t_menu, r_picklist);
    vcTable.Add (t_menu, r_buttons);

    /* Create new window, set appropriate action routine, return window    */
    w_menu = vcWin.Create (GetMenuWindow(par_menu), t_menu,
                           vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
    vcWin.SetTitle(w_menu, title_str);

    /* Function which is called when the window is closed by means other
       than the accept button.  Either by unpinning it in OpenLook, or
       selecting "close" from the X windows menubar. */
    function close(menuwin) {
        accept(menuwin, r_picklist);
    }

    vcWin.CallOnClose(w_menu, close);

    function null_win(menuwin) {
        NullWindow(GetSubMenu(entry));
    }        

    vcWin.DestroyNotify(w_menu, null_win);
    SetMenuWindow(GetSubMenu(entry), w_menu);
    return (w_menu);
}

function remove_from_list(entry) {
    declare rows, cols;

    vcTable.Dimensions(t_list, &rows, &cols);
    for (i := 0; i < rows; ++i) {
        str := GetStr(vcTable.Get(t_list, i, 0));
        if (string_eq(GetEntryName(entry), str)) {
            vcTable.DeleteRows(t_list, i, 1);
            break;
        }
    }
}

/* f_picked: The notify routine to be run when an item is picked from      */
/*           the menu picklist.  This will either create a sub window for  */
/*           sub-menus, or add entry to selected list if selected.  If     */
/*           deselected, the sub-menu will be cleared, or the entry will   */
/*           be removed from selected list                                 */
function f_picked(data, onoff, index) {

    if (HasSubMenu(data) == TRUE) {
        /* Check if sub-menu has a window that already exists. If so, and  */
        /* that entry is selected, just pop the window to the front.  If   */
        /* so and that entry is deselected, re-highlight the pick.         */

        win_there := FALSE;
        sub_win := GetMenuWindow(GetSubMenu(data));
        if ((sub_win != nil) && (vcWin.IsAlive(sub_win))) 
            win_there = TRUE;

        if (onoff && PopSubWindows) {
            if (win_there)
                vcWin.Show(sub_win);
            else {
                title_str := GetMenuTitle(data);
                m_parent := GetParentMenu(data);
                w_picked := build_menu_window(m_parent, title_str, data);
            }
        }
    }
    else {
        add_entry(data,onoff);
    }
}

function add_entry(data,onoff) {

    if (onoff) {
        /* Add entry name to list table - always at top of list table  */
        name_str := GetEntryName(data);
        space := GetEntrySpace(data);
        space_str := sprintf("%d", space);
        r_name := vcRegion.Create1(name_str, vcStaticText.fac, 
                                   name_str, nil);
        r_space := vcRegion.Create1(space_str, vcStaticText.fac, 
                                    space_str, nil); 
        vcTable.InsertRows(t_list, 0, 1);
        vcTable.Set(t_list, 0, 0, r_name);
        vcTable.Set(t_list, 0, 1, r_space); 

        update_nec_space(space);

        /* Add entry to data structure */
        AddSel(data);
    }
    else {
        /* Remove entry name from list table */
        remove_from_list(data);

        /* Remove entry from data structure */
        DelSel(data);

        /* Subtract from necessary space */
        space := GetEntrySpace(data);
        update_nec_space(0 - space);
    }
}

/* make_top_region: Creates the main region with top picklist, selected    */
/*                  entries list,                                          */
function make_top_region(entry) {

    HeadEntry = entry;
    r_title := vcRegion.Create1("menu_title", vcStaticText.fac,
                                GetMenuTitle(entry), nil);
    picklist := MakeSubPicklist(entry);
    vcPicklist.Notify(picklist, f_picked);
    vcPicklist.HighlightNotify(picklist, win_status);

    t_left := vcTable.Create("t_left", 5, 1);
    vcTable.Add (t_left, vcRubber.Create("above", 0, 0));
    vcTable.Add (t_left, r_title);
    vcTable.Add (t_left, vcRubber.Create("toppick", 0, 0));
    vcTable.Add (t_left, picklist);
    vcTable.Add (t_left, vcRubber.Create("botpick", 0, 0));

    t_right := vcTable.Create("t_right", 5, 1);
    vcTable.Add(t_right, vcRegion.Create1("title", vcStaticText.fac, 
                                          "Selected Entries:", nil));
    t_list = vcTable.Create("t_list", 1, 2);
    /* Fill up the table with spaces                                       */
    empty_str := vcRegion.Create1("empty_str", vcStaticText.fac, 
                                  "", nil);
    for (i := 0; i < NUMBLANKS; ++i) {
        vcTable.Set(t_list, i, 0, empty_str);
    }

    vcTable.Add(t_right, vcTable.Scroll("t_scroll", t_list, t_list,
                                        15, 15, 15));
    s_availspace = vcRegion.Create1("s_availspace", vcStaticText.fac, "", nil);
    update_avail_space(GetRootpath()); 
    vcTable.Add(t_right, s_availspace);
    vcTable.Row(t_right, 2, vcTable.Border.Visible, 1, 1, 1, 
                vgAlign.MiddleLeft);
    s_necspace = vcRegion.Create1("s_necspace", vcStaticText.fac, "", nil);
    update_nec_space(0); 
    vcTable.Add(t_right, s_necspace);


    l_label := vcRegion.Create("l_label", vcStaticText.fac,"");
    vcTable.Add(t_right,l_label);

    /* Create a picklist between cd-rom and tape, so that the user
       can specify which he is using, and the various drives & directories
       that are needed
     */
    t_options := vcTable.Create("t_options",5,2);

    l_drive := vcRegion.Create("l_drive", vcStaticText.fac,
                               "Installing From: ");
    vcTable.Add(t_options, l_drive);

    p_drive := vcRegion.Create("Exclusive", vcPicklist.fac);
    vcPicklist.Add(p_drive,"CD-ROM","cdrom");
    vcPicklist.Add(p_drive,"Tape","tape");

    vcTable.Add(t_options, p_drive);

    l_cdrompath := vcRegion.Create("l_cdrompath", vcStaticText.fac,
                                     "CD-ROM mount point: ");
    vcTable.Add(t_options, l_cdrompath);

    s_cdrompath := vcRegion.Create("s_cdrompath", vcCmdline.fac);
    vcCmdline.Set(s_cdrompath,GetCdromPath());
    vcCmdline.CallOnChange(s_cdrompath,SetCdromPath);
    vcTable.Add(t_options, s_cdrompath);

    l_drive := vcRegion.Create("l_drive", vcStaticText.fac,
                                  "Tape drive: ");
    vcTable.Add(t_options, l_drive);

    s_drive := vcRegion.Create("s_drive", vcCmdline.fac);
    vcCmdline.Set(s_drive,GetDrive());
    vcCmdline.CallOnChange(s_drive,SetDrive);
    vcTable.Add(t_options, s_drive);

    l_norewdrive := vcRegion.Create("l_norewdrive", vcStaticText.fac,
                                  "No-rewind drive: ");
    vcTable.Add(t_options, l_norewdrive);

    s_norewdrive := vcRegion.Create("s_norewdrive", vcCmdline.fac);
    vcCmdline.Set(s_norewdrive,GetNorewDrive());
    vcCmdline.CallOnChange(s_norewdrive,SetNorewDrive);
    vcTable.Add(t_options, s_norewdrive);

    l_rootpath := vcRegion.Create("l_rootpath", vcStaticText.fac,
                                  "Install Directory: ");
    vcTable.Add(t_options, l_rootpath);

    s_rootpath = vcRegion.Create("s_rootpath", vcCmdline.fac);
    vcCmdline.Set(s_rootpath,GetRootpath());
    vcCmdline.CallOnChange(s_rootpath,SetRootpath);
    vcTable.Add(t_options, s_rootpath);

    /* This function enables/disables various options depending on if the
       user is installing from a cd-rom or a tape drive.  It is declared
       here so that the scope will be correct for all of these silly region
       names.
     */
    function drive_status(data, onoff) {
        if (onoff) {
            if (string_eq(data,"cdrom")) {
                SetCdrom();
                vcRegion.Enable(l_cdrompath, TRUE, TRUE);
                vcRegion.Enable(s_cdrompath, TRUE, TRUE);
                vcRegion.Enable(l_drive, FALSE, TRUE);
                vcRegion.Enable(s_drive, FALSE, TRUE);
                vcRegion.Enable(l_norewdrive, FALSE, TRUE);
                vcRegion.Enable(s_norewdrive, FALSE, TRUE);
            }
            else {
                UnsetCdrom();
                vcRegion.Enable(l_cdrompath, FALSE, TRUE);
                vcRegion.Enable(s_cdrompath, FALSE, TRUE);
                vcRegion.Enable(l_drive, TRUE, TRUE);
                vcRegion.Enable(s_drive, TRUE, TRUE);
                vcRegion.Enable(l_norewdrive, TRUE, TRUE);
                vcRegion.Enable(s_norewdrive, TRUE, TRUE);
            }
        }   
    }
    vcPicklist.Notify(p_drive,drive_status);

    /* Figure out what the user specified on the command line and set
       the flags accordingly.
     */
    if (string_eq(IsCdrom(),"cdrom"))
        vcPicklist.SetPick(p_drive,0);
    else
        vcPicklist.SetPick(p_drive,1);

    vcTable.Add(t_right,t_options);

    /* Create table region for top half of region                          */
    t_top := vcTable.Create("t_top", 1, 2);
    vcTable.Add(t_top, t_left);
    vcTable.Add(t_top, t_right);

    shell = vcRegion.Create1("DTI", vcTxShell.fac, "",
                             list(macro_on <- TRUE));

    t_main := vcTable.Create("t_top", 2, 1);
    vcTable.Add(t_main, t_top);
    vcTable.Add(t_main, shell);
    return(t_main);
}
    
/* do_setup: Sets the arguments from the c_options command */
function do_setup(window, event, cdrom_path, drive, norew_drive, root_path) {
    SetCdromPath(cdrom_path);
    SetDrive(drive);
    SetNorewDrive(norew_drive);
    SetRootpath(root_path);
    update_avail_space(root_path);
    update_rootpath(root_path);
}

function busy_dti_on() {
    vcWin.Busy(w_main, TRUE);
}

function busy_dti_off() {
    vcWin.Busy(w_main, FALSE);
}

function system_script(command_str) {
    vcWin.System(w_main, command_str, nil, TRUE, vcSysOutput.Viewtext, 
                 nil, nil, nil, nil);
}

/************************* Command objects *********************************/

/* c_accept cleans up the UI structures, and calls the DTI with the chosen */
/* options.  The CallDTI callback does these things.                       */
function DoInstall (d,e) {

    tmp_cmd := vcWv.Cmd(fac, func (d,e) {return(0);});

    /* If we're installing from the cdrom, make sure that the user has */
    /* correctly specified the cdrom path.                             */
    if (string_eq(IsCdrom(),"cdrom"))
    {
        status_string := CheckCdRomPath();
        if (!string_eq(status_string,"A-OK"))
        {
            tmp_string := sprintf("Cannot find CD-ROM in directory: %s\n",
                                  GetCdromPath());
            vcWv.OkOrCancel(w_main, tmp_string, tmp_cmd, tmp_cmd);
            return(0);
        }
    }

    /* First check to make sure that the install directory doesn't exist. */
    /* If it does, then give the user a warning and ask him it he really  */
    /* wants to overwrite that directory.                                 */
    /* If the directory doesn't exist, then try to create it.  If it can't*/
    /* be created, the also complain.                                     */

    status_string := CheckRootPath();

    if (string_eq(status_string,"A-OK")) {
        CallDTI(c_install,c_clear);
    }

    if (string_eq(status_string,"can't create")) {
        tmp_string := sprintf("Cannot create directory: %s\n",GetRootpath());
        vcWv.OkOrCancel(w_main, tmp_string, tmp_cmd, tmp_cmd);
    }

    if (string_eq(status_string,"is not dir")) {
        tmp_string := sprintf("%s is not a directory.", GetRootpath());
        vcWv.OkOrCancel(w_main,tmp_string, tmp_cmd, tmp_cmd);
    }

    if (string_eq(status_string,"already exists")) {
        vcWv.OkOrCancel(w_main,
     "Install directory already exists.\n Do you really want to install here?",
            vcWv.Cmd(fac, func (d,e) {CallDTI(c_install,c_clear);}), tmp_cmd);
    }
}

/************************** Menu objects **********************************/

c_install := vcWv.Cmd (fac, DoInstall);

/* c_clear clears the selected entries list by toggling all menu entries   */
c_clear := vcWv.Cmd (fac, func (d,e) { ClearMenu(GetSubMenu(HeadEntry));});

/* Set help strings for each command */
vcCmd.SetHelp (c_install, "Install Selected Files");
vcCmd.SetHelp (c_clear, "Clear Menu Selections");

install_menus := vcWv.MakeMenu("install_menus",
                      vcWin.RedSquareEntryMinimal,
                      list ("Install_Selected_Files", c_install),
                      list ("Clear_Selected_List", c_clear));

vcFac.SetMenus (fac, install_menus);
