/* ------------------------------------------------------------------------ *\
 * $Id: vnavpick.vs,v 1.10 1993/12/01 09:59:39 dave Stab $
 * The definition of the pick facility for all popup selection windows in
 * the navigator.
 *
 * Author: Bob Davis
 *
 * Copyright (c) 1992 by Viewlogic Systems, Inc.
 * ---- change log ----
 *
\* ------------------------------------------------------------------------ */
/*function DEBUG(args...);
DEBUG := printf;*/

function CarIndex(a, n) {
    while((n > 0) && (a != nil)) {
        a = cdr(a);
        --n;
    }
    return(car(a));
} /* CarIndex */

function
_Create(args, region)
{
/*   DEBUG ("Creating a context!!!!!!\n"); */
   
    Context := struct { Title; Nav; Picklist; FindType; List;
                        Index; Str; HasBeenFound };

    Context.Title = car(args);
    Context.Nav = cadr(args);
    Context.List = caddr(args);
    Context.FindType = cadddr(args);

/*    DEBUG ("Context.List: %s\n", Context.List); */
/*    DEBUG ("caddr vs carindex2: %s %s\n", caddr(args), CarIndex(args, 2)); */

    Context.Picklist = nil;

    /* these will be changed by vcPicklistNotify & SetSelectedString */
    Context.Index = 0;
    Context.Str = car(Context.List);
/*    DEBUG("Context.STR at create %s\n", Context.Str); */
   
    Context.HasBeenFound = FALSE;

    /* set up the subtitle */
    declare SubTitle;
    if (   Context.FindType == vnav.vlnFindType.Net
        || Context.FindType == vnav.vlnFindType.Component)
        SubTitle = vcRegion.Create("", vcLabel.fac,
                                   "UIDPath - NamePath");
    else if (Context.FindType == vnav.vlnFindType.RefDes)
        SubTitle = vcRegion.Create("", vcLabel.fac,
                                   "Reference Designators");
    else if (Context.FindType == vnav.vlnFindType.RefDes_PinNumber)
        SubTitle = vcRegion.Create("", vcLabel.fac,
                                   "RefDes[PinNumber]");
    else if (Context.FindType == vnav.vlnFindType.Component_PinName)
        SubTitle = vcRegion.Create("", vcLabel.fac,
                                   "ComponentPath.PinName");
    else if (Context.FindType == vnav.vlnFindType.Sheet)
        SubTitle = vcRegion.Create("", vcLabel.fac,
                                   "NetPath - Schematic.SheetNumber");
    vcRegion.Font(SubTitle, vgFont.mediumb);

    /* set up the picklist */
    /* create picklist region */
    Picklist := vcRegion.Create("picklist", vcPicklist.fac, nil,
                                rows <- 10,
                                columns <- 1,
                                exclusive <- TRUE,
                                enhanced_notify <- TRUE,
                                align <- 6, /* vgAlignLC from vgraph.h*/
                                will_scroll <- TRUE);
    declare NArray;
    {
        item := Context.List;
        declare str;
        NArray = 0;
        while (item != nil) {
            str = car(item);
            vcPicklist.Add(Picklist, str, str);
            item = cdr(item);
            ++NArray;
        } /* while */
    }
    /* upon calls via vcPicklistNotify, this updates the PicklistContext stuff
     * above so the correct item is indicated.
     */
    vcPicklist.Notify(Picklist, func(Str, on_or_off, index) {
        if (on_or_off) {
            Context.Index = index;
/*            DEBUG ("Setting a context string %s\n", Str); */            
            Context.Str = Str;
            Context.HasBeenFound = FALSE;
        } /* if */
        /*DumpIt("SetSelectedString", Context);*/
    });
    Context.Picklist = Picklist;

    PickSelector := vcPicklist.Scroll("pickselector", Picklist,
                                  /* min           ideal            max */
                                      1, (NArray < 24) ? NArray : 24, 0);

    /* not a basketball technique */
    vcPicklist.SetPick(Picklist, 0);

    /* set up the buttons */
    AttachedB := vcWv.LabelButtonCreate("Attached", "Attached",
                                        nil, AttachedCmd, nil);

    EquivalentB := vcWv.LabelButtonCreate("Equivalent", "Equivalent",
                                          nil, EquivalentCmd, nil);

    if (Context.FindType != vnav.vlnFindType.RefDes)
        SheetsOrContentsB := vcWv.LabelButtonCreate("Sheets", "Sheets",
                                                    nil, SheetsCmd, nil);
    else
        SheetsOrContentsB := vcWv.LabelButtonCreate("Contents", "Contents",
                                                    nil, ContentsCmd, nil);

    VisitB := vcWv.LabelButtonCreate("Visit", "Visit(next)",
                                     nil, VisitCmd, nil);

    CancelB := vcWv.LabelButtonCreate("Cancel", "Cancel",
                                      nil, CancelCmd,
                                      list(close_window <- TRUE));

    HelpB := vcWv.LabelButtonCreate("Help", "Help",
                                    nil, HelpCmd, nil);

    GreyButtons(Context.FindType, AttachedB, EquivalentB, SheetsOrContentsB);

    TheButtons := vcTable.Make("buttons", 2, 3,
                               AttachedB, EquivalentB, SheetsOrContentsB,
                               VisitB, CancelB, HelpB);

    /* the table
     * find-cancel-help on the last line similar to the arrangements
     * of other dialogs.
     */
    ListSelector := vcTable.Make("listselector", 3, 1,
                                 SubTitle,
                                 PickSelector,
                                 TheButtons);

    vcRegion.AddChild(region, ListSelector);

    return(Context);
} /* _Create */

function
GreyButtons(FindType, AttachedB, EquivalentB, SheetsOrContentsB)
{
    Attached := FALSE;
    Equivalent := FALSE;
    SheetsOrContents := FALSE;

    if (FindType == vnav.vlnFindType.Net) {
        Attached = TRUE;
        Equivalent = TRUE;
        SheetsOrContents = TRUE; /* sheets */
    } /* if */
    else if (   FindType == vnav.vlnFindType.Component
             || FindType == vnav.vlnFindType.Component_PinName
             || FindType == vnav.vlnFindType.RefDes_PinNumber) {
        Attached = TRUE;
    } /* else if */
    else if (FindType == vnav.vlnFindType.RefDes) {
        Attached = TRUE;
        SheetsOrContents = TRUE; /* contents */
    } /* else if */

    /* last arg of these indicates whether or not to refresh */
    vcRegion.Enable(AttachedB, Attached, FALSE);
    vcRegion.Enable(EquivalentB, Equivalent, FALSE);
    vcRegion.Enable(SheetsOrContentsB, SheetsOrContents, FALSE);
} /* GreyButtons */

/* ---------------- */

function
Visit(Context, Event)
{
  /*  DumpIt("Visit", Context); */
/*    DEBUG ("---Inside vnavpick.vs ----Context Found? %s\n", Context.HasBeenFound); */

    if (Context.HasBeenFound)
        vcPicklist.SetPick(Context.Picklist, Context.Index + 1);

    Context.HasBeenFound = TRUE;
/*    DEBUG ("ft=%s, str=%s\n", Context.FindType, Context.Str); */
    if (Context.FindType == vnav.vlnFindType.RefDes_PinNumber)
    {
        /* 
        ** If the pin number has been stripped by the time that we
        ** get here force the search down to RefDes 
        */
        if (!lindex(Context.Str,'['))
            Context.FindType = vnav.vlnFindType.RefDes;
    }  
    vnav.DoVisit(Context.Nav, Context.FindType, Context.Str);
} /* Visit */

VisitCmd := vcWv.Cmd(fac, Visit, "Visit");

function
Attached(Context, Event)
{
    /*DumpIt("Attached", Context);*/

/*set_stdout(TRUE);*/
/*printf("event:\"%s\"\nwin:\"%s\"\nparent:\"%s\"\n",
       Event, vcEvent.Window(Event),
       vcWin.Parent(vcEvent.Window(Event)));*/
/*set_stdout(FALSE);*/

    vnav.AttachedFcn(Context.Nav, vcWin.Parent(vcEvent.Window(Event)),
                     Context.FindType, Context.Str);
} /* Attached */

AttachedCmd := vcWv.Cmd(fac, Attached, "Attached");

function
Equivalent(Context, Event)
{
    /*DumpIt("Equivalent", Context);*/

    if (Context.FindType == vnav.vlnFindType.Net)
        vnav.EquivalentFcn(Context.Nav, vcWin.Parent(vcEvent.Window(Event)),
                           Context.Str);
} /* Equivalent */

EquivalentCmd := vcWv.Cmd(fac, Equivalent, "Equivalent");

function
Sheets(Context, Event)
{
    /*DumpIt("Sheets", Context);*/

    if (Context.FindType == vnav.vlnFindType.Net)
        vnav.SheetsFcn(Context.Nav, vcWin.Parent(vcEvent.Window(Event)),
                       Context.Str);
} /* Sheets */

SheetsCmd := vcWv.Cmd(fac, Sheets, "Sheets");

function
Contents(Context, Event)
{
    /*DumpIt("Contents", Context);*/

    if (Context.FindType == vnav.vlnFindType.RefDes)
        vnav.ContentsFcn(Context.Nav, vcWin.Parent(vcEvent.Window(Event)),
                         Context.Str);
} /* Contents */

ContentsCmd := vcWv.Cmd(fac, Contents, "Contents");

/* ---------------- */

CancelCmd := vcWv.Cmd(fac, func(d,e);, "Cancel");

HelpCmd := vcWv.Cmd(fac, func(d,e) vcAlert.SpawnHelp(vcEvent.Window(e),
                                                     "vnav", "Picklists");,
                    "Help");

/* ---------------- */

MAXCOL := 78;

function
DumpIt(str, Context)
{
    declare col, length;
    declare ThisStr;
    Item := Context.List;

    printf("%s\n", str);
    printf("%s {\n   ", vnav.FindType2String(Context.FindType));

    col = 4;                    /* indent level */
    while(1) {
        ThisStr = car(Item);
        length = string_length(ThisStr);
        col += length + 1;
        if (col > MAXCOL) {
            printf("\n    ");
            col = 4 + length;
        } /* if */
        else
            printf(" ");
        printf(ThisStr);

        Item = cdr(Item);
        if (!Item)
            break;
    } /* while(1) */
    printf("\n}\n");

    printf("%d %s\n", Context.Index, Context.Str);
} /* DumpIt */

load("pickht.vs");              /* help text for buttons */
