/*
 * $Revision: 1.5.4.1 $
 */

package vspscale (create_scale_area, set_scale_cmdline_val, reset_value) {

cur_ftrt_str := nil;

vspSCALE := "SCALE";
vspFITRECT := "FITRECT";

SCALE_XPRMPT := "Scale X> ";
FITRECT_XPRMPT := "Fitrect X> ";
YPRMPT := "Y> ";

declare scale_xcmdline, scale_ycmdline;
declare ftrt_xcmdline, ftrt_ycmdline;
declare init_xscale, init_yscale;

/* This queries the software for the current values of x and y fitrect
 * and sets the 2 command lines accordingly.
 */
function update_ftrt_cmdlines () {
        vcCmdline.Set(ftrt_xcmdline, 
                fac_pack.get_def_ftrt(vspFIRSTVAL));
        vcCmdline.Set(ftrt_ycmdline, 
                fac_pack.get_def_ftrt(vspSECONDVAL));
}

/* This is called when other options are changed and they need to update
 * the scale.  If the sizing is set to single, then everything is updated.
 * If it is set to X/Y, however, only the x dimension changes.  The latter
 * is of dubious use and should indicate to the user that the sizing needs
 * to change to single.
 */
function set_scale_cmdline_val (str) {

        vcCmdline.Set(scale_xcmdline, str);
        get_scale_cmdline_val (`scalex, str);

        vcCmdline.Set(scale_ycmdline, str);
        get_scale_cmdline_val (`scaley, str);
}

/* In event of cancel, revert to original scale factor.
 */
function reset_value () {

        fac_pack.sp_sapopt(string_append(vspFITRECT, " ",string_append (
                fac_pack.conv_to_uunits (vspFIRSTVAL, init_xscale), " ", 
                fac_pack.conv_to_uunits (vspSECONDVAL, init_yscale))));
}

/* Set scale temporarily in case of cancel
 */
function set_vals (str) {
        cur_ftrt_str = str;
        set_graphics_modified ();
        if (cur_ftrt_str != nil)
                fac_pack.sp_sapopt(string_append(vspFITRECT, " ",cur_ftrt_str));
        update_ftrt_cmdlines ();
}

/* This is called when either the x or y scale command line is changed
 * by the user (and a <cr> has been entered).  The corresponding fitrect
 * values(s) are updated according to how the single/double picklist is set.
 */
function get_scale_cmdline_val (which, str) {

        /* Make sure is a number */
        convert (float, str);

        /* Need both x and y scales for a fitrect */
        if (which == `scalex) {

               /* Form full fitrect string (including 2nd value) */
               ftrt_str := string_append (
               fac_pack.conv_to_uunits (vspFIRSTVAL, str), " ", 
                        vcRegion.Get(ftrt_ycmdline, vlStringVT));
        }        
        else if (which == `scaley) {

                /* Form full fitrect string (including 1st value) */
                ftrt_str := string_append (
                         vcRegion.Get (ftrt_xcmdline, vlStringVT), " ", 
                                 fac_pack.conv_to_uunits (vspSECONDVAL, str));
        }
        else
                return (nil);

        set_vals (ftrt_str);
}

/* This is called when either the x or y Fitrect command line is changed
 * by the user (and a <cr> has been entered).  The corresponding scale
 * factor(s) are updated according to how the single/double picklist is set.
 */
function get_ftrt_cmdline_val (which, str) {

        /* Make sure is a number */
        convert (float, str);

        /* Need both x and y scales for a fitrect */
        if (which == `scalex) {

                /* form fitrect string w/ new val and old value */
                ftrt_str := string_append(str, " ",
                        vcRegion.Get (ftrt_ycmdline, vlStringVT));

                /* Convert 1st dimension fitrect into scale */
                scale_str := fac_pack.conv_from_uunits (vspFIRSTVAL, 
                                str);
                /* Make sure x scale is displayed */
                vcCmdline.Set(scale_xcmdline, scale_str);
        }        
        else if (which == `scaley) {

                /* form fitrect string w/ old val and new val */
                ftrt_str := string_append(
                           vcRegion.Get (ftrt_xcmdline, vlStringVT),
                           " ", str);

                /* Convert 2nd dimension fitrect into scale */      
                scale_str := fac_pack.conv_from_uunits (vspSECONDVAL, 
                           str);

                /* Make sure y scale is displayed */
                vcCmdline.Set(scale_ycmdline, scale_str);

        }
        else
                return (nil);

        set_vals (ftrt_str);

}

/* Creates a table which contains 4 numeric command lines (2 for
 * scale and 2 for fitrect).
 */
function create_scale_area ()
{
        ttab := vcTable.Create ("scale.tab", 2, 1);

        /* Add 2 command lines for scale factors */
        tlst := create_double_cmdline_list (SCALE_XPRMPT, YPRMPT, 
                        fac_pack.get_def_scale(vspFIRSTVAL), 
                        fac_pack.get_def_scale(vspSECONDVAL),
                        get_scale_cmdline_val, `scalex, `scaley, FALSE);

        scale_xcmdline = car (tlst);
        scale_ycmdline = cadr (tlst);
        vcTable.Add (ttab, caddr (tlst));

        init_xscale = fac_pack.get_def_scale (vspFIRSTVAL);
        init_yscale = fac_pack.get_def_scale (vspSECONDVAL);

        /* Add 2 command lines for fitrect values */
        tlst = create_double_cmdline_list (FITRECT_XPRMPT, YPRMPT,
                        fac_pack.get_def_ftrt (vspFIRSTVAL), 
                        fac_pack.get_def_ftrt (vspSECONDVAL),
                        get_ftrt_cmdline_val, `scalex, `scaley, FALSE);
        ftrt_xcmdline = car (tlst);
        ftrt_ycmdline = cadr (tlst);
        vcTable.Add (ttab, caddr (tlst));

        return(vcRegion.Create1("scale.bord", vcBorder.fac, ttab, nil));
}

} /* vspscale package */





