/* MATH stuff */

/* Functions to grab the beginning and ending values of the H axis */
function
GetHAxisVMin(display)
{
    type := HAxisType(display, 0);
    value := HAxisVMin(display, 0);
    if (type == vtrDispType.Log)
        value = exp10(value);
    value;
} /* GetHAxisVMin */

function
GetHAxisVMax(display)
{
    type := HAxisType(display, 0);
    value := HAxisVMax(display, 0);
    if (type == vtrDispType.Log)
        value = exp10(value);
    value;
} /* GetHAxisVMax */

/* ---- first we have the command line version of things ---- */

/* assign-operator function that makes command */
function
MakeAssignCmd(cmd_name)
{
    prompt_cmd := vcWv.Cmd(fac, func(d, e, s1, s2, start, end)
                           math(d, e, cmd_name, s1, s2, "", start, end);,
                           "_" + cmd_name + "_pr", cmd_name,
                           vcWv.Param("First Signal Name:",
                                      string, DYNAMIC, get_first_signal),
                           vcWv.Param("Second Signal Name/Constant:",
                                      string, DYNAMIC, get_next_signal),
                           vcWv.Param("Starting X-axis value:",
                                      float, DYNAMIC, GetHAxisVMin,
                                      vcWv.Secondary),
                           vcWv.Param("Ending X-axis value:",
                                      float, DYNAMIC, GetHAxisVMax,
                                      vcWv.Secondary));

    noprompt_cmd := vcWv.Cmd(fac, func(d, e)
                             math(d, e, cmd_name,
                                  get_first_signal(d), get_next_signal(d),
                                  "", -HUGEREAL, 
                                  HUGEREAL);,
                             "_" + cmd_name + "_nopr");

    function should_prompt(d, e, rest)
    {
        if (NumberOfSignalsSelected(d) < 2)
            vcWin.RunCmdline(vcEvent.Window(e),
                             vcCmd.Name(prompt_cmd) + " "
                             + convert(string, rest), e);
        else
            vcCmd.Run(noprompt_cmd, d, e, nil, TRUE);
    }
    return(vcWv.Cmd(fac, should_prompt, cmd_name, cmd_name,
                    vcWv.Param("", vlStringRestOfLineVT, STICKY, "")));
} /* MakeAssignCmd */

AddAssignCmd := MakeAssignCmd("+=");
vcCmd.EnableMacro(AddAssignCmd, vlOnOff.Off);
SubAssignCmd := MakeAssignCmd("-=");
vcCmd.EnableMacro(SubAssignCmd, vlOnOff.Off);
MultAssignCmd := MakeAssignCmd("*=");
vcCmd.EnableMacro(MultAssignCmd, vlOnOff.Off);
DivAssignCmd := MakeAssignCmd("/=");
vcCmd.EnableMacro(DivAssignCmd, vlOnOff.Off);

/* binary-operator function that makes command */
function
MakeBinaryCmd(cmd_name)
{
    prompt_cmd := vcWv.Cmd(fac, 
                           func(d, e, s1, s2, s3, start, end)
                           math(d, e, cmd_name, s1, s2, s3, start, end);,
                           "_" + cmd_name + "_pr", cmd_name,
                           vcWv.Param("First Signal Name:",
                                      string, DYNAMIC, get_first_signal),
                           vcWv.Param("Second Signal Name/Constant:",
                                      string, DYNAMIC, get_next_signal),
                           vcWv.Param("New Signal Name:",
                                      string, CONSTANT, ""),
                           vcWv.Param("Starting X-axis value:",
                                      float, DYNAMIC, GetHAxisVMin,
                                      vcWv.Secondary),
                           vcWv.Param("Ending X-axis value:",
                                      float, DYNAMIC, GetHAxisVMax,
                                      vcWv.Secondary));
    
    noprompt_cmd := vcWv.Cmd(fac, func(d, e)
                             math(d, e, cmd_name,
                                  get_first_signal(d),
                                  get_next_signal(d), "", 
                                  -HUGEREAL, 
                                  HUGEREAL);,
                             "_" + cmd_name + "_nopr");

    function should_prompt(d, e, rest)
    {
        if (NumberOfSignalsSelected(d) < 2)
            vcWin.RunCmdline(vcEvent.Window(e),
                             vcCmd.Name(prompt_cmd) + " "
                             + convert(string, rest), e);
        else
            vcCmd.Run(noprompt_cmd, d, e, nil, TRUE);
    }
    return(vcWv.Cmd(fac, should_prompt, cmd_name, cmd_name,
                    vcWv.Param("", vlStringRestOfLineVT, STICKY, "")));
} /* MakeBinaryCmd */

AddCmd := MakeBinaryCmd("+");
vcCmd.EnableMacro(AddCmd, vlOnOff.Off);
SubCmd := MakeBinaryCmd("-");
vcCmd.EnableMacro(SubCmd, vlOnOff.Off);
MultCmd := MakeBinaryCmd("*");
vcCmd.EnableMacro(MultCmd, vlOnOff.Off);
DivCmd := MakeBinaryCmd("/");
vcCmd.EnableMacro(DivCmd, vlOnOff.Off);

AddBindCmd := vcWv.Cmd(fac, func(d,e) 
                       {
                           vcCmd.Run(AddCmd, d, e, list(""), FALSE); 
                       }, "_AddBind");                
SubBindCmd := vcWv.Cmd(fac, func(d,e) 
                       {
                           vcCmd.Run(SubCmd, d, e, list(""), FALSE); 
                       }, "_SubBind");                 
MultBindCmd := vcWv.Cmd(fac, func(d,e) 
                        {
                            vcCmd.Run(MultCmd, d, e, list(""), FALSE); 
                        }, "_MultBind");
DivBindCmd := vcWv.Cmd(fac, func(d,e) 
                       {
                           vcCmd.Run(DivCmd, d, e, list(""), FALSE); 
                       }, "_DivBind");

PowerCmd := MakeBinaryCmd("power");
vcCmd.EnableMacro(PowerCmd, vlOnOff.Off);
ModuloCmd := MakeBinaryCmd("modulo");
vcCmd.EnableMacro(ModuloCmd, vlOnOff.Off);
Atan2Cmd := MakeBinaryCmd("atan2");
vcCmd.EnableMacro(Atan2Cmd, vlOnOff.Off);
HypotCmd := MakeBinaryCmd("hypot");
vcCmd.EnableMacro(HypotCmd, vlOnOff.Off);
YnCmd := MakeBinaryCmd("yn");
vcCmd.EnableMacro(YnCmd, vlOnOff.Off);
JnCmd := MakeBinaryCmd("jn");
vcCmd.EnableMacro(JnCmd, vlOnOff.Off);

/* unary-operator function that makes command */
function
MakeUnaryCmd(cmd_name)
{
    prompt_cmd := vcWv.Cmd(fac, func(d, e, s1, s2, start, end)
                           math(d, e, cmd_name, s1, s2, "", start, end);,
                           "_" + cmd_name + "_pr", cmd_name,
                           vcWv.Param("First Signal Name:",
                                      string, DYNAMIC, get_first_signal),
                           vcWv.Param("New Signal Name:",
                                      vlStringRestOfLineVT, CONSTANT, 
                                      ""),
                           vcWv.Param("Starting X-axis value:",
                                      float, DYNAMIC, GetHAxisVMin,
                                      vcWv.Secondary),
                           vcWv.Param("Ending X-axis value:",
                                      float, DYNAMIC, GetHAxisVMax,
                                      vcWv.Secondary));

    noprompt_cmd := vcWv.Cmd(fac, func(d, e)
                             math(d, e, cmd_name, get_first_signal(d),
                                  "", "", -HUGEREAL, 
                                  HUGEREAL);,
                             "_" + cmd_name + "_nopr");

    function should_prompt(d, e, rest)
    {
        if (NumberOfSignalsSelected(d) < 1)
            vcWin.RunCmdline(vcEvent.Window(e),
                             vcCmd.Name(prompt_cmd) + " "
                             + convert(string, rest), e);
        else
            vcCmd.Run(noprompt_cmd, d, e, nil, TRUE);
    }
    return(vcWv.Cmd(fac, should_prompt, cmd_name, cmd_name,
                    vcWv.Param("", vlStringRestOfLineVT, STICKY, "")));
} /* MakeUnaryCmd */

MInverseCmd := MakeUnaryCmd("minverse");
vcCmd.EnableMacro(MInverseCmd, vlOnOff.Off);
ExpCmd := MakeUnaryCmd("exp");
vcCmd.EnableMacro(ExpCmd, vlOnOff.Off);
SinCmd := MakeUnaryCmd("sin");
vcCmd.EnableMacro(SinCmd, vlOnOff.Off);
SinhCmd := MakeUnaryCmd("sinh");
vcCmd.EnableMacro(SinhCmd, vlOnOff.Off);
DerivCmd := MakeUnaryCmd("deriv");
vcCmd.EnableMacro(DerivCmd, vlOnOff.Off);
J0Cmd := MakeUnaryCmd("j0");
vcCmd.EnableMacro(J0Cmd, vlOnOff.Off);
MagnitudeCmd := MakeUnaryCmd("magnitude");
vcCmd.EnableMacro(MagnitudeCmd, vlOnOff.Off);
GDelayCmd := MakeUnaryCmd("gdelay");
vcCmd.EnableMacro(GDelayCmd, vlOnOff.Off);
AInverseCmd := MakeUnaryCmd("ainverse");
vcCmd.EnableMacro(AInverseCmd, vlOnOff.Off);
LogCmd := MakeUnaryCmd("log");
vcCmd.EnableMacro(LogCmd, vlOnOff.Off);
CosCmd := MakeUnaryCmd("cos");
vcCmd.EnableMacro(CosCmd, vlOnOff.Off);
CoshCmd := MakeUnaryCmd("cosh");
vcCmd.EnableMacro(CoshCmd, vlOnOff.Off);
IntegCmd := MakeUnaryCmd("integ");
vcCmd.EnableMacro(IntegCmd, vlOnOff.Off);
J1Cmd := MakeUnaryCmd("j1");
vcCmd.EnableMacro(J1Cmd, vlOnOff.Off);
RealCmd := MakeUnaryCmd("real");
vcCmd.EnableMacro(RealCmd, vlOnOff.Off);
ImagCmd := MakeUnaryCmd("imag");
vcCmd.EnableMacro(ImagCmd, vlOnOff.Off);
AbsCmd := MakeUnaryCmd("abs");
vcCmd.EnableMacro(AbsCmd, vlOnOff.Off);
Exp10Cmd := MakeUnaryCmd("exp10");
vcCmd.EnableMacro(Exp10Cmd, vlOnOff.Off);
TanCmd := MakeUnaryCmd("tan");
vcCmd.EnableMacro(TanCmd, vlOnOff.Off);
TanhCmd := MakeUnaryCmd("tanh");
vcCmd.EnableMacro(TanhCmd, vlOnOff.Off);
AverageCmd := MakeUnaryCmd("average");
vcCmd.EnableMacro(AverageCmd, vlOnOff.Off);
DegreesCmd := MakeUnaryCmd("degrees");
vcCmd.EnableMacro(DegreesCmd, vlOnOff.Off);
CDegreesCmd := MakeUnaryCmd("cdegrees");
vcCmd.EnableMacro(CDegreesCmd, vlOnOff.Off);
SignCmd := MakeUnaryCmd("sign");
vcCmd.EnableMacro(SignCmd, vlOnOff.Off);
Log10Cmd := MakeUnaryCmd("log10");
vcCmd.EnableMacro(Log10Cmd, vlOnOff.Off);
ASinCmd := MakeUnaryCmd("asin");
vcCmd.EnableMacro(ASinCmd, vlOnOff.Off);
RMSCmd := MakeUnaryCmd("rms");
vcCmd.EnableMacro(RMSCmd, vlOnOff.Off);
Y0Cmd := MakeUnaryCmd("y0");
vcCmd.EnableMacro(Y0Cmd, vlOnOff.Off);
RadiansCmd := MakeUnaryCmd("radians");
vcCmd.EnableMacro(RadiansCmd, vlOnOff.Off);
CRadiansCmd := MakeUnaryCmd("cradians");
vcCmd.EnableMacro(CRadiansCmd, vlOnOff.Off);
SqrtCmd := MakeUnaryCmd("sqrt");
vcCmd.EnableMacro(SqrtCmd, vlOnOff.Off);
DbCmd := MakeUnaryCmd("db");
vcCmd.EnableMacro(DbCmd, vlOnOff.Off);
ACosCmd := MakeUnaryCmd("acos");
vcCmd.EnableMacro(ACosCmd, vlOnOff.Off);
MinCmd := MakeUnaryCmd("min");
vcCmd.EnableMacro(MinCmd, vlOnOff.Off);
Y1Cmd := MakeUnaryCmd("y1");
vcCmd.EnableMacro(Y1Cmd, vlOnOff.Off);
GradiansCmd := MakeUnaryCmd("gradians");
vcCmd.EnableMacro(GradiansCmd, vlOnOff.Off);
CGradiansCmd := MakeUnaryCmd("cgradians");
vcCmd.EnableMacro(CGradiansCmd, vlOnOff.Off);
CeilCmd := MakeUnaryCmd("ceil");
vcCmd.EnableMacro(CeilCmd, vlOnOff.Off);
ATanCmd := MakeUnaryCmd("atan");
vcCmd.EnableMacro(ATanCmd, vlOnOff.Off);
MaxCmd := MakeUnaryCmd("max");
vcCmd.EnableMacro(MaxCmd, vlOnOff.Off);
FloorCmd := MakeUnaryCmd("floor");
vcCmd.EnableMacro(FloorCmd, vlOnOff.Off);
ConjugateCmd := MakeUnaryCmd("conjugate");
vcCmd.EnableMacro(ConjugateCmd, vlOnOff.Off);

/* ---- here's the stuff for the math dialog ---- */

/* this variable needs to be global */
math_dialog := nil;
w := nil;

/* Some handy shortcuts for specify different types of regions */
function label(str) vcRegion.Create1(str, vcLabel.fac, str, nil);
function border(btn) vcRegion.Create1(btn, vcBorder.fac, btn, nil);
function rubber(x, y) vcRubber.Create("MathRubber", x, y);
function cmdline(default) vcRegion.Create1("MathCmdLine" + 
                                           convert(string, default),
                                           vcCmdline.fac, default,
                                           list(cons(`width, 10),
                                                cons(`macro_on, FALSE)));
function picklist(l) vcRegion.Create1("MathPickList", vcPicklist.fac, l, nil);
function macro_off(r) 
{
    vcRegion.PrefSet(r,list(macro_on <- FALSE));
    return(r);
}

function
button(name, fun)
{
    b := vcRegion.Create1("Button" + name, vcButton.fac, fun, nil);
    vcRegion.AddChild(b, label(name));
    border(b);
} /* button */

/* build up list of math  operations for creating picklist */
function m(s) cons(s, s);

math_ops := list(
list(m("+"), m("-"), m("*"), m("/"),
     m("="), m("+="), m("-="), m("*="), m("/=")),
list(cons("1/x", "minverse"), m("exp"), m("sin"), m("sinh"), m("deriv"),
     m("j0"), cons("magn", "magnitude"), m("gdelay")),
list(cons("-x", "ainverse"), m("log"), m("cos"), m("cosh"), m("integ"),
     m("j1"), m("real"), m("imag")),
list(m("abs"), m("exp10"), m("tan"), m("tanh"), cons("avg", "average"),
     m("jn"), cons("deg", "degrees"), cons("cdeg", "cdegrees")),
list(m("sign"), m("log10"), m("asin"), m("rms"), m("y0"),
     cons("rad", "radians"), cons("crad", "cradians")),
list(m("sqrt"), m("db"), m("acos"), m("min"), m("y1"),
     cons("grad", "gradians"), cons("cgrad", "cgradians")),
list(m("ceil"), m("power"), m("atan"), m("max"), m("yn")),
list(m("floor"), m("modulo"), m("atan2"), m("hypot"),
     cons("z*", "conjugate")));

function
update_math_signals(display)
{
    if (math_dialog != nil) {
        signals := vcTable.Get(math_dialog, 2, 0);
        c1 := vcRegion.Data(vcTable.Get(signals, 0, 1), vcCmdlineVT);
        c2 := vcRegion.Data(vcTable.Get(signals, 1, 1), vcCmdlineVT);
        c3 := vcRegion.Data(vcTable.Get(signals, 2, 1), vcCmdlineVT);
        vcCmdline.Set(c1, get_first_signal(display));
        vcCmdline.Set(c2, get_next_signal(display));
        vcCmdline.Set(c3, "");
    } /* if */
} /* update_math_signals */

/* This routine brings up the math dialog box */
function
display_math_dialog(display, event)
{
    declare math_operation;

    parent := vcEvent.Window(event);
    save_start := nil;
    save_end := nil;

    /* set the command line value whenever the user picks a math operation */
    function set_operation(val, on_off)
    {
        if (on_off == vlOnOff.On)
            math_operation = val;
        vcWin.ClearStatus(parent);
    } /* set_operation */

    function do_math_operation()
    {
        signals := vcTable.Get(math_dialog, 2, 0);
        if (math_operation != nil) {
            s1 := vcRegion.Get(vcTable.Get(signals, 0, 1), string);
            s2 := vcRegion.Get(vcTable.Get(signals, 1, 1), string);
            s3 := vcRegion.Get(vcTable.Get(signals, 2, 1), string);
            if (save_start == nil)
                s4 := -HUGEREAL;
            else
                s4 := vcRegion.Get(vcTable.Get(signals, 3, 1), float);
            if (save_end == nil)
                s5 := HUGEREAL;
            else
                s5 := vcRegion.Get(vcTable.Get(signals, 4, 1), float);

            /* RSM - run appropriate math command in order to record it */
            if (string_length(s3) == 0) {
                if (string_length(s2) == 0)
                    vcCmd.Run(Math1Cmd, display, event,
                              list(math_operation, s1, s4, s5), TRUE);
                else
                    vcCmd.Run(Math2Cmd, display, event,
                              list(math_operation, s1, s2, s4, s5), TRUE);
            } /* if */
            else if (string_length(s2) == 0)
                vcCmd.Run(Math4Cmd, display, event,
                          list(math_operation, s1, s3, s4, s5), TRUE);
            else
                vcCmd.Run(Math3Cmd, display, event,
                          list(math_operation, s1, s2, s3, s4, s5), TRUE);
        } /* if */
        else {
            vcWin.ClearStatus(parent);
            vcWin.AddStatus(parent, "Please select a math function");
        } /* else */
        return(0);
    } /* do_math_operation */

    function accept(/*d, e*/)
    {
/*
        if (do_math_operation() == 0)
          vcWin.Close(w);
*/
        do_math_operation();
    } /* accept */

    /* if symbols_dialog doesn't exist then create it */
    if (math_dialog == nil) {

        function check_start_value(value) {
            save_start = value;
        } /* check_start_value */

        function check_end_value(value) {
            save_end = value;
        } /* check_end_value */

        /* create the main symbols dialog box table */
        math_dialog = vcTable.Create("Math Operations", 5, 1);
        macro_off(math_dialog);

        /* create buttons */
        buttons := vcRegion.Create("Buttons", vcButnRow.fac,
                                   list (list ("Accept", 
                                               "Accept",
                                               accept, 
                                               list (close_window <- 
                                                     TRUE)),
                                         list ("Apply", 
                                               "Apply",
                                               do_math_operation,
                                               nil),
                                         list ("Cancel",
                                               "Cancel",
                                               func() vcWin.Close(w);,
                                               nil)));
        
        /* bob adds this to handle return in last cmd line [3/27/92] */

        /* create picklist of paths */
        picklist := picklist(math_ops);
        macro_off(picklist);
        vcPicklist.Notify(picklist, set_operation);
        scroll_picklist := vcScroll.Create(picklist, 20, 20, nil);

        signals := vcTable.Create("Signals", 3, 2);
        macro_off(signals);
        vcTable.Add(signals, label("Signal1:"));
        vcTable.Add(signals, cmdline(get_first_signal(display)));
        vcTable.Add(signals, label("Signal2/Constant:"));
        vcTable.Add(signals, cmdline(get_next_signal(display)));
        vcTable.Add(signals, label("New Signal:"));
        {
            NewSigCmdline := cmdline("");
            vcTable.Add(signals, NewSigCmdline);
            vcCmdline.CallOnReturn(NewSigCmdline,
                                   func(event, str) { accept(); });
        }
        vcTable.Add(signals, label("Starting X-axis value:"));
        {
            type := HAxisType(display, 0);
            value := HAxisVMin(display, 0);
            if (type == vtrDispType.Log && value > 0.0)
                value = exp10(value);
            vcTable.Add(signals, StartCmdline := cmdline(value));
            vcCmdline.CallOnChange(StartCmdline, check_start_value);
        }
        vcTable.Add(signals, label("Ending X-axis value:"));
        {
            type := HAxisType(display, 0);
            value := HAxisVMax(display, 0);
            if (type == vtrDispType.Log && value > 0.0)
                value = exp10(value);
            vcTable.Add(signals, EndCmdline := cmdline(value));
            vcCmdline.CallOnChange(EndCmdline, check_end_value);
        }
        
        /* add regions to main dialog box region */
        vcTable.Add(math_dialog, scroll_picklist);
        vcTable.Add(math_dialog, rubber(0, 5));
        vcTable.Add(math_dialog, signals);
        vcTable.Add(math_dialog, buttons);

        /* Create a window with no cmdline and no status */
        w = vcWin.Create(parent, math_dialog, vcWin.NoCmdline + vcWin.NoStatus);

        /* if main window closed and dialogs still opened, close it */
        vcWin.DestroyNotify(vcEvent.Window(event),
                            func(a)
                            {
                                if (math_dialog != nil)
                                    vcWin.Close(w);
                            } );

        /* set the math_dialog box to nil if window closed */
        vcWin.DestroyNotify(w, func(a) math_dialog = nil;);
    } /* if */
    else
        /* If it already exists, show it */
        vcWin.Show(w);
} /* display_math_dialog */

SignalMathCmd := vcWv.Cmd(fac, display_math_dialog, "math");
vcCmd.EnableMacro(SignalMathCmd, vlOnOff.Off);

/* RSM - 11/26/91 These commands are needed in order to record the operation */
Math1Cmd := vcWv.Cmd(fac, func(d,e,o,s1,start,end) 
                     math(d,e,o,s1,"","",start,end);,
                     "math1","Math Operation",
                     vcWv.Param("Operation",string,STICKY,""),
                     vcWv.Param("Signal",string,STICKY,""),
                     vcWv.Param("Start",float,STICKY,-1,vcWv.Secondary),
                     vcWv.Param("End",float,STICKY,-1,vcWv.Secondary));
Math2Cmd := vcWv.Cmd(fac,func(d,e,o,s1,s2,start,end) 
                     math(d,e,o,s1,s2,"",start,end);,
                     "math2","Math Operation",
                     vcWv.Param("Operation",string,STICKY,""),
                     vcWv.Param("First Signal",string,STICKY,""),
                     vcWv.Param("Second Signal",string,STICKY,""),
                     vcWv.Param("Start",float,STICKY,-1,vcWv.Secondary),
                     vcWv.Param("End",float,STICKY,-1,vcWv.Secondary));
Math3Cmd := vcWv.Cmd(fac,func(d,e,o,s1,s2,s3,start,end) 
                     math(d,e,o,s1,s2,s3,start,end);,
                     "math3","Math Operation",
                     vcWv.Param("Operation",string,STICKY,""),
                     vcWv.Param("First Signal",string,STICKY,""),
                     vcWv.Param("Second Signal",string,STICKY,""),
                     vcWv.Param("Resultant Signal",string,STICKY,""),
                     vcWv.Param("Start",float,STICKY,-1,vcWv.Secondary),
                     vcWv.Param("End",float,STICKY,-1,vcWv.Secondary));
Math4Cmd := vcWv.Cmd(fac,func(d,e,o,s1,s3,start,end) 
                     math(d,e,o,s1,"",s3,start,end);,
                     "math4","Math Operation",
                     vcWv.Param("Operation",string,STICKY,""),
                     vcWv.Param("Signal",string,STICKY,""),
                     vcWv.Param("Resultant Signal",string,STICKY,""),
                     vcWv.Param("Start",float,STICKY,-1,vcWv.Secondary),
                     vcWv.Param("End",float,STICKY,-1,vcWv.Secondary));

/*
function after(display, event)
{
    update_math_signals(display);
}*/ /* after */
