/*
 *  $Revision: 1.61.1.2 $
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 */


/*
 * Define the message in a string variable "msg".  Note the use of
 * := to both declare and assign the variable in one step.  Also,
 * note the multiple line string constant.
 */
msg :=
"                                      Welcome to Powerview Core!

This demonstration is designed to introduce you to the basic concepts
of programming in Powerview Core, using ViewScript.

There are several demonstrations prepared.  Each is implemented using
ViewScript.  Select a demo below, and then press \"Run\" or \"Show\".";

/*
 * Auxiliary function used in demo array below.  This function builds
 * a function that loads a facility and creates a window for it, using
 * the specified argument and window flags.
 */
function do_fac(name, arg, window_flags) {
    function foo() {
        fac := vcFac.Create (name, vscmVT);
        vcWin.Create (vcRegion.Create1
                      ("demo",                      /* Name of region */
                       fac,                         /* Facility ID */
                       arg,                         /* Arg to facility */
                       list (macro_on <- TRUE)),    /* Customizations */
                      window_flags);
    } /* func */

    foo;

} /* function do_fac */

/*
 * We're going to create an array of demos.  Each demo is represented
 * by a list of three objects:  the button label, the filename (without
 * extension) that should be used for examining the demo, and an action
 * object, which is either a ViewScript filename to be loaded, or a
 * function to be run.
 *
 *                   Button Label       Filename        Action
 *                   ------------       --------        ------
 */
row0 := list (list ("Hello World",     "hello",        "hello"),
              list ("Menus",           "menu",         do_fac("menu",nil,
                                                             vcWin.NoCmdline)),
              list ("Icon",            "icon",         "icon"),
              list ("Numerical Scale", "scale",        "scale"));

row1 := list (list ("Pick from list",  "picklist",     "picklist"),
             list ("Drawing Editor",  "draw",         do_fac("draw",nil,0)),
              list ("Run Demo",        "rundemo",      "rundemo"),
              list ("Demo Code",       "demo",         "rundemo"));

row2 := list (list ("File List",       "file",         "file"),
              list ("Hide 1",          "hide1",        "hide1"),
              list ("Hide 2",          "hide2",        "hide2"),
              list ("Typein",          "typein",       "typein"));

row3 := list (list ("Table",           "table",        "table"),
              list ("Command",         "command",      "command"),
              list ("C-Shell",         "csh",          do_fac("csh",nil,0)),
              list ("Arrows",          "arrows",       "arrows"));

row4 := list (list ("Events",          "events",      do_fac("events",nil,0)),
              list ("Table Insert/Delete",     "tabinsdl",     "tabinsdl"),
              list ("Cursors",         "cursors",      "cursors"),
              list ("Colors",          "colors",       "colors"));

row5 := list (list ("Clock",           "clock",        "clock"),
              list ("Analog Clock",    "aclock",       "aclock"),
              list ("Temperature",     "temp",         "temp"),
              list ("Backing Store",   "bstore",       "bstore"));

row6 := list (list ("Ipc",             "vipctst",  do_fac("vipctst",nil,0)),
              list ("Cube",            "cube",         "cube"),
              list ("Histogram",       "hist",do_fac("hist","/etc/passwd",0)),
              list ("Fonts",           "fonts",        "fonts"));

row7 := list (list ("List Editor",     "listedit",     "listedit"),
              list ("Pick Param",      "pickedit", do_fac ("pickedit", nil,
                                                           vcWin.NoBindings)),
              list ("Confirm dialog",  "dialogs",      "dialogs"),
              list ("Margins",         "margins",      "margins"));

row8 := list (list ("Drag/Drop",       "rundnd",        "rundnd"),
              list ("Toggle Btns",     "toggle",        "toggle"),
              list ("Notebook",        "notebook",      "notebook"),
              list ("Browse",          "browse",    do_fac ("browse",nil,0)));

demos := list (row0,
               row1,
               row2,
               row3,
               row4,
               row5,
               row6,
               row7,
               row8);

demo_filename := car;
demo_action := cadr;

function _Create(data, region) {
    /*
     * Create a facility instance using the generic creation procedure
     * vcRegion.Create1.  This creates an area within a window under
     * the control of a facility called "StaticText", which displays
     * multiple-lines of text.  Viewtext could have been used to display
     * this as well, but it this is more efficient if the text is
     * read-only, and can also display in a nicer font.  Viewtext
     * only works with fixed-width fonts.
     *
     * Creating this region does not cause anything to be shown on the
     * screen.  This region is later combined with other regions, and
     * a window is created for the group of regions, that appears on the
     * screen.
     */
    welcome_msg := vcRegion.Create1 ("msg",             /* Name of region */
                                     vcStaticText.fac,  /* Facility name */
                                     msg,               /* Contents of fac */
                                     nil);              /* Customization */

    /*
     * Note that we are nesting the welcome_msg region inside the region
     * controlled by the vcMargin facility.  That facility adds a few pixels
     * of background color around the text, so that when it is displayed, the
     * text doesn't show up squashed against the borders.
     */
    margin := vcRegion.Create ("msg_margin", vcMargin.fac, welcome_msg);

    /*
     * Now we're going to create some buttons for each of the demo
     * choices.  A button for each demo will be arrayed horizontally
     * at the bottom of the window.  Hitting the left button on a
     * demo will select it, making it the "current" demo.  Another
     * array of buttons will give the four action items:  "Run
     * ViewScript", "Show ViewScript", "Run C", and "Show C".  The
     * user hits the buttons he wants, and the software below makes
     * it happen!
     */
    choices := vcRegion.Create1 ("choices", vcPicklist.fac, demos,
                                 list (macro_on <- TRUE));

    the_table := vcTable.Create ("Core Demo", 3, 1);
    vcTable.Add (the_table, vcScroll.Create (margin, 10, 10, nil));
    vcTable.Add (the_table, choices);
    run_cmd := vcWv.Cmd (fac, runpack.run, "Run");
    vcCmd.SetHelp (run_cmd, "Execute the selected demo");
    vcCmd.SetTip (run_cmd, "Run the demo");
    show_cmd := vcWv.Cmd (fac, show, "Show");
    vcCmd.SetHelp (show_cmd, "Display the source code of the selected demo");
    vcCmd.SetTip (show_cmd, "Display source");
    vcTable.Add (the_table,
                 vcRegion.Create ("button row", vcButnRow.fac,
                                  list (list ("Run", "Run",
                                              run_cmd,
                                              nil),
                                        list ("Show", "Show",
                                              show_cmd,
                                              nil))));
    vcTable.Row (the_table, 2, vcTable.Border.Visible /* Border.3d */,
                 1, 1, 1, vgAlign.UpperLeft);
    vcRegion.AddChild (region, the_table);

    /*
     * Return `choices' as the data object to be associated with this
     * facility instance.
     */
    choices;

} /* function _Create */


/*
 * The filename associated with a current demo state is the current
 * demo name, which is the current string choice in the picklist in
 * choices, plus the extension, which is passed in.
 */
function filename(choices, ext)
    vscript.vsGetini (demo_filename (vcRegion.Get (choices, vscmVT)) + ext);

/*
 * Bring up a viewtext window on the file.  At some point, we need to
 * set this file read only, which Viewtext can do, but not from ViewScript.
 */
function show(d, event) {
    vt := vcRegion.Create1 ("Show demo", vtx.fac, filename (d, ".vs"),
                            list (load_file <- vlYesNo.Yes, macro_on <- TRUE));
    vtx.ReadOnly (vt, TRUE);
    vcWin.Create (vt, 0x0);
}

/*
 * Read and execute the ViewScript demo name, using the environment defined
 * by the runpack package.
 */
package runpack(run) 
{
    function run(choices, event) {
        obj := demo_action (vcRegion.Get (choices, vscmVT));

        /* If the object is a string, assume its a filename and load it */
        if (istype (obj, vlStringVT))
            load (obj + ".vs");

        /* Otherwise, assume the object is a function and invoke it */
        else
            obj ();
    } /* function run */
}

/*
 * Construct a menu offering the Quit, Show, and Run options.
 */
menus := vcWv.MakeMenu ("demo",
                         vcWin.RedSquareEntryNoPlot);

/*
 * Set this up as the standard menu bar for the facility.
 */
vcFac.SetMenus (fac, menus);
