/*
 *  $Revision: 1.5.7.1 $
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

/*
 * This demonstrates how two regions can alternately occupy a single area,
 * using the vcOverlay facility.  Two labeled buttons are created, and they
 * are both added as children of an overlay region.  vcOverlay.SetCurrent is
 * called immediately to indicate which button is shown first.  Note how the
 * window is made large enough to accomodate either button.  Also note that
 * explicit reframing and refreshing does not need to occur as it does in the
 * "Hide 1" example.
 */

overlay := vcRegion.Create ("Hide2", vcOverlay.fac, vgAlign.MiddleCenter);

/*
 * Note that the two functions are written as action routines which will be
 * passed in as arguments to the vcWv.Cmd function.  Thus, both the `hide2'
 * function and the `show2' function take two required arguments, the facility
 * data object and an event object.  Both arguments are ingored in our case,
 * but they need to be there since Core *will* always pass in these arguments
 * to any action routine.
 */

function hide(win, event)
    vcOverlay.SetCurrent (overlay, button2);

hide_cmd := vcWv.Cmd (vcWin.fac, hide, "hide2");

    
function show(win, event)
    vcOverlay.SetCurrent (overlay, button1);

show_cmd := vcWv.Cmd (vcWin.fac, show, "show2");


button1 := vcWv.LabelButtonCreate ("button1", "Button 1", nil, hide_cmd, nil);
button2 := vcWv.LabelButtonCreate ("button2", "Button Two", nil, show_cmd,
                                   nil);

vcRegion.AddChild (overlay, button1);
vcRegion.AddChild (overlay, button2);
vcOverlay.SetCurrent (overlay, button1);


vcWin.Create (overlay, vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
