/*
 *  $Revision: 1.14.1.1 $
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 *
 *  Shows how to build menus.  This is done bottom-up, because the sub-menus
 *  are passed in as the "action" items for the top level menu.
 */


/*
 * Define msg_function, a function which generates a function that
 * prints a message to the status window of an event when called.
 * This function is used to generate functions that display a particular
 * string when called.  Note that the argument "str" is visible to
 * the returned function "display_msg", even after "msg_function"
 * returns.  This is due to the unlimited extent of Viewscript objects
 * and the lexical scoping of its names.
 */
function msg_function(str) {
    function display_msg(menu_stuff, event) {
        window := vcEvent.Window (event);
        vcWin.ClearStatus (window);
        vcWin.AddStatus (window, str);
    }

    display_msg;
}

/*
 * First, define a few commands that just echo to the status window
 * the fact that they have been run.  These will be used for the
 * menu leaves.
 */
cmd1 := vcWv.Cmd (fac, msg_function ("Cmd1"), "Cmd1");
/* vcCmd.SetDisabledMsg (cmd1, "Command1 doesn't work."); */

cmd2 := vcWv.Cmd (fac, msg_function ("Cmd2"), "Cmd2");
vcCmd.SetDisabledMsg (cmd2, "I can't figure out how to do Command2.");

cmd3 := vcWv.Cmd (fac, msg_function ("Cmd3"), "Cmd3");
vcCmd.SetDisabledMsg (cmd3, "Show me $1M and I'll show you Command 3.");

disabled := vcWv.Cmd (fac, func (m,e);, "Disabled");
vcCmd.Enable (disabled, vlOnOff.Off);

function toggle_state(d, e, state) {
    w := vcEvent.Window (e);
    vcWin.ClearStatus (w);
    vcWin.AddStatus (w, sprintf ("Toggle state = %s\n",
                                 convert (vlStringVT, state)));
} /* function toggle_state */
toggle_cmd := vcWv.Cmd (fac, toggle_state, "toggle", "Toggle State",
                        vcWv.Param ("State", vlOnOffVT, vcWv.Sticky, TRUE));

menu1 := vcWv.MakeMenu ("Menu1",
                         list ("&Alpha", cmd1),
                         vcWv.CheckedOption ("Checked Option", toggle_cmd,
                                             TRUE),
                         list ("&Disabled", disabled));

menu2 := vcWv.MakeMenu ("Menu2",
                         list ("&Beta",  cmd2),
                         list ("&Gamma", menu1));

pinup := vcWv.MakeMenu ("Pinned Menu",
                         vcWv.Pin (),
                         list ("&This",  cmd1),
                         list ("&Menu",  cmd2),
                         list ("&Is",    cmd3),
                         vcWv.Separator (),
                         list ("&Pinned", cmd1));                         

icon_menu := vcWv.MakeMenu ("Icon Menu",
                            list (list (vcWv.Icon ("empty"), cmd1),
                                  list (vcWv.Icon ("full"), cmd2)),
                            list (list (vcWv.Icon ("terminal"), cmd3),
                                  list (vcWv.Icon ("letters"), cmd1)));


function dis(c, val)
    func(d,e) {
        vcCmd.Enable (c, val);
        vcWin.UpdateMenus (vcEvent.Window (e));
    };

disable :=
    vcWv.MakeMenu ("Disable",
                  list ("&1", vcWv.Cmd (fac, dis (cmd1, vlOnOff.Off), "Dis1")),
                  list ("&2", vcWv.Cmd (fac, dis (cmd2, vlOnOff.Off), "Dis2")),
                  list ("&3", vcWv.Cmd (fac, dis (cmd3,vlOnOff.Off), "Dis3")));

enable_menu :=
    vcWv.MakeMenu ("Enable",
                   list ("&1", vcWv.Cmd (fac, dis (cmd1, vlOnOff.On), "En1")),
                   list ("&2", vcWv.Cmd (fac, dis (cmd2, vlOnOff.On), "En2")),
                   list ("&3", vcWv.Cmd (fac, dis (cmd3,vlOnOff.On), "En3")));

odd_menu :=
    vcWv.MakeMenu ("Odd Entries",
                 list (list ("&Odd", cmd1)),
                 list (list ("&Number", cmd2), list ("Of &Choices", cmd3)));
                                 
menu3 := vcWv.MakeMenu ("Menu3",
                        list ("&Delta",      cmd3),
                        list ("&Epsilon",    menu2),
                        list ("&Icon",       icon_menu),
                        list ("&Pinup",      pinup),
                        list ("&Odd",        odd_menu),
                        list ("&Close",      vcWin.wclose),
                        list ("Di&sable",    disable),
                        list ("&Enable",     enable_menu));

/*
 * Now, make this menu the main menu for the facility
 */
vcFac.SetMenus (fac, menu3);

/*
 * Also, make the same menu pop-up when the user hits the right button.
 * The variable "bindings" is automatically initialized by Core for
 * each facility.  It holds the binding table that is used when the
 * input focus is set to the facility's region.  The auxiliary function
 * popup_menu is created for clarity, although for brevity and speed
 * we'd have ordinarily used "func(s, e) vcMenuPop.create (menu3, e);"
 * in place of "popup_menu" in the call to vcWv.Cmd below.
 */
function popup_menu(stuff, event)
    vcMenuPop.Create (menu3, event);


vcBind.Set (bindings, '\rightdown', vcWv.Cmd (fac, popup_menu));

/*
 * Provide the facility creation method.  There's no data associated with
 * this facility, so we'll just return nil.
 */
function _Create(obj, region) {
    msg := vcRegion.Create1 ("msg",                     /* Name of region */
                             vcLabel.fac,               /* Facility name */
                             "Press RightDown",         /* Contents of fac */
                             list (macro_on <- TRUE));  /* Customizations */
    vcRegion.AddChild (region, msg);
    nil;
}
