/*
 *  $Revision: 1.1.5.1 $
 *
 *  Using the vcPicklist facility to edit command parameters.
 *
 *  Copyright (c) 1993 by Viewlogic Systems, Inc.
 */


/*
 * The build_dflt_picklist functions define the shape of the picklist.  Both
 * use the list function to build a 2-D representation of the picklist.  For
 * more info on picklist creation, see the vcPicklist. man page.
 */
function build_dflt_picklist1(data)
    list (list ("Hungary",  "Budapest"),
          list ("Brasil",   "Brasil"),
          list ("Canada",   "Ottawa"),
          list ("Mongolia", "Ulan Bator"),
          list ("Sudan",    "Khartum"),
          list ("New Zeland", "Wellington"));


function build_dflt_picklist2(data)
    list (list ("Minsk",    "Belarus"),
          list ("Santiago", "Chile"),
          list ("Managua",  "Nicaragua"),
          list ("Islamabad","Pakistan"),
          list ("Nairobi",  "Kenya"),
          list ("Djakarta", "Indonesia"));


/*
 * Both the display_capital and display_notify functions perform the same
 * task: displaying the input parameter in the status window.
 * NOTE: the _Get method for the vcPicklist routine returns a ViewScript list
 * of selected entries.  Since we know that our picklist has only one column
 * and is exclusive, that list must have only one entry.
 */
function display_capital(data, event, capital)
{
    win := vcEvent.Window (event);
    vcWin.ClearStatus (win);
    vcWin.AddStatus (win, sprintf ("The capital is %s", car (capital)));
}

function display_nation(data, event, nation)
{
    win := vcEvent.Window (event);
    vcWin.ClearStatus (win);
    vcWin.AddStatus (win, sprintf ("It's the capital of %s", car (nation)));
}


/*
 * Both commands use the vcPicklist facility as the parameter editor.  Note
 * the usage of the vcWv.Dynamic default strategy to build the default
 * picklist and allow the user to make a choice.
 */
capital_cmd := vcWv.Cmd (fac, display_capital, "capital", "Capital",
                         vcWv.Param ("What's the capital of", vscmVT,
                                     vcWv.Dynamic, build_dflt_picklist1,
                                     vcPicklist.fac, exclusive <- TRUE));

nation_cmd := vcWv.Cmd (fac, display_nation, "nations", "Nations",
                        vcWv.Param ("Which nation's capital is this?", vscmVT,
                                    vcWv.Dynamic, build_dflt_picklist2,
                                    vcPicklist.fac, exclusive <- TRUE));


quiz_menu := vcWv.MakeMenu ("quiz",
                            list ("Capitals",   capital_cmd),
                            list ("Nations",    nation_cmd));

main_menu := vcWv.MakeMenu ("main",
                            vcWin.RedSquareEntryNoPlotNoQuit,
                            list ("Quiz", quiz_menu));

vcFac.SetMenus (fac, main_menu);


/*
 * This demo is a separate facility, thus it must have a _Create method.  This
 * facility is needed only to show how the commands can be invoked.
 */
function _Create(data, region)
{
    msg := "This demo shows the use of the vcPicklist facility as an editor
facility for a command parameter.  You can see it when you select
any of the menu choices in the Quiz menu.";

    reg := vcRegion.Create ("vcPicklist As Editor", vcStaticText.fac, msg);
    vcRegion.AddChild (region, reg);
    reg;
}
